﻿/*********************************************************************************

	RosettaFioSie.c

	Alphabet Rosetta

	(c)Copyright 2017-2020  Alphabet AB, Sweden (Jan Bergstrom)

	http://www.alphabet.se

	Licensed under the Apache License, Version 2.0 (the "License");
	you may not use this file except in compliance with the License.
	You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.

	We ask for attribution to us, when used.

******************************************************************************************

The RosettaFioSie.c file contains functions to interpret and write SIE-files (www.sie.se)
into the Alphabet Rosetta Accounting basic data logics and database system.

The RosettaFioSie.h file contains the external declarations and definitions.

Content of RosettaFioSie.c
1.) Interpret SIE file data
2.) Create SIE file data
IB
**********************************************************************************/
#include "RosettaFioTypes.h"

/********************************************************************

Interpret SIE file data

*********************************************************************/

extern char *CopyString(char **ppTarget, const char *pData);

char TaxYear[32] = { 0 };

extern char TextRefNote[];
extern char TextRefError[];
extern char TextRefOptional[];
extern char TextRefMandatory[];
extern char TextRefDefinition[];
extern char TextRefJustSpaces[];
extern char TextRefExtField[];
extern char TextRefExtTag[];
extern char TextRefNoSieData[];

char SIE_0[3] = { "0\0\0" };
char SIE_1[3] = { "1\0\0" };
char SIE_2[3] = { "2\0\0" };
char SIE_3[3] = { "3\0\0" };
char SIE_4E[3] = { "4E\0" };
char SIE_4I[3] = { "4I\0" };

char Ill_SIE_0[] = { "Is not allowed for SIE 0" };
char Ill_SIE_1[] = { "Is not allowed for SIE 1" };
char Ill_SIE_2[] = { "Is not allowed for SIE 2" };
char Ill_SIE_3[] = { "Is not allowed for SIE 3" };
char Ill_SIE_4E[] = { "Is not allowed for SIE 4E" };
char Ill_SIE_4I[] = { "Is not allowed for SIE 4I" };

#ifndef CP_ACP
#define CP_ACP							0		   // default to ANSI code page
#endif
#ifndef CP_OEMCP
#define CP_OEMCP						1		   // default to OEM  code page
#endif
#ifndef CP_UTF8
#define CP_UTF8							65001	   // UTF-8 translation
#endif

//#define LANGUAGE						psSieParam->pAccounting->pDefaultLanguage
#define LANGUAGE						0

#define MB_OK							0x00000000L
#define MB_OKCANCEL						0x00000001L
#define MB_ABORTRETRYIGNORE				0x00000002L
#define MB_YESNOCANCEL					0x00000003L
#define MB_YESNO						0x00000004L
#define MB_RETRYCANCEL					0x00000005L

#define FIELD_SEPARATOR					0x09
#define ASCII_TAB						0x09

#define MAX_COMMAND_FIELDS				8 // Number of parameters of a SIE-line except the Command tag

#define TA_MODE_OFF						0 // Nothing
#define TA_MODE_PENDING					1 // After Verification
#define TA_MODE_ACTIVE					2 // After { after Verification (Ready to accept Transactions)

#define SIE_PARAM_STATUS_0				0x00000001
#define SIE_PARAM_STATUS_1				0x00000010
#define SIE_PARAM_STATUS_2				0x00000100
#define SIE_PARAM_STATUS_3				0x00001000
#define SIE_PARAM_STATUS_4				0x00010000
#define SIE_PARAM_STATUS_5				0x00100000
#define SIE_PARAM_STATUS_6				0x01000000
#define SIE_PARAM_STATUS_7				0x10000000

#define SIE_PARAM_STATUS_ALL			0x11111111

#define SIE_PARAM_STATUS_MAX			SIE_PARAM_STATUS_7
#define OBJECT_SEPARATOR				0x1e // For multiple objects in #TRANS
#define OBJECT_SET_SEPARATOR			0x1f // For multiple objects in #TRANS

#define DT_ANY							0
#define DT_ALPHA						1
#define DT_UNSIGNED						2
#define DT_NUMERIC						3
#define DT_AMOUNT						4
#define DT_QUANTITY						5
#define DT_DATE							6
#define DT_PERIOD						7
#define DT_YEAR							8
#define DT_YEAR_CODE					9
#define DT_OBJECT						10

#define INTERPRET_SYNTAX_NONE			0x00000000
#define INTERPRET_SYNTAX_ALL			0xff000000
#define INTERPRET_SYNTAX_ADRESS			0x10000000
#define INTERPRET_SYNTAX_BKOD			0x11000000
#define INTERPRET_SYNTAX_DIM			0x12000000
#define INTERPRET_SYNTAX_ENHET			0x13000000
#define INTERPRET_SYNTAX_FLAGGA			0x14000000
#define INTERPRET_SYNTAX_FNAMN			0x15000000
#define INTERPRET_SYNTAX_FNR			0x16000000
#define INTERPRET_SYNTAX_FORMAT			0x17000000
#define INTERPRET_SYNTAX_GEN			0x18000000
#define INTERPRET_SYNTAX_IB				0x19000000
#define INTERPRET_SYNTAX_KONTO			0x20000000
#define INTERPRET_SYNTAX_KPTYP			0x21000000
#define INTERPRET_SYNTAX_KTYP			0x22000000
#define INTERPRET_SYNTAX_OBJEKT			0x23000000
#define INTERPRET_SYNTAX_OIB			0x24000000
#define INTERPRET_SYNTAX_OMFATTN		0x25000000
#define INTERPRET_SYNTAX_ORGNR			0x26000000
#define INTERPRET_SYNTAX_OUB			0x27000000
#define INTERPRET_SYNTAX_PBUDGET		0x28000000
#define INTERPRET_SYNTAX_PROGRAM		0x29000000
#define INTERPRET_SYNTAX_PROSA			0x30000000
#define INTERPRET_SYNTAX_PSALDO			0x31000000
#define INTERPRET_SYNTAX_RAR			0x32000000
#define INTERPRET_SYNTAX_RES			0x33000000
#define INTERPRET_SYNTAX_SIETYP			0x34000000
#define INTERPRET_SYNTAX_SRU			0x35000000
#define INTERPRET_SYNTAX_TAXAR			0x36000000
#define INTERPRET_SYNTAX_TRANS			0x37000000
#define INTERPRET_SYNTAX_UB				0x38000000
#define INTERPRET_SYNTAX_UNDERDIM		0x39000000
#define INTERPRET_SYNTAX_VER			0x40000000
#define INTERPRET_SYNTAX_FTYP			0x41000000
#define INTERPRET_SYNTAX_VALUTA			0x42000000
#define INTERPRET_SYNTAX_RTRANS			0x43000000
#define INTERPRET_SYNTAX_BTRANS			0x44000000
#define INTERPRET_SYNTAX_NON_SIE_DATA	0x98000000
#define INTERPRET_SYNTAX_EXT_TAG		0x99000000

#define INTERPRET_SYNTAX_OK				0
#define INTERPRET_SYNTAX_ERR_MANDATORY	1 // Mandatory data field is missing
#define INTERPRET_SYNTAX_ERR_DEFINE		2 // Double of by demand no double id unit
#define INTERPRET_SYNTAX_ERR_NOTE		4 // Double of by demand no double id unit
#define INTERPRET_SYNTAX_EXT_FIELD		8 // Something necessary is not defined
#define INTERPRET_SYNTAX_MASK_ALL		0xf // System error no memory

#define INTERPRET_SYNTAX_ERR_FATAL		(BITMASK) (-1) // System error no memory

static VoucherType *pVoucher = 0;
//static char NilTitle[]={"-"};

typedef struct {
	BOOL		bPeriodicBalancesExists;
	BOOL		bDimObjExists;
	BOOL		bDimObjBalancesExists;
	BOOL		bLedgerEntriesExists;
	BOOL		bQuantitiesExists;
	BOOL		bBudgetExists;
	int			FileType;
	char		*AccountChartType;
	NUMERIC		iSieFileLines;
	NUMERIC		iSieFileErrorLines;
	LLIST		pListFileReadResultLines;		// Sie Test Results
} AuxType;

typedef struct {
	NUMERIC	dwLineNumber;
	//	char	*pRawTagLine;
	char	*pCommandFields[MAX_COMMAND_FIELDS];
	NUMERIC	dwRefErrorLine;
	NUMERIC	dwErrorLine[MAX_COMMAND_FIELDS];
	//	char	*pErrorLines[MAX_COMMAND_FIELDS][MAX_ERROR_FIELDS];
} FileTestLinesType;

AMOUNT BalanceAmount = 0;

typedef struct {
	BOOL FLAGGA;
	BOOL PROGRAM;
	BOOL FORMAT;
	BOOL GEN;
	BOOL FNAMN;
	BOOL RAR;
	BOOL OMFATTN;
	BOOL KONTO;
	BOOL KTYP;
	BOOL SRU;
	BOOL OBJEKT;
	BOOL IB;
	BOOL UB;
	BOOL RES;
	BOOL OIB;
	BOOL OUB;
	BOOL PBUDGET;
	BOOL PSALDO;
} SIEMANDATORYTYPE;

typedef struct {
	NUMERIC	dwLineNumber;
	//	char *pRawTagLine;
	char *pFields[MAX_COMMAND_FIELDS];
	AccountingType *pAccounting;
	BITMASK bmCommand;
	BITMASK bmResult;
	BITMASK bmExtResult;
	char *Message[MAX_COMMAND_FIELDS];
	SIEMANDATORYTYPE *SieMandatory;
} SIEPARAMTYPE;


static BITMASK SieInterpretation(char *pSieTabRecord, SIEPARAMTYPE *psSieParam);
static void SieSyntax(char *pSieTabRecord);
static BITMASK SieContent(SIEPARAMTYPE *psSieParam, int Content);

static BITMASK	SieCommandADRESS(SIEPARAMTYPE *psSieParam, int Content);
static BITMASK	SieCommandBKOD(SIEPARAMTYPE *psSieParam, int Content);
static BITMASK	SieCommandDimension(SIEPARAMTYPE *psSieParam, int Content);
static BITMASK	SieCommandENHET(SIEPARAMTYPE *psSieParam, int Content);

static BITMASK	SieCommandFLAGGA(SIEPARAMTYPE *psSieParam, int Content);
static BITMASK	SieCommandFNAMN(SIEPARAMTYPE *psSieParam, int Content);
static BITMASK	SieCommandFNR(SIEPARAMTYPE *psSieParam, int Content);
static BITMASK	SieCommandFORMAT(SIEPARAMTYPE *psSieParam, int Content);
static BITMASK	SieCommandGEN(SIEPARAMTYPE *psSieParam, int Content);

static BITMASK	SieCommandKONTO(SIEPARAMTYPE *psSieParam, int Content);
static BITMASK	SieCommandKPTYP(SIEPARAMTYPE *psSieParam, int Content);
static BITMASK	SieCommandKTYP(SIEPARAMTYPE *psSieParam, int Content);
static BITMASK	SieCommandOBJEKT(SIEPARAMTYPE *psSieParam, int Content);
static BITMASK	SieCommandBalance(SIEPARAMTYPE *psSieParam, int Content);
static BITMASK	SieCommandOMFATTN(SIEPARAMTYPE *psSieParam, int Content);

static BITMASK	SieCommandORGNR(SIEPARAMTYPE *psSieParam, int Content);
static BITMASK	SieCommandPROGRAM(SIEPARAMTYPE *psSieParam, int Content);
static BITMASK	SieCommandPROSA(SIEPARAMTYPE *psSieParam, int Content);

static BITMASK	SieCommandRAR(SIEPARAMTYPE *psSieParam, int Content);
static BITMASK	SieCommandSIETYP(SIEPARAMTYPE *psSieParam, int Content);
static BITMASK	SieCommandSRU(SIEPARAMTYPE *psSieParam, int Content);

static BITMASK	SieCommandTAXAR(SIEPARAMTYPE *psSieParam, int Content);
static BITMASK	SieCommandVER(SIEPARAMTYPE *psSieParam, int Content);

static BITMASK	SieCommandVALUTA(SIEPARAMTYPE *psSieParam, int Content);
static BITMASK	SieCommandFTYP(SIEPARAMTYPE *psSieParam, int Content);

static BITMASK	SieCommandVTStart(SIEPARAMTYPE *psSieParam, int Content);
static BITMASK	SieCommandTRANS(SIEPARAMTYPE *psSieParam, int Content);
static BITMASK	SieCommandVTEnd(SIEPARAMTYPE *psSieParam, int Content);

static BITMASK SieCommandExtended(SIEPARAMTYPE *psSieParam, int Content);
static BITMASK SieCommandNoSieData(SIEPARAMTYPE *psSieParam, int Content);

static BOOL StoreSieTestMessages(SIEPARAMTYPE *psSieParam, LLIST plErrorList);
static void MandatoryCheck(AccountingType *pAccounting, SIEMANDATORYTYPE	*SieMandatory, NUMERIC *pdwErrorLines);
static void CheckOMFATTN(AccountingType *pAccounting, NUMERIC *pdwErrorLines);
static BOOL CheckMandatoryField(char **ppFields, BITMASK bmField, int iDataType, BITMASK *bmResult);
static BOOL CheckSieCommandExtField(char **ppFields, BITMASK bmField, BITMASK *bmResult);


static BOOL CheckAlpha(char *String);
static BOOL CheckNumeric(char *String);
static BOOL CheckUnsigned(char *String);
static BOOL CheckDecimal(char *String);
static BOOL CheckAmount(char *String);

void *InterpretSie(char *pFilePathName, char *pBuffer, LLIST plErrorList);
static char *ConvertTextCp858ToCp1252(char *Text);
static char *ConvertTextCp1252ToCp858(char *Text);
char *ConvertTextCp1252ToUtf8(char *pWinText);
static char *ConvertTextUtf8ToCp1252(char *pUtf8Text);

/********************************************************************

void *OpenSieFile(char *pFilePathName, LLIST plErrorList)

*********************************************************************/
void *OpenSieFile(char *pFilePathName, LLIST plErrorList)
{
	if (strstr(pFilePathName, ".SE")
		|| strstr(pFilePathName, ".SI")
		|| strstr(pFilePathName, ".SIE")
		|| strstr(pFilePathName, ".si")
		|| strstr(pFilePathName, ".se")
		|| strstr(pFilePathName, ".sie")
		|| strstr(pFilePathName, ".Si")
		|| strstr(pFilePathName, ".Se")
		|| strstr(pFilePathName, ".Sie")
		|| strstr(pFilePathName, ".sI")
		|| strstr(pFilePathName, ".sE")
		|| strstr(pFilePathName, ".sIe")
		|| strstr(pFilePathName, ".siE")
		|| strstr(pFilePathName, ".sIE")
		|| strstr(pFilePathName, ".SiE")
		|| strstr(pFilePathName, ".SIe")) {
		// Read the file
		char	*pBuffer = FileRead(pFilePathName);
		if (pBuffer) {
			void *vpAccounting = InterpretSie(pFilePathName, pBuffer, plErrorList);
			free(pBuffer);
			return vpAccounting;
		}
		return 0;
	}
	return 0;
}


/********************************************************************

void *InterpretSie(char *pFilePathName, char *pBuffer, LLIST plErrorList)

*********************************************************************/
void *InterpretSie(char *pFilePathName, char *pBuffer, LLIST plErrorList)
{
	// Read the file
	*TaxYear = 0;
	if (pBuffer) {
		if (strstr(pBuffer, "#KONTO") || strstr(pBuffer, "#VER")) { // Must be in a SIE-file
			char *pConvertBuffer = pBuffer;
			if ((!strstr(pBuffer, "#FORMAT UTF8"))) // #FORMAT Must be in a SIE-file (if UTF8 not need converting, do not exist)
				pConvertBuffer = ConvertTextCp1252ToUtf8(ConvertTextCp858ToCp1252(pBuffer));
			size_t ztBufferLength = strlen(pConvertBuffer);
			char *StartPtr = pConvertBuffer;
			char *EndPtr = strchr(StartPtr, 0x0d); //CR
			AccountingType *pAccounting = CreateAccounting(0, 0);

			if (EndPtr) {
				// We work internally with LF
				if (strchr(StartPtr, 0x0a)) {
					// If LF are in the file we make ' ' of any CR
					while (EndPtr) {
						*EndPtr = ' ';
						EndPtr = strchr(EndPtr, 0x0d); //CR 
					}
				}
				else {
					// If LF are in the file we make LF of any CR
					while (EndPtr) {
						*EndPtr = 0x0a;
						EndPtr = strchr(EndPtr, 0x0d); //CR 
					}
				}
				EndPtr = strchr(StartPtr, 0x0a); //LF 
			}
			else
				EndPtr = strchr(StartPtr, 0x0a); //LF 

			if (!EndPtr) {
				if (pConvertBuffer != pBuffer)
					free(pConvertBuffer);
				DestroyAccounting(&pAccounting);
				return 0;
			}

			if ((!strstr(pConvertBuffer, "#UB")) && strstr(pConvertBuffer, "#VER"))
				CopyString(&pAccounting->pContent, CONTENT_LABEL_VOUCHER);
			else if (strstr(pConvertBuffer, "#VER"))
				CopyString(&pAccounting->pContent, CONTENT_LABEL_LEDGER);
			else if (strstr(pConvertBuffer, "#IB") || strstr(pConvertBuffer, "#UB"))
				CopyString(&pAccounting->pContent, CONTENT_LABEL_BALANCE);
			else
				CopyString(&pAccounting->pContent, CONTENT_LABEL_CHARTS);
			CopyString(&pAccounting->pGaap, "BAS96");
			{
				SIEMANDATORYTYPE	SieMandatory;
				NUMERIC				dwLineNumber = 0;
				NUMERIC				dwErrorLines = 0;
				SIEPARAMTYPE		SieParam;
				char				*pSieTabRecord;
				BITMASK				bmResult = 0;

				if (!pAccounting) {
					if (pConvertBuffer != pBuffer)
						free(pConvertBuffer);
					DestroyAccounting(&pAccounting);
					return 0;
				}

				memset(&SieMandatory, 0, sizeof(SIEMANDATORYTYPE));
				while (EndPtr) {
					*EndPtr = 0; // Termintate Item string

					memset(&SieParam, 0, sizeof(SIEPARAMTYPE));

					pSieTabRecord = StartPtr;
					// SieParam.pRawTagLine	= calloc(strlen(pSieTabRecord)+1,1);
					// strcpy(SieParam.pRawTagLine, pSieTabRecord);

					SieParam.pAccounting = pAccounting;
					SieParam.SieMandatory = &SieMandatory;

					// Have the lines interpret one by one
					dwLineNumber++; // starts with 1st line =1, and so on
					SieParam.dwLineNumber = dwLineNumber;

					bmResult = SieInterpretation(pSieTabRecord, &SieParam);
					if (bmResult == INTERPRET_SYNTAX_ERR_FATAL) {
						if (pConvertBuffer != pBuffer)
							free(pConvertBuffer);
						DestroyAccounting(&pAccounting);
						return 0; // Error;
					}

					// Put the result on the screen (if asked for)
					StoreSieTestMessages(&SieParam, plErrorList);

					EndPtr++; // Step over the NUL
					// Wind to next line
					while ((*EndPtr <= ' ')
						&& (EndPtr <= (pConvertBuffer + ztBufferLength))) // Not over the file end
						EndPtr++;
					StartPtr = EndPtr;
					if (EndPtr <= (pConvertBuffer + ztBufferLength))
						EndPtr = strchr(StartPtr, 0x0a); //LF 
					else
						EndPtr = 0;
				}
				if (StartPtr <= (pConvertBuffer + ztBufferLength)) { // Left over data without LF, should not be
					// try to interpret it
					pSieTabRecord = StartPtr;
					//SieParam.pRawTagLine	= calloc(strlen(pSieTabRecord)+1,1);
					//strcpy(SieParam.pRawTagLine, pSieTabRecord);

					// We try to interpret the left over data
					memset(&SieParam, 0, sizeof(SIEPARAMTYPE));

					pSieTabRecord = StartPtr;
					//SieParam.pRawTagLine	= calloc(strlen(pSieTabRecord)+1,1);
					//strcpy(SieParam.pRawTagLine, pSieTabRecord);

					SieParam.pAccounting = pAccounting;
					SieParam.SieMandatory = &SieMandatory;

					// Have the lines interpret one by one
					dwLineNumber++; // starts with 1st line =1, and so on
					SieParam.dwLineNumber = dwLineNumber;

					bmResult = SieInterpretation(pSieTabRecord, &SieParam);
					/*				if(bmResult==INTERPRET_SYNTAX_ERR_FATAL) {
										SetGeneralErrorMessage(pAccounting, "", TextRefError, TextRefMandatory, "There are more non-interpreted data the end, data without LF (must be)", &dwErrorLines);
										}

									StoreSieTestMessages(&SieParam, plErrorList);

									SetGeneralErrorMessage(pAccounting, "", TextRefError, TextRefMandatory, "Last line of interpreted data the end is without LF (must be)", &dwErrorLines);
					*/
				}

				MandatoryCheck(pAccounting, SieParam.SieMandatory, &dwErrorLines);

				if (SieParam.SieMandatory->PSALDO)
					CheckOMFATTN(pAccounting, &dwErrorLines);
			}
			//			SumUpErrors(pAccounting, &dwErrorLines, TRUE, TRUE);
			if (pConvertBuffer != pBuffer)
				free(pConvertBuffer);
			//			strncpy(pAccounting->FileName, pFilePathName, ERROR_MESSAGE_LENGTH);
			{
				AccountingType *pSourceAccounting = ExpandSieAccountTypes(pAccounting, FILE_TYPE_STD);
				if (pSourceAccounting != pAccounting) {
					DestroyAccounting(&pAccounting);
					{ // We have to mark the file it is read
						char *pFlagBuffer = FileRead(pFilePathName);
						if (pFlagBuffer) {
							char *Ptr = strstr(pFlagBuffer, "#FLAGGA");
							if (Ptr) {
								while ((*Ptr != '0') && (*Ptr != 0x0a) && (*Ptr != 0x0d))
									Ptr++;
								if (*Ptr == '0') {
									*Ptr = '1';
									FileWrite(pFilePathName, pFlagBuffer);
								}
							}
							free(pFlagBuffer);
						}
					}
					return pSourceAccounting;
				}
			}
			return pAccounting;
		}
		else
			return 0;
	}
	else
		return 0;
	return 0;
}


/********************************************************************

static BITMASK SieInterpretation(	char *pSieTabRecord, SIEPARAMTYPE *psSieParam)
																	*
*********************************************************************/
static BITMASK SieInterpretation(char *pSieTabRecord, SIEPARAMTYPE *psSieParam)
{
	// Check the syntax make a tabbed record of the raw tagged line
	SieSyntax(pSieTabRecord);

	// Put the Fields in slots
	{
		char		*pField = pSieTabRecord;
		int					iParam = 0;

		psSieParam->pFields[iParam++] = pField;
		pField = strchr(++pField, FIELD_SEPARATOR);
		while (pField && (iParam < MAX_COMMAND_FIELDS)) { // Look for TABs
			psSieParam->pFields[iParam++] = pField + 1;
			*pField = 0;
			pField = strchr(++pField, FIELD_SEPARATOR);
		}
		// If maximum number of fields delete 
		if (pField)
			*pField = 0;

		// Interpret the content
		SieContent(psSieParam, 4);
		return psSieParam->bmResult;
	}

}


/********************************************************************

static void SieSyntax(				char *pSieTabRecord)

*********************************************************************/
static void SieSyntax(char *pSieTabRecord)
{
	size_t zt = 0;
	size_t StringLength = strlen(pSieTabRecord);

	if (!strncmp(pSieTabRecord, "#VER", 4))
		pSieTabRecord = pSieTabRecord;

	while (zt < StringLength) {
		if ((pSieTabRecord[zt] == ' ') || (pSieTabRecord[zt] == ASCII_TAB)) {
			// space between fields are converted to TABs 0x09
			pSieTabRecord[zt] = FIELD_SEPARATOR; // VK_TAB;
			zt++;

			// Remove extra space
			while ((zt < StringLength) && ((pSieTabRecord[zt] == ' ') || (pSieTabRecord[zt] == ASCII_TAB))) {
				memmove(&pSieTabRecord[zt], &pSieTabRecord[zt + 1], strlen(&pSieTabRecord[zt]));
				StringLength--;
			}
		}
		else if ((pSieTabRecord[zt] == '{') && (strchr(pSieTabRecord, '#'))) {
			BOOL Dim = TRUE;
			// Start if an object field ({)
			memmove(&pSieTabRecord[zt], &pSieTabRecord[zt + 1], strlen(&pSieTabRecord[zt]));
			StringLength--;
			while ((pSieTabRecord[zt] == ' ') || (pSieTabRecord[zt] == ASCII_TAB)) {
				memmove(&pSieTabRecord[zt], &pSieTabRecord[zt + 1], strlen(&pSieTabRecord[zt]));
				StringLength--;
			}

			while ((zt < StringLength) && (pSieTabRecord[zt] != '}')) {
				while ((pSieTabRecord[zt] == '\\') && (pSieTabRecord[zt + 1] == '"')) {
					memmove(&pSieTabRecord[zt], &pSieTabRecord[zt + 1], strlen(&pSieTabRecord[zt]));
					StringLength--;
					zt++; // Text ", no field delimiter
				}
				if (pSieTabRecord[zt] == '"') {
					memmove(&pSieTabRecord[zt], &pSieTabRecord[zt + 1], strlen(&pSieTabRecord[zt]));
					StringLength--;
					while ((pSieTabRecord[zt] != '"') || ((pSieTabRecord[zt] == '"') && (pSieTabRecord[zt - 1] == '\\'))) {
						if (pSieTabRecord[zt] == ASCII_TAB)
							pSieTabRecord[zt] = ' ';
						else if ((pSieTabRecord[zt] == '"') && (pSieTabRecord[zt - 1] == '\\')) {
							memmove(&pSieTabRecord[zt - 1], &pSieTabRecord[zt], strlen(&pSieTabRecord[zt - 1]));
							StringLength--;
							zt--;
						}
						zt++;
					}
					memmove(&pSieTabRecord[zt], &pSieTabRecord[zt + 1], strlen(&pSieTabRecord[zt]));
					StringLength--;
				}
				if ((pSieTabRecord[zt] == ' ') || (pSieTabRecord[zt] == ASCII_TAB)) {
					// space between fields are converted to field separators
					if (Dim) {
						pSieTabRecord[zt] = OBJECT_SET_SEPARATOR; // internal Dim/obj pair separator (-);
						Dim = FALSE;
						zt++;
						while ((pSieTabRecord[zt] == ' ') || (pSieTabRecord[zt] == ASCII_TAB)) {
							memmove(&pSieTabRecord[zt], &pSieTabRecord[zt + 1], strlen(&pSieTabRecord[zt]));
							StringLength--;
							//							zt--; 
						}
					}
					else {
						pSieTabRecord[zt] = OBJECT_SEPARATOR; // separator between Dim/obj pairs, US;
						Dim = TRUE;
						zt++;
						while ((pSieTabRecord[zt] == ' ') || (pSieTabRecord[zt] == ASCII_TAB)) {
							memmove(&pSieTabRecord[zt], &pSieTabRecord[zt + 1], strlen(&pSieTabRecord[zt]));
							StringLength--;
							//							zt--; 
						}
					}
				}

				else if (pSieTabRecord[zt] != '}')
					zt++; // Step ahead
			}
			// End of Object field (})
			memmove(&pSieTabRecord[zt], &pSieTabRecord[zt + 1], strlen(&pSieTabRecord[zt]));
			StringLength--;
			if (pSieTabRecord[zt - 1] == OBJECT_SEPARATOR) { // Empty junk
				memmove(&pSieTabRecord[zt - 1], &pSieTabRecord[zt], strlen(&pSieTabRecord[zt - 1]));
				StringLength--;
				zt--;
			}
		}
		else if (pSieTabRecord[zt] == '"') {
			// if a Textstring field
			memmove(&pSieTabRecord[zt], &pSieTabRecord[zt + 1], strlen(&pSieTabRecord[zt]));
			StringLength--;
			while ((pSieTabRecord[zt] != '"') || ((pSieTabRecord[zt] == '"') && (pSieTabRecord[zt - 1] == '\\'))) {
				if (pSieTabRecord[zt] == ASCII_TAB)
					pSieTabRecord[zt] = ' ';
				else if ((pSieTabRecord[zt] == '"') && (pSieTabRecord[zt - 1] == '\\')) {
					memmove(&pSieTabRecord[zt - 1], &pSieTabRecord[zt], strlen(&pSieTabRecord[zt - 1]));
					StringLength--;
					zt--;
				}
				zt++;
			}
			memmove(&pSieTabRecord[zt], &pSieTabRecord[zt + 1], strlen(&pSieTabRecord[zt]));
			StringLength--;
		}
		else if (!((pSieTabRecord[zt] == ' ') || (pSieTabRecord[zt] == ASCII_TAB))) {
			// other data field
			while ((zt < StringLength) && (!((pSieTabRecord[zt] == ' ') || (pSieTabRecord[zt] == ASCII_TAB))))
				zt++;
		}
		else // dirt, step over it
			zt++;
	}
	// SIE interpretaion 
}


/********************************************************************

static BITMASK SieContent(		SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieContent(SIEPARAMTYPE *psSieParam, int Content)
{
	if (!strcmp(psSieParam->pFields[0], "#ADRESS"))				SieCommandADRESS(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#BKOD"))			SieCommandBKOD(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#DIM"))			SieCommandDimension(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#ENHET"))			SieCommandENHET(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#FLAGGA"))		SieCommandFLAGGA(psSieParam, Content);

	else if (!strcmp(psSieParam->pFields[0], "#FNAMN"))			SieCommandFNAMN(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#FNR"))			SieCommandFNR(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#FORMAT"))		SieCommandFORMAT(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#GEN"))			SieCommandGEN(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#IB"))			SieCommandBalance(psSieParam, Content);

	else if (!strcmp(psSieParam->pFields[0], "#KONTO"))			SieCommandKONTO(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#KPTYP"))			SieCommandKPTYP(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#KTYP"))			SieCommandKTYP(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#OBJEKT"))		SieCommandOBJEKT(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#OIB"))			SieCommandBalance(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#OMFATTN"))		SieCommandOMFATTN(psSieParam, Content);

	else if (!strcmp(psSieParam->pFields[0], "#ORGNR"))			SieCommandORGNR(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#OUB"))			SieCommandBalance(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#PBUDGET"))		SieCommandBalance(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#PROGRAM"))		SieCommandPROGRAM(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#PROSA"))			SieCommandPROSA(psSieParam, Content);

	else if (!strcmp(psSieParam->pFields[0], "#PSALDO"))		SieCommandBalance(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#RAR"))			SieCommandRAR(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#RES"))			SieCommandBalance(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#SIETYP"))		SieCommandSIETYP(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#SRU"))			SieCommandSRU(psSieParam, Content);

	else if (!strcmp(psSieParam->pFields[0], "#TAXAR"))			SieCommandTAXAR(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#TRANS"))			SieCommandTRANS(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#UB"))			SieCommandBalance(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#UNDERDIM"))		SieCommandDimension(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#VER"))			SieCommandVER(psSieParam, Content);

	else if (!strcmp(psSieParam->pFields[0], "#FTYP"))			SieCommandFTYP(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#RTRANS"))		SieCommandTRANS(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#BTRANS"))		SieCommandTRANS(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "#VALUTA"))		SieCommandVALUTA(psSieParam, Content);

	else if (!strcmp(psSieParam->pFields[0], "{"))				SieCommandVTStart(psSieParam, Content);
	else if (!strcmp(psSieParam->pFields[0], "}"))				SieCommandVTEnd(psSieParam, Content);
	else if (!strncmp(psSieParam->pFields[0], "#KSUMMA", 7))	INTERPRET_SYNTAX_NONE;	// Not used SIE Commands
	else if (!strncmp(psSieParam->pFields[0], "#", 1))			SieCommandExtended(psSieParam, Content);
	else														SieCommandNoSieData(psSieParam, Content);

	return psSieParam->bmResult;

}

/********************************************************************

static BITMASK SieCommandADRESS(SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandADRESS(SIEPARAMTYPE *psSieParam, int Content)
{
	// #ADRESS   kontakt(0..n)  utdelningsadr(0..n)  postadr(0..n)  tel(N0..16)

	// Contact informtion of the client company
	psSieParam->bmCommand = INTERPRET_SYNTAX_ADRESS;

	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_5, &psSieParam->bmResult);
	// Store data
	if (!psSieParam->pAccounting->pEntity)
		psSieParam->pAccounting->pEntity = CreateParty(psSieParam->pAccounting, psSieParam->pAccounting);

	LListDeleteData(psSieParam->pAccounting->pEntity->pListAddress);
	FreeString(&psSieParam->pAccounting->pEntity->pPostalCode);
	if (psSieParam->pFields[2] && *psSieParam->pFields[2])
		LListAddRecord(psSieParam->pAccounting->pEntity->pListAddress, CopyString(0, psSieParam->pFields[2]));
	if (psSieParam->pFields[3] && *psSieParam->pFields[3])
		CopyString(&psSieParam->pAccounting->pEntity->pPostalAddress, psSieParam->pFields[3]);

	if ((psSieParam->pFields[1] && *psSieParam->pFields[1])
		&& (2 > strlen(psSieParam->pFields[1]) && *psSieParam->pFields[1] != ' ')) {
		ContactType *pContact
			= LListGetFirstRecordData(psSieParam->pAccounting->pEntity->pListContact);
		while (pContact) {
			DestroyContact(&pContact);		// sie:ContactType
			pContact = LListGetNextRecordData(psSieParam->pAccounting->pEntity->pListContact);
		}
		LListClear(psSieParam->pAccounting->pEntity->pListContact);
		if ((psSieParam->pFields[1] && *psSieParam->pFields[1])
			|| (psSieParam->pFields[4] && *psSieParam->pFields[4])) {
			pContact = CreateContact(psSieParam->pAccounting, psSieParam->pAccounting->pEntity);
			CopyString(&pContact->pName, psSieParam->pFields[1]);
			if (psSieParam->pFields[4] && *psSieParam->pFields[4])
				LListAddRecord(pContact->pListPhone, CopyString(0, psSieParam->pFields[4]));		// sie:Phone
			LListAddRecord(psSieParam->pAccounting->pEntity->pListContact, pContact);
		}
	}
	else if (psSieParam->pFields[4] && *psSieParam->pFields[4])
		CopyString(&psSieParam->pAccounting->pEntity->pEntityPhone, psSieParam->pFields[4]);	// sie:Phone
	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandBKOD(SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandBKOD(SIEPARAMTYPE *psSieParam, int Content)
{
	// #BKOD	 SNI-kod(1..n) // (Is normally numeric but it is not up to SIE but the statistic autority that can change whenever they like)

	// SNI is the Swedish equivalent to NACE company coding for GNP accounting etc 

	psSieParam->bmCommand = INTERPRET_SYNTAX_BKOD;

	if (!strcmp(psSieParam->pAccounting->pContent, CONTENT_LABEL_VOUCHER)) {
		psSieParam->bmResult = INTERPRET_SYNTAX_ERR_DEFINE;
		psSieParam->Message[0] = "Is not allowed for SIE4i";
		return INTERPRET_SYNTAX_ERR_DEFINE; // Fatal field error
	}

	// Check datatypes
	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_2, &psSieParam->bmResult);

	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_1, DT_ANY, &psSieParam->bmResult)) {
		psSieParam->Message[1] = "\"SNI-kod\" data, missing. If no data don't write the #BKOD";
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error
	}

	// Store data
	if (!psSieParam->pAccounting->pEntity)
		psSieParam->pAccounting->pEntity = CreateParty(psSieParam->pAccounting, psSieParam->pAccounting);
	CopyString(&psSieParam->pAccounting->pEntity->pStatisticalClass, psSieParam->pFields[1]);

	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandDimension(	SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandDimension(SIEPARAMTYPE *psSieParam, int Content)
{
	// #DIM	  dimensionsnr(N1..n) namn(1..n)
	// #UNDERDIM dimensionsnr(N1..n) namn(1..n)  superdimension(N1..n)

	// Dimensions are groups of objects in accounting, one dim-sub-level is allowed

	if (strcmp("#DIM", psSieParam->pFields[0]))
		psSieParam->bmCommand = INTERPRET_SYNTAX_DIM;
	else if (strcmp("#UNDERDIM", psSieParam->pFields[0]))
		psSieParam->bmCommand = INTERPRET_SYNTAX_UNDERDIM;

	// Check if allowed command
/*	if((psSieParam->pAccounting->pContent==SIE_1) || (psSieParam->pAccounting->pContent==SIE_2)) {
		psSieParam->bmResult=INTERPRET_SYNTAX_ERR_DEFINE;
		if(psSieParam->pAccounting->pContent==SIE_1)
			psSieParam->Message[0]= Ill_SIE_1;
		else // if(psSieParam->pAccounting->pContent==SIE_2)
			psSieParam->Message[0]= Ill_SIE_2;
		return INTERPRET_SYNTAX_ERR_DEFINE; // Fatal field error
		}
*/
// Check datatypes
	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_1, DT_NUMERIC, &psSieParam->bmResult))
		psSieParam->Message[1] = "\"dimensionsnr\" data, missing or non-numeric";
	if (!strcmp("#UNDERDIM", psSieParam->pFields[0])) {
		if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_3, DT_NUMERIC, &psSieParam->bmResult))
			psSieParam->Message[3] = "\"superdimension\" data, missing or non-numeric";
		if (psSieParam->bmResult & SIE_PARAM_STATUS_ALL)
			return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error

		CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_4, &psSieParam->bmResult);
	}
	else {
		if (psSieParam->bmResult & SIE_PARAM_STATUS_ALL)
			return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error

		CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_3, &psSieParam->bmResult);
	}
	// We continue even though name is missing
	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_2, DT_ANY, &psSieParam->bmResult))
		psSieParam->Message[2] = "\"namn\" is missing";

	// Store data
	{
		DimensionType *pDimension = CreateDimension(psSieParam->pAccounting, psSieParam->pAccounting->pObjectsChart);
		pDimension->Id = StringToCounter(psSieParam->pFields[1]);		// xs:positiveInteger
		pDimension = SearchSetDimension(psSieParam->pAccounting->pObjectsChart->pListDimension, pDimension);
		if (!strcmp("#UNDERDIM", psSieParam->pFields[0]) && psSieParam->pFields[3] && *psSieParam->pFields[3]) {
			DimensionType *pSuperDimension = GetDimension(psSieParam->pAccounting->pObjectsChart->pListDimension, StringToCounter(psSieParam->pFields[3]));
			if (pSuperDimension)
				pDimension->pSuperDimension = pSuperDimension;
		}
		if (psSieParam->pFields[2] && *psSieParam->pFields[2])
			SearchSetText(pDimension->pListName,
				AssembleText(psSieParam->pAccounting, pDimension, psSieParam->pFields[2], LANGUAGE));
	}
	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandENHET(SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandENHET(SIEPARAMTYPE *psSieParam, int Content)
{
	// #ENHET	kontonr(N1..n)  enhet(1..n)

	// Quantity units for bookkeeping accounts, for quantity accouting

	psSieParam->bmCommand = INTERPRET_SYNTAX_ENHET;

	// Check datatypes
	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_1, DT_UNSIGNED, &psSieParam->bmResult))
		psSieParam->Message[1] = "\"kontonr\" data, missing or non-unsigned numeric";
	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_2, DT_ANY, &psSieParam->bmResult))
		psSieParam->Message[2] = "\"enhet\" data, missing. If no Quant unit don't write the #ENHET";

	if (psSieParam->bmResult & SIE_PARAM_STATUS_ALL)
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error

	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_3, &psSieParam->bmResult);


	// Store data
	{
		AccountType *pAccount = GetAccount(psSieParam->pAccounting, psSieParam->pFields[1]);
		if (!pAccount) {
			// The account is not defined by by #KONTO
			psSieParam->Message[1] = "#KONTO The account is not defined";
			psSieParam->bmResult |= (SIE_PARAM_STATUS_2*INTERPRET_SYNTAX_ERR_DEFINE);
			pAccount = CreateAccount(psSieParam->pAccounting, psSieParam->pAccounting->pAccountsChart);
			CopyString(&pAccount->pId, psSieParam->pFields[1]);
			pAccount = SearchSetAccount(psSieParam->pAccounting->pAccountsChart->pListAccount, pAccount);
		}
		if (psSieParam->pFields[2] && *psSieParam->pFields[2])
			SearchSetText(pAccount->pListQuantityUnit,
				AssembleText(psSieParam->pAccounting, pAccount, psSieParam->pFields[2], LANGUAGE));
	}
	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandFLAGGA(SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandFLAGGA(SIEPARAMTYPE *psSieParam, int Content)
{
	// #FLAGGA   x(B1)

	// This is the flag telling if file is read or not, used by some reading SW avoidning double reading
	psSieParam->bmCommand = INTERPRET_SYNTAX_FLAGGA;
	psSieParam->SieMandatory->FLAGGA = TRUE;

	if (psSieParam->dwLineNumber != 1) {
		psSieParam->bmResult |= (INTERPRET_SYNTAX_ERR_DEFINE); // Error, Account not defined
		psSieParam->Message[0] = "This tag should be on the first line of the file, is not";
	}



	// Check datatypes
	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_2, &psSieParam->bmResult);

	if (!((psSieParam->pFields[1][0] == '0') || (psSieParam->pFields[1][0] == '1'))) {
		psSieParam->bmResult |= (SIE_PARAM_STATUS_1*INTERPRET_SYNTAX_ERR_MANDATORY);
		psSieParam->Message[1] = "\"x\" data, missing or not 0 or 1";
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error
	}

	// Store data
	// We do nothing	
	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandFNAMN(SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandFNAMN(SIEPARAMTYPE *psSieParam, int Content)
{
	// #FNAMN	f�retagsnamn(1..n)

	// The client companys name

	psSieParam->bmCommand = INTERPRET_SYNTAX_FNAMN;
	psSieParam->SieMandatory->FNAMN = TRUE;

	// Check datatypes
	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_2, &psSieParam->bmResult);

	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_1, DT_ANY, &psSieParam->bmResult)) {
		psSieParam->Message[1] = "\"ftgsnamn\" data, missing, mandatory";
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error
	}

	// Store data
	if (!psSieParam->pAccounting->pEntity)
		psSieParam->pAccounting->pEntity = CreateParty(psSieParam->pAccounting, psSieParam->pAccounting);
	CopyString(&psSieParam->pAccounting->pEntity->pName, psSieParam->pFields[1]);
	return TRUE;
}


/********************************************************************

static BITMASK SieCommandFNR(	SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandFNR(SIEPARAMTYPE *psSieParam, int Content)
{
	// #FNR	  f�retagsid(1..n) // Sound like numeric but can be plain text as well (=alphanumeric)

	// The clients client number by the accounting or audit consultant

	psSieParam->bmCommand = INTERPRET_SYNTAX_FNR;

	// Check datatypes
	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_2, &psSieParam->bmResult);

	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_1, DT_ANY, &psSieParam->bmResult)) {
		psSieParam->Message[1] = "\"id\" data, missing. If no ID don't write the #FNR";
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error
	}

	// Store data
	if (!psSieParam->pAccounting->pEntity)
		psSieParam->pAccounting->pEntity = CreateParty(psSieParam->pAccounting, psSieParam->pAccounting);
	CopyString(&psSieParam->pAccounting->pEntity->pProprietaryId, psSieParam->pFields[1]);
	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandFORMAT(SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandFORMAT(SIEPARAMTYPE *psSieParam, int Content)
{
	// #FORMAT   PC8(3) 

	// The character set of this file, only DOS PC8 =CP_858 is alloed (=CP437 =CP850)

	psSieParam->bmCommand = INTERPRET_SYNTAX_FORMAT;
	psSieParam->SieMandatory->FORMAT = TRUE;

	// Checking it has the PC8 context (only allowed)
	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_2, &psSieParam->bmResult);

	if (strcmp("PC8", psSieParam->pFields[1]) && strcmp("UTF8", psSieParam->pFields[1])) {
		// Error, report it must be PC8 
		psSieParam->Message[1] = "\"PC8\" data, must be PC8";
		psSieParam->bmResult |= (SIE_PARAM_STATUS_1*INTERPRET_SYNTAX_ERR_MANDATORY);
	}
	// We do nothing
	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandFTYP(SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandFTYP(SIEPARAMTYPE *psSieParam, int Content)
{
	// #FTYP	 F�retagstyp(1) 

	// F�retagstyp is the Swedish Bolagsverkets (BV) definition of company coding 

	psSieParam->bmCommand = INTERPRET_SYNTAX_FTYP;

	// Check datatypes
	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_2, &psSieParam->bmResult);

	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_1, DT_ANY, &psSieParam->bmResult)) {
		psSieParam->Message[1] = "\"typ\" data, missing. If no data don't write the #FTYP";
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error
	}

	// Store data
	if (!psSieParam->pAccounting->pEntity)
		psSieParam->pAccounting->pEntity = CreateParty(psSieParam->pAccounting, psSieParam->pAccounting);
	CopyString(&psSieParam->pAccounting->pEntity->pEntityType, psSieParam->pFields[1]);
	return psSieParam->bmResult;

}


/********************************************************************

static BITMASK SieCommandGEN(	SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandGEN(SIEPARAMTYPE *psSieParam, int Content)
{
	// #GEN	  datum(N8 (YYYYMMDD))  sign(1..n)

	// The date and the user producing this SIE-file

	psSieParam->bmCommand = INTERPRET_SYNTAX_GEN;
	psSieParam->SieMandatory->GEN = TRUE;

	// Non-fatal errors
	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_3, &psSieParam->bmResult);

	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_1, DT_DATE, &psSieParam->bmResult))
		psSieParam->Message[1] = "\"datum\" must be a correct YYYYMMDD date, is not";

	// Store data
	DestroyOperatorInfo(&psSieParam->pAccounting->pGeneratingUser);
	psSieParam->pAccounting->pGeneratingUser
		= AseembleOperatorInfo(psSieParam->pAccounting, psSieParam->pAccounting,
			psSieParam->pFields[2],
			StringToDate(psSieParam->pFields[1]));
	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandKONTO(SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandKONTO(SIEPARAMTYPE *psSieParam, int Content)
{
	// #KONTO	kontonr(N1..n)  kontonamn(1..n)

	// The Bookkeeping account also classified by #KTYP and #SRU and its quantity units defined by #ENHET 

	psSieParam->bmCommand = INTERPRET_SYNTAX_KONTO;
	psSieParam->SieMandatory->KONTO = TRUE;

	// Check datatypes
	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_1, DT_UNSIGNED, &psSieParam->bmResult)) {
		psSieParam->Message[1] = "\"kontonr\" data, missing or non-unsigned numeric";
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error
	}
	// We continue even though name is missing
	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_2, DT_ANY, &psSieParam->bmResult))
		psSieParam->Message[2] = "\"kontonamn\" is missing";

	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_3, &psSieParam->bmResult);

	// Store data
	{
		AccountType *pAccount = CreateAccount(psSieParam->pAccounting, psSieParam->pAccounting->pAccountsChart);
		CopyString(&pAccount->pId, psSieParam->pFields[1]);
		pAccount = SearchSetAccount(psSieParam->pAccounting->pAccountsChart->pListAccount, pAccount);
		// In SIE-files default account types are according to BAS chart of accounts
		switch (*psSieParam->pFields[1]) {
		case '2':
			// Store data
			CopyString(&pAccount->pType, ACCOUNT_TYPE_LIABILITIES);
			break;
		case '3':
			// Store data
			CopyString(&pAccount->pType, ACCOUNT_TYPE_REVENUES);
			break;
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			// Store data
			CopyString(&pAccount->pType, ACCOUNT_TYPE_EXPENSES);
			break;
		default:
		case '0':
		case '1':
			// Store data
			CopyString(&pAccount->pType, ACCOUNT_TYPE_ASSETS);
			break;
		}
		if (psSieParam->pFields[2] && *psSieParam->pFields[2])
			SearchSetText(pAccount->pListName,
				AssembleText(psSieParam->pAccounting, pAccount, psSieParam->pFields[2], LANGUAGE));
	}
	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandKPTYP(SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandKPTYP(SIEPARAMTYPE *psSieParam, int Content)
{
	// #KPTYP	kontoplanstyp(1..n)

	// Account plan type

	psSieParam->bmCommand = INTERPRET_SYNTAX_KPTYP;

	// Check datatypes
	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_2, &psSieParam->bmResult);

	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_1, DT_ANY, &psSieParam->bmResult)) {
		psSieParam->Message[1] = "\"kontoplanstyp\" data, missing. If no data don't write the #KPTYP";
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error
	}

	// Store data
	CopyString(&psSieParam->pAccounting->pGaap, psSieParam->pFields[1]);
	if (!strncmp(psSieParam->pFields[1], "EUBAS97", 7))
		CopyString(&psSieParam->pAccounting->pGaap, BAS2000);
	else
		CopyString(&psSieParam->pAccounting->pGaap, psSieParam->pFields[1]);
	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandKTYP(SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandKTYP(SIEPARAMTYPE *psSieParam, int Content)
{
	// #KTYP	 kontonr  kontotyp(1 (can only be TSI or K))

	// Account codes of Assets (T), Liabilities (S), Revenues (I),Costs (K)

	psSieParam->bmCommand = INTERPRET_SYNTAX_KTYP;
	psSieParam->SieMandatory->KTYP = TRUE;

	// Check datatypes
	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_3, &psSieParam->bmResult);

	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_1, DT_UNSIGNED, &psSieParam->bmResult)) {
		psSieParam->Message[1] = "\"kontonr\" data, missing or non-unsigned numeric";
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error
	}

	// Checking it has data is T, S, I or K
	if (!((*psSieParam->pFields[2] == 'T')
		|| (*psSieParam->pFields[2] == 'S')
		|| (*psSieParam->pFields[2] == 'I')
		|| (*psSieParam->pFields[2] == 'K'))) {
		psSieParam->bmResult |= (SIE_PARAM_STATUS_2*INTERPRET_SYNTAX_ERR_MANDATORY);
		psSieParam->Message[2] = "\"kontotyp\" only T, S, I, K are allowed";
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error
	}
	{
		AccountType *pAccount = GetAccount(psSieParam->pAccounting, psSieParam->pFields[1]);
		if (!pAccount) {
			// The account is not defined by by #KONTO
			psSieParam->Message[1] = "#KONTO The account is not defined";
			psSieParam->bmResult |= (SIE_PARAM_STATUS_2*INTERPRET_SYNTAX_ERR_DEFINE);
			pAccount = CreateAccount(psSieParam->pAccounting, psSieParam->pAccounting->pAccountsChart);
			CopyString(&pAccount->pId, psSieParam->pFields[1]);
			pAccount = SearchSetAccount(psSieParam->pAccounting->pAccountsChart->pListAccount, pAccount);
		}
		switch (*psSieParam->pFields[2]) {
		case 'S':
			// Store data
			CopyString(&pAccount->pType, ACCOUNT_TYPE_LIABILITIES);
			break;
		case 'I':
			// Store data
			CopyString(&pAccount->pType, ACCOUNT_TYPE_REVENUES);
			break;
		case 'K':
			// Store data
			CopyString(&pAccount->pType, ACCOUNT_TYPE_EXPENSES);
			break;
		default:
		case 'T':
			// Store data
			CopyString(&pAccount->pType, ACCOUNT_TYPE_ASSETS);
			break;
		}
	}
	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandOBJEKT(SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandOBJEKT(SIEPARAMTYPE *psSieParam, int Content)
{
	// #OBJEKT   dimensionsnr(N1..n)  objektkod(1..n)   objektnamn(1..n)

		// Objects is a general procedure for, user set flag filters, for reports. 
		// Objects are organised in clusters in the dimensions
	psSieParam->bmCommand = INTERPRET_SYNTAX_OBJEKT;
	psSieParam->SieMandatory->OBJEKT = TRUE;

	/*	if((psSieParam->pAccounting->pContent==SIE_1) || (psSieParam->pAccounting->pContent==SIE_2)) {
			psSieParam->bmResult=INTERPRET_SYNTAX_ERR_DEFINE;
			if(psSieParam->pAccounting->pContent==SIE_1)
				psSieParam->Message[0]= Ill_SIE_1;
			else // if(psSieParam->pAccounting->pContent==SIE_2)
				psSieParam->Message[0]= Ill_SIE_2;
			return INTERPRET_SYNTAX_ERR_DEFINE; // Fatal field error
			}
	*/
	// Check datatypes
	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_1, DT_NUMERIC, &psSieParam->bmResult))
		psSieParam->Message[1] = "\"dimensionsnr\" data, missing or non-numeric";
	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_2, DT_ANY, &psSieParam->bmResult))
		psSieParam->Message[2] = "\"objektkod\" data, missing";

	if (psSieParam->bmResult & SIE_PARAM_STATUS_ALL)
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error

	// We continue even though name is missing
	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_3, DT_ANY, &psSieParam->bmResult))
		psSieParam->Message[3] = "\"objektnamn\" is missing, is mandatory";

	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_4, &psSieParam->bmResult);

	// Store data
	{
		DimensionType *pDimension = CreateDimension(psSieParam->pAccounting, psSieParam->pAccounting->pObjectsChart);
		pDimension->Id = StringToCounter(psSieParam->pFields[1]);
		pDimension = SearchSetDimension(psSieParam->pAccounting->pObjectsChart->pListDimension, pDimension);
		{
			ObjectType *pObject = CreateObject(psSieParam->pAccounting, pDimension);
			CopyString(&pObject->pId, psSieParam->pFields[2]);
			pObject->pDimension = pDimension;
			pObject = SearchSetObject(psSieParam->pAccounting->pObjectsChart->pListDimension, pObject);
			if (psSieParam->pFields[3] && *psSieParam->pFields[3])
				SearchSetText(pObject->pListName,
					AssembleText(psSieParam->pAccounting, pObject, psSieParam->pFields[3], LANGUAGE));
		}
	}
	return psSieParam->bmResult;
}

/********************************************************************

static BITMASK SieCommandBalance(	SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandBalance(SIEPARAMTYPE *psSieParam, int Content)
{
	// #IB	   �rsnr(n1, 0 or -1,-2 etc)  konto(N1..n)  saldo(N1..n,2)  kvantitet(N1..n,n)
	// #UB	   �rsnr(n1, 0 or -1,-2 etc)  konto(N1..n)  saldo(N1..n,2)  kvantitet(N1..n,n)
	// #RES	  �rsnr(n1, 0 or -1,-2 etc)  konto(N1..n)  saldo(N1..n,2)  kvantitet(N1..n,n)
	// #OIB	  �rsnr(n1, 0 or -1,-2 etc)  konto(N1..n)  {dimensionsnr(N1..n)  objektnr(1..n)}  saldo(N1..n,2)  kvantitet(N1..n,n)
	// #OUB	  �rsnr(n1, 0 or -1,-2 etc)  konto(N1..n)  {dimensionsnr(N1..n)  objektnr(1..n)}  saldo(N1..n,2)  kvantitet(N1..n,n)
	// #PBUDGET  �rsnr(n1, 0 or -1,-2 etc)  period(N6, YYYYMM)  konto(N1..n)  {dimensionsnr(N1..n)  objektnr(1..n)}  saldo(N1..n,2)  kvantitet(N1..n,n)
	// #PSALDO   �rsnr(n1, 0 or -1,-2 etc)  period(N6, YYYYMM)  konto(N1..n)  {dimensionsnr(N1..n)  objektnr(1..n)}  saldo(N1..n,2)  kvantitet(N1..n,n)

	NUMERIC LedgerID = 0;
	DATEINT MonthEnd = 0;
	char *pAccountID = 0;
	NUMERIC cDimensionId = 0;	//  0  is All
	char *pObjectId = "0";		// "0" is All
	AMOUNT AmountIn = 0;
	QUANT QuantityIn = 0;
	AMOUNT AmountOut = 0;
	QUANT QuantityOut = 0;
	AMOUNT *pAmountIn = 0;
	QUANT *pQuantityIn = 0;
	AMOUNT *pAmountOut = 0;
	QUANT *pQuantityOut = 0;

	// Balances of an object or the entire ledger
	if (!strcmp("#IB", psSieParam->pFields[0])) {
		psSieParam->bmCommand = INTERPRET_SYNTAX_IB;
		psSieParam->SieMandatory->IB = TRUE;

		LedgerID = StringToNumeric(psSieParam->pFields[1]);
		pAccountID = psSieParam->pFields[2];
		AmountIn = StringToAmount(psSieParam->pFields[3]);
		QuantityIn = StringToDecimal(psSieParam->pFields[4]);
		// Trigger update of data
		if (psSieParam->pFields[3] && *psSieParam->pFields[3])
			pAmountIn = &AmountIn;
		if (psSieParam->pFields[4] && *psSieParam->pFields[4])
			pQuantityIn = &QuantityIn;
	}
	else if (!strcmp("#UB", psSieParam->pFields[0])) {
		psSieParam->bmCommand = INTERPRET_SYNTAX_UB;
		psSieParam->SieMandatory->UB = TRUE;

		LedgerID = StringToNumeric(psSieParam->pFields[1]);
		pAccountID = psSieParam->pFields[2];
		AmountOut = StringToAmount(psSieParam->pFields[3]);
		QuantityOut = StringToDecimal(psSieParam->pFields[4]);
		// Trigger update of data
		if (psSieParam->pFields[3] && *psSieParam->pFields[3])
			pAmountOut = &AmountOut;
		if (psSieParam->pFields[4] && *psSieParam->pFields[4])
			pQuantityOut = &QuantityOut;
	}
	else if (!strcmp("#RES", psSieParam->pFields[0])) {
		psSieParam->bmCommand = INTERPRET_SYNTAX_RES;
		psSieParam->SieMandatory->RES = TRUE;

		LedgerID = StringToNumeric(psSieParam->pFields[1]);
		pAccountID = psSieParam->pFields[2];
		AmountOut = StringToAmount(psSieParam->pFields[3]);
		QuantityOut = StringToDecimal(psSieParam->pFields[4]);
		// Trigger update of data
		if (psSieParam->pFields[3] && *psSieParam->pFields[3])
			pAmountOut = &AmountOut;
		if (psSieParam->pFields[4] && *psSieParam->pFields[4])
			pQuantityOut = &QuantityOut;
	}
	else if (!strcmp("#OIB", psSieParam->pFields[0])) {
		psSieParam->bmCommand = INTERPRET_SYNTAX_OIB;
		psSieParam->SieMandatory->OIB = TRUE;
		//		psSieParam->pAccounting->Aux.bDimObjBalancesExists=TRUE;

		LedgerID = StringToNumeric(psSieParam->pFields[1]);
		pAccountID = psSieParam->pFields[2];
		if (psSieParam->pFields[3] && *psSieParam->pFields[3]) {
			char *Ptr = strchr(psSieParam->pFields[3], OBJECT_SET_SEPARATOR);
			if (Ptr && (*(Ptr + 1)) && *psSieParam->pFields[3]) {
				*Ptr = 0;
				cDimensionId = StringToNumeric(psSieParam->pFields[3]);
				*Ptr = OBJECT_SET_SEPARATOR;
				pObjectId = Ptr + 1;
			}
		}
		AmountIn = StringToAmount(psSieParam->pFields[4]);
		QuantityIn = StringToDecimal(psSieParam->pFields[5]);
		// Trigger update of data
		if (psSieParam->pFields[4] && *psSieParam->pFields[4])
			pAmountIn = &AmountIn;
		if (psSieParam->pFields[5] && *psSieParam->pFields[5])
			pQuantityIn = &QuantityIn;
	}
	else if (!strcmp("#OUB", psSieParam->pFields[0])) {
		psSieParam->bmCommand = INTERPRET_SYNTAX_OUB;
		psSieParam->SieMandatory->OUB = TRUE;
		//		psSieParam->pAccounting->Aux.bDimObjBalancesExists=TRUE;

		LedgerID = StringToNumeric(psSieParam->pFields[1]);
		pAccountID = psSieParam->pFields[2];
		if (psSieParam->pFields[3] && *psSieParam->pFields[3]) {
			char *Ptr = strchr(psSieParam->pFields[3], OBJECT_SET_SEPARATOR);
			if (Ptr && (*(Ptr + 1)) && *psSieParam->pFields[3]) {
				*Ptr = 0;
				cDimensionId = StringToNumeric(psSieParam->pFields[3]);
				*Ptr = OBJECT_SET_SEPARATOR;
				pObjectId = Ptr + 1;
			}
		}
		AmountOut = StringToAmount(psSieParam->pFields[4]);
		QuantityOut = StringToDecimal(psSieParam->pFields[5]);
		// Trigger update of data
		if (psSieParam->pFields[4] && *psSieParam->pFields[4])
			pAmountOut = &AmountOut;
		if (psSieParam->pFields[5] && *psSieParam->pFields[5])
			pQuantityOut = &QuantityOut;
	}
	else if ((!strcmp("#PBUDGET", psSieParam->pFields[0]))
		|| (!strcmp("#PSALDO", psSieParam->pFields[0]))) {
		if (!strcmp("#PBUDGET", psSieParam->pFields[0])) {
			psSieParam->bmCommand = INTERPRET_SYNTAX_PBUDGET;
			psSieParam->SieMandatory->PBUDGET = TRUE;
		}
		else { // if (!strcmp("#PSALDO", psSieParam->pFields[0]))
			psSieParam->bmCommand = INTERPRET_SYNTAX_PSALDO;
			psSieParam->SieMandatory->PSALDO = TRUE;
		}
		LedgerID = StringToNumeric(psSieParam->pFields[1]);
		{
			char Date[LENGTH_LONG_DATE];
			strcpy(Date, psSieParam->pFields[2]);
			strcpy(&Date[strlen(Date)], "01");
			MonthEnd = StringToDate(Date);
			MonthEnd = GetLastDateInMonth(MonthEnd);
		}
		pAccountID = psSieParam->pFields[3];
		if (psSieParam->pFields[4] && *psSieParam->pFields[4]) {
			char *Ptr = strchr(psSieParam->pFields[4], OBJECT_SET_SEPARATOR);
			if (Ptr && (*(Ptr + 1)) && *psSieParam->pFields[4]) {
				*Ptr = 0;
				cDimensionId = StringToNumeric(psSieParam->pFields[4]);
				*Ptr = OBJECT_SET_SEPARATOR;
				pObjectId = Ptr + 1;
			}
		}
		AmountIn = StringToAmount(psSieParam->pFields[5]);
		QuantityIn = StringToDecimal(psSieParam->pFields[6]);
		// Trigger update of data
		if (!strcmp("#PBUDGET", psSieParam->pFields[0])) {
			if (psSieParam->pFields[5] && *psSieParam->pFields[5])
				pAmountIn = &AmountIn;
			if (psSieParam->pFields[6] && *psSieParam->pFields[6])
				pQuantityIn = &QuantityIn;
		}
		else {
			if (psSieParam->pFields[5] && *psSieParam->pFields[5])
				pAmountOut = &AmountIn;
			if (psSieParam->pFields[6] && *psSieParam->pFields[6])
				pQuantityOut = &QuantityIn;
		}

	}
	else
		return INTERPRET_SYNTAX_ERR_DEFINE; // Fatal field error

	{
		// Store data
		LedgerType *pLedger = GetLedger(psSieParam->pAccounting, LedgerID);
		DimensionType *pDimension = CreateDimension(psSieParam->pAccounting, psSieParam->pAccounting->pObjectsChart);
		if (!pLedger) { // #RAR is missing
			// The ledger year of the period is not defined by #RAR
			psSieParam->Message[0] = "\"period\" #RAR for the the year of the period is not defined";
			psSieParam->bmResult |= (SIE_PARAM_STATUS_2*INTERPRET_SYNTAX_ERR_DEFINE);
			pLedger = CreateLedger(psSieParam->pAccounting, psSieParam->pAccounting->pLedgersChart);
			{
				LedgerType *pLedgerPrevious = GetLedger(psSieParam->pAccounting, LedgerID + 1);
				if (pLedgerPrevious) {
					pLedger->YearEnd = GetPrevMonthLastDate(pLedgerPrevious->YearStart);
					pLedger->YearStart = ((GetNextMonthLastDate(pLedger->YearEnd - 1200) / 100) * 100) + 1;
				}
			}
			pLedger->Id = LedgerID;
			pLedger = SearchSetLedger(psSieParam->pAccounting->pLedgersChart->pListLedger, pLedger);
		}
		if(MonthEnd && (!((pLedger->YearStart <= MonthEnd) && (pLedger->YearEnd >= MonthEnd))))
			return INTERPRET_SYNTAX_ERR_DEFINE; // Fatal field error Month data out of Ledger rage
		if (*pObjectId && strcmp(pObjectId, "0") && (!cDimensionId))
			return INTERPRET_SYNTAX_ERR_DEFINE; // Fatal field error crazy object with balance, skipp it
		pDimension->Id = cDimensionId;
		pDimension = SearchSetDimension(psSieParam->pAccounting->pObjectsChart->pListDimension, pDimension);
		{
			ObjectType *pObject = CreateObject(psSieParam->pAccounting, pDimension);
			CopyString(&pObject->pId, pObjectId);
			pObject->pDimension = pDimension;
			pObject = SearchSetObject(psSieParam->pAccounting->pObjectsChart->pListDimension, pObject);
			{
				BalanceReportType *pBalanceReport = CreateBalanceReport(psSieParam->pAccounting, pLedger);
				pBalanceReport->pObject = pObject;
				pBalanceReport = SearchSetBalanceReport(pLedger->pListReport, pBalanceReport);
				{
					AccountType *pAccount = GetAccount(psSieParam->pAccounting, pAccountID);
					if (!pAccount) {
						// The account is not defined by by #KONTO
						psSieParam->Message[1] = "#KONTO The account is not defined";
						psSieParam->bmResult |= (SIE_PARAM_STATUS_2*INTERPRET_SYNTAX_ERR_DEFINE);
						pAccount = CreateAccount(psSieParam->pAccounting, psSieParam->pAccounting->pAccountsChart);
						CopyString(&pAccount->pId, pAccountID);
						pAccount = SearchSetAccount(psSieParam->pAccounting->pAccountsChart->pListAccount, pAccount);
					}
					{
						BalanceAccountType *pBalanceAccount = CreateBalanceAccount(psSieParam->pAccounting, pBalanceReport);
						pBalanceAccount->pAccount = pAccount;
						pBalanceAccount = SearchSetBalanceAccount(pBalanceReport->pListBalanceAccount, pBalanceAccount);
						if ((psSieParam->bmCommand == INTERPRET_SYNTAX_PBUDGET)
							|| (psSieParam->bmCommand == INTERPRET_SYNTAX_PSALDO)) {
							BalanceAccountPeriodType *pBalanceAccountPeriod = CreateBalanceAccountPeriod(psSieParam->pAccounting, pBalanceAccount);
							pBalanceAccountPeriod->End = MonthEnd;
							if (pAmountIn)
								pBalanceAccountPeriod->Budget = *pAmountIn;
							if (pQuantityIn)
								pBalanceAccountPeriod->BudgetQuantity = *pQuantityIn;
							if (pAmountOut)
								pBalanceAccountPeriod->Outcome = *pAmountOut;
							if (pQuantityOut)
								pBalanceAccountPeriod->OutcomeQuantity = *pQuantityOut;
							pBalanceAccountPeriod = SearchSetBalanceAccountPeriod(pBalanceAccount->pListMonth, pBalanceAccountPeriod);
							pBalanceAccountPeriod = pBalanceAccountPeriod;
						}
						else {
							if (pAmountIn)
								pBalanceAccount->Opening = *pAmountIn;
							if (pQuantityIn)
								pBalanceAccount->OpeningQuantity = *pQuantityIn;
							if (pAmountOut)
								pBalanceAccount->Forward = *pAmountOut;
							if (pQuantityOut)
								pBalanceAccount->ForwardQuantity = *pQuantityOut;
							// This is due to no demand i discipline in in and out notations by the SIE group
							if (pLedger->Id < 0) {
								LedgerType *pNextLedger = GetLedger(pLedger->pStructDocTop, pLedger->Id + 1);
								{
									BalanceReportType *pNextBalanceReport = CreateBalanceReport(psSieParam->pAccounting, pNextLedger);
									pNextBalanceReport->pObject = pObject;
									pNextBalanceReport = SearchSetBalanceReport(pNextLedger->pListReport, pNextBalanceReport);
									{
										BalanceAccountType *pNextBalanceAccount = CreateBalanceAccount(psSieParam->pAccounting, pNextBalanceReport);
										pNextBalanceAccount->pAccount = pAccount;
										pNextBalanceAccount = SearchSetBalanceAccount(pNextBalanceReport->pListBalanceAccount, pNextBalanceAccount);
										if ((!strcmp(pNextBalanceAccount->pAccount->pType, ACCOUNT_TYPE_TANGIBLE)) ||
											(!strcmp(pNextBalanceAccount->pAccount->pType, ACCOUNT_TYPE_STOCK)) ||
											(!strcmp(pNextBalanceAccount->pAccount->pType, ACCOUNT_TYPE_RECEIVABLES)) ||
											(!strcmp(pNextBalanceAccount->pAccount->pType, ACCOUNT_TYPE_ASSETS)) ||
											(!strcmp(pNextBalanceAccount->pAccount->pType, ACCOUNT_TYPE_EQUITIES)) ||
											(!strcmp(pNextBalanceAccount->pAccount->pType, ACCOUNT_TYPE_PROVISIONS)) ||
											(!strcmp(pNextBalanceAccount->pAccount->pType, ACCOUNT_TYPE_LIABILITIES))) {
											if (pAmountOut && (!pNextBalanceAccount->Opening))
												pNextBalanceAccount->Opening = *pAmountOut;
											if (pQuantityOut && (!pNextBalanceAccount->OpeningQuantity))
												pNextBalanceAccount->OpeningQuantity = *pQuantityOut;
										}
									}
								}
							}
							{
								LedgerType *pPreviousLedger = GetLedger(pLedger->pStructDocTop, pLedger->Id - 1);
								if (pPreviousLedger) {
									BalanceReportType *pPreviousBalanceReport = CreateBalanceReport(psSieParam->pAccounting, pPreviousLedger);
									pPreviousBalanceReport->pObject = pObject;
									pPreviousBalanceReport = SearchSetBalanceReport(pPreviousLedger->pListReport, pPreviousBalanceReport);
									{
										BalanceAccountType *pPreviousBalanceAccount = CreateBalanceAccount(psSieParam->pAccounting, pPreviousBalanceReport);
										pPreviousBalanceAccount->pAccount = pAccount;
										pPreviousBalanceAccount = SearchSetBalanceAccount(pPreviousBalanceReport->pListBalanceAccount, pPreviousBalanceAccount);
										if (pAmountIn && (!pPreviousBalanceAccount->Forward))
											pPreviousBalanceAccount->Forward = *pAmountIn;
										if (pQuantityIn && (!pPreviousBalanceAccount->ForwardQuantity))
											pPreviousBalanceAccount->ForwardQuantity = *pQuantityIn;
									}
								}
							}
						}
					}
				}
			}
		}
	}
	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandOMFATTN(SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandOMFATTN(SIEPARAMTYPE *psSieParam, int Content)
{
	// #OMFATTN  datum(N8 (YYYYMMDD))

	// The last bookeeping date of this accounting file

	psSieParam->bmCommand = INTERPRET_SYNTAX_OMFATTN;
	psSieParam->SieMandatory->OMFATTN = TRUE;

	if (!strcmp(psSieParam->pAccounting->pContent, CONTENT_LABEL_VOUCHER)) {
		psSieParam->bmResult = INTERPRET_SYNTAX_ERR_DEFINE;
		//		if(psSieParam->pAccounting->pContent==SIE_4I)
		psSieParam->Message[0] = "Is not allowed for SIE4i";
		//		else if(psSieParam->pAccounting->pContent==SIE_1)
		//			psSieParam->Message[0]= "Is not allowed for SIE1";
		return INTERPRET_SYNTAX_ERR_DEFINE; // Fatal field error
	}

	// Check datatypes
	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_2, &psSieParam->bmResult);

	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_1, DT_DATE, &psSieParam->bmResult)) {
		psSieParam->Message[1] = "\"datum\" must be a correct YYYYMMDD date, is not";
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error
	}


	// Store data
	psSieParam->pAccounting->LastEntryDate = StringToDate(psSieParam->pFields[1]);
	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandORGNR(SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandORGNR(SIEPARAMTYPE *psSieParam, int Content)
{
	// #ORGNR	orgnr(1..n)  f�rvnr(N2)  verknr(not used) 
	// orgnr in Sweden is normally 10 numeric digits but could have a leading ISO 3166-1 alpha-2 country pCode and a final separator pCode (f�rvnr)

	// The organisational ID-number of this company

	psSieParam->bmCommand = INTERPRET_SYNTAX_ORGNR;

	// Check datatypes
	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_4, &psSieParam->bmResult);

	if (psSieParam->pFields[3] && *psSieParam->pFields[3]) {
		psSieParam->Message[3] = "\"verknr\" data, is not used since 1994";
		psSieParam->bmResult |= (SIE_PARAM_STATUS_3*INTERPRET_SYNTAX_ERR_NOTE);
	}

	/* Depricated from the SIE4b spec
		if(psSieParam->pFields[3] && *psSieParam->pFields[3] &&
			(!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_3, DT_NUMERIC, &psSieParam->bmResult)))
			psSieParam->Message[3]= "\"verknr\" data, non-numeric";
		if(psSieParam->pFields[2] && *psSieParam->pFields[2] &&
			(!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_2, DT_NUMERIC, &psSieParam->bmResult)))
			psSieParam->Message[2]= "\"forvnr\" data, non-numeric";
	*/
	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_1, DT_ANY, &psSieParam->bmResult)) {
		psSieParam->Message[1] = "\"orgnr\" data, missing or non-numeric";
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error
	}

	// Store data
	if (!psSieParam->pAccounting->pEntity)
		psSieParam->pAccounting->pEntity = CreateParty(psSieParam->pAccounting, psSieParam->pAccounting);
	CopyString(&psSieParam->pAccounting->pEntity->pLegalRegId, psSieParam->pFields[1]);
	CopyString(&psSieParam->pAccounting->pEntity->pVatRegId, psSieParam->pFields[1]);
	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandPROGRAM(SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandPROGRAM(SIEPARAMTYPE *psSieParam, int Content)
{
	// #PROGRAM  programnamn(1..n)  version(1..n)

	// Information about the program that has been writing this file

	psSieParam->bmCommand = INTERPRET_SYNTAX_PROGRAM;
	psSieParam->SieMandatory->PROGRAM = TRUE;

	// Check datatypes
	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_3, &psSieParam->bmResult);

	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_2, DT_ANY, &psSieParam->bmResult))
		psSieParam->Message[2] = "\"version\" data, missing";

	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_1, DT_ANY, &psSieParam->bmResult))
		psSieParam->Message[1] = "\"programnamn\" data, missing";

	if (psSieParam->bmResult & SIE_PARAM_STATUS_ALL)
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error

	// Store data
	DestroyProgramInfo(&psSieParam->pAccounting->pProgram);
	psSieParam->pAccounting->pProgram = CreateProgramInfo(psSieParam->pAccounting, psSieParam->pAccounting);
	CopyString(&psSieParam->pAccounting->pProgram->pProgramInfo, psSieParam->pFields[1]);		// xs:positiveInteger
	CopyString(&psSieParam->pAccounting->pProgram->pversion, psSieParam->pFields[2]);		// xs:positiveInteger
	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandPROSA(SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandPROSA(SIEPARAMTYPE *psSieParam, int Content)
{
	// #PROSA	text(1..n)

	// Free text information 

	psSieParam->bmCommand = INTERPRET_SYNTAX_PROSA;

	// Check datatypes
	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_2, &psSieParam->bmResult);

	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_1, DT_ANY, &psSieParam->bmResult)) {
		psSieParam->Message[1] = "\"text\" data, missing. If no message don't write the #PROSA";
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error
	}

	// Store data
	if (psSieParam->pFields[1] && *psSieParam->pFields[1])
		LListAddRecord(psSieParam->pAccounting->pListComment,
			AssembleText(psSieParam->pAccounting, psSieParam->pAccounting, psSieParam->pFields[1], LANGUAGE));
	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandRAR(	SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandRAR(SIEPARAMTYPE *psSieParam, int Content)
{
	// #RAR	  �rsnr(N1)  start(N8 (YYYYMMDD))  slut(N8 (YYYYMMDD))
	// Definition of an accounting year (0 = this year, -1 = last year, -2 ... previous years

	psSieParam->bmCommand = INTERPRET_SYNTAX_RAR;
	psSieParam->SieMandatory->RAR = TRUE;

	// Check datatypes
	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_1, DT_YEAR_CODE, &psSieParam->bmResult))
		psSieParam->Message[1] = "\"arsnr\" must be 0 or a lesser digit";
	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_2, DT_DATE, &psSieParam->bmResult))
		psSieParam->Message[2] = "\"start\" must be a correct YYYYMMDD date, is not";
	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_3, DT_DATE, &psSieParam->bmResult))
		psSieParam->Message[3] = "\"slut\" must be a correct YYYYMMDD date, is not";

	if (psSieParam->bmResult & SIE_PARAM_STATUS_ALL)
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error

	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_4, &psSieParam->bmResult);

	if (0 >= atoi(psSieParam->pFields[1])) {
		// Store data
		LedgerType *pLedger = CreateLedger(psSieParam->pAccounting, psSieParam->pAccounting->pLedgersChart);
		pLedger->Id = StringToNumeric(psSieParam->pFields[1]);
		pLedger->YearStart = StringToDate(psSieParam->pFields[2]);
		pLedger->YearEnd = StringToDate(psSieParam->pFields[3]);
		pLedger = SearchSetLedger(psSieParam->pAccounting->pLedgersChart->pListLedger, pLedger);
	}
	else {
		psSieParam->Message[1] = "\"arsnr\" values more than 0 are not allowed";
		psSieParam->bmResult |= (SIE_PARAM_STATUS_1*INTERPRET_SYNTAX_ERR_DEFINE);
	}
	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandSIETYP(SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandSIETYP(SIEPARAMTYPE *psSieParam, int Content)
{
	// #SIETYP   typnr(N1)
	char *pSieTyp = psSieParam->pFields[1];

	if (strcmp(psSieParam->pAccounting->pContent, CONTENT_LABEL_VOUCHER)) {
		// Information of what type of SIE-file this file is
		psSieParam->bmCommand = INTERPRET_SYNTAX_SIETYP;

		// Check datatypes
		CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_2, &psSieParam->bmResult);

		if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_1, DT_UNSIGNED, &psSieParam->bmResult)) {
			psSieParam->Message[1] = "\"typnr\" data, missing or non-unsigned numeric";
			return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error
		}

		// Store data
		if (atoi(pSieTyp) == 4)
			CopyString(&psSieParam->pAccounting->pContent, CONTENT_LABEL_LEDGER);
		else if (atoi(pSieTyp) == 0)
			CopyString(&psSieParam->pAccounting->pContent, CONTENT_LABEL_CHARTS);
		else
			CopyString(&psSieParam->pAccounting->pContent, CONTENT_LABEL_BALANCE);
	}
	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandSRU(	SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandSRU(SIEPARAMTYPE *psSieParam, int Content)
{
	// #SRU	  konto  SRU-kod(1..n) (1..n) // was 3 from 2008 4 digits (Is normally numeric but it is not up to SIE but the tax autority that can change whenever they like)

	// Swedish Income Tax declaration sheet field codes of bookkeeping accounts the #TAXAR Year
	// Note, they can be multiple

	psSieParam->bmCommand = INTERPRET_SYNTAX_SRU;
	psSieParam->SieMandatory->SRU = TRUE;

	// If #TAXAR is not defined above this is useless
	if (!*TaxYear) {
		psSieParam->bmResult |= (INTERPRET_SYNTAX_ERR_DEFINE);
		psSieParam->Message[0] = "#TAXAR, Tax year is not previously defined, must be, SRU-tax pCode is not interpret";
		return INTERPRET_SYNTAX_ERR_DEFINE;
	}

	if (!psSieParam->pAccounting->pEntity->pEntityType) {
		psSieParam->bmResult |= (INTERPRET_SYNTAX_ERR_NOTE);
		psSieParam->Message[0] = "#FTYP, Company type is not previously defined, ought to be, SRU-tax pCode is invalid";
	}

	// Check datatypes
	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_3, &psSieParam->bmResult);

	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_1, DT_UNSIGNED, &psSieParam->bmResult)) {
		psSieParam->Message[1] = "\"konto\" data, missing or non-unsigned numeric";
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error
	}

	{
		// Store data
		AccountType *pAccount = GetAccount(psSieParam->pAccounting, psSieParam->pFields[1]);
		if (!pAccount) {
			// The account is not defined by by #KONTO
			psSieParam->Message[1] = "#KONTO The account is not defined";
			psSieParam->bmResult |= (SIE_PARAM_STATUS_2*INTERPRET_SYNTAX_ERR_DEFINE);
			pAccount = CreateAccount(psSieParam->pAccounting, psSieParam->pAccounting->pAccountsChart);
			CopyString(&pAccount->pId, psSieParam->pFields[1]);
			pAccount = SearchSetAccount(psSieParam->pAccounting->pAccountsChart->pListAccount, pAccount);
		}
		{
			AccountCodeType *pAccountCode = CreateAccountCode(psSieParam->pAccounting, pAccount);
			CopyString(&pAccountCode->pCodeType, "SRU");
			LListAddRecord(pAccountCode->pListApplicableDomain, CopyString(0, "SE"));
			CopyString(&pAccountCode->pIssuer, "SKV");
			CopyString(&pAccountCode->pCode, psSieParam->pFields[2]);
			CopyString(&pAccountCode->pValidYear, TaxYear);
			pAccountCode = SearchSetAccountCode(pAccount->pListAccountCode, pAccountCode);
		}
	}
	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandTAXAR(SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandTAXAR(SIEPARAMTYPE *psSieParam, int Content)
{
	// #TAXAR	�r(N4 (YYYY))   // the uppstform(3 BLA/�RL) is Deprecated
	// The taxation year of the SRU-codes in #SRU

		// The valid year of the SRU account codes below
	psSieParam->bmCommand = INTERPRET_SYNTAX_TAXAR;

	// Check datatypes
	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_3, &psSieParam->bmResult);

	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_1, DT_YEAR, &psSieParam->bmResult)) {
		psSieParam->Message[1] = "\"ar\" must be YYYY, is not";
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error
	}

	// Store data
	strcpy(TaxYear, psSieParam->pFields[1]);

	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandVALUTA(	SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandVALUTA(SIEPARAMTYPE *psSieParam, int Content)
{
	// #VALUTA Valutakod(3) // The Currency pCode should be defined by the three digit ISO 4217 
	// If not used the Ledger is supposed to be in SEK

	psSieParam->bmCommand = INTERPRET_SYNTAX_VALUTA;

	// Check datatypes
	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_2, &psSieParam->bmResult);

	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_1, DT_ALPHA, &psSieParam->bmResult)) {
		psSieParam->Message[1] = "\"valutakod\" data, missing. If no data don't write the #VALUTA";
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error
	}
	else if (3 != strlen(psSieParam->pFields[1])) {
		psSieParam->Message[1] = "\"valutakod\" must have three letters, an ISO 4217 currency pCode";
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error
	}

	// Store data
	CopyString(&psSieParam->pAccounting->pCurrency, psSieParam->pFields[1]);
	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandVER(	SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandVER(SIEPARAMTYPE *psSieParam, int Content)
{
	// #VER	  serie(1..n)  vernr(N1..n)  verdatum(N8 (YYYYMMDD))  vertext(1..n)  regdatum(N8 (YYYYMMDD)) sign(1..n)

	// Voucher entry of the ledger

	psSieParam->bmCommand = INTERPRET_SYNTAX_VER;

	if (!((!strcmp(psSieParam->pAccounting->pContent, CONTENT_LABEL_VOUCHER)) || (!strcmp(psSieParam->pAccounting->pContent, CONTENT_LABEL_LEDGER)))) {
		psSieParam->bmResult = INTERPRET_SYNTAX_ERR_DEFINE;
		/*		if(psSieParam->pAccounting->pContent==SIE_1)
					psSieParam->Message[0]= Ill_SIE_1;
				else if(psSieParam->pAccounting->pContent==SIE_2)
					psSieParam->Message[0]= Ill_SIE_2;
				else // if(psSieParam->pAccounting->pContent==SIE_3)
		*/			psSieParam->Message[0] = Ill_SIE_3;
		return INTERPRET_SYNTAX_ERR_DEFINE; // Fatal field error
	}

	// Check datatypes
	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_7, &psSieParam->bmResult);

	/* Not valid in SIE4b Voucher lists, issue should be handled else in other ways
		if(psSieParam->pFields[2] && *psSieParam->pFields[2] && (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_2, DT_UNSIGNED, &psSieParam->bmResult)))
			psSieParam->Message[2]= "\"vernr\" must be unsigned numeric";
	*/
	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_3, DT_DATE, &psSieParam->bmResult))
		psSieParam->Message[3] = "\"verdatum\" must be a correct YYYYMMDD date, is not";
	if (psSieParam->pFields[5] && *psSieParam->pFields[5] && (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_5, DT_DATE, &psSieParam->bmResult)))
		psSieParam->Message[5] = "\"regdatum\" must be a correct YYYYMMDD date, is not";

	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_3, DT_DATE, &psSieParam->bmResult))
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error

	// Store data
	{
		LedgerType *pLedger = GetLedgerDynamic(psSieParam->pAccounting, StringToDate(psSieParam->pFields[3]));
		if (!pLedger) {
			psSieParam->bmResult = INTERPRET_SYNTAX_ERR_DEFINE;
			return INTERPRET_SYNTAX_ERR_DEFINE; // Fatal field error
		}
		pVoucher = CreateVoucher(psSieParam->pAccounting, pLedger);
		pVoucher->pId = CreateSeriesId(psSieParam->pAccounting, pVoucher);
		if (psSieParam->pFields[1] && *psSieParam->pFields[1])
			CopyString(&pVoucher->pId->pseries, psSieParam->pFields[1]);
		if (psSieParam->pFields[2] && *psSieParam->pFields[2])
			pVoucher->pId->SeriesId = StringToNumeric(psSieParam->pFields[2]);
		if (psSieParam->pFields[3] && *psSieParam->pFields[3])
			pVoucher->Date = StringToDate(psSieParam->pFields[3]);
		if (psSieParam->pFields[4] && *psSieParam->pFields[4])
			SearchSetText(pVoucher->pListTitle,
				AssembleText(psSieParam->pAccounting, pLedger, psSieParam->pFields[4], LANGUAGE));
		if ((psSieParam->pFields[5] && *psSieParam->pFields[5]) || (psSieParam->pFields[6] && *psSieParam->pFields[6]))
			pVoucher->pAccountant
			= AseembleOperatorInfo(psSieParam->pAccounting, pVoucher,
				psSieParam->pFields[6],
				StringToDate(psSieParam->pFields[5]));
	}
	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandVTStart(SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandVTStart(SIEPARAMTYPE *psSieParam, int Content)
{
	// {(1)

	// The initial holder bracket of the transaction lines of a Ledger entry (mandatory)

	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_1, &psSieParam->bmResult);

	if (pVoucher) {
		BalanceAmount = 0;
		psSieParam->bmResult |= (SIE_PARAM_STATUS_0*INTERPRET_SYNTAX_OK);
		return INTERPRET_SYNTAX_OK;
	}
	else {
		psSieParam->Message[0] = "Syntax error no #VER is recognised";
		psSieParam->bmResult |= (SIE_PARAM_STATUS_0*INTERPRET_SYNTAX_ERR_DEFINE);
		return INTERPRET_SYNTAX_ERR_DEFINE;
	}
}

/********************************************************************

static BITMASK MakeDimensionObjectList(char *pDimObjListField, void *pStructDocTop, void *pStructParent, LLIST pListDimension, LLIST pListObjectRef)

*********************************************************************/
static BITMASK MakeDimensionObjectList(char *pDimObjListField, void *pStructDocTop, void *pStructParent, LLIST pListDimension, LLIST pListObjectRef)
{ // Getting the DImension and Object out of an DimObj field
	int iRetVal = INTERPRET_SYNTAX_OK;

	char *pDimObjSeparator = strchr(pDimObjListField, OBJECT_SET_SEPARATOR);
	char *pSetObjSeparator = pDimObjListField;
	while (pSetObjSeparator && pDimObjSeparator) {
		*pDimObjSeparator = 0;
		{
			COUNTER	cDimensionId = StringToNumeric(pSetObjSeparator);
			*pDimObjSeparator = OBJECT_SET_SEPARATOR;
			{
				char *pObjectId = pDimObjSeparator + 1;

				pSetObjSeparator = strchr(pDimObjSeparator + 1, OBJECT_SEPARATOR);
				if (pSetObjSeparator) {
					*pSetObjSeparator = 0;
					pSetObjSeparator++;
					pDimObjSeparator = strchr(pSetObjSeparator, OBJECT_SET_SEPARATOR);
				}
				{
					// Store data
					extern DimensionType *pDimensionRef;
					DimensionType *pDimension = CreateDimension(pStructDocTop, pStructParent);
					pDimension->Id = cDimensionId;
					pDimension = SearchSetDimension(pListDimension, pDimension);
					{
						extern ObjectType *pObjectRef;
						ObjectType *pObject = CreateObject(pStructDocTop, pDimension);
						CopyString(&pObject->pId, pObjectId);
						pObject->pDimension = pDimension;
						pObject = SearchSetObject(pListDimension, pObject);
						LListAddRecord(pListObjectRef, pObject);
					}
				}
			}
		}
	}
	return iRetVal;
}


/********************************************************************

static BITMASK SieCommandTRANS(SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandTRANS(SIEPARAMTYPE *psSieParam, int Content)
{
	// #TRANS	kontonr(1..n)  {objektlista(0..n)}  belopp(N1..n,2)  transdat(0/N8 (YYYYMMDD))  transtext(0..n)  kvantitet(N0..n,n) sign(0..n)
	// #RTRANS	kontonr(1..n)  {objektlista(0..n)}  belopp(N1..n,2)  transdat(0/N8 (YYYYMMDD))  transtext(0..n)  kvantitet(N0..n,n) sign(0..n)
	// #BTRANS	kontonr(1..n)  {objektlista(0..n)}  belopp(N1..n,2)  transdat(0/N8 (YYYYMMDD))  transtext(0..n)  kvantitet(N0..n,n) sign(0..n)

	// Transaction line of a Ledger Entry

	if (!strcmp(psSieParam->pFields[0], "#TRANS"))
		psSieParam->bmCommand = INTERPRET_SYNTAX_TRANS;
	else if (!strcmp(psSieParam->pFields[0], "#RTRANS")) { // Supplemented accounting entry 
	// RTRANS (Note that Supplemented accounting entries are NOT recognised by XBRL or UN/CEFACT and should by them be considered as any accounting entry)
	// The Swedish BFN avoids international contacts and has no clear statement. In this case XBRL or UN/CEFACT will set the standard.
	// SIE is influenced here by member SW-suppliers having this information field in their applications, it should be removed. )
		psSieParam->bmCommand = INTERPRET_SYNTAX_RTRANS;
		return psSieParam->bmResult; // Solution of a deprecated extended entry is not solved by SIE. 
	}
	else if (!strcmp(psSieParam->pFields[0], "#BTRANS"))	// Deprecated accounting entry
		psSieParam->bmCommand = INTERPRET_SYNTAX_BTRANS;

	if (!((!strcmp(psSieParam->pAccounting->pContent, CONTENT_LABEL_VOUCHER)) || (!strcmp(psSieParam->pAccounting->pContent, CONTENT_LABEL_LEDGER)))) {
		psSieParam->bmResult = INTERPRET_SYNTAX_ERR_DEFINE;
		psSieParam->Message[0] = Ill_SIE_3;
		return INTERPRET_SYNTAX_ERR_DEFINE; // Fatal field error
	}

	// Check datatypes
 //	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_7, &psSieParam->bmResult);

	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_1, DT_UNSIGNED, &psSieParam->bmResult))
		psSieParam->Message[1] = "\"kontonr\" data, missing or non-unsigned numeric";
	if (psSieParam->pFields[2] && *psSieParam->pFields[2] && (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_2, DT_OBJECT, &psSieParam->bmResult)))
		psSieParam->Message[2] = "\"objektlista\"  data, syntax error";
	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_3, DT_AMOUNT, &psSieParam->bmResult))
		psSieParam->Message[3] = "\"belopp\" data, missing or non-dot-decimal numeric, max two decmials";
	if (psSieParam->pFields[4] && *psSieParam->pFields[4] && (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_4, DT_DATE, &psSieParam->bmResult)))
		psSieParam->Message[4] = "\"transdat\" must be a correct YYYYMMDD date, is not";
	if (psSieParam->pFields[6] && *psSieParam->pFields[6] && (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_6, DT_QUANTITY, &psSieParam->bmResult)))
		psSieParam->Message[6] = "\"kvantitet\" data, non-numeric";

	// We do not quit if fatalm error until all fields are checked for the report
	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_1, DT_UNSIGNED, &psSieParam->bmResult))
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error
	if (!CheckMandatoryField(psSieParam->pFields, SIE_PARAM_STATUS_3, DT_AMOUNT, &psSieParam->bmResult))
		return INTERPRET_SYNTAX_ERR_MANDATORY; // Fatal field error

	if (!pVoucher) {
		psSieParam->Message[0] = "Syntax error no #VER is recognised";
		psSieParam->bmResult |= INTERPRET_SYNTAX_ERR_DEFINE;
		return INTERPRET_SYNTAX_ERR_FATAL; // Fatal Error
	}
	{
		// Store data
		VoucherEntryType *pVoucherEntry = CreateVoucherEntry(psSieParam->pAccounting, pVoucher);
		if (psSieParam->bmCommand == INTERPRET_SYNTAX_BTRANS)
			pVoucherEntry->pDeprecated = AseembleOperatorInfo(psSieParam->pAccounting, pVoucherEntry, "Sie", GetSystemDate());
		if (psSieParam->pFields[1] && *psSieParam->pFields[1]) {
			pVoucherEntry->pAccount = GetAccount(psSieParam->pAccounting, psSieParam->pFields[1]);
			if (!pVoucherEntry->pAccount) {
				// The account is not defined by by #KONTO
				psSieParam->Message[1] = "#KONTO The account is not defined";
				psSieParam->bmResult |= (SIE_PARAM_STATUS_2*INTERPRET_SYNTAX_ERR_DEFINE);
				pVoucherEntry->pAccount = CreateAccount(psSieParam->pAccounting, psSieParam->pAccounting->pAccountsChart);
				CopyString(&pVoucherEntry->pAccount->pId, psSieParam->pFields[1]);
				pVoucherEntry->pAccount = SearchSetAccount(psSieParam->pAccounting->pAccountsChart->pListAccount, pVoucherEntry->pAccount);
			}
		}
		if (psSieParam->pFields[2] && *psSieParam->pFields[2])
			MakeDimensionObjectList(
				psSieParam->pFields[2],
				psSieParam->pAccounting,
				psSieParam->pAccounting->pObjectsChart,
				psSieParam->pAccounting->pObjectsChart->pListDimension,
				pVoucherEntry->pListObject);
		if (psSieParam->pFields[3] && *psSieParam->pFields[3])
			pVoucherEntry->Amount = StringToAmount(psSieParam->pFields[3]);
		if (psSieParam->pFields[4] && *psSieParam->pFields[4])
			pVoucherEntry->AuxDate = StringToDate(psSieParam->pFields[4]);
		if (psSieParam->pFields[5] && *psSieParam->pFields[5])
			SearchSetText(pVoucherEntry->pListTitle,
				AssembleText(psSieParam->pAccounting, pVoucherEntry, psSieParam->pFields[5], LANGUAGE));
		if (psSieParam->pFields[6] && *psSieParam->pFields[6])
			pVoucherEntry->Quantity = StringToDecimal(psSieParam->pFields[6]);
		if (psSieParam->pFields[7] && *psSieParam->pFields[7])
			pVoucherEntry->pAccountant = AseembleOperatorInfo(psSieParam->pAccounting, pVoucher,
				psSieParam->pFields[7],
				0);
		LListAddRecord(pVoucher->pListEntry, pVoucherEntry);
	}
	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandVTEnd(SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandVTEnd(SIEPARAMTYPE *psSieParam, int Content)
{
	// }(1)

	// The final holder bracket of the transaction lines of a Ledger entry (mandatory)

	CheckSieCommandExtField(psSieParam->pFields, SIE_PARAM_STATUS_1, &psSieParam->bmResult);

	if (pVoucher) {
		if (BalanceAmount) {
			psSieParam->Message[0] = "#VER This Ledger entry is not balanced";
			BalanceAmount = 0;
			psSieParam->bmResult |= (SIE_PARAM_STATUS_0*INTERPRET_SYNTAX_ERR_MANDATORY);
			//			return INTERPRET_SYNTAX_ERR_MANDATORY;
		}
		LedgerType *Ledger = pVoucher->pStructParent;
		LListAddRecord(Ledger->pListVoucher, pVoucher);

		BalanceAmount = 0;
		pVoucher = 0;

		psSieParam->bmResult |= (SIE_PARAM_STATUS_0*INTERPRET_SYNTAX_OK);
		return INTERPRET_SYNTAX_OK;
	}
	else {
		psSieParam->Message[0] = "Syntax error no #VER is recognised";
		psSieParam->bmResult |= (SIE_PARAM_STATUS_0*INTERPRET_SYNTAX_ERR_DEFINE);
		return INTERPRET_SYNTAX_ERR_DEFINE;
	}
}


/********************************************************************

static BITMASK SieCommandExtended(	SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandExtended(SIEPARAMTYPE *psSieParam, int Content)
{
	psSieParam->bmCommand = INTERPRET_SYNTAX_EXT_TAG;
	psSieParam->Message[0] = "Extended tag, is allowed, but not interpret";
	psSieParam->bmResult |= (SIE_PARAM_STATUS_0*INTERPRET_SYNTAX_ERR_NOTE);
	return psSieParam->bmResult;
}


/********************************************************************

static BITMASK SieCommandNoSieData(	SIEPARAMTYPE *psSieParam, int Content)

*********************************************************************/
static BITMASK SieCommandNoSieData(SIEPARAMTYPE *psSieParam, int Content)
{
	psSieParam->bmCommand = INTERPRET_SYNTAX_NON_SIE_DATA;
	psSieParam->Message[0] = "Non SIE syntax data, is not allowed";
	psSieParam->bmResult |= (SIE_PARAM_STATUS_0*INTERPRET_SYNTAX_ERR_DEFINE);
	return psSieParam->bmResult;
}


/********************************************************************

static void MandatoryCheck(AccountingType *pAccounting, SIEMANDATORYTYPE	*SieMandatory, NUMERIC *pdwErrorLines)

*********************************************************************/
static void MandatoryCheck(AccountingType *pAccounting, SIEMANDATORYTYPE	*SieMandatory, NUMERIC *pdwErrorLines)
{
#if 0
	static char Message[] = { "Is missing in the SIE-file and must be included" };
	if (!SieMandatory->FLAGGA)
		SetGeneralErrorMessage(pAccounting, "#FLAGGA", TextRefError, TextRefMandatory, Message, pdwErrorLines);

	if (!SieMandatory->PROGRAM)
		SetGeneralErrorMessage(pAccounting, "#PROGRAM", TextRefError, TextRefMandatory, Message, pdwErrorLines);

	if (!SieMandatory->FORMAT)
		SetGeneralErrorMessage(pAccounting, "#FORMAT", TextRefError, TextRefMandatory, Message, pdwErrorLines);

	if (!SieMandatory->GEN)
		SetGeneralErrorMessage(pAccounting, "#GEN", TextRefError, TextRefMandatory, Message, pdwErrorLines);

	if (!SieMandatory->FNAMN)
		SetGeneralErrorMessage(pAccounting, "#FNAMN", TextRefError, TextRefMandatory, Message, pdwErrorLines);

	if ((!SieMandatory->RAR) && ((!strcmp(pAccounting->pContent, CONTENT_LABEL_BALANCE)) || (!strcmp(pAccounting->pContent, CONTENT_LABEL_LEDGER))))
		SetGeneralErrorMessage(pAccounting, "#RAR", TextRefError, TextRefMandatory, Message, pdwErrorLines);

	/*	if((!SieMandatory->OMFATTN)
			&& (!strcmp(pAccounting->pContent, CONTENT_LABEL_BALANCE))
			SetGeneralErrorMessage(pAccounting, "#OMFATTN", TextRefError, TextRefMandatory, Message, pdwErrorLines);
	*/
	if ((!SieMandatory->KONTO) && ((!strcmp(pAccounting->pContent, CONTENT_LABEL_BALANCE)) || (!strcmp(pAccounting->pContent, CONTENT_LABEL_LEDGER))))
		SetGeneralErrorMessage(pAccounting, "#KONTO", TextRefError, TextRefMandatory, Message, pdwErrorLines);

	/*	if((!SieMandatory->SRU)
			 && (pAccounting->pContent==SIE_1)
			 && (pAccounting->pContent==SIE_2))
			SetGeneralErrorMessage(pAccounting, "#SRU", TextRefError, TextRefMandatory, Message, pdwErrorLines);
	*/
	if ((!SieMandatory->OBJEKT)
		&& (!strcmp(pAccounting->pContent, CONTENT_LABEL_BALANCE)))
		SetGeneralErrorMessage(pAccounting, "#OBJEKT", TextRefError, TextRefMandatory, Message, pdwErrorLines);

	if ((!SieMandatory->IB) && ((!strcmp(pAccounting->pContent, CONTENT_LABEL_BALANCE)) || (!strcmp(pAccounting->pContent, CONTENT_LABEL_LEDGER))))
		SetGeneralErrorMessage(pAccounting, "#IB", TextRefError, TextRefMandatory, Message, pdwErrorLines);

	if ((!SieMandatory->UB) && ((!strcmp(pAccounting->pContent, CONTENT_LABEL_BALANCE)) || (!strcmp(pAccounting->pContent, CONTENT_LABEL_LEDGER))))
		SetGeneralErrorMessage(pAccounting, "#UB", TextRefError, TextRefMandatory, Message, pdwErrorLines);

	/*	if((!SieMandatory->OIB)
			&& (pAccounting->pContent==SIE_3))
			SetGeneralErrorMessage(pAccounting, "#OIB", TextRefError, TextRefMandatory, Message, pdwErrorLines);

		if((!SieMandatory->OUB)
			&& (pAccounting->pContent==SIE_3))
			SetGeneralErrorMessage(pAccounting, "#OUB", TextRefError, TextRefMandatory, Message, pdwErrorLines);
	*/
	if ((!SieMandatory->RES) && ((!strcmp(pAccounting->pContent, CONTENT_LABEL_BALANCE)) || (!strcmp(pAccounting->pContent, CONTENT_LABEL_LEDGER))))
		SetGeneralErrorMessage(pAccounting, "#RES", TextRefError, TextRefMandatory, Message, pdwErrorLines);

	/*	if((!SieMandatory->PBUDGET)
			&& ((pAccounting->pContent==SIE_2)
				||(pAccounting->pContent==SIE_3)))
			SetGeneralErrorMessage(pAccounting, "#PBUDGET", TextRefError, TextRefMandatory, Message, pdwErrorLines);
	*/
	/*	if((!SieMandatory->PSALDO)
			&& ((pAccounting->pContent==SIE_2)
				||(pAccounting->pContent==SIE_3)))
			SetGeneralErrorMessage(pAccounting, "#PSALDO", TextRefError, TextRefMandatory, Message, pdwErrorLines);
	*/
	if ((!SieMandatory->KTYP) && ((!strcmp(pAccounting->pContent, CONTENT_LABEL_BALANCE)) || (!strcmp(pAccounting->pContent, CONTENT_LABEL_LEDGER))))
		SetGeneralErrorMessage(pAccounting, "#KTYP", TextRefNote, TextRefOptional, "Is missing, is strongly recommended", pdwErrorLines);
#endif		
}


/********************************************************************

static void CheckOMFATTN(AccountingType *pAccounting, NUMERIC *pdwErrorLines)

*********************************************************************/
static void CheckOMFATTN(AccountingType *pAccounting, NUMERIC *pdwErrorLines)
{
#if 0
	if ((!pAccounting->LastEntryDate) && (pAccounting->GeneratedDate < pAccounting->pListBalanceReport.AccountingYearEnd))
		SetGeneralErrorMessage(pAccounting, "#OMFATTN", TextRefNote, TextRefMandatory, "is missing, the bookkeeping year is active", pdwErrorLines);
#endif
}


/********************************************************************

static BOOL StoreSieTestMessages(SIEPARAMTYPE *psSieParam, LLIST plErrorList)

*********************************************************************/
static BOOL StoreSieTestMessages(SIEPARAMTYPE *psSieParam, LLIST plErrorList)
{
#if 0
	FileTestLinesType *FileTestLine = calloc(1, sizeof(FileTestLinesType));

	memset(FileTestLine, 0, sizeof(FileTestLinesType));

	// Sie File Line
	FileTestLine->dwLineNumber = psSieParam->dwLineNumber;

	// Raw Line
	FileTestLine->pRawTagLine = psSieParam->pRawTagLine;

	// Command Line Fields
	{
		int iField = 0;
		while ((psSieParam->pFields[iField]) && (iField < MAX_COMMAND_FIELDS)) {
			FileTestLine->pCommandFields[iField] = psSieParam->pFields[iField];
			iField++;
		}
	}

	{
		BITMASK bmMask = (SIE_PARAM_STATUS_0 * 0x0f);
		int iSieField = 0;
		for (iSieField = 0; iSieField < MAX_COMMAND_FIELDS; iSieField++) {
			BITMASK bmCheck = (psSieParam->bmResult & bmMask);

			while (bmCheck / 0x10)
				bmCheck /= 0x10;

			if (bmCheck) {
				(*pdwErrorLines)++;
				FileTestLine->dwErrorLine[iSieField] = *pdwErrorLines;
				if (!FileTestLine->dwRefErrorLine)
					FileTestLine->dwRefErrorLine = *pdwErrorLines;
				psSieParam->pAccounting->Aux.iSieFileErrorLines++;

				FileTestLine->pErrorLines[iSieField][0] = psSieParam->pFields[0];

				if ((bmCheck & INTERPRET_SYNTAX_EXT_FIELD)
					|| (bmCheck & INTERPRET_SYNTAX_ERR_NOTE))
					FileTestLine->pErrorLines[iSieField][1] = TextRefNote;
				else
					FileTestLine->pErrorLines[iSieField][1] = TextRefError;

				if (bmCheck & INTERPRET_SYNTAX_ERR_MANDATORY) {
					FileTestLine->pErrorLines[iSieField][2] = TextRefMandatory;
					FileTestLine->pErrorLines[iSieField][3] = psSieParam->Message[iSieField];
				}
				else if (bmCheck & INTERPRET_SYNTAX_ERR_DEFINE) {
					FileTestLine->pErrorLines[iSieField][2] = TextRefDefinition;
					FileTestLine->pErrorLines[iSieField][3] = psSieParam->Message[iSieField];
				}
				else if (bmCheck & INTERPRET_SYNTAX_EXT_FIELD) {
					FileTestLine->pErrorLines[iSieField][2] = TextRefDefinition;
					FileTestLine->pErrorLines[iSieField][3] = TextRefExtField;
				}
				else if (bmCheck & INTERPRET_SYNTAX_ERR_NOTE) {
					FileTestLine->pErrorLines[iSieField][2] = TextRefOptional;
					if (psSieParam->Message[iSieField] && psSieParam->Message[iSieField][0])
						FileTestLine->pErrorLines[iSieField][3] = psSieParam->Message[iSieField];
					else
						FileTestLine->pErrorLines[iSieField][3] = TextRefJustSpaces;
				}
			}
			bmMask *= 0x10;
		}
	}

	if (RET_OK != LListAddRecord(psSieParam->pAccounting->Aux.pListFileReadResultLines, FileTestLine)) {
		// realse the last data when we can't go on, due to memory fuzz
		free(FileTestLine);
		// Error, report it
		return FALSE; // Fatal Error
	}
#endif
	return TRUE; // OK

}




/********************************************************************

static BOOL CheckMandatoryField(char **ppFields, BITMASK bmField, int iDataType, BITMASK *bmResult)

*********************************************************************/
static BOOL CheckMandatoryField(char **ppFields, BITMASK bmField, int iDataType, BITMASK *bmResult)
{ // Checking if the field has data and is numerical
	int iField = 1;
	BITMASK bmField2 = bmField;

	while (bmField2 /= 0x10)
		iField++;
	iField--;

	if (!ppFields[iField]) {
		*bmResult |= (bmField*INTERPRET_SYNTAX_ERR_MANDATORY);
		return FALSE;
	}
	if (strlen(ppFields[iField])) {
		switch (iDataType) {
		case DT_ALPHA:
			if (CheckAlpha(ppFields[iField]))
				// No error, report it
				*bmResult |= (bmField * INTERPRET_SYNTAX_OK);
			else
				// Error, Mandatory numeric field, report it
				*bmResult |= (bmField * INTERPRET_SYNTAX_ERR_MANDATORY);
			break;
		case DT_UNSIGNED:
			if (CheckUnsigned(ppFields[iField]))
				// No error, report it
				*bmResult |= (bmField * INTERPRET_SYNTAX_OK);
			else
				// Error, Mandatory numeric field, report it
				*bmResult |= (bmField * INTERPRET_SYNTAX_ERR_MANDATORY);
			break;
		case DT_NUMERIC:
			if (CheckNumeric(ppFields[iField]))
				// No error, report it
				*bmResult |= (bmField * INTERPRET_SYNTAX_OK);
			else
				// Error, Mandatory numeric field, report it
				*bmResult |= (bmField * INTERPRET_SYNTAX_ERR_MANDATORY);
			break;
		case DT_AMOUNT:
			if (CheckAmount(ppFields[iField]))
				// No error, report it
				*bmResult |= (bmField * INTERPRET_SYNTAX_OK);
			else
				// Error, Mandatory numeric field, report it
				*bmResult |= (bmField * INTERPRET_SYNTAX_ERR_MANDATORY);
			break;
		case DT_QUANTITY:
			if (CheckDecimal(ppFields[iField]))
				// No error, report it
				*bmResult |= (bmField * INTERPRET_SYNTAX_OK);
			else
				// Error, Mandatory numeric field, report it
				*bmResult |= (bmField * INTERPRET_SYNTAX_ERR_MANDATORY);
			break;
		case DT_DATE:
			if ((CheckUnsigned(ppFields[iField]))
				&& (25000000 >= atoi(ppFields[iField]))		// Year
				&& (12 >= ((atoi(ppFields[iField]) / 100) % 100))	// Months
				&& (31 >= (atoi(ppFields[iField]) % 100))		// Days
				)
				// No error, report it
				*bmResult |= (bmField*INTERPRET_SYNTAX_OK);
			else
				// Error, Mandatory numeric field, report it
				*bmResult |= (bmField*INTERPRET_SYNTAX_ERR_MANDATORY);
			break;
		case DT_PERIOD:
			if ((CheckUnsigned(ppFields[iField]))
				&& (250000 >= atoi(ppFields[iField]))			// Year
				&& (12 >= (atoi(ppFields[iField]) % 100))		// Months
				)
				// No error, report it
				*bmResult |= (bmField*INTERPRET_SYNTAX_OK);
			else
				// Error, Mandatory numeric field, report it
				*bmResult |= (bmField*INTERPRET_SYNTAX_ERR_MANDATORY);
			break;
		case DT_YEAR:
			if ((CheckUnsigned(ppFields[iField])) && (2500 >= atoi(ppFields[iField])))
				// No error, report it
				*bmResult |= (bmField*INTERPRET_SYNTAX_OK);
			else
				// Error, Mandatory numeric field, report it
				*bmResult |= (bmField*INTERPRET_SYNTAX_ERR_MANDATORY);
			break;
		case DT_YEAR_CODE:
			if ((CheckNumeric(ppFields[iField])) && (0 >= atoi(ppFields[iField])))
				// No error, report it
				*bmResult |= (bmField*INTERPRET_SYNTAX_OK);
			else
				// Error, Mandatory numeric field, report it
				*bmResult |= (bmField*INTERPRET_SYNTAX_ERR_MANDATORY);
			break;
		case DT_OBJECT:
		{
			// We are only checking the dimension is numeric 
			// and that there is data in the object field 
			// (if the object is defined we check later)
			char *pDim = ppFields[iField];
			char *pObj = 0;
			pDim--;
			while (pDim) {
				pDim++;
				pObj = strchr(pDim, OBJECT_SET_SEPARATOR);
				if (pObj) {
					*pObj = 0;
					if (!CheckNumeric(pDim))
						// Error, Mandatory numeric field, report it
						*bmResult |= (bmField*INTERPRET_SYNTAX_ERR_MANDATORY);
					*pObj = OBJECT_SET_SEPARATOR;
					pObj++; // Pointing at the object part
					// Checking object data
					if ((!strlen(pObj)) || (*pObj == OBJECT_SEPARATOR))
						*bmResult |= (bmField*INTERPRET_SYNTAX_ERR_MANDATORY);
					if ((!strcmp(pObj, "0")) && (!strcmp(pDim, "0"))) // Not allowed object ID Dim 0 obj 0
						*bmResult |= (bmField*INTERPRET_SYNTAX_ERR_MANDATORY);
					pDim = strchr(pObj, OBJECT_SEPARATOR); // CHecking for more objects
				}
				else { // No object seprator (an object is not defined
					*bmResult |= (bmField*INTERPRET_SYNTAX_ERR_MANDATORY);
					pDim = 0; // Quit
				}
			}
		}
		break;
		case DT_ANY:
		default:
			if (!ppFields[iField][0])
				*bmResult |= (bmField*INTERPRET_SYNTAX_ERR_MANDATORY);
			else {
				// Check for only spaced fields
				*bmResult |= (bmField*INTERPRET_SYNTAX_ERR_NOTE);
				{
					int i = 0;
					while (ppFields[iField][i]) {
						if (ppFields[iField][i] > ' ')
							*bmResult &= ~(bmField*INTERPRET_SYNTAX_ERR_NOTE);
						i++;
					}
				}
			}
			// No error, report it
			break;
		}
	}
	else
		// Error, Mandatory field, report it
		*bmResult |= (bmField*INTERPRET_SYNTAX_ERR_MANDATORY);

	if ((*bmResult) & bmField)
		return FALSE;
	return TRUE;
}


/********************************************************************

static BOOL CheckSieCommandExtField(char *ppFields, BITMASK bmField, BITMASK *bmResult)

*********************************************************************/
static BOOL CheckSieCommandExtField(char **ppFields, BITMASK bmField, BITMASK *bmResult)
{
	BOOL More = TRUE;
	BOOL Ext = FALSE;

	if (bmField > SIE_PARAM_STATUS_MAX)
		return FALSE;

	if (bmField <= 0)
		return FALSE;

	while (More) {
		More = CheckMandatoryField(ppFields, bmField, DT_ANY, bmResult);
		if (More) {
			*bmResult &= ~(bmField*INTERPRET_SYNTAX_MASK_ALL);
			*bmResult |= (bmField*INTERPRET_SYNTAX_EXT_FIELD);
			Ext = TRUE;
		}
		else {
			*bmResult &= ~(bmField*INTERPRET_SYNTAX_MASK_ALL);
			*bmResult |= (bmField*INTERPRET_SYNTAX_OK);
			More = FALSE;
		}
		bmField *= 0x10;

		if (bmField > SIE_PARAM_STATUS_MAX)
			More = FALSE;
	}
	if (Ext)
		return TRUE;
	return FALSE;

}

/********************************************************************

static BOOL CheckAlpha(char *String)

*********************************************************************/
static BOOL CheckAlpha(char *String)
{
	int i = 0;

	while (String[i]) {
		if (isdigit(String[i] & 0x7f))
			return FALSE;
		i++;
	}
	return TRUE;
}


/********************************************************************

static BOOL CheckNumeric(char *String)

*********************************************************************/
static BOOL CheckNumeric(char *String)
{
	int i = 0;

	while (String[i]) {
		if (!(isdigit(String[i] & 0x7f)
			|| (String[i] == '-')))
			return FALSE;
		i++;
	}
	return TRUE;
}


/********************************************************************

static BOOL CheckUnsigned(char *String)

*********************************************************************/
static BOOL CheckUnsigned(char *String)
{
	int i = 0;

	while (String[i]) {
		if (!isdigit(String[i] & 0x7f))
			return FALSE;
		i++;
	}
	return TRUE;
}


/********************************************************************

static BOOL CheckDecimal(char *String)

*********************************************************************/
static BOOL CheckDecimal(char *String)
{
	int i = 0;

	while (String[i]) {
		if (!(isdigit(String[i] & 0x7f)
			|| (String[i] == '-')
			|| (String[i] == '.')))
			return FALSE;
		i++;
	}
	return TRUE;
}


/********************************************************************

static BOOL CheckAmount(char *String)

*********************************************************************/
static BOOL CheckAmount(char *String)
{
	int i = 0;
	int DecimalCounter = 0;

	while (String[i]) {
		if (!(isdigit(String[i] & 0x7f)
			|| (String[i] == '-')
			|| (String[i] == '.')))
			return FALSE;
		if (DecimalCounter)
			DecimalCounter++;
		if (String[i] == '.')
			DecimalCounter = 1;
		i++;
	}
	if (DecimalCounter > 3)
		return FALSE; // More than two decimals, not allowed
	return TRUE;
}


/********************************************************************
*
* Characterset conversion CP_OEMCP: // PC8 CP_858 (PC DOS-ASCII) CP_ACP: // CP_1252 (Windows-ASCII)
* (CP_OEMCP: // PC8 CP_858 is the same as CP_437 except IBM fix of some danish/norwegian and portuguese chars)
*
*********************************************************************/
static char *ConvertTextCp858ToCp1252(char *Text)
{
	unsigned char *UnsignedText = (unsigned char *)Text;

	static unsigned char Cp858toCp1252[0x80] = {
		0xc7,  0xfc,  0xe9,  0xe2,  0xe4,  0xe0,  0xe5,  0xe7,	/* 80 - 87 */
		0xea,  0xeb,  0xe8,  0xef,  0xee,  0xec,  0xc4,  0xc5,	/* 88 - 8F */
		0xc9,  0xe6,  0xc6,  0xf4,  0xf6,  0xf2,  0xfb,  0xf9,	/* 90 - 97 */
		0xff,  0xd6,  0xdc,  0xf8,  0xa3,  0xd8,  0xd7,  0x83,	/* 98 - 9F */
		0xe1,  0xed,  0xf3,  0xfa,  0xf1,  0xd1,  0xaa,  0xba,	/* A0 - A7 */
		0xbf,  0xae,  0xac,  0xbd,  0xbc,  0xa1,  0xab,  0xbb,	/* A8 - AF */
		0x20,  0x20,  0x20,  0x20,  0x20,  0xc1,  0xc2,  0xc0,	/* B0 - B7 */
		0xa9,  0x20,  0x20,  0x20,  0x20,  0xa2,  0xa5,  0x20,	/* B8 - BF */
		0x20,  0x20,  0x20,  0x20,  0x20,  0x20,  0xe3,  0xc3,	/* C0 - C7 */
		0x20,  0x20,  0x20,  0x20,  0x20,  0x20,  0x20,  0xa4,	/* C8 - CF */
		0xf0,  0xd0,  0xca,  0xcb,  0xc8,  0x80,  0xcd,  0xce,	/* D0 - D7 */
		0xcf,  0x20,  0x20,  0x20,  0x20,  0xa6,  0xcc,  0x20,	/* D8 - DF */
		0xd3,  0xdf,  0xd4,  0xd2,  0xf5,  0xd5,  0xb5,  0xfe,	/* E0 - E7 */
		0xde,  0xda,  0xdb,  0xd9,  0xfd,  0xdd,  0xaf,  0xb4,	/* E8 - EF */
		0xad,  0xb1,  0x3d,  0xbe,  0xb6,  0xa7,  0xf7,  0xb8,	/* F0 - F7 */
		0xb0,  0xa8,  0xb7,  0xb9,  0xb3,  0xb2,  0x20,  0x20	/* F8 - FF */
	};

	size_t zt;
	size_t ztTextLength = strlen((const char *)Text);

	for (zt = 0; zt < ztTextLength; zt++) {
		if (UnsignedText[zt] >= 0x80)
			UnsignedText[zt] = Cp858toCp1252[UnsignedText[zt] - 0x80];
	}
	return (char *)Text;
}


/********************************************************************
*
* Characterset conversion CP_ACP: // CP_1252 (Windows-ASCII) to CP_OEMCP: // PC8 CP_858 (PC DOS-ASCII)
* (CP_OEMCP: // PC8 CP_858 is the same as CP_437 except IBM fix of some danish/norwegian and portuguese chars)
*
*********************************************************************/
static char *ConvertTextCp1252ToCp858(char *Text)
{
	unsigned char *UnsignedText = (unsigned char *)Text;

	static unsigned char Cp1252toCp858[0x80] = {
		0xd5,  0x20,  0x20,  0x9f,  0x20,  0x20,  0x20,  0x20,	/* 80 - 87 */
		0x20,  0x20,  0x20,  0x20,  0x20,  0x20,  0x20,  0x20,	/* 88 - 8F */
		0x20,  0x20,  0x20,  0x20,  0x20,  0x20,  0x20,  0x20,	/* 90 - 97 */
		0x20,  0x20,  0x20,  0x20,  0x20,  0x20,  0x20,  0x20,	/* 98 - 9F */
		0xff,  0xad,  0xbd,  0x9c,  0xcf,  0xbe,  0xdd,  0xf5,	/* A0 - A7 */
		0xf9,  0xb8,  0xa6,  0xae,  0xaa,  0xf0,  0xa9,  0xee,	/* A8 - AF */
		0xf8,  0xf1,  0xfd,  0xfc,  0xef,  0xe6,  0xf4,  0xfa,	/* B0 - B7 */
		0xf7,  0xfb,  0xa7,  0xaf,  0xac,  0xab,  0xf3,  0xa8,	/* B8 - BF */
		0xb7,  0xb5,  0xb6,  0xc7,  0x8e,  0x8f,  0x92,  0x80,	/* C0 - C7 */
		0xd4,  0x90,  0xd2,  0xd3,  0xde,  0xd6,  0xd7,  0xd8,	/* C8 - CF */
		0xd1,  0xa5,  0xe3,  0xe0,  0xe2,  0xe5,  0x99,  0x9e,	/* D0 - D7 */
		0x9d,  0xeb,  0xe9,  0xea,  0x9a,  0xed,  0xe8,  0xe1,	/* D8 - DF */
		0x85,  0xa0,  0x83,  0xc6,  0x84,  0x86,  0x91,  0x87,	/* E0 - E7 */
		0x8a,  0x82,  0x88,  0x89,  0x8d,  0xa1,  0x8c,  0x8b,	/* E8 - EF */
		0xd0,  0xa4,  0x95,  0xa2,  0x93,  0xe4,  0x94,  0xf6,  /* F0 - F7 */
		0x9b,  0x97,  0xa3,  0x96,  0x81,  0xec,  0xe7,  0x98	/* F8 - FF */
	};

	size_t zt;
	size_t ztTextLength = strlen((const char *)Text);

	for (zt = 0; zt < ztTextLength; zt++) {
		if (UnsignedText[zt] >= 0x80)
			UnsignedText[zt] = Cp1252toCp858[UnsignedText[zt] - 0x80];
	}
	return (char *)Text;
}


/********************************************************************
*
* Characterset conversion CP_ACP: // CP_1252 (Windows-ASCII) to UTF8 (Unicode)
*
*********************************************************************/
char *ConvertTextCp1252ToUtf8(char *pWinText)
{
	int iNewExtraLength = 0;
	UCHAR *ucpWinText = (UCHAR *)pWinText;
	char *pReturnBuffer = pWinText;
	int i = 0;

	while (ucpWinText[i]) {
		if (ucpWinText[i] > 0x7f)
			iNewExtraLength += 6;
		i++;
	}
	if (iNewExtraLength)
		pReturnBuffer = calloc(1, strlen(pWinText) + iNewExtraLength + 1);
	if (pReturnBuffer) {
		UCHAR *ucpReturnBufferEntry = (UCHAR *)pReturnBuffer;
		i = 0;
		while (ucpWinText[i]) {
			if (ucpWinText[i] > 0x7f) {
				*ucpReturnBufferEntry = (0xC0 | (ucpWinText[i] >> 6));
				ucpReturnBufferEntry++;
				*ucpReturnBufferEntry = (0x80 | (ucpWinText[i] & 0x3F));
			}
			else
				*ucpReturnBufferEntry = ucpWinText[i];
			i++;
			ucpReturnBufferEntry++;
		}
	}
	return pReturnBuffer;
}


/********************************************************************
*
static char *ConvertTextUtf8ToCp1252(char *pUtf8Text)
*
*********************************************************************/
static char *ConvertTextUtf8ToCp1252(char *pUtf8Text)
{
	UCHAR *ucpUtf8Text = (UCHAR *)pUtf8Text;
	UCHAR *ucpReturnBufferEntry = ucpUtf8Text;
	int i = 0;
	{
		while (ucpUtf8Text[i]) {
			if (ucpUtf8Text[i] == 0xC3) {
				i++;
				*ucpReturnBufferEntry = (0xc0 | (ucpUtf8Text[i] & 0x3F));
			}
			else
				*ucpReturnBufferEntry = ucpUtf8Text[i];
			i++;
			ucpReturnBufferEntry++;
		}
	}
	*ucpReturnBufferEntry = 0;
	return pUtf8Text;
}

/********************************************************************

Create SIE file data

*********************************************************************/
#define SIE_ASSETS		((!strcmp(pAccount->pType, ACCOUNT_TYPE_TANGIBLE)) || (!strcmp(pAccount->pType, ACCOUNT_TYPE_STOCK)) || (!strcmp(pAccount->pType, ACCOUNT_TYPE_RECEIVABLES)) || (!strcmp(pAccount->pType, ACCOUNT_TYPE_ASSETS)))
#define SIE_LIABILITIES	((!strcmp(pAccount->pType, ACCOUNT_TYPE_EQUITIES)) || (!strcmp(pAccount->pType, ACCOUNT_TYPE_PROVISIONS)) || (!strcmp(pAccount->pType, ACCOUNT_TYPE_LIABILITIES)))
#define SIE_INCOMES		(!strcmp(pAccount->pType, ACCOUNT_TYPE_REVENUES))
#define SIE_COSTS		((!strcmp(pAccount->pType, ACCOUNT_TYPE_SUPPLIES)) || (!strcmp(pAccount->pType, ACCOUNT_TYPE_EXPENSES)) || (!strcmp(pAccount->pType, ACCOUNT_TYPE_STAFF)) || (!strcmp(pAccount->pType, ACCOUNT_TYPE_LOSS_GAINS)) || (!strcmp(pAccount->pType, ACCOUNT_TYPE_FINANCIAL)) || (!strcmp(pAccount->pType, ACCOUNT_TYPE_AUX)))

static int WriteSieAccountStatements(FILE *pStream, AccountingType *pAccounting, BOOL QuantityOption, BOOL LeanChartsOption, BOOL AccountSortOption, DATEINT InitialDate, DATEINT FinalDate, char *FilterEnteredBy, char *pContent, char *pLanguage);
static int WriteSieDimObjStatements(FILE *pStream, AccountingType *pAccounting, BOOL LeanChartsOption, BOOL DimensionSortOption, BOOL ObjectSortOption, DATEINT InitialDate, DATEINT FinalDate, char * FilterEnteredBy, char *pContent, char *pLanguage);
static int WriteSieBalanceStatements(FILE *pStream, AccountingType *pAccounting, BOOL QuantityOption, BOOL ObjectOption, BOOL PeriodOption, BOOL AccountSortOption, BOOL DimensionSortOption, BOOL ObjectSortOption, DATEINT FinalDate, char *pContent, char *pLanguage);
static int WriteSieBalanceStatementLines(FILE *pStream, NUMERIC LedgerId, BalanceAccountType *pBalanceAccount, ObjectType *pObject, BOOL QuantityOption, BOOL PeriodOption, DATEINT FinalDate, char *pContent, char *pLanguage);
static int WriteSieVoucherStatements(FILE *pStream, AccountingType *pAccounting, BOOL CompactFileOption, BOOL QuantityOption, BOOL ObjectOption, BOOL PeriodOption, int VoucherSortOption, DATEINT InitialDate, DATEINT FinalDate, char *FilterEnteredBy, char *pContent, char *pLanguage);
static int WriteTransObjectList(FILE *pStream, LLIST pListObjectRef);
static int WriteTransOptionals(FILE *pStream, BOOL CompactFileOption, DATEINT dTransDate, char *pSieTransTitle, char *pQuantity, char *pSieTransOperator);
int StreamWriteSie(
	FILE *pStream,
	void *vpAccounting,
	char *pContent,
	char *pGenUserName,
	BOOL CompactFileOption,
	BOOL LeanChartsOption,
	BOOL QuantityOption,
	BOOL PeriodOption,
	BOOL ObjectOption,
	BOOL AccountSortOption,
	BOOL DimensionSortOption,
	BOOL ObjectSortOption,
	int  VoucherSortOption,
	char *pInitialDate,
	char *pFinalDate,
	char *FilterEnteredBy,
	char *pProgramName,
	char *pProgramVersion,
	char *pLanguage);

/*************************************************************

Write a Sie File

*************************************************************/
int FileWriteSie(
	char *pFilePathName,
	void *vpAccounting,
	char *pContent,
	char *pGenUserName,
	BOOL CompactFileOption,
	BOOL LeanChartsOption,
	BOOL QuantityOption,
	BOOL PeriodOption,
	BOOL ObjectOption,
	BOOL AccountSortOption,
	BOOL DimensionSortOption,
	BOOL ObjectSortOption,
	int  VoucherSortOption,
	char *pInitialDate,
	char *pFinalDate,
	char *FilterEnteredBy,
	char *pProgramName,
	char *pProgramVersion,
	char *pLanguage,
	BOOL bUtf8)
{
	if ((!pFilePathName) || (!*pFilePathName))
		return ERR_FILE_NAME;

	if ((!pContent) || (!*pContent))
		return ERR_FILE_CONTENT;

	// Open the file
	{
		FILE *pStream = FileOpen(pFilePathName, "wb");
		if (!pStream) {
#if 0 //#ifndef ANDROID // JNI can't take it
			int iError = ferror(pStream);
			iError = iError;
			clearerr(pStream);
#endif
			return ERR_OPEN_FILE; // Bad
		}

		// Write the SIE file
		{
			int iReturn = StreamWriteSie(
				pStream,
				vpAccounting,
				pContent,
				pGenUserName,
				CompactFileOption,
				LeanChartsOption,
				QuantityOption,
				PeriodOption,
				ObjectOption,
				AccountSortOption,
				DimensionSortOption,
				ObjectSortOption,
				VoucherSortOption,
				pInitialDate,
				pFinalDate,
				FilterEnteredBy,
				pProgramName,
				pProgramVersion,
				pLanguage);

			// Close the file 
			if (fclose(pStream) && (!iReturn))
				iReturn = ERR_CLOSE_FILE;

			// If bad delete the file 
			if (iReturn) // Bad
				remove(pFilePathName);

			// Convert it to the right characterset
			if (!bUtf8) {
				char	*pBuffer = FileRead(pFilePathName);
				if (pBuffer && *pBuffer) {
					char *pConvertBuffer = ConvertTextCp1252ToCp858(ConvertTextUtf8ToCp1252(pBuffer));
					FILE *pStreamConvert = FileOpen(pFilePathName, "wb");
					if (!pStreamConvert) {
#if 0//#if 0 //#ifndef ANDROID // JNI can't take it
						int iError = ferror(pStreamConvert);
						iError = iError;
						clearerr(pStreamConvert);
#endif
						return ERR_OPEN_FILE; // Bad
					}
					if (0 > fprintf(pStreamConvert, "%s", pConvertBuffer))
						return ERR_TECHNICAL;

					// Close the file 
					if (fclose(pStreamConvert) && (!iReturn))
						iReturn = ERR_CLOSE_FILE;
					// Free Data
					if (pConvertBuffer != pBuffer)
						free(pConvertBuffer);
					free(pBuffer);
				}

			}
			return iReturn;
		}
	}
}


/*************************************************************

Write a Sie Stream

*************************************************************/
int StreamWriteSie(
	FILE *pStream,
	void *vpAccounting,
	char *pContent,
	char *pGenUserName,
	BOOL CompactFileOption,
	BOOL LeanChartsOption,
	BOOL QuantityOption,
	BOOL PeriodOption,
	BOOL ObjectOption,
	BOOL AccountSortOption,
	BOOL DimensionSortOption,
	BOOL ObjectSortOption,
	int  VoucherSortOption,
	char *pInitialDate,
	char *pFinalDate,
	char *FilterEnteredBy,
	char *pProgramName,
	char *pProgramVersion,
	char *pLanguage)
{
	if (!vpAccounting)
		return (-1);	// Nothing to save
	if (!pStream)
		return (-2);	// File not found or opened
	{
		// Need to make a copy of Accounting to recode the accounts for a SIE-file (has fewer account types)
		AccountingType *pSieAccounting = ConvertAccountCoding(vpAccounting, FILE_TYPE_SIE);

		DATEINT InitialDate = 0;
		DATEINT FinalDate = 0;

		LedgerType *pDefaultLedger = GetLedger(pSieAccounting, 0);

		if (strcmp(pContent, CONTENT_LABEL_CHARTS)) { // Not just Charts
			if ((!pGenUserName) || (!*pGenUserName))
				pGenUserName = NulString;

			if (!strcmp(pContent, CONTENT_LABEL_VOUCHER))
				LeanChartsOption = TRUE;
			else
				FilterEnteredBy = 0;

			// OMFATTN as switch??

			if ((!pSieAccounting->pEntity->pName) || (!*pSieAccounting->pEntity->pName)) {
				// Company name iss mandatory
				DestroyAccounting(&pSieAccounting);
				return (-2);
			}


			if ((!pDefaultLedger)
				|| (((!strcmp(pContent, CONTENT_LABEL_BALANCE)) || (!strcmp(pContent, CONTENT_LABEL_LEDGER)))
					&& ((!pDefaultLedger->YearStart) || (!pDefaultLedger->YearEnd)))) {
				DestroyAccounting(&pSieAccounting);
				return 3;
			}

			if (pInitialDate && *pInitialDate) { // Only used in SIE_4I
				InitialDate = StringToDate(pInitialDate);
				if (InitialDate < pDefaultLedger->YearStart)
					InitialDate = pDefaultLedger->YearStart;
			}
			//		else
			//			InitialDate = pDefaultLedger->YearStart;

			if (pFinalDate && *pFinalDate) { // The complete year
				char szTempFinalDate[LENGTH_LONG_DATE];
				memset(szTempFinalDate, 0, LENGTH_LONG_DATE);
				strncpy(szTempFinalDate, pFinalDate, LENGTH_LONG_DATE);
				if ((!strcmp(pContent, CONTENT_LABEL_BALANCE)) || (!strcmp(pContent, CONTENT_LABEL_LEDGER))) {
					szTempFinalDate[strlen(szTempFinalDate) - 2] = 0;
					strcat(szTempFinalDate, "31");
				}
				FinalDate = StringToDate(szTempFinalDate);
				if (FinalDate > pDefaultLedger->YearEnd)
					FinalDate = pDefaultLedger->YearEnd;
			}
			else if (pDefaultLedger->YearEnd)
				FinalDate = pDefaultLedger->YearEnd;
			else
				FinalDate = (DATEINT)(-1);
		}
		// #FLAGGA x	
		if (0 > fprintf(pStream, "#FLAGGA 0\r\n")) {
			DestroyAccounting(&pSieAccounting);
			return (-2);
		}
		//#FORMAT PC8
		if (0 > fprintf(pStream, "#FORMAT PC8\r\n")) {
			DestroyAccounting(&pSieAccounting);
			return (-2);
		}
		// #GEN datum sign
		if (0 > fprintf(pStream, "#GEN %llu \"%s\"\r\n",
			GetSystemDate(),
			pGenUserName)) {
			DestroyAccounting(&pSieAccounting);
			return (-2);
		}
		// #PROSA text	
		if (pSieAccounting->pListComment) { // Optional
			TextType *pComment
				= LListGetFirstRecordData(pSieAccounting->pListComment);
			while (pComment) {
				if (0 > fprintf(pStream, "#PROSA \"%s\"\r\n", pComment->pText)) {
					DestroyAccounting(&pSieAccounting);
					return (-2);
				}
				pComment = LListGetNextRecordData(pSieAccounting->pListComment);
			}
		}
		// #SIETYP typnr
		if (!strcmp(pContent, CONTENT_LABEL_CHARTS)) {
			if (0 > fprintf(pStream, "#SIETYP 0\r\n")) {
				DestroyAccounting(&pSieAccounting);
				return (-2);
			}
		}
		else if (!strcmp(pContent, CONTENT_LABEL_BALANCE)) {
			if (ObjectOption) {
				if (0 > fprintf(pStream, "#SIETYP 3\r\n")) {
					DestroyAccounting(&pSieAccounting);
					return (-2);
				}
			}
			else if (PeriodOption) {
				if (0 > fprintf(pStream, "#SIETYP 2\r\n")) {
					DestroyAccounting(&pSieAccounting);
					return (-2);
				}
			}
			else {
				if (0 > fprintf(pStream, "#SIETYP 1\r\n")) {
					DestroyAccounting(&pSieAccounting);
					return (-2);
				}
			}
		}
		else {
			if (0 > fprintf(pStream, "#SIETYP 4\r\n")) {
				DestroyAccounting(&pSieAccounting);
				return (-2);
			}
		}
		// #PROGRAM programnamn version
		if (0 > fprintf(pStream, "#PROGRAM \"%s\" \"%s\"\r\n",
			pProgramName,
			pProgramVersion)) {
			DestroyAccounting(&pSieAccounting);
			return (-2);
		}
		if (strcmp(pContent, CONTENT_LABEL_CHARTS)) { // Not just Charts
			// #OMFATTN datum  
			if ((!strcmp(pContent, CONTENT_LABEL_BALANCE)) || (!strcmp(pContent, CONTENT_LABEL_LEDGER))) {
				// The #OMFATTN information is about the content extension of the file written
				if (0 > fprintf(pStream, "#OMFATTN %llu\r\n", GetLastEntryDate(pSieAccounting))) {
					DestroyAccounting(&pSieAccounting);
					return (-2);
				}
			}
			// #FTYP Företagstyp	
			if (((!strcmp(pContent, CONTENT_LABEL_BALANCE)) || (!strcmp(pContent, CONTENT_LABEL_LEDGER)))
				|| ((!CompactFileOption) && (!strcmp(pContent, CONTENT_LABEL_VOUCHER)))) {
				if (pSieAccounting->pEntity->pEntityType && *pSieAccounting->pEntity->pEntityType) { // Optional 
					if (0 > fprintf(pStream, "#FTYP %s\r\n", pSieAccounting->pEntity->pEntityType)) {
						DestroyAccounting(&pSieAccounting);
						return (-2);
					}
				}
			}
			// #ORGNR orgnr förvnr verknr
			if (((!strcmp(pContent, CONTENT_LABEL_BALANCE)) || (!strcmp(pContent, CONTENT_LABEL_LEDGER)))
				|| ((!CompactFileOption) && (!strcmp(pContent, CONTENT_LABEL_VOUCHER)))) {
				if (pSieAccounting->pEntity->pLegalRegId && *pSieAccounting->pEntity->pLegalRegId) { // Optional
					// verknr Not used since many years
					if (0 > fprintf(pStream, "#ORGNR %s\r\n", pSieAccounting->pEntity->pLegalRegId)) {
						DestroyAccounting(&pSieAccounting);
						return (-2);
					}
				}
			}
			// #FNR företagsid
			if (((!strcmp(pContent, CONTENT_LABEL_BALANCE)) || (!strcmp(pContent, CONTENT_LABEL_LEDGER)))
				|| ((!CompactFileOption) && (!strcmp(pContent, CONTENT_LABEL_VOUCHER)))) {
				if (pSieAccounting->pEntity->pProprietaryId && *pSieAccounting->pEntity->pProprietaryId) { // Optional
					if (0 > fprintf(pStream, "#FNR \"%s\"\r\n", pSieAccounting->pEntity->pProprietaryId)) {
						DestroyAccounting(&pSieAccounting);
						return (-2);
					}
				}
			}
			// #BKOD SNI-kod	
			if ((!strcmp(pContent, CONTENT_LABEL_BALANCE)) || (!strcmp(pContent, CONTENT_LABEL_LEDGER))) {
				if ((pSieAccounting->pEntity->pStatisticalClass && *pSieAccounting->pEntity->pStatisticalClass)) { // Optional 
					if (0 > fprintf(pStream, "#BKOD %s\r\n", pSieAccounting->pEntity->pStatisticalClass)) {
						DestroyAccounting(&pSieAccounting);
						return (-2);
					}
				}
			}
			// #FNAMN företagsnamn	
			if (strcmp(pContent, CONTENT_LABEL_CHARTS)) { // Not just Charts
				if (pSieAccounting->pEntity->pName && *pSieAccounting->pEntity->pName) { // Mandatory
					if (0 > fprintf(pStream, "#FNAMN \"%s\"\r\n", pSieAccounting->pEntity->pName)) {
						DestroyAccounting(&pSieAccounting);
						return (-2);
					}
				}
				else
					return 1;
			}
			// #ADRESS kontakt utdelningsadr postadr tel	
			if (((!strcmp(pContent, CONTENT_LABEL_BALANCE)) || (!strcmp(pContent, CONTENT_LABEL_LEDGER)))
				|| ((!CompactFileOption) && (!strcmp(pContent, CONTENT_LABEL_VOUCHER)))) {
				ContactType *pContact
					= LListGetFirstRecordData(pSieAccounting->pEntity->pListContact);
				char *pContactName = NulString;
				if (pContact && pContact->pName && *pContact->pName)
					pContactName = pContact->pName;
				// Contact name
				char *pAddress
					= LListGetFirstRecordData(pSieAccounting->pEntity->pListAddress);
				if (pContact
					|| pAddress
					|| (pSieAccounting->pEntity->pPostalCode && *pSieAccounting->pEntity->pPostalCode)
					|| (pSieAccounting->pEntity->pPostalAddress && *pSieAccounting->pEntity->pPostalAddress)
					|| (pSieAccounting->pEntity->pPostalCountry && *pSieAccounting->pEntity->pPostalCountry)) {
					if (0 > fprintf(pStream, "#ADRESS \"%s\" \"", pContactName)) {
						DestroyAccounting(&pSieAccounting);
						return (-2);
					}
					// Street address
					{
						pAddress
							= LListGetFirstRecordData(pSieAccounting->pEntity->pListAddress);
						while (pAddress) {
							BOOL bSpace = FALSE;
							if (*pAddress) {
								if (0 > fprintf(pStream, "%s", pAddress)) {
									DestroyAccounting(&pSieAccounting);
									return (-2);
								}
							}
							if (pAddress[strlen(pAddress) - 1] == ' ')
								bSpace = TRUE;
							pAddress = LListGetNextRecordData(pSieAccounting->pEntity->pListAddress);
							if (pAddress && (!bSpace) && (0 > fprintf(pStream, " "))) {
								DestroyAccounting(&pSieAccounting);
								return (-2);
							}
						}
					}
					if (0 > fprintf(pStream, "\" \"")) {
						DestroyAccounting(&pSieAccounting);
						return (-2);
					}
					// Postal adress pPostalCode
					{
						if (pSieAccounting->pEntity->pPostalCode && *pSieAccounting->pEntity->pPostalCode) {
							if (0 > fprintf(pStream, "%s", pSieAccounting->pEntity->pPostalCode)) {
								DestroyAccounting(&pSieAccounting);
								return (-2);
							}
							if ((pSieAccounting->pEntity->pPostalAddress && *pSieAccounting->pEntity->pPostalAddress)
								|| (pSieAccounting->pEntity->pPostalCountry && *pSieAccounting->pEntity->pPostalCountry)) {
								if (0 > fprintf(pStream, " ")) {
									DestroyAccounting(&pSieAccounting);
									return (-2);
								}
							}
						}
						if (pSieAccounting->pEntity->pPostalAddress && *pSieAccounting->pEntity->pPostalAddress) {
							if (0 > fprintf(pStream, "%s", pSieAccounting->pEntity->pPostalAddress)) {
								DestroyAccounting(&pSieAccounting);
								return (-2);
							}
						}
						if ((pSieAccounting->pEntity->pPostalAddress && *pSieAccounting->pEntity->pPostalAddress)
							&& (pSieAccounting->pEntity->pPostalCountry && *pSieAccounting->pEntity->pPostalCountry)) {
							if (0 > fprintf(pStream, " ")) {
								DestroyAccounting(&pSieAccounting);
								return (-2);
							}
						}
						if (pSieAccounting->pEntity->pPostalCountry && *pSieAccounting->pEntity->pPostalCountry) {
							if (0 > fprintf(pStream, "%s", pSieAccounting->pEntity->pPostalCountry)) {
								DestroyAccounting(&pSieAccounting);
								return (-2);
							}
						}
						if (0 > fprintf(pStream, "\" \"")) {
							DestroyAccounting(&pSieAccounting);
							return (-2);
						}
						// Phone
						if (pContact) {
							char *pContactPhone
								= LListGetFirstRecordData(pContact->pListPhone);
							while (pContactPhone) {
								BOOL bSpace = FALSE;
								if (*pContactPhone) {
									if (0 > fprintf(pStream, "%s", pContactPhone)) {
										DestroyAccounting(&pSieAccounting);
										return (-2);
									}
								}
								if (pContactPhone[strlen(pContactPhone)] == ' ')
									bSpace = TRUE;
								pContactPhone = LListGetNextRecordData(pContact->pListPhone);
								if (pContactPhone && (!bSpace) && (0 > fprintf(pStream, " "))) {
									DestroyAccounting(&pSieAccounting);
									return (-2);
								}
							}
						}
						// End
						if (0 > fprintf(pStream, "\"\r\n")) {
							DestroyAccounting(&pSieAccounting);
							return (-2);
						}
					}
				}
			}
			// #RAR årsnr start slut
			if ((!strcmp(pContent, CONTENT_LABEL_BALANCE)) || (!strcmp(pContent, CONTENT_LABEL_LEDGER))
				|| (!strcmp(pContent, CONTENT_LABEL_VOUCHER))) {
				LedgerType *pLedger
					= LListGetFirstRecordData(pSieAccounting->pLedgersChart->pListLedger);
				while (pLedger) {
					if (strcmp(pContent, CONTENT_LABEL_VOUCHER) || (!pLedger->Id)) {
						if (0 > fprintf(pStream, "#RAR %lli %llu %llu\r\n",
							pLedger->Id,
							pLedger->YearStart,
							pLedger->YearEnd)) {
							DestroyAccounting(&pSieAccounting);
							return (-2);
						}
					}
					pLedger = LListGetNextRecordData(pSieAccounting->pLedgersChart->pListLedger);
				} // End looking for dimension
			}
			// #TAXAR år
			if ((((!strcmp(pContent, CONTENT_LABEL_BALANCE)) || (!strcmp(pContent, CONTENT_LABEL_LEDGER)))
				|| ((!CompactFileOption) && (!strcmp(pContent, CONTENT_LABEL_VOUCHER))))
				&& ((pDefaultLedger->YearEnd / 10000) <= 2008)) {
				char szTaxDate[LENGTH_LONG_DATE];
				if (0 > fprintf(pStream, "#TAXAR %s\r\n", NumericToStr(pDefaultLedger->YearEnd / 10000, szTaxDate))) {
					DestroyAccounting(&pSieAccounting);
					return (-2);
				}
			}
			// #KPTYP kontoplanstyp
			if (pSieAccounting->pGaap && *pSieAccounting->pGaap) {
				if (pSieAccounting->pGaap && *pSieAccounting->pGaap) { // Optional
					if (!strncmp(pSieAccounting->pGaap, BAS2000, 4))
					{
						if (0 > fprintf(pStream, "#KPTYP EUBAS97\r\n")) {
							DestroyAccounting(&pSieAccounting);
							return (-2);
						}
					}
					else {
						if (0 > fprintf(pStream, "#KPTYP %s\r\n", pSieAccounting->pGaap)) {
							DestroyAccounting(&pSieAccounting);
							return (-2);
						}
					}
				}
			}
			// #VALUTA Valutakod
			if (strcmp(pContent, CONTENT_LABEL_CHARTS)) { // Not just Charts {
				if (pSieAccounting->pCurrency && *pSieAccounting->pCurrency) {
					if (0 > fprintf(pStream, "#VALUTA %s\r\n", pSieAccounting->pCurrency)) {
						DestroyAccounting(&pSieAccounting);
						return (-2);
					}
				}
			}
		}
		// #KONTO kontonr kontonamn
		// #KTYP kontonr kontotyp
		// #SRU konto SRU-kod
		// #ENHET kontonr enhet
		WriteSieAccountStatements(pStream, pSieAccounting, QuantityOption, LeanChartsOption, AccountSortOption, InitialDate, FinalDate, FilterEnteredBy, pContent, pLanguage);

		// #DIM dimensionsnr namn
		// #UNDERDIM dimensionsnr namn superdimension
		// #OBJEKT dimensionsnr objektnr objektnamn
		if (ObjectOption)
			WriteSieDimObjStatements(pStream, pSieAccounting, LeanChartsOption, DimensionSortOption, ObjectSortOption, InitialDate, FinalDate, FilterEnteredBy, pContent, pLanguage);

		// #IB årsnr konto saldo kvantitet
		// #UB årsnr konto saldo kvantitet
		// #RES årsnr konto saldo kvantitet
		// #PSALDO årsnr period konto {dimensionsnr objektnr} saldo kvantitet 
		// #PBUDGET årsnr period konto {dimensionsnr objektnr} saldo kvantitet
		if ((!strcmp(pContent, CONTENT_LABEL_BALANCE)) || (!strcmp(pContent, CONTENT_LABEL_LEDGER)))
			WriteSieBalanceStatements(pStream, pSieAccounting, QuantityOption, ObjectOption, PeriodOption, AccountSortOption, DimensionSortOption, ObjectSortOption, FinalDate, pContent, pLanguage);

		// #VER serie vernr verdatum vertext regdatum sign
		// #TRANS kontonr {objektlista} belopp transdat transtext kvantitet sign			
		// #BTRANS kontonr {objektlista} belopp transdat transtext kvantitet sign			
		// #RTRANS kontonr {objektlista} belopp transdat transtext kvantitet sign			
		if ((!strcmp(pContent, CONTENT_LABEL_VOUCHER)) || (!strcmp(pContent, CONTENT_LABEL_LEDGER)))
			WriteSieVoucherStatements(pStream, pSieAccounting, CompactFileOption, QuantityOption, ObjectOption, PeriodOption, VoucherSortOption, InitialDate, FinalDate, FilterEnteredBy, pContent, pLanguage);

		DestroyAccounting(&pSieAccounting);
		return 0; // Good
	}
}


/********************************************************************
*
static int WriteSieAccountStatements(FILE *pStream, AccountingType *pAccounting, BOOL QuantityOption, BOOL LeanChartsOption, BOOL AccountSortOption, DATEINT InitialDate, DATEINT FinalDate, char *FilterEnteredBy, char *pContent, char *pLanguage)
*
*********************************************************************/
static int WriteSieAccountStatements(FILE *pStream, AccountingType *pAccounting, BOOL QuantityOption, BOOL LeanChartsOption, BOOL AccountSortOption, DATEINT InitialDate, DATEINT FinalDate, char *FilterEnteredBy, char *pContent, char *pLanguage)
{
	// #KONTO kontonr kontonamn
	// #KTYP kontonr kontotyp
	// #SRU konto SRU-kod
	// #ENHET kontonr enhet

	if (!pAccounting->pAccountsChart)
		return 0; // No data

	{
		AccountType *pAccount
			= LListGetFirstRecordData(pAccounting->pAccountsChart->pListAccount);
		while (pAccount) {
			if ((!LeanChartsOption) || (!pAccount->InActive)) {
				// #KONTO
				if (0 > fprintf(pStream, "#KONTO %s \"%s\"\r\n", pAccount->pId, GetTextListText(pAccount->pListName, pLanguage)))
					return (-2);

				// #KYP
				if (strcmp(pContent, CONTENT_LABEL_VOUCHER)
					|| ((!LeanChartsOption) && (!strcmp(pContent, CONTENT_LABEL_VOUCHER)))) {
					if (pAccount->pType) {
						if (SIE_ASSETS) {
							if (0 > fprintf(pStream, "#KTYP %s T\r\n", pAccount->pId))
								return (-2);
						}
						else if (SIE_LIABILITIES) {
							if (0 > fprintf(pStream, "#KTYP %s S\r\n", pAccount->pId))
								return (-2);
						}
						else if (SIE_INCOMES) {
							if (0 > fprintf(pStream, "#KTYP %s I\r\n", pAccount->pId))
								return (-2);
						}
						else if (SIE_COSTS) {
							if (0 > fprintf(pStream, "#KTYP %s K\r\n", pAccount->pId))
								return (-2);
						}
					}
				}

				// #ENHET
				if (QuantityOption && *GetTextListText(pAccount->pListQuantityUnit, pLanguage)) {
					if (GetTextListText(pAccount->pListQuantityUnit, pLanguage)) {
						if (0 > fprintf(pStream, "#ENHET %s \"%s\"\r\n",
							pAccount->pId,
							GetTextListText(pAccount->pListQuantityUnit, pLanguage)))
							return (-2);
					}
				}

				// #SRU
				if (strcmp(pContent, CONTENT_LABEL_VOUCHER)
					|| ((!LeanChartsOption) && (!strcmp(pContent, CONTENT_LABEL_VOUCHER)))) {
					LedgerType *pDefaultLedger = GetLedger(pAccounting, 0);
					NUMERIC	ValidYear = (NUMERIC)(pDefaultLedger->YearEnd / 10000);
					//BOOL TaxcodesNoted=FALSE;
					AccountCodeType *pAccountCode
						= LListGetFirstRecordData(pAccount->pListAccountCode);
					while (pAccountCode) {
						if (pAccountCode->pValidYear
							&& (StringToNumeric(pAccountCode->pValidYear) == ValidYear)
							&& ((ValidYear <= 2008) || (pAccounting->pEntity && pAccounting->pEntity->pEntityType && *pAccounting->pEntity->pEntityType))
							&& (!strncmp(pAccountCode->pCodeType, "SRU", 3))
							&& ((ValidYear <= 2008) || (pAccounting->pEntity && pAccounting->pEntity->pEntityType && strstr(pAccountCode->pCodeType, pAccounting->pEntity->pEntityType)))) {
							if (0 > fprintf(pStream, "#SRU %s \"%s\"\r\n", pAccount->pId, pAccountCode->pCode))
								return (-2);
						}
						pAccountCode = LListGetNextRecordData(pAccount->pListAccountCode);
					}
				}
			}
			pAccount = LListGetNextRecordData(pAccounting->pAccountsChart->pListAccount);
		}
	}
	return 0;
}


/********************************************************************
*
static int WriteSieDimObjStatements(FILE *pStream, AccountingType *pAccounting, BOOL LeanChartsOption, BOOL DimensionSortOption, BOOL ObjectSortOption, DATEINT InitialDate, DATEINT FinalDate, char * FilterEnteredBy, char *pContent, char *pLanguage)
*
*********************************************************************/
static int WriteSieDimObjStatements(FILE *pStream, AccountingType *pAccounting, BOOL LeanChartsOption, BOOL DimensionSortOption, BOOL ObjectSortOption, DATEINT InitialDate, DATEINT FinalDate, char * FilterEnteredBy, char *pContent, char *pLanguage)
{
	// #DIM dimensionsnr namn
	// #UNDERDIM dimensionsnr namn superdimension
	// #OBJEKT dimensionsnr objektnr objektnamn

	if (!pAccounting->pObjectsChart)
		return 0; // No data

	{
		DimensionType *pDimension
			= LListGetFirstRecordData(pAccounting->pObjectsChart->pListDimension);
		while (pDimension) {
			if ((((!LeanChartsOption) || IsDimensionActive(pAccounting, pDimension)) && (pDimension->Id != 0))
				&& LListGetFirstRecordData(pDimension->pListObject)) {
				if (pDimension->pSuperDimension) {
					if (0 > fprintf(pStream, "#UNDERDIM %llu \"%s\" %llu\r\n",
						pDimension->Id,
						GetTextListText(pDimension->pListName, pLanguage),
						((DimensionType *)pDimension->pSuperDimension)->Id))
						return (-2);
				}
				else {
					if (0 > fprintf(pStream, "#DIM %llu \"%s\"\r\n",
						pDimension->Id,
						GetTextListText(pDimension->pListName, pLanguage)))
						return (-2);
				}

				// Objects in the dimension
				{
					ObjectType *pObject
						= LListGetFirstRecordData(pDimension->pListObject);
					while (pObject) {
						if ((!LeanChartsOption) || (!pObject->InActive)) {
							if (0 > fprintf(pStream, "#OBJEKT %llu \"%s\" \"%s\"\r\n",
								pDimension->Id,
								pObject->pId,
								GetTextListText(pObject->pListName, pLanguage)))
								return (-2);
						}
						pObject = LListGetNextRecordData(pDimension->pListObject);
					}
				}
			}
			pDimension = LListGetNextRecordData(pAccounting->pObjectsChart->pListDimension);
		}
	}
	return 0;
}


/********************************************************************
*
static int WriteSieBalanceStatements(FILE *pStream, AccountingType *pAccounting, BOOL QuantityOption, BOOL ObjectOption, BOOL PeriodOption, BOOL AccountSortOption, BOOL DimensionSortOption, BOOL ObjectSortOption, DATEINT FinalDate, char *pContent, char *pLanguage)
*
*********************************************************************/
static int WriteSieBalanceStatements(FILE *pStream, AccountingType *pAccounting, BOOL QuantityOption, BOOL ObjectOption, BOOL PeriodOption, BOOL AccountSortOption, BOOL DimensionSortOption, BOOL ObjectSortOption, DATEINT FinalDate, char *pContent, char *pLanguage)
{
	// #IB årsnr konto saldo kvantitet
	// #UB årsnr konto saldo kvantitet
	// #RES årsnr konto saldo kvantitet
	// #PSALDO årsnr period konto {dimensionsnr objektnr} saldo kvantitet 
	// #PBUDGET årsnr period konto {dimensionsnr objektnr} saldo kvantitet
	if (pAccounting->pAccountsChart) {
		LedgerType *pLedger
			= LListGetFirstRecordData(pAccounting->pLedgersChart->pListLedger);
		while (pLedger) {
			BalanceReportType *pReport
				= LListGetFirstRecordData(pLedger->pListReport);
			while (pReport) {
				ObjectType *pObject = 0;
				if (pReport->pObject)
					pObject = pReport->pObject;
				if ((!pObject) /* All */ || ObjectOption) {
					BalanceAccountType *pBalanceAccount
						= LListGetFirstRecordData(pReport->pListBalanceAccount);
					while (pBalanceAccount) {
						WriteSieBalanceStatementLines(
							pStream,
							pLedger->Id,
							pBalanceAccount,
							pObject,
							QuantityOption,
							PeriodOption,
							FinalDate,
							pContent,
							pLanguage);

						pBalanceAccount = LListGetNextRecordData(pReport->pListBalanceAccount);
					}
				}
				pReport = LListGetNextRecordData(pLedger->pListReport);
			}
			pLedger = LListGetNextRecordData(pAccounting->pLedgersChart->pListLedger);
		}
		pLedger = GetLedger(pAccounting, 0);
	}
	return 0; // OK
}


/********************************************************************
*
static int WriteSieBalanceStatementLines(FILE *pStream, NUMERIC LedgerId, BalanceAccountType *pBalanceAccount, ObjectType *pObject, BOOL QuantityOption, BOOL PeriodOption, DATEINT FinalDate, char *pContent, char *pLanguage)
*
*********************************************************************/
static int WriteSieBalanceStatementLines(FILE *pStream, NUMERIC LedgerId, BalanceAccountType *pBalanceAccount, ObjectType *pObject, BOOL QuantityOption, BOOL PeriodOption, DATEINT FinalDate, char *pContent, char *pLanguage)
{
	// #IB årsnr konto saldo kvantitet
	// #UB årsnr konto saldo kvantitet
	// #RES årsnr konto saldo kvantitet
	// #PSALDO årsnr period konto {dimensionsnr objektnr} saldo kvantitet 
	// #PBUDGET årsnr period konto {dimensionsnr objektnr} saldo kvantitet
	char szAmount[LENGTH_AMOUNT_STRING];
	char szQuantity[LENGTH_QUANT_STRING];

	if (pBalanceAccount && pBalanceAccount->pAccount) {
		AccountType *pAccount = pBalanceAccount->pAccount;

		if (pObject && *pObject->pId && strcmp(pObject->pId, "0") && (!(pObject->pDimension && pObject->pDimension->Id)))
			return (-1); //Fatal error

		if (pObject && (!(pObject->pDimension && pObject->pDimension->Id)))
			pObject = 0;

		// #IB, #OIB
		if ((pBalanceAccount->Opening || (!DecimalCmp(pBalanceAccount->OpeningQuantity, 0)))
			&& (SIE_ASSETS || SIE_LIABILITIES)) { //Non-coded Swedish result accounts 
			memset(szAmount, 0, LENGTH_AMOUNT_STRING);
			memset(szQuantity, 0, LENGTH_QUANT_STRING);
			if (pBalanceAccount->Opening)
				AmountToStr(pBalanceAccount->Opening, szAmount);
			else
				strcpy(szAmount, "0");

			if (QuantityOption) {
				strcpy(szQuantity, " ");
				if (!DecimalCmp(pBalanceAccount->OpeningQuantity, 0))
					DecimalToStr(pBalanceAccount->OpeningQuantity, &szQuantity[strlen(szQuantity)]);
				else if (!(*GetTextListText(pAccount->pListQuantityUnit, pLanguage)))
					strcpy(szQuantity, "\0");
				else
					strcat(szQuantity, "0");
			}
			else
				strcpy(szQuantity, "\0");

			if (pObject && pObject->pDimension && pObject->pDimension->Id) {
				// An object
				if (0 > fprintf(pStream, "#OIB %lli %s {%llu \"%s\"} %s%s\r\n",
					LedgerId,
					pAccount->pId,
					pObject->pDimension->Id,
					pObject->pId,
					szAmount,
					szQuantity))
					return (-2);
			}
			else {
				// All Result (icomes or costs)
				if (0 > fprintf(pStream, "#IB %lli %s %s%s\r\n",
					LedgerId,
					pAccount->pId,
					szAmount,
					szQuantity))
					return (-2);
			}
		}
		// #UB, #OUB, #RES
		if (pBalanceAccount->Forward || (!DecimalCmp(pBalanceAccount->ForwardQuantity, 0))) {
			memset(szAmount, 0, LENGTH_AMOUNT_STRING);
			memset(szQuantity, 0, LENGTH_QUANT_STRING);
			if (pBalanceAccount->Forward)
				AmountToStr(pBalanceAccount->Forward, szAmount);
			else
				strcpy(szAmount, "0");

			if (QuantityOption) {
				strcpy(szQuantity, " ");
				if (!DecimalCmp(pBalanceAccount->ForwardQuantity, 0))
					DecimalToStr(pBalanceAccount->ForwardQuantity, &szQuantity[strlen(szQuantity)]);
				else if (!(*GetTextListText(pAccount->pListQuantityUnit, pLanguage)))
					strcpy(szQuantity, "\0");
				else
					strcat(szQuantity, "0");
			}
			else
				strcpy(szQuantity, "\0");
			{
				if (pObject && pObject->pDimension && pObject->pDimension->Id) {
					// An object
					if (0 > fprintf(pStream, "#OUB %lli %s {%llu \"%s\"} %s%s\r\n",
						LedgerId,
						pAccount->pId,
						pObject->pDimension->Id,
						pObject->pId,
						szAmount,
						szQuantity))
						return (-2);
				}
				else if (SIE_INCOMES || SIE_COSTS) {
					// All Result (icomes or costs)
					if (0 > fprintf(pStream, "#RES %lli %s %s%s\r\n",
						LedgerId,
						pAccount->pId,
						szAmount,
						szQuantity))
						return (-2);
				}
				else if (SIE_ASSETS || SIE_LIABILITIES) {
					if (0 > fprintf(pStream, "#UB %lli %s %s%s\r\n",
						LedgerId,
						pAccount->pId,
						szAmount,
						szQuantity))
						return (-2);
				}
			}
		}
		if (PeriodOption) {
			char szYearMonth[LENGTH_LONG_DATE];
			memset(szYearMonth, 0, LENGTH_LONG_DATE);
			{
				BalanceAccountPeriodType *pMonth
					= LListGetFirstRecordData(pBalanceAccount->pListMonth);
				while (pMonth) {
					NumericToStr(pMonth->End, szYearMonth);
					szYearMonth[6] = 0; // No days

					// #PSALDO
					if (pMonth->Outcome || (!DecimalCmp(pMonth->OutcomeQuantity, 0))) {
						memset(szAmount, 0, LENGTH_AMOUNT_STRING);
						memset(szQuantity, 0, LENGTH_QUANT_STRING);
						if (pMonth->Outcome)
							AmountToStr(pMonth->Outcome, szAmount);
						else
							strcpy(szAmount, "0");

						if (QuantityOption) {
							strcpy(szQuantity, " ");
							if (!DecimalCmp(pMonth->OutcomeQuantity, 0))
								DecimalToStr(pMonth->OutcomeQuantity, &szQuantity[strlen(szQuantity)]);
							else if (!(*GetTextListText(pAccount->pListQuantityUnit, pLanguage)))
								strcpy(szQuantity, "\0");
							else
								strcat(szQuantity, "0");
						}
						else
							strcpy(szQuantity, "\0");

						if (pObject && pObject->pDimension && pObject->pDimension->Id) {
							if (0 > fprintf(pStream, "#PSALDO %lli %s %s {%llu \"%s\"} %s%s\r\n",
								LedgerId,
								szYearMonth,
								pAccount->pId,
								pObject->pDimension->Id,
								pObject->pId,
								szAmount,
								szQuantity))
								return (-2);
						}
						else {
							if (0 > fprintf(pStream, "#PSALDO %lli %s %s {} %s%s\r\n",
								LedgerId,
								szYearMonth,
								pAccount->pId,
								szAmount,
								szQuantity))
								return (-2);
						}
					}

					// #PBUDGET
					if (pMonth->Budget || (!DecimalCmp(pMonth->BudgetQuantity, 0))) {
						memset(szAmount, 0, LENGTH_AMOUNT_STRING);
						memset(szQuantity, 0, LENGTH_QUANT_STRING);
						if (!DecimalCmp(pMonth->BudgetQuantity, 0))
							AmountToStr(pMonth->Budget, szAmount);
						else
							strcpy(szAmount, "0");

						if (QuantityOption) {
							strcpy(szQuantity, " ");
							if (pMonth->BudgetQuantity)
								DecimalToStr(pMonth->BudgetQuantity, &szQuantity[strlen(szQuantity)]);
							else if (!(*GetTextListText(pAccount->pListQuantityUnit, pLanguage)))
								strcpy(szQuantity, "\0");
							else
								strcat(szQuantity, "0");
						}
						else
							strcpy(szQuantity, "\0");

						if (pObject && pObject->pDimension && pObject->pDimension->Id) {
							if (0 > fprintf(pStream, "#PBUDGET %lli %s %s {%llu \"%s\"} %s%s\r\n",
								LedgerId,
								szYearMonth,
								pAccount->pId,
								pObject->pDimension->Id,
								pObject->pId,
								szAmount,
								szQuantity))
								return (-2);
						}
						else
							if (0 > fprintf(pStream, "#PBUDGET %lli %s %s {} %s%s\r\n",
								LedgerId,
								szYearMonth,
								pAccount->pId,
								szAmount,
								szQuantity))
								return (-2);
					}

					pMonth = LListGetNextRecordData(pBalanceAccount->pListMonth);
				}
			}
		}
	}
	return 0;
}


/********************************************************************
*
static int WriteSieVoucherStatements(FILE *pStream, AccountingType *pAccounting, BOOL CompactFileOption, BOOL QuantityOption, BOOL ObjectOption, BOOL PeriodOption, int VoucherSortOption, DATEINT InitialDate, DATEINT FinalDate, char * FilterEnteredBy, char *pContent, char *pLanguage)
*
*********************************************************************/
static int WriteSieVoucherStatements(FILE *pStream, AccountingType *pAccounting, BOOL CompactFileOption, BOOL QuantityOption, BOOL ObjectOption, BOOL PeriodOption, int VoucherSortOption, DATEINT InitialDate, DATEINT FinalDate, char * FilterEnteredBy, char *pContent, char *pLanguage)
{
	// #VER serie vernr verdatum vertext regdatum sign

	// Ledger Vouchers
	if ((!strcmp(pContent, CONTENT_LABEL_LEDGER)) || (!strcmp(pContent, CONTENT_LABEL_VOUCHER))) {
		LedgerType *pLedger
			= LListGetLastRecordData(pAccounting->pLedgersChart->pListLedger);
		while (pLedger) {
			VoucherType *pVoucherWrite
				= LListGetFirstRecordData(pLedger->pListVoucher);
			while (pVoucherWrite) {
				if ((pVoucherWrite->pId)
					&& (((InitialDate <= pVoucherWrite->Date)
						&& (FinalDate >= pVoucherWrite->Date)))
					&& ((!FilterEnteredBy) ||
					(!(*FilterEnteredBy)) ||
						(pVoucherWrite->pAccountant && pVoucherWrite->pAccountant->pOperatorInfo && (!strcmp(pVoucherWrite->pAccountant->pOperatorInfo, FilterEnteredBy))))) {
					if ((!FinalDate) || (pVoucherWrite->Date <= FinalDate)) { // The complete year 
						char *pVoucherpIdSeries = NulString;
						char *pSieVerTitle = GetTextListText(pVoucherWrite->pListTitle, pLanguage);
						DATEINT dVoucherAccountantDate = 0;
						char szVoucherAccountantDate[LENGTH_LONG_DATE] = { "\"\"" };
						char *pVoucherAccountantOperatorInfo = 0;

						if (pVoucherWrite->pId->SeriesId)
							pVoucherpIdSeries = pVoucherWrite->pId->pseries;
						if ((!CompactFileOption) && (!*pSieVerTitle)) {
							// In non-compact mode, if no Voucher title use Voucher entry title if that exists
							VoucherEntryType *pVoucherEntry
								= LListGetFirstRecordData(pVoucherWrite->pListEntry);
							while (pVoucherEntry && (!*pSieVerTitle)) {
								pSieVerTitle = GetTextListText(pVoucherEntry->pListTitle, pLanguage);
								pVoucherEntry = LListGetNextRecordData(pVoucherWrite->pListEntry);
							}
						}

						if ((pVoucherWrite->pAccountant) && (pVoucherWrite->pAccountant->date))
							dVoucherAccountantDate = pVoucherWrite->pAccountant->date;
						if ((pVoucherWrite->pAccountant) && (pVoucherWrite->pAccountant->pOperatorInfo))
							pVoucherAccountantOperatorInfo = pVoucherWrite->pAccountant->pOperatorInfo;
						if (dVoucherAccountantDate)
							sprintf(szVoucherAccountantDate, "%llu", dVoucherAccountantDate);

						if (pVoucherAccountantOperatorInfo && *pVoucherAccountantOperatorInfo) {
							if (0 > fprintf(pStream, "#VER \"%s\" %llu %llu \"%s\" %s \"%s\"\r\n{\r\n",
								pVoucherpIdSeries,
								pVoucherWrite->pId->SeriesId,
								pVoucherWrite->Date,
								pSieVerTitle,
								szVoucherAccountantDate,
								pVoucherAccountantOperatorInfo))
								return (-2);
						}
						else if (dVoucherAccountantDate) {
							if (0 > fprintf(pStream, "#VER \"%s\" %llu %llu \"%s\" %s\r\n{\r\n",
								pVoucherpIdSeries,
								pVoucherWrite->pId->SeriesId,
								pVoucherWrite->Date,
								pSieVerTitle,
								szVoucherAccountantDate))
								return (-2);
						}
						else if (pSieVerTitle && *pSieVerTitle) {
							if (0 > fprintf(pStream, "#VER \"%s\" %llu %llu \"%s\"\r\n{\r\n",
								pVoucherpIdSeries,
								pVoucherWrite->pId->SeriesId,
								pVoucherWrite->Date,
								pSieVerTitle))
								return (-2);
						}
						else if (0 > fprintf(pStream, "#VER \"%s\" %llu %llu\r\n{\r\n",
							pVoucherpIdSeries,
							pVoucherWrite->pId->SeriesId,
							pVoucherWrite->Date))
							return (-2);



						// #TRANS kontonr {objektlista} belopp transdat transtext kvantitet sign			
						// #BTRANS kontonr {objektlista} belopp transdat transtext kvantitet sign			
						// #RTRANS kontonr {objektlista} belopp transdat transtext kvantitet sign			

						// Objects in the dimension, prepare
						{
							char szAmount[LENGTH_AMOUNT_STRING];
							char szQuantity[LENGTH_QUANT_STRING];

							VoucherEntryType *pVoucherEntry
								= LListGetFirstRecordData(pVoucherWrite->pListEntry);
							while (pVoucherEntry) {
								DATEINT dTransdate = 0;
								memset(szAmount, 0, LENGTH_AMOUNT_STRING);
								memset(szQuantity, 0, LENGTH_QUANT_STRING);
								if (CompactFileOption) {
									if ((pVoucherEntry->AuxDate) && (pVoucherEntry->AuxDate != pVoucherWrite->Date))
										dTransdate = pVoucherEntry->AuxDate;
								}
								else {
									if (pVoucherEntry->AuxDate)
										dTransdate = pVoucherEntry->AuxDate;
									else
										dTransdate = pVoucherWrite->Date;
								}

								AmountToStr(pVoucherEntry->Amount, szAmount);

								if (QuantityOption) {
									if (!DecimalCmp(pVoucherEntry->Quantity, 0))
										DecimalToStr(pVoucherEntry->Quantity, szQuantity);
									else if (!(CompactFileOption || (!(*GetTextListText(pVoucherEntry->pAccount->pListQuantityUnit, pLanguage)))))
										strcpy(szQuantity, "0");
								}
								{
									char *pSieTransTitle = GetTextListText(pVoucherEntry->pListTitle, pLanguage);
									char *pSieTransOperator = NulString;

									if (!CompactFileOption) {
										// For full notification 
										// and for support of programs not having support for 
										// both pVoucherWrite->pTitle and pVoucherEntry->pTitle
										if (((!pSieTransTitle) || (!*pSieTransTitle)) && (pSieVerTitle && *pSieVerTitle))
											// Setting an empty pVoucherEntry->pTitle as pVoucherWrite->pTitle
											pSieTransTitle = pSieVerTitle;
									}
									// In compact mode we do not repeat pVoucherEntry->pTitle if pVoucherWrite->pTitle is the same
									else if ((!pSieTransTitle) || (pSieTransTitle && pSieVerTitle && (!strcmp(pSieTransTitle, pSieVerTitle))))
										pSieTransTitle = NulString;

									if (!pSieTransOperator)
										pSieTransOperator = NulString;

									if (pVoucherWrite->pAccountant
										&& pVoucherEntry->pAccountant
										&& pVoucherWrite->pAccountant->pOperatorInfo
										&& *pVoucherWrite->pAccountant->pOperatorInfo
										&& pVoucherEntry->pAccountant->pOperatorInfo
										&& *pVoucherEntry->pAccountant->pOperatorInfo
										&& (!strcmp(pVoucherEntry->pAccountant->pOperatorInfo, pVoucherWrite->pAccountant->pOperatorInfo)))
										// Same
										pSieTransOperator = NulString;
									else if(pVoucherEntry->pAccountant && pVoucherEntry->pAccountant->pOperatorInfo)// Different
										pSieTransOperator = pVoucherEntry->pAccountant->pOperatorInfo;
									/*
									// If Extended it is noted a second time, normal as well
									// Depricated feature due implementation debate in the SIE union.
									if(pVoucherEntry->Extended) // Non-deprecated added entry
									// Deprecated pVoucherEntry can not be Extended according to SIE4B			 supplemet appendix spec
									{
									if(0>fprintf(pStream, "#RTRANS %s {", pVoucherEntry->pAccount->pId))
									return (-2);

									if(ObjectOption)
									WriteTransObjectList(pStream, pVoucherEntry->pListObject);
									if(0>fprintf(pStream, "} "))
									return (-2);

									// Amount is mandatory (but could be 0
									if(0>fprintf(pStream, "%s", szAmount))
									return (-2);

									WriteTransOptionals(pStream, CompactFileOption, dTransDate, pSieTransTitle, szQuantity, pSieTransOperator);
									if(0>fprintf(pStream, "\r\n"))
									return (-2);
									}
									*/

									if (pVoucherEntry->pDeprecated) {
										if (0 > fprintf(pStream, "#BTRANS %s {", pVoucherEntry->pAccount->pId))
											return (-2);

										if (ObjectOption)
											WriteTransObjectList(pStream, pVoucherEntry->pListObject);
										if (0 > fprintf(pStream, "} "))
											return (-2);

										// Amount is mandatory (but could be 0
										if (0 > fprintf(pStream, "%s", szAmount))
											return (-2);

										WriteTransOptionals(pStream, CompactFileOption, dTransdate, pSieTransTitle, szQuantity, pSieTransOperator);
										if (0 > fprintf(pStream, "\r\n"))
											return (-2);
									}
									else { // Normal
										if (0 > fprintf(pStream, "#TRANS %s {", pVoucherEntry->pAccount->pId))
											return (-2);

										if (ObjectOption)
											WriteTransObjectList(pStream, pVoucherEntry->pListObject);
										if (0 > fprintf(pStream, "} "))
											return (-2);

										// Amount is mandatory (but could be 0
										if (0 > fprintf(pStream, "%s", szAmount))
											return (-2);

										WriteTransOptionals(pStream, CompactFileOption, dTransdate, pSieTransTitle, szQuantity, pSieTransOperator);
										if (0 > fprintf(pStream, "\r\n"))
											return (-2);
									}
								}
								pVoucherEntry = LListGetNextRecordData(pVoucherWrite->pListEntry);
							}
						}

						if (0 > fprintf(pStream, "}\r\n"))
							return (-2);
					}
				}
				pVoucherWrite = LListGetNextRecordData(pLedger->pListVoucher);
			}
			pLedger = LListGetPreviousRecordData(pAccounting->pLedgersChart->pListLedger);
		}
	}
	return 0; // No balances for iSIE_4I
}


/********************************************************************
*
static int WriteTransObjectList(FILE *pStream, LLIST pListObjectRef)
*
*********************************************************************/
static int WriteTransObjectList(FILE *pStream, LLIST pListObjectRef)
{
	ObjectType *pObject
		= LListGetFirstRecordData(pListObjectRef);
	while (pObject) {
		if (0 > fprintf(pStream, "%llu \"%s\"", pObject->pDimension->Id, pObject->pId))
			return (-2);
		pObject = LListGetNextRecordData(pListObjectRef);
		if (pObject) {
			if (0 > fprintf(pStream, " "))
				return (-2);
		}
	}
	return 0;
}


/********************************************************************
*
static int WriteTransOptionals(
*
*********************************************************************/
static int WriteTransOptionals(
	FILE *pStream,
	BOOL CompactFileOption,
	DATEINT dTransDate,
	char *pSieTransTitle,
	char *pQuantity,
	char *pSieTransOperator)
{
	// Optional Trans stuff
	if ((!CompactFileOption)
		|| (dTransDate)
		|| (pSieTransTitle && *pSieTransTitle)
		|| (pQuantity && *pQuantity)
		|| (pSieTransOperator && *pSieTransOperator)) {
		if (dTransDate) {
			if (0 > fprintf(pStream, " %llu", dTransDate))
				return (-2);
		}
		else { // No dTransDate 
			if (0 > fprintf(pStream, " \"\"")) // Must be something behind
				return (-2);
		}
		//  pSieTransTitle 
		if ((!CompactFileOption)
			|| (pSieTransTitle && *pSieTransTitle)
			|| (pQuantity && *pQuantity)
			|| (pSieTransOperator && *pSieTransOperator)) {
			if (0 > fprintf(pStream, " \""))
				return (-2);

			if (pSieTransTitle && *pSieTransTitle) {
				if (0 > fprintf(pStream, "%s", pSieTransTitle))
					return (-2);
			}
			if (0 > fprintf(pStream, "\""))
				return (-2);

			//  pSieTransOperator
			if ((!CompactFileOption)
				|| (pQuantity && *pQuantity)
				|| (pSieTransOperator && *pSieTransOperator)) {
				if (0 > fprintf(pStream, " "))
					return (-2);

				if (pQuantity && *pQuantity) {
					if (0 > fprintf(pStream, "%s", pQuantity))
						return (-2);
				}
				else { // No pQuantity 
					if (0 > fprintf(pStream, "\"\""))  // Must be something behind
						return (-2);
				}
				// End No pQuantity 
				if ((!CompactFileOption)
					|| (pSieTransOperator && *pSieTransOperator)) {
					if (0 > fprintf(pStream, " \""))
						return (-2);

					if (pSieTransOperator  && *pSieTransOperator) {
						if (0 > fprintf(pStream, "%s", pSieTransOperator))
							return (-2);
					}
					if (0 > fprintf(pStream, "\"")) // End of it all
						return (-2);
				}
				// pSieTransOperator the last or no CompactFileOption
			}
			// pSieTransTitle the last and CompactFileOption
		}
		// szTransDate the last and CompactFileOption
	}
	// Amount the last and CompactFileOption

	return 0;
}

