/*********************************************************************************

	RosettaFioProcess.c

	Alphabet Rosetta

	(c)Copyright 2017-2020  Alphabet AB, Sweden (Jan Bergstrom)

	http://www.alphabet.se

	Licensed under the Apache License, Version 2.0 (the "License");
	you may not use this file except in compliance with the License.
	You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.

	We ask for attribution to us, when used.

******************************************************************************************

The RosettaFioProcess.c file contains the basic OS independent logics for the application.

The RosettaFioProcess.h file contains the external declarations and definitions.

The basic logics is tight to the tab data and the interpretations sections
(at the end in this file), the rest of the RosettaFioProcess.c file is OS independent
application specific support functions.

The entrance is the Interpret File() to where a buffer of file content is
sent after being read. The file data is stored in a cloud of fragments of
data in shape of data structures, defined and handled in the RosettaFio.c and RosettaFio.h.
The RosettaFio.c and RosettaFio.h is machine generated code from XSD data specifications.

The LList.c handles the database and the Util.c app independent support functions.

The SearchSet-functions (in RosettaFioProcess.c add up new data in the linked list
database or updates existing structures of data contents.

Files starting with OS identification contains OS specific UI functions. The
basic is very limited in App dependency while the app file is much app dependent.

The Tab-data represent a report (of various types) and the screen data of the
application is stored in a cell system in the RosettaView.c file. This data is
retried to fill the OS dependent screens of the applications. Only the last
part is OS dependent. Even tab data and menus are stored OS independently.

Many main tabs can be displayed (also be added or removed on the fly) and
each main tab point to its basic data and its display window data mirror.

The basic implementation is a Linked list data base solution in LList.c because
modern computers have some much memory that it is not an issue. It allows much
more reporting on the fly appearance of the application, like a word processor
file application. It gives the opportunity to make a version with more specific
data base systems like SQL is needed (for instance for real heavy huge accounting
use).

Content of RosettaFioProcess.c
1.) Basic program support features
2.) Locales
3.) Account coding
4.) Program texts
5.) Sum up Voucher data to Balances, month periods, quarter periods and section totals
6.) Search and get Data structures etc
7.) Sort functions
8.) Interpret, write, copy, merge and destroy the major data set

**********************************************************************************/
#include "RosettaFioTypes.h"

static BalanceTotalPeriodType *SearchSetBalanceTotalPeriod(LLIST pListBalanceTotalPeriod, BalanceTotalPeriodType *pSearchBalanceTotalPeriod);
static BOOL IsQuantity(void *vpAccounting);
static BOOL IsEntity(void *vpAccounting);
BOOL IsObject(void *vpAccounting);
void CheckActive(void *vpAccounting, void *vpObject, DATEINT dPeriod, char *pPeriodTabText, BOOL bVoucherOnly, char *pVoucherSeries);
static void CheckPeriodRange(AccountingType *pAccounting);
static void CheckAccounting(InstanceDataType *pInstanceData);
BOOL AccountTypeCmp(char *pAccountType, char *pAccountTypeCmp);
static int AccountLength(void* vpAccounting);
static int CalcThousandSets(AMOUNT Amount);
static int ThousandSets(void *vpAccounting);
static InstanceDataType *CopyInstanceDataBody(InstanceDataType *pInstanceData, void *vpAccounting);
size_t MergeAccountingInstanceData(size_t ztOldInstanceData, size_t ztNewInstanceData);
int ExternalWriteWorkData(InstanceDataType *pInstanceData, BITMASK bmCommand, char *pFilePathName);
static size_t ExternalInterpretFile(char *pFilePathName, InstanceDataType **ppInstanceData, char *pBuffer);
size_t InterpretFile(char *pBuffer, char *pFilePathName, LocaleDataType *pLocaleData);
	/********************************************************************

Basic program support features

*********************************************************************/
/********************************************************************

char *ProgramTextsLanguageSupport(

*********************************************************************/
char *ProgramTextsLanguageSupport(
	char *pLang)
{
	static char DefLang[]={ "en" };
	if (!pLang)
		pLang = DefLang;
	// We only support one Norwegian language implementation 
	// in program lead texts
	if (!strncmp(pLang, "nb", 2))
		pLang = "no";
	if (!strncmp(pLang, "nn", 2))
		pLang = "no";
	return pLang;
}

/********************************************************************

char *GetSystemProgram(char **ppTarget)

********************************************************************/
char *GetSystemProgram(char **ppTarget)
{
	return CopyString(ppTarget, PROGRAM_NAME);
}

/********************************************************************

char *GetSystemProgramVersion(char **ppTarget)

********************************************************************/
char *GetSystemProgramVersion(char **ppTarget)
{
	return CopyString(ppTarget, PROGRAM_VERSION);
}

/********************************************************************

char *GetWorkDataName(

*********************************************************************/
char *GetWorkDataName(
	void *vpWorkData)
{
	if (((AccountingType *)vpWorkData)->pEntity)
		return ((AccountingType *)vpWorkData)->pEntity->pName;
	return NulString;
}

/********************************************************************

char *GetContent(

*********************************************************************/
char *GetContent(
	void *vpAccounting)
{
	AccountingType *pAccounting = vpAccounting;
	if (pAccounting)
		return pAccounting->pContent;
	else
		return NulString;
}

/********************************************************************

char *AppGetContentFromBm(

*********************************************************************/
char *AppGetContentFromBm(
	BITMASK bmCommand)
{
	static char szContent[32] = { 0 };

	switch (bmCommand & CONTENT_MASK) {
	case CONTENT_CHARTS:
		strcpy(szContent, CONTENT_LABEL_CHARTS);
		break;
	case CONTENT_BALANCE:
		strcpy(szContent, CONTENT_LABEL_BALANCE);
		break;
	case CONTENT_VOUCHERS:
		strcpy(szContent, CONTENT_LABEL_VOUCHER);
		break;
	case CONTENT_INVOICE:
		strcpy(szContent, CONTENT_LABEL_INVOICE);
		break;
	case CONTENT_BANKACS:
		strcpy(szContent, CONTENT_LABEL_BANKACS);
		break;
	default:
	case CONTENT_LEDGER:
		strcpy(szContent, CONTENT_LABEL_LEDGER);
		break;
	}
	return szContent;
}

/********************************************************************

BITMASK AppGetBmFromContent(

*********************************************************************/
BITMASK AppGetBmFromContent(
	char *pContent)
{
	if (!strcmp(pContent, CONTENT_LABEL_CHARTS))
		return CONTENT_CHARTS;
	else if (!strcmp(pContent, CONTENT_LABEL_BALANCE))
		return CONTENT_BALANCE;
	else if (!strcmp(pContent, CONTENT_LABEL_VOUCHER))
		return CONTENT_VOUCHERS;
	else if (!strcmp(pContent, CONTENT_LABEL_LEDGER))
		return CONTENT_LEDGER;
	else if (!strcmp(pContent, CONTENT_LABEL_INVOICE))
		return CONTENT_INVOICE;
	else if (!strcmp(pContent, CONTENT_LABEL_BANKACS))
		return CONTENT_BANKACS;
	else
		return CONTENT_NOTHING;
}

/********************************************************************

BITMASK GetContentParameters(

*********************************************************************/
BITMASK GetContentParameters(
	void *vpWorkData,
	int iFiletype)
{
	AccountingType *pAccounting = vpWorkData;
	BITMASK bmContent = AppGetBmFromContent(pAccounting->pContent);
	return bmContent | iFiletype;
}

/********************************************************************

char *GetWorkDataLogotype(

*********************************************************************/
char *GetWorkDataLogotype(
	void *vpWorkData)
{
	if((vpWorkData) 
		&&  (((AccountingType *)vpWorkData)->pEntity) 
		&& ((AccountingType *)vpWorkData)->pEntity->pListLogotypeFileUrl)
		return LListGetFirstRecordData(((AccountingType *)vpWorkData)->pEntity->pListLogotypeFileUrl);
	return 0;
}

/********************************************************************

char *GetWorkDataCurrency(

*********************************************************************/
char *GetWorkDataCurrency(
	void *vpWorkData)
{
	if (vpWorkData)
		return ((AccountingType *)vpWorkData)->pCurrency;
	return 0;
}

/********************************************************************

char *GetWorkDataEnityName(

*********************************************************************/
char *GetWorkDataEnityName(
	void *vpWorkData)
{
	if (!((AccountingType *)vpWorkData)->pEntity)
		return "";
	return ((AccountingType *)vpWorkData)->pEntity->pName;
}

/********************************************************************

char *GetWorkDataEnityLegalRegId(

*********************************************************************/
char *GetWorkDataEnityLegalRegId(
	void *vpWorkData)
{
	if (!((AccountingType *)vpWorkData)->pEntity)
		return "";
	return ((AccountingType *)vpWorkData)->pEntity->pLegalRegId;
}

/********************************************************************
*
DATEINT GetLastEntryDate(
*
*********************************************************************/
DATEINT GetLastEntryDate(
	AccountingType *pAccounting)
{
	LLISTITEMTYPE *CursorRecord = pAccounting->pLedgersChart->pListLedger->CursorRecord;
	LedgerType *pDefaultLedger = GetLedger(pAccounting, 0);
	if (pDefaultLedger) {
		DATEINT LastEntryDate = pDefaultLedger->YearStart;
		if (LListGetLastRecordData(pDefaultLedger->pListVoucher)) {
			LLISTITEMTYPE *VoucherCursorRecord = pDefaultLedger->pListVoucher->CursorRecord;
			VoucherType *pVoucher
				= LListGetLastRecordData(pDefaultLedger->pListVoucher);
			while (pVoucher) {
				if (LastEntryDate < pVoucher->Date)
					LastEntryDate = pVoucher->Date;

				pVoucher = LListGetPreviousRecordData(pDefaultLedger->pListVoucher);
			}
			pDefaultLedger->pListVoucher->CursorRecord = VoucherCursorRecord;
		}
		pAccounting->pLedgersChart->pListLedger->CursorRecord = CursorRecord;
		return LastEntryDate;
	}
	return 0;
}

/********************************************************************

static COUNTER GetLastVoucherId(

*********************************************************************/
static COUNTER GetLastVoucherId(
	AccountingType *pAccounting,
	char *pSeries)
{
	COUNTER cVoucherId = 0;

	LedgerType *pLedger
		= LListGetLastRecordData(pAccounting->pLedgersChart->pListLedger);
	while (pLedger) {
		VoucherType *pVoucher
			= LListGetLastRecordData(pLedger->pListVoucher);
		while (pVoucher) {
			if ((strcmp(pVoucher->pId->pseries, pSeries)) && (cVoucherId < pVoucher->pId->SeriesId))
				cVoucherId = pVoucher->pId->SeriesId;
			pVoucher = LListGetPreviousRecordData(pLedger->pListVoucher);
		}
		pLedger = LListGetPreviousRecordData(pAccounting->pLedgersChart->pListLedger);
	}
	return cVoucherId;
}

/********************************************************************

static DATEINT GetAccountingStart(

*********************************************************************/
static DATEINT GetAccountingStart(
	void *vpAccounting)
{
	AccountingType *pAccounting = vpAccounting;
	NUMERIC Id = 0;
	LedgerType *pLedger = GetLedger(pAccounting, Id);
	while (pLedger && LListGetFirstRecordData(pLedger->pListVoucher)) {
		pLedger = GetLedger(pAccounting, --Id);
	}
	pLedger = GetLedger(pAccounting, ++Id);
	if (strcmp(pAccounting->pContent, CONTENT_LABEL_CHARTS) && pLedger)
		return pLedger->YearStart;
	else
		return 0;
}

/********************************************************************

static DATEINT GetLedgerStart(

*********************************************************************/
static DATEINT GetLedgerStart(
	void *vpAccounting,
	NUMERIC nId)
{
	AccountingType *pAccounting = vpAccounting;
	LedgerType *pLedger = GetLedger(pAccounting, nId);

	if (strcmp(pAccounting->pContent, CONTENT_LABEL_CHARTS) && pLedger)
		return pLedger->YearStart;
	else
		return 0;
}

/********************************************************************

static DATEINT GetLedgerEnd(

*********************************************************************/
static DATEINT GetLedgerEnd(
	void *vpAccounting,
	NUMERIC nId)
{
	AccountingType *pAccounting = vpAccounting;
	LedgerType *pLedger = GetLedger(pAccounting, nId);
	if (strcmp(pAccounting->pContent, CONTENT_LABEL_CHARTS) && pLedger)
		return pLedger->YearEnd;
	else
		return 0;
}

/********************************************************************

DATEINT GetLedgerLastEntryDate(

*********************************************************************/
DATEINT GetLedgerLastEntryDate(
	void *vpAccounting)
{
	if(vpAccounting) {
		AccountingType *pAccounting = vpAccounting;
		LedgerType *pLedger = GetLedger(pAccounting, 0);
		if (pLedger) {
			DATEINT Date = pAccounting->LastEntryDate;
			if (Date != pLedger->YearEnd)
				return Date;
		}
	}
	return 0;
}

/********************************************************************

char *GetWorkDataRange(

*********************************************************************/
char *GetWorkDataRange(
	void *vpWorkData)
{
	static char RangeTxt[64];
	DATEINT StartYear = GetLedgerStart(vpWorkData, 0) / 10000;
	DATEINT EndYear = GetLedgerEnd(vpWorkData, 0) / 10000;
	if (StartYear == EndYear)
		sprintf(RangeTxt, "%d", (int)StartYear);
	else
		sprintf(RangeTxt, "%d-%d", (int)StartYear, (int)EndYear);
	return RangeTxt;
}

#if defined(ANDROID) || defined(IOS)
/********************************************************************

Aux functions

*********************************************************************/
/********************************************************************

int GetDimTotals(

*********************************************************************/
int GetDimTotals(
	void *vpWorkData,
	BOOL bAudit)
{
	int cDims = 0;
	AccountingType *pAccounting = vpWorkData;
	DimensionType *pDimension
		= LListGetFirstRecordData(pAccounting->pObjectsChart->pListDimension);
	while (pDimension) {
		if (bAudit || (!pDimension->InActive) || (!pDimension->Id))
			cDims++;
		pDimension = LListGetNextRecordData(pAccounting->pObjectsChart->pListDimension);
	}
	return cDims;
}

/********************************************************************

COUNTER *GetDimIds(

*********************************************************************/
COUNTER *GetDimIds(
	void *vpWorkData,
	BOOL bAudit)
{
	size_t ztDims = GetDimTotals(vpWorkData, bAudit);
	COUNTER *pcDims = calloc(ztDims + 1, sizeof(COUNTER));
	if (pcDims) {
		COUNTER iSlot = 0;
		AccountingType *pAccounting = vpWorkData;
		DimensionType *pDimension
			= LListGetFirstRecordData(pAccounting->pObjectsChart->pListDimension);
		while (pDimension) {
			if (bAudit || (!pDimension->InActive) || (!pDimension->Id))
				pcDims[iSlot++] = (COUNTER)pDimension->Id;
			pDimension = LListGetNextRecordData(pAccounting->pObjectsChart->pListDimension);
		}
	}
	return pcDims;
}

/********************************************************************

char *GetDimTitle(

*********************************************************************/
char *GetDimTitle(
	void *vpWorkData,
	char *pLanguage,
	COUNTER cDimId)
{
	AccountingType *pAccounting = vpWorkData;
	DimensionType *pDimension = GetDimension(pAccounting->pObjectsChart->pListDimension, cDimId);
	if (pDimension)
		return GetTextListText(pDimension->pListName, pLanguage); // OK
	return NulString; // Not found
}

/********************************************************************

int GetObjTotals(

*********************************************************************/
int GetObjTotals(
	void *vpWorkData,
	BOOL bAudit,
	COUNTER cDimId)
{
	int iObjs = 0;
	AccountingType *pAccounting = vpWorkData;
	DimensionType *pDimension = GetDimension(pAccounting->pObjectsChart->pListDimension, cDimId);
	if (pDimension) {
		ObjectType *pObject
			= LListGetFirstRecordData(pDimension->pListObject);
		while (pObject) {
			if (bAudit || (!pObject->InActive))
				iObjs++;
			pObject = LListGetNextRecordData(pDimension->pListObject);
		}
	}
	return iObjs;
}

/********************************************************************

char **GetObjIds(

*********************************************************************/
char **GetObjIds(
	void *vpWorkData,
	BOOL bAudit,
	COUNTER cDimId)
{
	int iObjs = GetObjTotals(vpWorkData, bAudit, cDimId);
	char **ppObjs = calloc(iObjs + 1, sizeof(char *));
	if (ppObjs) {
		int i = 0;
		AccountingType *pAccounting = vpWorkData;
		DimensionType *pDimension = GetDimension(pAccounting->pObjectsChart->pListDimension, cDimId);
		if (pDimension) {
			ObjectType *pObject
				= LListGetFirstRecordData(pDimension->pListObject);
			while (pObject) {
				if (bAudit || (!pObject->InActive))
					ppObjs[i++] = pObject->pId;
				pObject = LListGetNextRecordData(pDimension->pListObject);
			}
		}
	}
	return ppObjs;
}

/********************************************************************

char *GetObjTitle(

*********************************************************************/
char *GetObjTitle(
	void *vpWorkData,
	char *pLanguage,
	COUNTER cDimId,
	char *pObjId)
{
	ObjectType *pObject = GetDimObjectAccounting(vpWorkData, cDimId, pObjId);
	if (pObject)
		return GetTextListText(pObject->pListName, pLanguage); // OK
	return NulString; // Not found
}
#endif

/********************************************************************

Locales

*********************************************************************/
/********************************************************************

static LocaleDataType *DestroyLocaleData(

*********************************************************************/
static LocaleDataType *DestroyLocaleData(
	LocaleDataType *pLocaleData)
{
	FreeString(&pLocaleData->pLocaleCountry);
	FreeString(&pLocaleData->pLocaleLanguage);
	FreeString(&pLocaleData->pLocaleCurrency);
	FreeString(&pLocaleData->pLocaleShortDateTimePattern);
	FreeString(&pLocaleData->pLocaleDecimalSeparator);
	FreeString(&pLocaleData->pLocaleThousandSeparator);
	FreeString(&pLocaleData->pLocaleThousandGrouping);
	FreeString(&pLocaleData->pLocaleNegativePrefix);
	FreeString(&pLocaleData->pLocaleNegativeSuffix);
	{
		int i = 0;
		for (i = 0; i < NORMAL_MONTHS_PER_YEAR; i++)
			FreeString(&pLocaleData->pMonth[i]);
	}
	FreeString(&pLocaleData->pOperator);
	memset(pLocaleData, 0, sizeof(LocaleDataType));
	return pLocaleData;
}

/********************************************************************

LocaleDataType *DestroyLocales(

*********************************************************************/
LocaleDataType *DestroyLocales(
	LocaleDataType *pLocaleData)
{
	DestroyLocaleData(pLocaleData);
	free(pLocaleData);
	return 0;
	//InstanceData.DefaultGeneralChartOfAccounts = GetVDUAccountingGeneralChartOfAccounts(CP_APP);
}

/********************************************************************

Account coding

*********************************************************************/
/********************************************************************

static COUNTER CounterAccountCoding(

*********************************************************************/
static COUNTER CounterAccountCoding(
	char *pAccountType)
{
	if (!strcmp(pAccountType, ACCOUNT_TYPE_TOTAL))
		return MENU_VIEW_TOTAL;
	if (!strcmp(pAccountType, ACCOUNT_TYPE_BALANCE))
		return MENU_VIEW_BALANCE;
	if (!strcmp(pAccountType, ACCOUNT_TYPE_RESULT))
		return MENU_VIEW_RESULT;
	if (!strcmp(pAccountType, ACCOUNT_TYPE_ACTIVE))
		return MENU_VIEW_ACTIVE;
	if (!strcmp(pAccountType, ACCOUNT_TYPE_PASSIVE))
		return MENU_VIEW_PASSIVE;
	if (!strcmp(pAccountType, ACCOUNT_TYPE_PROFIT))
		return MENU_VIEW_PROFIT;
	if (!strcmp(pAccountType, ACCOUNT_TYPE_EARNING))
		return MENU_VIEW_EARNING;
	if (!strcmp(pAccountType, ACCOUNT_TYPE_OUTCOME))
		return MENU_VIEW_OUTCOME;
	if (!strcmp(pAccountType, ACCOUNT_TYPE_TANGIBLE))
		return MENU_VIEW_TANGIBLE;
	if (!strcmp(pAccountType, ACCOUNT_TYPE_STOCK))
		return MENU_VIEW_STOCK;
	if (!strcmp(pAccountType, ACCOUNT_TYPE_RECEIVABLES))
		return MENU_VIEW_RECEIVABLES;
	if (!strcmp(pAccountType, ACCOUNT_TYPE_ASSETS))
		return MENU_VIEW_ASSETS;
	if (!strcmp(pAccountType, ACCOUNT_TYPE_EQUITIES))
		return MENU_VIEW_EQUITIES;
	if (!strcmp(pAccountType, ACCOUNT_TYPE_PROVISIONS))
		return MENU_VIEW_PROVISIONS;
	if (!strcmp(pAccountType, ACCOUNT_TYPE_LIABILITIES))
		return MENU_VIEW_LIABILITIES;
	if (!strcmp(pAccountType, ACCOUNT_TYPE_REVENUES))
		return MENU_VIEW_REVENUES;
	if (!strcmp(pAccountType, ACCOUNT_TYPE_SUPPLIES))
		return MENU_VIEW_SUPPLIES;
	if (!strcmp(pAccountType, ACCOUNT_TYPE_EXPENSES))
		return MENU_VIEW_EXPENSES;
	if (!strcmp(pAccountType, ACCOUNT_TYPE_STAFF))
		return MENU_VIEW_STAFF;
	if (!strcmp(pAccountType, ACCOUNT_TYPE_LOSS_GAINS))
		return MENU_VIEW_LOSSGAINES;
	if (!strcmp(pAccountType, ACCOUNT_TYPE_FINANCIAL))
		return MENU_VIEW_FINANCIAL;
	if (!strcmp(pAccountType, ACCOUNT_TYPE_AUX))
		return MENU_VIEW_AUXILIARY_COSTS;
	return MENU_VIEW_ASSETS; // Default dummy
}

/********************************************************************

AccountingType *ConvertAccountCoding(

*********************************************************************/
AccountingType *ConvertAccountCoding(
	AccountingType *pSourceAccounting,
	BITMASK bmFileType)
{
	AccountingType *pAccounting = CopyAccounting(0, 0, pSourceAccounting);
	AccountType *pAccount
		= LListGetFirstRecordData(pAccounting->pAccountsChart->pListAccount);
	while (pAccount) {
		switch (CounterAccountCoding(pAccount->pType)) {
		case MENU_VIEW_TANGIBLE:
		case MENU_VIEW_STOCK:
		case MENU_VIEW_RECEIVABLES:
			if ((bmFileType == FILE_TYPE_SIE) || (bmFileType == FILE_TYPE_XGL))
				CopyString(&pAccount->pType, ACCOUNT_TYPE_ASSETS);
			break;
		case MENU_VIEW_EQUITIES:
			if (bmFileType == FILE_TYPE_SIE)
				CopyString(&pAccount->pType, ACCOUNT_TYPE_LIABILITIES);
			break;
		case MENU_VIEW_PROVISIONS:
				CopyString(&pAccount->pType, ACCOUNT_TYPE_LIABILITIES);
			break;
		case MENU_VIEW_SUPPLIES:
		case MENU_VIEW_STAFF:
			if ((bmFileType == FILE_TYPE_SIE) || (bmFileType == FILE_TYPE_XGL))
				CopyString(&pAccount->pType, ACCOUNT_TYPE_EXPENSES);
			break;
		case MENU_VIEW_LOSSGAINES:
		case MENU_VIEW_FINANCIAL:
		case MENU_VIEW_AUXILIARY_COSTS:
			if (bmFileType == FILE_TYPE_SIE)
				CopyString(&pAccount->pType, ACCOUNT_TYPE_EXPENSES);
			break;
		case MENU_VIEW_TOTAL:
		case MENU_VIEW_BALANCE:
		case MENU_VIEW_RESULT:
		case MENU_VIEW_ACTIVE:
		case MENU_VIEW_PASSIVE:
		case MENU_VIEW_PROFIT:
		case MENU_VIEW_EARNING:
		case MENU_VIEW_OUTCOME:
		case MENU_VIEW_ASSETS:
		case MENU_VIEW_LIABILITIES:
		case MENU_VIEW_REVENUES:
		case MENU_VIEW_EXPENSES:
		default:
			break;
		}
		pAccount = LListGetNextRecordData(pAccounting->pAccountsChart->pListAccount);
	}
	SumUpBalanceTotals(pAccounting);
	return pAccounting;
}


/********************************************************************

AccountingType *ExpandSieAccountTypes(

*********************************************************************/
AccountingType *ExpandSieAccountTypes(
	AccountingType *pSourceAccounting,
	BITMASK bmFileType)
{
	AccountingType *pAccounting = CopyAccounting(0, 0, pSourceAccounting);
	//	if(strcmp(pAccounting->pGaap, BAS2000)) // If not the BAS GAAP
	//		return pSourceAccounting;
	if ((!strcmp(pAccounting->pGaap, "EUBAS97"))
		|| (!strncmp(pAccounting->pGaap, BAS2000, 4))
		|| (!strncmp(pAccounting->pGaap, "NE2", 3))) {
		AccountType *pAccount
			= LListGetFirstRecordData(pAccounting->pAccountsChart->pListAccount);
		while (pAccount) {
			if (((bmFileType & FILE_TYPE_MASK) == FILE_TYPE_STD) || ((bmFileType & FILE_TYPE_MASK) == FILE_TYPE_XGL)) {
				switch (CounterAccountCoding(pAccount->pType)) {
				case MENU_VIEW_ASSETS:
					if ((bmFileType & FILE_TYPE_MASK) == FILE_TYPE_STD) {
						switch (*pAccount->pId) {
						case '1':
							switch (pAccount->pId[1]) {
							case '0':
							case '1':
							case '2':
							case '3':
								CopyString(&pAccount->pType, ACCOUNT_TYPE_TANGIBLE);
								break;
							case '4':
								CopyString(&pAccount->pType, ACCOUNT_TYPE_STOCK);
								break;
							case '5':
							case '6':
							case '7':
								CopyString(&pAccount->pType, ACCOUNT_TYPE_RECEIVABLES);
								break;
							default:
								break;
							}
							break;
						case '2':
							switch (pAccount->pId[1]) {
							case '0':
								CopyString(&pAccount->pType, ACCOUNT_TYPE_EQUITIES);
								break;
							case '1':
							case '2':
								CopyString(&pAccount->pType, ACCOUNT_TYPE_PROVISIONS);
								break;
							default:
								CopyString(&pAccount->pType, ACCOUNT_TYPE_LIABILITIES);
								break;
							}
							break;
						case '3':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_REVENUES);
							break;
						case '4':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_SUPPLIES);
							break;
						case '5':
						case '6':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_EXPENSES);
							break;
						case '7':
							switch (pAccount->pId[1]) {
							case '0':
							case '1':
							case '2':
							case '3':
							case '4':
							case '5':
							case '6':
								CopyString(&pAccount->pType, ACCOUNT_TYPE_STAFF);
								break;
							case '7':
							case '8':
							case '9':
								CopyString(&pAccount->pType, ACCOUNT_TYPE_LOSS_GAINS);
								break;
							default:
								break;
							}
							break;
						case '8':
							switch (pAccount->pId[1]) {
							case '0':
							case '1':
							case '2':
								CopyString(&pAccount->pType, ACCOUNT_TYPE_LOSS_GAINS);
								break;
							case '3':
							case '4':
							case '5':
							case '6':
							case '7':
								CopyString(&pAccount->pType, ACCOUNT_TYPE_FINANCIAL);
								break;
							case '8':
							case '9':
								CopyString(&pAccount->pType, ACCOUNT_TYPE_AUX);
								break;
							default:
								CopyString(&pAccount->pType, ACCOUNT_TYPE_EXPENSES);
								break;
							}
							break;
						case '9':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_AUX);
							break;
						default:
							break;
						}
					}
					break;
				case MENU_VIEW_LIABILITIES:
					switch (*pAccount->pId) {
					case '2':
						switch (pAccount->pId[1]) {
						case '0':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_EQUITIES);
							break;
						case '1':
						case '2':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_PROVISIONS);
							break;
						default:
							break;
						}
						break;
					default:
						break;
					}
					break;
				case MENU_VIEW_EXPENSES:
					switch (*pAccount->pId) {
					case '4':
						CopyString(&pAccount->pType, ACCOUNT_TYPE_SUPPLIES);
						break;
					case '7':
						switch (pAccount->pId[1]) {
						case '0':
						case '1':
						case '2':
						case '3':
						case '4':
						case '5':
						case '6':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_STAFF);
							break;
						case '7':
						case '8':
						case '9':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_LOSS_GAINS);
							break;
						default:
							break;
						}
						break;
					case '8':
						switch (pAccount->pId[1]) {
						case '0':
						case '1':
						case '2':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_LOSS_GAINS);
							break;
						case '3':
						case '4':
						case '5':
						case '6':
						case '7':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_FINANCIAL);
							break;
						case '8':
						case '9':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_AUX);
							break;
						default:
							break;
						}
						break;
					case '9':
						CopyString(&pAccount->pType, ACCOUNT_TYPE_AUX);
						break;
					default:
						CopyString(&pAccount->pType, ACCOUNT_TYPE_EXPENSES);
						break;
					}
					break;
				default:
					break;
				}
			}
			pAccount = LListGetNextRecordData(pAccounting->pAccountsChart->pListAccount);
		}
		SumUpBalanceTotals(pAccounting);
	}
	else if (!strncmp(pAccounting->pGaap, "BAS9", 3)) {
		AccountType *pAccount
			= LListGetFirstRecordData(pAccounting->pAccountsChart->pListAccount);
		while (pAccount) {
			if (((bmFileType & FILE_TYPE_MASK) == FILE_TYPE_STD) || ((bmFileType & FILE_TYPE_MASK) == FILE_TYPE_XGL)) {
				switch (CounterAccountCoding(pAccount->pType)) {
				case MENU_VIEW_ASSETS:
					if ((bmFileType & FILE_TYPE_MASK) == FILE_TYPE_STD) {
						switch (*pAccount->pId) {
						case '1':
							switch (pAccount->pId[1]) {
							case '6':
							case '7':
							case '8':
							case '9':
								CopyString(&pAccount->pType, ACCOUNT_TYPE_TANGIBLE);
								break;
							case '5':
								CopyString(&pAccount->pType, ACCOUNT_TYPE_STOCK);
								break;
							case '2':
							case '3':
							case '4':
								CopyString(&pAccount->pType, ACCOUNT_TYPE_RECEIVABLES);
								break;
							default:
								break;
							}
							break;
						case '2':
							switch (pAccount->pId[1]) {
							case '8':
								CopyString(&pAccount->pType, ACCOUNT_TYPE_PROVISIONS);
								break;
							case '9':
								CopyString(&pAccount->pType, ACCOUNT_TYPE_EQUITIES);
								break;
							default:
								CopyString(&pAccount->pType, ACCOUNT_TYPE_LIABILITIES);
								break;
							}
							break;
						case '3':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_REVENUES);
							break;
						case '4':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_SUPPLIES);
							break;
						case '5':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_STAFF);
							break;
						case '6':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_EXPENSES);
							break;
						case '7':
							switch (pAccount->pId[1]) {
							case '0':
							case '1':
							case '2':
							case '3':
							case '4':
							case '5':
							case '6':
							case '7':
								CopyString(&pAccount->pType, ACCOUNT_TYPE_EXPENSES);
								break;
							case '8':
							case '9':
								CopyString(&pAccount->pType, ACCOUNT_TYPE_LOSS_GAINS);
								break;
							default:
								break;
							}
							break;
						case '8':
							switch (pAccount->pId[1]) {
							case '0':
							case '1':
							case '2':
								CopyString(&pAccount->pType, ACCOUNT_TYPE_FINANCIAL);
								break;
							case '3':
							case '4':
								CopyString(&pAccount->pType, ACCOUNT_TYPE_LOSS_GAINS);
								break;
							case '5':
							case '6':
							case '7':
								CopyString(&pAccount->pType, ACCOUNT_TYPE_FINANCIAL);
								break;
							case '8':
							case '9':
								CopyString(&pAccount->pType, ACCOUNT_TYPE_AUX);
								break;
							default:
								CopyString(&pAccount->pType, ACCOUNT_TYPE_EXPENSES);
								break;
							}
							break;
						case '9':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_AUX);
							break;
						default:
							break;
						}
					}
					break;
				case MENU_VIEW_LIABILITIES:
					switch (*pAccount->pId) {
					case '2':
						switch (pAccount->pId[1]) {
						case '8':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_PROVISIONS);
							break;
						case '9':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_EQUITIES);
							break;
						default:
							break;
						}
						break;
					default:
						break;
					}
					break;
				case MENU_VIEW_EXPENSES:
					switch (*pAccount->pId) {
					case '4':
						CopyString(&pAccount->pType, ACCOUNT_TYPE_SUPPLIES);
						break;
					case '5':
						CopyString(&pAccount->pType, ACCOUNT_TYPE_STAFF);
						break;
					case '6':
						CopyString(&pAccount->pType, ACCOUNT_TYPE_EXPENSES);
						break;
					case '7':
						switch (pAccount->pId[1]) {
						case '0':
						case '1':
						case '2':
						case '3':
						case '4':
						case '5':
						case '6':
						case '7':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_EXPENSES);
							break;
						case '8':
						case '9':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_LOSS_GAINS);
							break;
						default:
							break;
						}
						break;
					case '8':
						switch (pAccount->pId[1]) {
						case '0':
						case '1':
						case '2':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_FINANCIAL);
							break;
						case '3':
						case '4':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_LOSS_GAINS);
							break;
						case '5':
						case '6':
						case '7':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_FINANCIAL);
							break;
						case '8':
						case '9':
							CopyString(&pAccount->pType, ACCOUNT_TYPE_AUX);
							break;
						default:
							CopyString(&pAccount->pType, ACCOUNT_TYPE_EXPENSES);
							break;
						}
						break;
					case '9':
						CopyString(&pAccount->pType, ACCOUNT_TYPE_AUX);
						break;
					default:
						break;
					}
					break;
				default:
					break;
				}
			}
			pAccount = LListGetNextRecordData(pAccounting->pAccountsChart->pListAccount);
		}
		SumUpBalanceTotals(pAccounting);
	}
	return pAccounting;
}

/********************************************************************

Program texts

*********************************************************************/
/********************************************************************

static char *GetEnumTypeText(

*********************************************************************/
static char *GetEnumTypeText(
	EnumType *pAccountTypeType,
	char *pType,
	char *pLang)
{
	size_t zt = 0;
	char *pEnumerationID = (pAccountTypeType + zt)->EnumerationID;
	pLang = ProgramTextsLanguageSupport(pLang);
	while (pEnumerationID && *pEnumerationID) {
		if (!strcmp(pEnumerationID, pType)) {
			size_t ztLang = 0;
			char *pEnText = 0;
			char *pNonText = 0;
			while (((pAccountTypeType + zt)->pListDescription + ztLang)
				&& ((TextType *)((pAccountTypeType + zt)->pListDescription + ztLang)->pText)) {
				TextType *pText = ((pAccountTypeType + zt)->pListDescription + ztLang);
				char *pLanguage = pText->plang;
				char *pTitle = pText->pText;
				if (!(pLanguage && *pLanguage))
					pNonText = pTitle;
				else if (!strcmp(pLanguage, pLang))
					return pTitle;
				else if (!strcmp(pLanguage, "en"))
					pEnText = pTitle;
				ztLang++;
			}
			if (pNonText && *pNonText)
				return pNonText;
			if (pEnText && *pEnText)
				return pEnText;
		}
		zt++;
		pEnumerationID = (pAccountTypeType + zt)->EnumerationID;
	}
	return NulString;
}

/********************************************************************

char *GetEnumApplicationTextType(

*********************************************************************/
char *GetEnumApplicationTextType(
	char *pType,
	char *pLang)
{
	char *pText = GetEnumTypeText(EnumApplicationTextType, pType, ProgramTextsLanguageSupport(pLang));
	if (pText && *pText)
		return pText; // OK
	return pType; // Failure
}

/********************************************************************

char *GetEnumDepreciationPeriodsType(

*********************************************************************/
char *GetEnumDepreciationPeriodsType(
	char *pType,
	char *pLang)
{
	char *pText = GetEnumTypeText(EnumDepreciationPeriodsType, pType, ProgramTextsLanguageSupport(pLang));
	if (pText && *pText)
		return pText;
	return pType; // Failure
}

/********************************************************************

char *GetEnumFolderTypeText(

*********************************************************************/
char *GetEnumFolderTypeText(
	char *pType,
	char *pLang)
{
	char *pText = GetEnumTypeText(EnumFolderType, pType, ProgramTextsLanguageSupport(pLang));
	if (pText && *pText)
		return pText;
	pText = GetEnumTypeText(EnumContentType, pType, ProgramTextsLanguageSupport(pLang));
	if (pText && *pText)
		return pText;
	pText = GetEnumTypeText(EnumDocumentContentType, pType, ProgramTextsLanguageSupport(pLang));
	if (pText && *pText)
		return pText;
	return pType; // Failure
}

/********************************************************************

char *GetEnumAccountTitleTypeText(

*********************************************************************/
char *GetEnumAccountTitleTypeText(
	char *pType,
	char *pLang)
{
	char *pText = GetEnumTypeText(EnumAccountTypeType, pType, ProgramTextsLanguageSupport(pLang));
	if (pText && *pText)
		return pText;
	pText = GetEnumTypeText(EnumAccountTotalTypeType, pType, ProgramTextsLanguageSupport(pLang));
	if (pText && *pText)
		return pText;
	pText = GetEnumTypeText(EnumDimensionTitleType, pType, ProgramTextsLanguageSupport(pLang));
	if (pText && *pText)
		return pText;
	return pType; // Failure
}

/********************************************************************

static char *GetEnumHelpTextType(

*********************************************************************/
static char *GetEnumHelpTextType(
	char *pType,
	char *pLang)
{
	char *pText = GetEnumTypeText(EnumHelpTextType, pType, ProgramTextsLanguageSupport(pLang));
	if (pText && *pText)
		return pText;
	return pType; // Failure
}

/********************************************************************

char *AppMajorTexts(

*********************************************************************/
char *AppMajorTexts(
	int iTextItem,
	char *pLocaleLanguage,
	BOOL bEmoji)
{
	static char MenuText[256] = { 0 };
	*MenuText = 0; // Wipe it
	switch (iTextItem) {
	case MENU_ERRORS:
		strcat(MenuText, TXT_ERRORS);
		return MenuText;
	}
	return NulString;
}

/********************************************************************

char *AppMenuTexts(

*********************************************************************/
char *AppMenuTexts(
	int iTextItem,
	LocaleDataType *pLocaleData,
	BOOL bEmoji,
	BOOL bHwKeyb)
{
	static char MenuText[4096] = { 0 };
	*MenuText = 0; // Wipe it
	if (bEmoji) {
		switch (iTextItem) {
		case MENU_EXIT:
			strcat(MenuText, EMOJI_MENU_EXIT);
			strcat(MenuText, " ");
			break;
		case MENU_REFRESH:
			strcat(MenuText, EMOJI_MENU_REFRESH);
			strcat(MenuText, " ");
			break;
		case MENU_OPEN:
			strcat(MenuText, EMOJI_MENU_OPEN);
			strcat(MenuText, " ");
			break;
		case MENU_SPREADSHEET:
			strcat(MenuText, EMOJI_MENU_SAVE_SPREADSHEET);
			strcat(MenuText, " ");
			break;
		case MENU_TEXT:
			strcat(MenuText, EMOJI_MENU_SAVE_TEXT);
			strcat(MenuText, " ");
			break;
		case MENU_SAVE:
			strcat(MenuText, EMOJI_MENU_SAVE);
			strcat(MenuText, " ");
			break;
		case MENU_SAVE_AS:
			strcat(MenuText, EMOJI_MENU_SAVE_AS);
			strcat(MenuText, " ");
			break;
#if(PROGRAM_FEATURES_MASK & PROGRAM_FEATURES_ROSETTA)
		case MENU_SAVE_AS_STD:
			strcat(MenuText, EMOJI_MENU_SAVE_AS);
			break;
		case MENU_SAVE_AS_STD_ALL:
			strcat(MenuText, EMOJI_MENU_SAVE_AS);
			break;
		case MENU_SAVE_AS_STD_CHART:
			strcat(MenuText, EMOJI_MENU_SAVE_AS);
			break;
		case MENU_SAVE_AS_STD_BALANCE:
			strcat(MenuText, EMOJI_MENU_SAVE_AS);
			break;
		case MENU_SAVE_AS_STD_VOUCHER:
			strcat(MenuText, EMOJI_MENU_SAVE_AS);
			break;
		case MENU_SAVE_AS_STD_LEDGER:
			strcat(MenuText, EMOJI_MENU_SAVE_AS);
			break;
		case MENU_SAVE_AS_STD_INVOICE:
			strcat(MenuText, EMOJI_MENU_SAVE_AS);
			break;
		case MENU_SAVE_AS_STD_BANKACS:
			strcat(MenuText, EMOJI_MENU_SAVE_AS);
			break;
		case MENU_SAVE_AS_STD_ERROR:
			strcat(MenuText, EMOJI_MENU_SAVE_AS);
			break;
		case MENU_SAVE_AS_STD_ANALYSE:
			strcat(MenuText, EMOJI_MENU_SAVE_AS);
			break;
#endif
#if(PROGRAM_FEATURES_MASK & PROGRAM_FEATURES_SIE)
		case MENU_SAVE_AS_SIE:
			strcat(MenuText, EMOJI_MENU_SAVE_AS);
			break;
		case MENU_SAVE_AS_SIE_ALL:
			strcat(MenuText, EMOJI_MENU_SAVE_AS);
			break;
		case MENU_SAVE_AS_SIE_CHART:
			strcat(MenuText, EMOJI_MENU_SAVE_AS);
			break;
		case MENU_SAVE_AS_SIE_BALANCE:
			strcat(MenuText, EMOJI_MENU_SAVE_AS);
			break;
		case MENU_SAVE_AS_SIE_VOUCHER:
			strcat(MenuText, EMOJI_MENU_SAVE_AS);
			break;
#endif
#if(PROGRAM_FEATURES_MASK & PROGRAM_FEATURES_XGL)
		case MENU_SAVE_AS_XGL:
			strcat(MenuText, EMOJI_MENU_SAVE_AS);
			break;
		case MENU_SAVE_AS_XGL_ALL:
			strcat(MenuText, EMOJI_MENU_SAVE_AS);
			break;
		case MENU_SAVE_AS_XGL_CHART:
			strcat(MenuText, EMOJI_MENU_SAVE_AS);
			break;
		case MENU_SAVE_AS_XGL_BALANCE:
			strcat(MenuText, EMOJI_MENU_SAVE_AS);
			break;
		case MENU_SAVE_AS_XGL_VOUCHER:
			strcat(MenuText, EMOJI_MENU_SAVE_AS);
			break;
#endif
		case MENU_PRINT:
			strcat(MenuText, EMOJI_MENU_PRINT);
			strcat(MenuText, " ");
			break;
			/*
		case MENU_OK:
			strcat(MenuText, EMOJI_MENU_OK);
			strcat(MenuText, " ");
			break;
		case MENU_CANCEL:
			strcat(MenuText, EMOJI_MENU_CANCEL);
			strcat(MenuText, " ");
			break;
			*/
			/*
		case MENU_COPY:
			strcat(MenuText, EMOJI_COPY_EDIT);
			strcat(MenuText, " ");
			break;*/
		case MENU_EDIT:
			strcat(MenuText, EMOJI_MENU_EDIT);
			strcat(MenuText, " ");
			break;
		case MENU_EDIT_ITEM:
			strcat(MenuText, EMOJI_MENU_EDIT);
			strcat(MenuText, " ");
			break;
		case MENU_DELETE_ITEM:
			strcat(MenuText, EMOJI_MENU_DELETE);
			strcat(MenuText, " ");
			break;
			/*			case MENU_EXAMINE:
			strcat(MenuText, EMOJI_MENU_EXAMINE);
			strcat(MenuText, " ");
			case MENU_EXPAND:
			strcat(MenuText, EMOJI_MENU_EXPAND);
			strcat(MenuText, " ");
			break;
			case MENU_LINE_SELECT:
			strcat(MenuText, EMOJI_MENU_LINE_SELECT);
			strcat(MenuText, " ");
			break;
			*/
		case MENU_SEARCH:
			strcat(MenuText, EMOJI_MENU_SEARCH);
			strcat(MenuText, " ");
			break;
		case MENU_SEARCH_BUTTON:
			strcat(MenuText, EMOJI_MENU_SEARCH);
			strcat(MenuText, " ");
			break;
		case MENU_FILE:
			strcat(MenuText, EMOJI_MENU_FILE);
			strcat(MenuText, " ");
			break;
		case MENU_ADD_ITEM:
			strcat(MenuText, EMOJI_MENU_ADD);
			strcat(MenuText, " ");
			break;
		case MENU_DO:
			strcat(MenuText, EMOJI_MENU_DO);
			strcat(MenuText, " ");
			break;
		case MENU_DO_NEW:
			strcat(MenuText, EMOJI_MENU_NEW);
			strcat(MenuText, " ");
			break;
		case MENU_DO_YEAR:
			strcat(MenuText, EMOJI_MENU_NEW2);
			strcat(MenuText, " ");
			break;
		case MENU_DO_IMPORT:
			strcat(MenuText, EMOJI_MENU_IMPORT);
			strcat(MenuText, " ");
			break;
		case MENU_IMPORT:
			strcat(MenuText, EMOJI_MENU_IMPORT);
			strcat(MenuText, " ");
			break;
		case MENU_VIEW:
			strcat(MenuText, EMOJI_MENU_VIEW);
			strcat(MenuText, " ");
			break;
		case MENU_VIEW_TAB_DEL:
			strcat(MenuText, EMOJI_MENU_DELETE);
			strcat(MenuText, " ");
			break;
		case MENU_VIEW_TAB_ADD:
			strcat(MenuText, EMOJI_MENU_ADD);
			strcat(MenuText, " ");
			break;
		case MENU_EDIT_ENTITY:
			strcat(MenuText, EMOJI_MENU_EDIT);
			strcat(MenuText, " ");
			break;
		case MENU_EDIT_ACCOUNTS:
			strcat(MenuText, EMOJI_MENU_EDIT);
			strcat(MenuText, " ");
			break;
		case MENU_EDIT_OBJECTS:
			strcat(MenuText, EMOJI_MENU_EDIT);
			strcat(MenuText, " ");
			break;
		case MENU_EDIT_SERIES:
			strcat(MenuText, EMOJI_MENU_EDIT);
			strcat(MenuText, " ");
			break;
		case MENU_ABOUT:
			strcat(MenuText, EMOJI_MENU_ABOUT);
			strcat(MenuText, " ");
			break;
		case MENU_HELP:
			strcpy(MenuText, EMOJI_MENU_HELP);
			strcat(MenuText, " ");
			break;
		case MENU_LOGO:
			strcpy(MenuText, EMOJI_MENU_LOGO);
			strcat(MenuText, " ");
			break;
		case MENU_URL:
			strcpy(MenuText, EMOJI_MENU_URL);
			strcat(MenuText, " ");
			break;
		case MENU_WEBPAGE:
			strcpy(MenuText, EMOJI_MENU_WEBPAGE);
			strcat(MenuText, " ");
			break;
		case MENU_ADDRESS:
			strcpy(MenuText, EMOJI_MENU_ADDRESS);
			strcat(MenuText, " ");
			break;
		case MENU_EMAIL:
			strcpy(MenuText, EMOJI_MENU_EMAIL);
			strcat(MenuText, " ");
			break;
		case MENU_PHONE:
			strcpy(MenuText, EMOJI_MENU_PHONE);
			strcat(MenuText, " ");
			break;
		case MENU_SOCIALMEDIA:
			strcpy(MenuText, EMOJI_MENU_SOCIALMEDIA);
			strcat(MenuText, " ");
			break;
		case MENU_CONTACTLOG:
			strcpy(MenuText, EMOJI_MENU_CONTACTLOG);
			strcat(MenuText, " ");
			break;
		case MENU_PKI:
			strcpy(MenuText, EMOJI_MENU_PKI);
			strcat(MenuText, " ");
			break;
		case MENU_CRM:
			strcpy(MenuText, EMOJI_MENU_CRM);
			strcat(MenuText, " ");
			break;
		case MENU_NOTE:
			strcpy(MenuText, EMOJI_MENU_NOTE);
			strcat(MenuText, " ");
			break;
		case MENU_LEGAL_ID:
			strcpy(MenuText, EMOJI_MENU_LEGAL_ID);
			strcat(MenuText, " ");
			break;
		case MENU_VAT_ID:
			strcpy(MenuText, EMOJI_MENU_VAT_ID);
			strcat(MenuText, " ");
			break;
		case MENU_VAT_CURRENCY:
			strcpy(MenuText, EMOJI_MENU_VAT_CURRENCY);
			strcat(MenuText, " ");
			break;
		case MENU_JURISDICTION:
			strcpy(MenuText, EMOJI_MENU_JURISDICTION);
			strcat(MenuText, " ");
			break;
		case MENU_ENTITYTYPE:
			strcpy(MenuText, EMOJI_MENU_ENTITYTYPE);
			strcat(MenuText, " ");
			break;
		case MENU_STATISTICAL:
			strcpy(MenuText, EMOJI_MENU_STATISTICAL);
			strcat(MenuText, " ");
			break;
		case MENU_CASHACCOUNTING:
			strcpy(MenuText, EMOJI_MENU_CASHACCOUNTING);
			strcat(MenuText, " ");
			break;
		case MENU_CONTACT:
			strcpy(MenuText, EMOJI_MENU_CONTACT);
			strcat(MenuText, " ");
			break;
		case MENU_TXT_NOTE:
			strcpy(MenuText, EMOJI_MENU_NOTE);
			strcat(MenuText, " ");
			break;
		default:
			break;
		}
	}
	switch (iTextItem) {
	case MENU_EXIT:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_EXIT, pLocaleData->pLocaleLanguage));
		break;
	case MENU_OK:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_OK, pLocaleData->pLocaleLanguage));
		break;
	case MENU_CANCEL:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_CANCEL, pLocaleData->pLocaleLanguage));
		break;
	case MENU_COPY:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_COPY, pLocaleData->pLocaleLanguage));
		break;
	case MENU_EDIT:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_EDIT, pLocaleData->pLocaleLanguage));
		break;
	case MENU_EDIT_ITEM:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_EDIT, pLocaleData->pLocaleLanguage));
		break;
	case MENU_DELETE_ITEM:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_DELETE, pLocaleData->pLocaleLanguage));
		break;
	case MENU_EXAMINE:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_EXAMINE, pLocaleData->pLocaleLanguage));
		break;
	case MENU_EXPAND:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_EXPAND, pLocaleData->pLocaleLanguage));
		break;
	case MENU_LINE_SELECT:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_LINE_SELECT, pLocaleData->pLocaleLanguage));
		break;
	case MENU_SEARCH:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_SEARCH, pLocaleData->pLocaleLanguage));
		break;
	case MENU_SEARCH_BUTTON:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_SEARCH, pLocaleData->pLocaleLanguage));
		strcat(MenuText, GetEnumApplicationTextType(" ", pLocaleData->pLocaleLanguage));
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_OK, pLocaleData->pLocaleLanguage));
		break;
	case MENU_FILE:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_FILE, pLocaleData->pLocaleLanguage));
		break;
	case MENU_ADD_ITEM:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_NEW, pLocaleData->pLocaleLanguage));
		break;
	case MENU_REFRESH:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_REFRESH, pLocaleData->pLocaleLanguage));
		break;
	case MENU_OPEN:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_OPEN, pLocaleData->pLocaleLanguage));
		break;
	case MENU_SAVE:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_SAVE, pLocaleData->pLocaleLanguage));
		break;
	case MENU_SAVE_AS:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_SAVE_AS, pLocaleData->pLocaleLanguage));
		break;
#if(PROGRAM_FEATURES_MASK & PROGRAM_FEATURES_ROSETTA)
	case MENU_SAVE_AS_STD:
		strcat(MenuText, FILE_TYPE_LABEL_STD);
		break;
	case MENU_SAVE_AS_STD_ALL:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_SAVE_AS_ALL, pLocaleData->pLocaleLanguage));
		break;
	case MENU_SAVE_AS_STD_CHART:
		strcat(MenuText, GetEnumFolderTypeText(CONTENT_LABEL_CHARTS, pLocaleData->pLocaleLanguage));
		break;
	case MENU_SAVE_AS_STD_BALANCE:
		strcat(MenuText, GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleData->pLocaleLanguage));
		break;
	case MENU_SAVE_AS_STD_VOUCHER:
		strcat(MenuText, GetEnumFolderTypeText(CONTENT_LABEL_VOUCHER, pLocaleData->pLocaleLanguage));
		break;
	case MENU_SAVE_AS_STD_LEDGER:
		strcat(MenuText, GetEnumFolderTypeText(CONTENT_LABEL_LEDGER, pLocaleData->pLocaleLanguage));
		break;
	case MENU_SAVE_AS_STD_INVOICE:
		strcat(MenuText, GetEnumFolderTypeText(CONTENT_LABEL_INVOICE, pLocaleData->pLocaleLanguage));
		break;
	case MENU_SAVE_AS_STD_BANKACS:
		strcat(MenuText, GetEnumFolderTypeText(CONTENT_LABEL_BANKACS, pLocaleData->pLocaleLanguage));
		break;
	case MENU_SAVE_AS_STD_ERROR:
		strcat(MenuText, GetEnumFolderTypeText(CONTENT_LABEL_ERROR, pLocaleData->pLocaleLanguage));
		break;
	case MENU_SAVE_AS_STD_ANALYSE:
		strcat(MenuText, GetEnumFolderTypeText(CONTENT_LABEL_ANALYSE, pLocaleData->pLocaleLanguage));
		break;
#endif
#if(PROGRAM_FEATURES_MASK & PROGRAM_FEATURES_SIE)
	case MENU_SAVE_AS_SIE:
		strcat(MenuText, FILE_TYPE_LABEL_SIE);
		break;
	case MENU_SAVE_AS_SIE_ALL:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_SAVE_AS_ALL, pLocaleData->pLocaleLanguage));
		break;
	case MENU_SAVE_AS_SIE_CHART:
		strcat(MenuText, GetEnumFolderTypeText(CONTENT_LABEL_CHARTS, pLocaleData->pLocaleLanguage));
		break;
	case MENU_SAVE_AS_SIE_BALANCE:
		strcat(MenuText, GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleData->pLocaleLanguage));
		break;
	case MENU_SAVE_AS_SIE_VOUCHER:
		strcat(MenuText, GetEnumFolderTypeText(CONTENT_LABEL_VOUCHER, pLocaleData->pLocaleLanguage));
		break;
#endif
#if(PROGRAM_FEATURES_MASK & PROGRAM_FEATURES_XGL)
	case MENU_SAVE_AS_XGL:
		strcat(MenuText, FILE_TYPE_LABEL_XGL);
		break;
	case MENU_SAVE_AS_XGL_ALL:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_SAVE_AS_ALL, pLocaleData->pLocaleLanguage));
		break;
	case MENU_SAVE_AS_XGL_CHART:
		strcat(MenuText, GetEnumFolderTypeText(CONTENT_LABEL_CHARTS, pLocaleData->pLocaleLanguage));
		break;
	case MENU_SAVE_AS_XGL_BALANCE:
		strcat(MenuText, GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleData->pLocaleLanguage));
		break;
	case MENU_SAVE_AS_XGL_VOUCHER:
		strcat(MenuText, GetEnumFolderTypeText(CONTENT_LABEL_VOUCHER, pLocaleData->pLocaleLanguage));
		break;
#endif
	case MENU_TEXT:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_TEXT, pLocaleData->pLocaleLanguage));
		break;
	case MENU_SPREADSHEET:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_SPREADSHEET, pLocaleData->pLocaleLanguage));
		break;
	case MENU_PRINT:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_PRINT, pLocaleData->pLocaleLanguage));
		break;
	case MENU_DO:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_DO, pLocaleData->pLocaleLanguage));
		break;
	case MENU_DO_NEW:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_DO_NEW, pLocaleData->pLocaleLanguage));
		break;
	case MENU_DO_YEAR:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_DO_YEAR, pLocaleData->pLocaleLanguage));
		break;
	case MENU_IMPORT:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_DO_IMPORT, pLocaleData->pLocaleLanguage));
		break;
	case MENU_DO_IMPORT:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_DO_IMPORT, pLocaleData->pLocaleLanguage));
		break;
	case MENU_DO_IMPORT_BALANCES:
		strcat(MenuText, GetEnumFolderTypeText(TXT_BALANCE, pLocaleData->pLocaleLanguage));
		break;
	case MENU_DO_IMPORT_VOUCHERS:
		strcat(MenuText, GetEnumFolderTypeText(TXT_VOUCHERS, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_VIEW, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_TAB_DEL:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_VIEW_TAB_DEL, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_TAB_ADD:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_VIEW_TAB_ADD, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_TAB_ADD_ENTITY:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_ENTITY, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_TAB_ADD_OBJECTS:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_OBJECT, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_TAB_ADD_BALANCE:
		strcat(MenuText, GetEnumFolderTypeText(TXT_BALANCE, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_TAB_ADD_LEDGER:
		strcat(MenuText, GetEnumFolderTypeText(TXT_LEDGER, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_TAB_ADD_VOUCHERS:
		strcat(MenuText, GetEnumFolderTypeText(TXT_VOUCHERS, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_TAB_ADD_INVOICE:
		strcat(MenuText, TXT_INVOICE);
		break;
	case MENU_VIEW_TAB_ADD_BANKACC:
		strcat(MenuText, TXT_BANKACS);
		break;
	case MENU_EDIT_ENTITY:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_ENTITY, pLocaleData->pLocaleLanguage));
		break;
	case MENU_EDIT_ACCOUNTS:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_ACCOUNTS, pLocaleData->pLocaleLanguage));
		break;
	case MENU_EDIT_OBJECTS:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_OBJECT, pLocaleData->pLocaleLanguage));
		break;
	case MENU_EDIT_SERIES:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_SERIES, pLocaleData->pLocaleLanguage));
		break;
	case MENU_SERIES:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_SERIES, pLocaleData->pLocaleLanguage));
		break;
	case MENU_EDIT_DATE:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_DATE, pLocaleData->pLocaleLanguage));
		break;
	case MENU_EDIT_DEBIT:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_DEBIT, pLocaleData->pLocaleLanguage));
		break;
	case MENU_EDIT_CREDIT:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_CREDIT, pLocaleData->pLocaleLanguage));
		break;
	case MENU_ABOUT:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_ABOUT, pLocaleData->pLocaleLanguage));
		break;
	case MENU_ABOUT_MESSAGE:
		strcat(MenuText, GetEnumHelpTextType(TXT_MENU_ABOUT_MESSAGE, pLocaleData->pLocaleLanguage));
		strcat(MenuText, "\n\n");
		strcat(MenuText, GetEnumHelpTextType(TXT_MENU_ABOUT_ROSETTA, pLocaleData->pLocaleLanguage));
		strcat(MenuText, "\n_________________________\n\n");
		strcat(MenuText, GetEnumHelpTextType(TXT_MENU_EDITION, pLocaleData->pLocaleLanguage));
		strcat(MenuText, "\n\n");
		strcat(MenuText, GetEnumHelpTextType(TXT_MENU_RESPONSIBILITIES, pLocaleData->pLocaleLanguage));
		strcat(MenuText, "\n\n");
		DateToStrLocale(EXPIRE_DATE, &MenuText[strlen(MenuText)], pLocaleData);
		strcat(MenuText, GetEnumHelpTextType(TXT_MENU_EXPIRE, pLocaleData->pLocaleLanguage));
		strcat(MenuText, "\n_________________________\n\n");
		strcat(MenuText, PRODUCT_DECLARATION"\n"PROGRAM_COPYRIGHT"\n");
		strcat(MenuText, GetEnumHelpTextType(TXT_MENU_ABOUT_CONTACT, pLocaleData->pLocaleLanguage));
		break;
	case MENU_HELP:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_HELP, pLocaleData->pLocaleLanguage));
		break;
	case MENU_HELP_MESSAGE:
		strcat(MenuText, GetEnumHelpTextType(TXT_MENU_HELP_MESSAGE, pLocaleData->pLocaleLanguage));
#if defined(ANDROID) || defined(IOS)
		strcat(MenuText, "\n\n");
		strcat(MenuText, GetEnumHelpTextType(TXT_MENU_HELP_HANDSET, pLocaleData->pLocaleLanguage));
#else
		strcat(MenuText, "\n\n");
		strcat(MenuText, GetEnumHelpTextType(TXT_MENU_HELP_DESKTOP, pLocaleData->pLocaleLanguage));
#endif
		break;
	case MENU_LOGO:
		strcpy(MenuText, TXT_MENU_LOGO);
		break;
	case MENU_URL:
		strcpy(MenuText, TXT_MENU_URL);
		break;
	case MENU_WEBPAGE:
		strcpy(MenuText, TXT_MENU_WEBPAGE);
		break;
	case MENU_ADDRESS:
		strcpy(MenuText, TXT_MENU_ADDRESS);
		break;
	case MENU_EMAIL:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_EMAIL, pLocaleData->pLocaleLanguage));
		break;
	case MENU_PHONE:
		strcpy(MenuText, TXT_MENU_PHONE);
		break;
	case MENU_SOCIALMEDIA:
		strcpy(MenuText, TXT_MENU_SOCIALMEDIA);
		break;
	case MENU_CONTACTLOG:
		strcpy(MenuText, TXT_MENU_CONTACTLOG);
		break;
	case MENU_PKI:
		strcpy(MenuText, TXT_MENU_PKI);
		break;
	case MENU_CRM:
		strcpy(MenuText, TXT_MENU_CRM);
		break;
	case MENU_NOTE:
		//strcat(MenuText, TXT_MENU_NOTE);
		break;
	case MENU_LEGAL_ID:
		strcpy(MenuText, TXT_MENU_LEGAL_ID);
		break;
	case MENU_VAT_ID:
		strcpy(MenuText, TXT_MENU_VAT_ID);
		break;
	case MENU_VAT_CURRENCY:
		strcpy(MenuText, TXT_MENU_VAT_CURRENCY);
		break;
	case MENU_JURISDICTION:
		strcpy(MenuText, TXT_MENU_JURISDICTION);
		break;
	case MENU_ENTITYTYPE:
		strcpy(MenuText, TXT_MENU_ENTITYTYPE);
		break;
	case MENU_STATISTICAL:
		strcpy(MenuText, TXT_MENU_STATISTICAL);
		break;
	case MENU_CASHACCOUNTING:
		strcpy(MenuText, TXT_MENU_CASHACCOUNTING);
		break;
	case MENU_CONTACT:
		strcpy(MenuText, TXT_MENU_CONTACT);
		break;
	case MENU_DRAGNDROP:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_DRAGNDROP, pLocaleData->pLocaleLanguage));
		break;
	case MENU_DIMOBJ:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_DIMOBJ, pLocaleData->pLocaleLanguage));
		break;
	case MENU_ERROR_NO_FILE_NAME:
		strcat(MenuText, GetEnumApplicationTextType(TXT_ERROR_NO_FILE_NAME, pLocaleData->pLocaleLanguage));
		break;
	case MENU_ERROR_NO_FILE_CONTENT:
		strcat(MenuText, GetEnumApplicationTextType(TXT_ERROR_NO_FILE_CONTENT, pLocaleData->pLocaleLanguage));
		break;
	case MENU_ERROR_EXPIRE:
		strcat(MenuText, GetEnumApplicationTextType(TXT_ERROR_EXPIRE, pLocaleData->pLocaleLanguage));
		break;
	case MENU_ERROR_OS_VER:
		strcat(MenuText, GetEnumApplicationTextType(TXT_ERROR_OS_VER, pLocaleData->pLocaleLanguage));
		break;
	case MENU_ERROR_FILE_CLOSE:
		strcat(MenuText, GetEnumApplicationTextType(TXT_ERROR_FILE_CLOSE, pLocaleData->pLocaleLanguage));
		break;
	case MENU_ERROR_FILE_OPEN:
		strcat(MenuText, GetEnumApplicationTextType(TXT_ERROR_FILE_OPEN, pLocaleData->pLocaleLanguage));
		break;
	case MENU_ERROR_MEMORY_MANAGEMENT:
		strcat(MenuText, GetEnumApplicationTextType(TXT_ERROR_MEMORY_MANAGEMENT, pLocaleData->pLocaleLanguage));
		break;
	case MENU_FAILURE:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_FAILURE, pLocaleData->pLocaleLanguage));
		break;
	case MENU_QUANT:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_QUANT, pLocaleData->pLocaleLanguage));
		break;
	case MENU_CHANGE:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_CHANGE, pLocaleData->pLocaleLanguage));
		break;
	case MENU_SUM_MONTHS:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_SUM_MONTHS, pLocaleData->pLocaleLanguage));
		break;
	case MENU_BUDGET:
		strcat(MenuText, GetEnumApplicationTextType(TXT_BUDGET, pLocaleData->pLocaleLanguage));
		break;
	case MENU_TXT_NOTE:
		strcat(MenuText, GetEnumApplicationTextType(TXT_NOTE, pLocaleData->pLocaleLanguage));
		break;
	case MENU_TXT_FILE_CONTENT:
		strcat(MenuText, GetEnumApplicationTextType(TXT_FILE_CONTENT, pLocaleData->pLocaleLanguage));
		break;
	case MENU_TXT_LAST_ENTRY_DATE:
		strcat(MenuText, GetEnumApplicationTextType(TXT_LAST_ENTRY_DATE, pLocaleData->pLocaleLanguage));
		break;
	case MENU_TXT_CURRENCY:
		strcat(MenuText, GetEnumApplicationTextType(TXT_CURRENCY, pLocaleData->pLocaleLanguage));
		break;
	case MENU_TXT_MAIN_LANGUAGE:
		strcat(MenuText, GetEnumApplicationTextType(TXT_MAIN_LANGUAGE, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_ACCOUNTTYPE:
		strcat(MenuText, GetEnumApplicationTextType(ACCOUNT_TYPE_SECTOR, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_TOTAL:
		strcat(MenuText, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_TOTAL, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_BALANCE:
		strcat(MenuText, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_BALANCE, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_RESULT:
		strcat(MenuText, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_RESULT, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_ACTIVE:
		strcat(MenuText, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_ACTIVE, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_PASSIVE:
		strcat(MenuText, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_PASSIVE, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_PROFIT:
		strcat(MenuText, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_PROFIT, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_EARNING:
		strcat(MenuText, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_EARNING, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_OUTCOME:
		strcat(MenuText, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_OUTCOME, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_TANGIBLE:
		strcat(MenuText, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_TANGIBLE, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_STOCK:
		strcat(MenuText, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_STOCK, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_RECEIVABLES:
		strcat(MenuText, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_RECEIVABLES, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_ASSETS:
		strcat(MenuText, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_ASSETS, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_EQUITIES:
		strcat(MenuText, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_EQUITIES, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_PROVISIONS:
		strcat(MenuText, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_PROVISIONS, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_LIABILITIES:
		strcat(MenuText, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_LIABILITIES, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_REVENUES:
		strcat(MenuText, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_REVENUES, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_SUPPLIES:
		strcat(MenuText, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_SUPPLIES, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_EXPENSES:
		strcat(MenuText, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_EXPENSES, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_STAFF:
		strcat(MenuText, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_STAFF, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_LOSSGAINES:
		strcat(MenuText, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_LOSS_GAINS, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_FINANCIAL:
		strcat(MenuText, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_FINANCIAL, pLocaleData->pLocaleLanguage));
		break;
	case MENU_VIEW_AUXILIARY_COSTS:
		strcat(MenuText, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_AUX, pLocaleData->pLocaleLanguage));
		break;
	default:
		return NulString;
	}
	if (bHwKeyb) {
		switch (iTextItem) {
		case MENU_EXIT:
			break;
		case MENU_OK:
			break;
		case MENU_CANCEL:
			break;
		case MENU_EDIT:
			break;
		case MENU_EDIT_ITEM:
			//		strcat(MenuText, GetEnumApplicationTextType(" - ", pLocaleData->pLocaleLanguage));
			//		strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_INSERT, pLocaleData->pLocaleLanguage));
			break;
		case MENU_DELETE_ITEM:
			strcat(MenuText, GetEnumApplicationTextType(" - ", pLocaleData->pLocaleLanguage));
			strcat(MenuText, GetEnumApplicationTextType(TXT_MENU_DELETE, pLocaleData->pLocaleLanguage));
			break;
		case MENU_EXAMINE:
			strcat(MenuText, GetEnumApplicationTextType(" - F12", pLocaleData->pLocaleLanguage));
			break;
		case MENU_EXPAND:
			strcat(MenuText, GetEnumApplicationTextType(" - F11", pLocaleData->pLocaleLanguage));
			break;
		case MENU_LINE_SELECT:
			break;
		case MENU_SEARCH:
			strcat(MenuText, GetEnumApplicationTextType(" - F3", pLocaleData->pLocaleLanguage));
			break;
		case MENU_SEARCH_BUTTON:
			strcat(MenuText, GetEnumApplicationTextType(" - F4", pLocaleData->pLocaleLanguage));
			break;
		case MENU_FILE:
			break;
		case MENU_ADD_ITEM:
			break;
		case MENU_OPEN:
			strcat(MenuText, GetEnumApplicationTextType(" - F2", pLocaleData->pLocaleLanguage));
			break;
		case MENU_SAVE:
			break;
		case MENU_SAVE_AS:
			break;
#if(PROGRAM_FEATURES_MASK & PROGRAM_FEATURES_ROSETTA)
		case MENU_SAVE_AS_STD:
			break;
		case MENU_SAVE_AS_STD_ALL:
			break;
		case MENU_SAVE_AS_STD_CHART:
			break;
		case MENU_SAVE_AS_STD_BALANCE:
			break;
		case MENU_SAVE_AS_STD_VOUCHER:
			break;
		case MENU_SAVE_AS_STD_LEDGER:
			break;
		case MENU_SAVE_AS_STD_INVOICE:
			break;
		case MENU_SAVE_AS_STD_BANKACS:
			break;
		case MENU_SAVE_AS_STD_ERROR:
			break;
		case MENU_SAVE_AS_STD_ANALYSE:
			break;
#endif
#if(PROGRAM_FEATURES_MASK & PROGRAM_FEATURES_SIE)
		case MENU_SAVE_AS_SIE:
			break;
		case MENU_SAVE_AS_SIE_ALL:
			break;
		case MENU_SAVE_AS_SIE_CHART:
			break;
		case MENU_SAVE_AS_SIE_BALANCE:
			break;
		case MENU_SAVE_AS_SIE_VOUCHER:
			break;
#endif
#if(PROGRAM_FEATURES_MASK & PROGRAM_FEATURES_XGL)
		case MENU_SAVE_AS_XGL:
			break;
		case MENU_SAVE_AS_XGL_ALL:
			break;
		case MENU_SAVE_AS_XGL_CHART:
			break;
		case MENU_SAVE_AS_XGL_BALANCE:
			break;
		case MENU_SAVE_AS_XGL_VOUCHER:
			break;
#endif
		case MENU_TEXT:
			break;
		case MENU_SPREADSHEET:
			break;
		case MENU_PRINT:
			break;
		case MENU_DO:
			break;
		case MENU_DO_NEW:
			break;
		case MENU_DO_YEAR:
			break;
		case MENU_IMPORT:
			break;
		case MENU_DO_IMPORT:
			break;
		case MENU_DO_IMPORT_BALANCES:
			break;
		case MENU_DO_IMPORT_VOUCHERS:
			break;
		case MENU_VIEW:
			break;
		case MENU_VIEW_TAB_DEL:
			strcat(MenuText, GetEnumApplicationTextType(" - -", pLocaleData->pLocaleLanguage));
			break;
		case MENU_VIEW_TAB_ADD:
			break;
		case MENU_VIEW_TAB_ADD_ENTITY:
			strcat(MenuText, GetEnumApplicationTextType(" - F9", pLocaleData->pLocaleLanguage));
			break;
		case MENU_VIEW_TAB_ADD_OBJECTS:
			strcat(MenuText, GetEnumApplicationTextType(" - F8", pLocaleData->pLocaleLanguage));
			break;
		case MENU_VIEW_TAB_ADD_BALANCE:
			strcat(MenuText, GetEnumApplicationTextType(" - F5", pLocaleData->pLocaleLanguage));
			break;
		case MENU_VIEW_TAB_ADD_LEDGER:
			strcat(MenuText, GetEnumApplicationTextType(" - F6", pLocaleData->pLocaleLanguage));
			break;
		case MENU_VIEW_TAB_ADD_VOUCHERS:
			strcat(MenuText, GetEnumApplicationTextType(" - F7", pLocaleData->pLocaleLanguage));
			break;
		case MENU_VIEW_TAB_ADD_INVOICE:
			break;
		case MENU_VIEW_TAB_ADD_BANKACC:
			break;
		case MENU_EDIT_ENTITY:
			break;
		case MENU_EDIT_ACCOUNTS:
			break;
		case MENU_EDIT_OBJECTS:
			break;
		case MENU_EDIT_SERIES:
			break;
		case MENU_SERIES:
			break;
		case MENU_EDIT_DATE:
			break;
		case MENU_EDIT_DEBIT:
			break;
		case MENU_EDIT_CREDIT:
			break;
		case MENU_ABOUT:
			break;
		case MENU_ABOUT_MESSAGE:
			break;
		case MENU_HELP:
			strcat(MenuText, GetEnumApplicationTextType(" - F1", pLocaleData->pLocaleLanguage));
			break;
		case MENU_HELP_MESSAGE:
			break;
		case MENU_LOGO:
			break;
		case MENU_URL:
			break;
		case MENU_WEBPAGE:
			break;
		case MENU_ADDRESS:
			break;
		case MENU_EMAIL:
			break;
		case MENU_PHONE:
			break;
		case MENU_SOCIALMEDIA:
			break;
		case MENU_CONTACTLOG:
			break;
		case MENU_PKI:
			break;
		case MENU_CRM:
			break;
		case MENU_NOTE:
			break;
		case MENU_LEGAL_ID:
			break;
		case MENU_VAT_ID:
			break;
		case MENU_VAT_CURRENCY:
			break;
		case MENU_JURISDICTION:
			break;
		case MENU_ENTITYTYPE:
			break;
		case MENU_STATISTICAL:
			break;
		case MENU_CASHACCOUNTING:
			break;
		case MENU_CONTACT:
			break;
		case MENU_DRAGNDROP:
			break;
		case MENU_DIMOBJ:
			break;
		case MENU_ERROR_NO_FILE_NAME:
			break;
		case MENU_ERROR_NO_FILE_CONTENT:
			break;
		case MENU_ERROR_EXPIRE:
			break;
		case MENU_ERROR_OS_VER:
			break;
		case MENU_ERROR_FILE_CLOSE:
			break;
		case MENU_ERROR_FILE_OPEN:
			break;
		case MENU_ERROR_MEMORY_MANAGEMENT:
			break;
		case MENU_FAILURE:
			break;
		case MENU_BUDGET:
			break;
		case MENU_TXT_NOTE:
			break;
		case MENU_TXT_FILE_CONTENT:
			break;
		case MENU_TXT_LAST_ENTRY_DATE:
			break;
		case MENU_TXT_CURRENCY:
			break;
		case MENU_TXT_MAIN_LANGUAGE:
			break;
		case MENU_VIEW_ACCOUNTTYPE:
			break;
		case MENU_VIEW_TOTAL:
			break;
		case MENU_VIEW_BALANCE:
			break;
		case MENU_VIEW_RESULT:
			break;
		case MENU_VIEW_ACTIVE:
			break;
		case MENU_VIEW_PASSIVE:
			break;
		case MENU_VIEW_PROFIT:
			break;
		case MENU_VIEW_EARNING:
			break;
		case MENU_VIEW_OUTCOME:
			break;
		case MENU_VIEW_TANGIBLE:
			break;
		case MENU_VIEW_STOCK:
			break;
		case MENU_VIEW_RECEIVABLES:
			break;
		case MENU_VIEW_ASSETS:
			break;
		case MENU_VIEW_EQUITIES:
			break;
		case MENU_VIEW_PROVISIONS:
			break;
		case MENU_VIEW_LIABILITIES:
			break;
		case MENU_VIEW_REVENUES:
			break;
		case MENU_VIEW_SUPPLIES:
			break;
		case MENU_VIEW_EXPENSES:
			break;
		case MENU_VIEW_STAFF:
			break;
		case MENU_VIEW_LOSSGAINES:
			break;
		case MENU_VIEW_FINANCIAL:
			break;
		case MENU_VIEW_AUXILIARY_COSTS:
			break;
		default:
			break;
		}
	}
	return MenuText;
}

/********************************************************************

char *CreateGeneralTitle(

*********************************************************************/
char *CreateGeneralTitle(
	InstanceDataType *pInstanceData,
	char *pFilePathName,
	BITMASK bmCommand,
	TabDataType *pTabData,
	char *pCurrency,
	DATEINT LastDate,
	size_t ztMaxPathLength)
{
	char *pSubText = 0;
	char *pPeriodText = 0;
	void *vpObject = 0;
	DATEINT StartDate = 0;

	if (!pInstanceData)
		return pFilePathName;

	if (!pFilePathName)
		return pFilePathName;

	char pLocaleLanguage[3] = "en";
	if (pInstanceData && pInstanceData->pLocaleData && pInstanceData->pLocaleData->pLocaleLanguage)
		strncpy(pLocaleLanguage, pInstanceData->pLocaleData->pLocaleLanguage, 2);

	if (pTabData) {
		pSubText = pTabData->pSelSeries;
		pPeriodText = pTabData->pPeriodLabel;
		vpObject = pTabData->vpObject;
	}

	//No file pFilePathName content is in use (in Android)
	{
		char *pPtr = strrchr(pFilePathName, '\\');
		if (!pPtr)
			pPtr = strrchr(pFilePathName, '/');
		if (!pPtr) {
			pPtr = pFilePathName;
		}
		else
			pPtr++;
		*pPtr = 0;
	}

	// The company name
	if (GetWorkDataName(pInstanceData->vpWorkData)) {
		strncat(pFilePathName, GetWorkDataName(pInstanceData->vpWorkData), ztMaxPathLength - strlen(pFilePathName));
		strncat(pFilePathName, " ", ztMaxPathLength - strlen(pFilePathName));
	}

	// Name Starts with start and last year
	if ((ztMaxPathLength - strlen(pFilePathName)) <= 32)
		return pFilePathName;
	if(((bmCommand & CONTENT_MASK) == CONTENT_LEDGER) 
		|| ((bmCommand & CONTENT_MASK) == CONTENT_VOUCHERS))
		StartDate = GetAccountingStart(pInstanceData->vpWorkData);
	else 
		StartDate = GetLedgerStart(pInstanceData->vpWorkData, 0);
	if (StartDate) {
		NumericToStr(StartDate, &pFilePathName[strlen(pFilePathName)]);
		memset(&pFilePathName[strlen(pFilePathName) - 4], 0, 5);
		if ((GetLedgerEnd(pInstanceData->vpWorkData, 0) / 10000) != (StartDate / 10000)) {
			if ((ztMaxPathLength - strlen(pFilePathName)) <= 16)
				return pFilePathName;
			strncat(pFilePathName, "-", ztMaxPathLength - strlen(pFilePathName));
			NumericToStr(GetLedgerEnd(pInstanceData->vpWorkData, 0), &pFilePathName[strlen(pFilePathName)]);
			memset(&pFilePathName[strlen(pFilePathName) - 4], 0, 5);
			memmove(&pFilePathName[strlen(pFilePathName) - 4], &pFilePathName[strlen(pFilePathName) - 2], 3);
		}
	}
	switch (bmCommand & CONTENT_MASK) {
	case CONTENT_CHARTS:
		strncat(pFilePathName, " ", ztMaxPathLength - strlen(pFilePathName));
		if (((bmCommand & FILE_TYPE_MASK) == FILE_TYPE_TXT)
			|| ((bmCommand & FILE_TYPE_MASK) == FILE_TYPE_CSV)
			|| ((bmCommand & FILE_TYPE_MASK) == FILE_TYPE_PDF)
			|| ((bmCommand & FILE_TYPE_MASK) == FILE_TYPE_PRINT)
			|| ((bmCommand & FILE_TYPE_MASK) == FILE_TYPE_TITLE))
			strncat(pFilePathName, GetEnumFolderTypeText(CONTENT_LABEL_OBJECTS, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
		else
			strncat(pFilePathName, GetEnumFolderTypeText(CONTENT_LABEL_CHARTS, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
		break;
	case CONTENT_BALANCE:
		strncat(pFilePathName, " ", ztMaxPathLength - strlen(pFilePathName));
		strncat(pFilePathName, GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
		break;
	case CONTENT_VOUCHERS:
		strncat(pFilePathName, " ", ztMaxPathLength - strlen(pFilePathName));
		strncat(pFilePathName, GetEnumFolderTypeText(CONTENT_LABEL_VOUCHER, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
		break;
	case CONTENT_INVOICE:
		strncat(pFilePathName, " ", ztMaxPathLength - strlen(pFilePathName));
		strncat(pFilePathName, CONTENT_LABEL_INVOICE, ztMaxPathLength - strlen(pFilePathName));
		break;
	case CONTENT_BANKACS:
		strncat(pFilePathName, " ", ztMaxPathLength - strlen(pFilePathName));
		strncat(pFilePathName, CONTENT_LABEL_BANKACS, ztMaxPathLength - strlen(pFilePathName));
		break;
	case CONTENT_NOTHING:
		strncat(pFilePathName, " ", ztMaxPathLength - strlen(pFilePathName));
		strncat(pFilePathName, CONTENT_LABEL_NOTHING, ztMaxPathLength - strlen(pFilePathName));
		break;
	case CONTENT_ERRORS:
		strncat(pFilePathName, " ", ztMaxPathLength - strlen(pFilePathName));
		strncat(pFilePathName, CONTENT_LABEL_ERROR, ztMaxPathLength - strlen(pFilePathName));
		break;
	case CONTENT_ENTITY:
		strncat(pFilePathName, " ", ztMaxPathLength - strlen(pFilePathName));
		strncat(pFilePathName, GetEnumFolderTypeText(CONTENT_LABEL_ENTITY, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
		break;
	case CONTENT_ANALYSE:
		strncat(pFilePathName, " ", ztMaxPathLength - strlen(pFilePathName));
		strncat(pFilePathName, GetEnumFolderTypeText(CONTENT_LABEL_ANALYSE, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
		break;
	case CONTENT_LEDGER:
		strncat(pFilePathName, " ", ztMaxPathLength - strlen(pFilePathName));
		strncat(pFilePathName, GetEnumFolderTypeText(CONTENT_LABEL_LEDGER, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
		break;
	default:
		break;
	}

	if (pTabData) {
		if ((pTabData->bmTabType & CONTENT_MASK) == CONTENT_VOUCHERS) {
			if (pSubText && *pSubText && (/*not*/strcmp(pSubText, ALL_BULLET))) {
				strncat(pFilePathName, ", ", ztMaxPathLength - strlen(pFilePathName));
				strncat(pFilePathName, pSubText, ztMaxPathLength - strlen(pFilePathName));
			}
		}
		else if ((((pTabData->bmTabType & CONTENT_MASK) == CONTENT_BALANCE) || ((pTabData->bmTabType & CONTENT_MASK) == CONTENT_LEDGER))
			&& (pTabData->nSubTab != MENU_VIEW_TOTAL)) {
			strncat(pFilePathName, ", ", ztMaxPathLength - strlen(pFilePathName));
			switch (pTabData->nSubTab) {
			case MENU_VIEW_TOTAL:
				strncat(pFilePathName, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_TOTAL, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				break;
			case MENU_VIEW_BALANCE:
				strncat(pFilePathName, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_BALANCE, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				break;
			case MENU_VIEW_RESULT:
				strncat(pFilePathName, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_RESULT, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				break;
			case MENU_VIEW_ACTIVE:
				strncat(pFilePathName, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_ACTIVE, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				break;
			case MENU_VIEW_PASSIVE:
				strncat(pFilePathName, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_PASSIVE, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				break;
			case MENU_VIEW_PROFIT:
				strncat(pFilePathName, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_PROFIT, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				break;
			case MENU_VIEW_EARNING:
				strncat(pFilePathName, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_EARNING, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				break;
			case MENU_VIEW_OUTCOME:
				strncat(pFilePathName, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_OUTCOME, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				break;
			case MENU_VIEW_TANGIBLE:
				strncat(pFilePathName, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_TANGIBLE, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				break;
			case MENU_VIEW_STOCK:
				strncat(pFilePathName, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_STOCK, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				break;
			case MENU_VIEW_RECEIVABLES:
				strncat(pFilePathName, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_RECEIVABLES, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				break;
			case MENU_VIEW_ASSETS:
				strncat(pFilePathName, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_ASSETS, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				break;
			case MENU_VIEW_EQUITIES:
				strncat(pFilePathName, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_EQUITIES, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				break;
			case MENU_VIEW_PROVISIONS:
				strncat(pFilePathName, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_PROVISIONS, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				break;
			case MENU_VIEW_LIABILITIES:
				strncat(pFilePathName, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_LIABILITIES, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				break;
			case MENU_VIEW_REVENUES:
				strncat(pFilePathName, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_REVENUES, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				break;
			case MENU_VIEW_SUPPLIES:
				strncat(pFilePathName, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_SUPPLIES, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				break;
			case MENU_VIEW_EXPENSES:
				strncat(pFilePathName, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_EXPENSES, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				break;
			case MENU_VIEW_STAFF:
				strncat(pFilePathName, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_STAFF, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				break;
			case MENU_VIEW_LOSSGAINES:
				strncat(pFilePathName, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_LOSS_GAINS, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				break;
			case MENU_VIEW_FINANCIAL:
				strncat(pFilePathName, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_FINANCIAL, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				break;
			case MENU_VIEW_AUXILIARY_COSTS:
				strncat(pFilePathName, GetEnumAccountTitleTypeText(ACCOUNT_TYPE_AUX, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				break;
			}
		}
		if (pPeriodText && *pPeriodText && (/*not*/strcmp(pPeriodText, ALL_BULLET))) {
			strncat(pFilePathName, ", ", ztMaxPathLength - strlen(pFilePathName));
			strncat(pFilePathName, pPeriodText, ztMaxPathLength - strlen(pFilePathName));
		}
		if (vpObject) {
			ObjectType *pObject = vpObject;
			if (pObject->pDimension->Id) {
				strncat(pFilePathName, ", ", ztMaxPathLength - strlen(pFilePathName));
				strncat(pFilePathName, GetTextListText(pObject->pDimension->pListName, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
				strncat(pFilePathName, "-", ztMaxPathLength - strlen(pFilePathName));
				strncat(pFilePathName, GetTextListText(pObject->pListName, pLocaleLanguage), ztMaxPathLength - strlen(pFilePathName));
			}
		}
	}
	switch (bmCommand & FILE_TYPE_MASK) {
	case FILE_TYPE_TXT:
	case FILE_TYPE_CSV:
	case FILE_TYPE_PDF:
		if (pCurrency && *pCurrency) {
			strncat(pFilePathName, " ", ztMaxPathLength - strlen(pFilePathName));
			strncat(pFilePathName, pCurrency, ztMaxPathLength - strlen(pFilePathName));
			strncat(pFilePathName, "", ztMaxPathLength - strlen(pFilePathName));
		}
		if (LastDate) {
			strncat(pFilePathName, " ", ztMaxPathLength - strlen(pFilePathName));
			if (ztMaxPathLength > (strlen(pFilePathName) + LENGTH_LONG_DATE))
				DateToStrLocale(LastDate, &pFilePathName[strlen(pFilePathName)], 0);
		}
		break;
	case FILE_TYPE_TITLE:
		if (pCurrency && *pCurrency) {
			strncat(pFilePathName, " (", ztMaxPathLength - strlen(pFilePathName));
			strncat(pFilePathName, pCurrency, ztMaxPathLength - strlen(pFilePathName));
			strncat(pFilePathName, ")", ztMaxPathLength - strlen(pFilePathName));
		}
		if ((LastDate)) {
			strncat(pFilePathName, ", ", ztMaxPathLength - strlen(pFilePathName));
			if (ztMaxPathLength > (strlen(pFilePathName) + LENGTH_LONG_DATE))
				DateToStrLocale(LastDate, &pFilePathName[strlen(pFilePathName)], pInstanceData->pLocaleData);
		}
		break;
	default:
		break;
	}
	switch (bmCommand & FILE_TYPE_MASK) {
	case FILE_TYPE_TXT:
		strncat(pFilePathName, FILE_TYPE_SUFFIX_TXT, ztMaxPathLength - strlen(pFilePathName));
		break;
	case FILE_TYPE_CSV:
		strncat(pFilePathName, FILE_TYPE_SUFFIX_CSV, ztMaxPathLength - strlen(pFilePathName));
		break;
#if(PROGRAM_FEATURES_MASK & PROGRAM_FEATURES_SIE)
	case FILE_TYPE_SIE:
		if ((bmCommand & CONTENT_MASK) == CONTENT_VOUCHERS)
			strncat(pFilePathName, FILE_TYPE_SUFFIX_SIE_I, ztMaxPathLength - strlen(pFilePathName));
		else
			strncat(pFilePathName, FILE_TYPE_SUFFIX_SIE_E, ztMaxPathLength - strlen(pFilePathName));
		break;
#endif
#if(PROGRAM_FEATURES_MASK & PROGRAM_FEATURES_XGL)
	case FILE_TYPE_XGL:
		strncat(pFilePathName, FILE_TYPE_SUFFIX_XGL, ztMaxPathLength - strlen(pFilePathName));
		break;
#endif
#if(PROGRAM_FEATURES_MASK & PROGRAM_FEATURES_ROSETTA)
	case FILE_TYPE_STD:
		strncat(pFilePathName, FILE_TYPE_SUFFIX_STD, ztMaxPathLength - strlen(pFilePathName));
		break;
#endif
	case FILE_TYPE_PDF:
		strncat(pFilePathName, FILE_TYPE_SUFFIX_PDF, ztMaxPathLength - strlen(pFilePathName));
		break;
	case FILE_TYPE_TITLE:
		break;
	default:
		break;
	}
	return pFilePathName;
}

/********************************************************************

char *NuberedTabTexts(

*********************************************************************/
char *NuberedTabTexts(
	char *pTabText,
	NUMERIC nIndex)
{
	char *pTabNumberedText = 0;
	pTabNumberedText = calloc(strlen(pTabText) + 32, sizeof(char));
	NumericToStr(nIndex + 1, pTabNumberedText);
	strcat(pTabNumberedText, ". ");
	strcat(pTabNumberedText, pTabText);
	return pTabNumberedText;
}

/********************************************************************

Sum up Voucher data to Balances, month periods, quarter periods and section totals

*********************************************************************/
/*************************************************************

static void WrapUpAccounting(

*************************************************************/
static void WrapUpAccounting(
	AccountingType *pAccounting)
{
	if (!pAccounting->pGaap || !*pAccounting->pGaap)
		CopyString(&pAccounting->pGaap, BAS2000);
	/*	else if (!strncmp(pAccounting->pGaap, "BAS2", 4))
			CopyString(&pAccounting->pGaap, BAS2000);
	*/
	{
		LedgerType *pLedger
			= LListGetFirstRecordData(pAccounting->pLedgersChart->pListLedger);
		while (pLedger) {
			if (!IsDataLedger(pLedger)) {
				DestroyLedger(&pLedger);
				pAccounting->pLedgersChart->pListLedger->CursorRecord->vpData = 0;
				LListDeleteCursorRecord(pAccounting->pLedgersChart->pListLedger);
			}
			pLedger = LListGetNextRecordData(pAccounting->pLedgersChart->pListLedger);
		}
	}
}

/********************************************************************

static void AddUpAccountPeriod(

*********************************************************************/
static void AddUpAccountPeriod(
	void *pStructDocTop,
	void *pStructParent,
	LLIST pList,
	DATEINT PeriodEnd,
	AMOUNT Amount,
	QUANT Quantity,
	AMOUNT BudgetAmount,
	QUANT BudgetQuantity,
	char *pPeriodLabel)
{
	BalanceAccountPeriodType *pBalanceAccountPeriod = CreateBalanceAccountPeriod(pStructDocTop, pStructParent);
	pBalanceAccountPeriod->End = GetLastDateInPeriod(PeriodEnd, pPeriodLabel);
	pBalanceAccountPeriod = SearchSetBalanceAccountPeriod(pList, pBalanceAccountPeriod);
	if (pBalanceAccountPeriod) {
		pBalanceAccountPeriod->Outcome += Amount;
		pBalanceAccountPeriod->OutcomeQuantity += Quantity;
		pBalanceAccountPeriod->Budget += BudgetAmount;
		pBalanceAccountPeriod->BudgetQuantity += BudgetQuantity;
	}
}

/********************************************************************

static void AddUpBalanceReport(

*********************************************************************/
static void AddUpBalanceReport(
	LedgerType *pLedger,
	ObjectType *pObject,
	AccountType *pAccount,
	DATEINT PeriodEnd,
	AMOUNT Amount,
	QUANT Quantity)
{
	if (pLedger && pAccount && PeriodEnd && (Amount || (!DecimalCmp(Quantity, 0)))) {
		BalanceReportType *pBalanceReport = GetBalanceReport(pLedger, pObject);
		if (!pBalanceReport) {
			pBalanceReport = CreateBalanceReport(pLedger->pStructDocTop, pLedger);
			pBalanceReport->pObject = pObject;
			pBalanceReport = SearchSetBalanceReport(pLedger->pListReport, pBalanceReport);
		}
		if (pBalanceReport) {
			BalanceAccountType *pBalanceAccount = GetBalanceAccount(pBalanceReport, pAccount);
			if (!pBalanceAccount) {
				pBalanceAccount = CreateBalanceAccount(pBalanceReport->pStructDocTop, pBalanceReport);
				pBalanceAccount->pAccount = pAccount;
				pBalanceAccount = SearchSetBalanceAccount(pBalanceReport->pListBalanceAccount, pBalanceAccount);
			}
			if (pBalanceAccount) {
				pBalanceAccount->Forward += Amount;
				pBalanceAccount->ForwardQuantity += Quantity;
				AddUpAccountPeriod(pBalanceAccount->pStructDocTop, pBalanceAccount, pBalanceAccount->pListMonth, PeriodEnd, Amount, Quantity, 0, 0, MONTHPERIOD);
			}
		}
	}
}

/********************************************************************

static void ResetQuarters(

*********************************************************************/
static void ResetQuarters(
	LedgerType *pLedger)
{
	BalanceReportType *pBalanceReport = LListGetFirstRecordData(pLedger->pListReport);
	while (pBalanceReport) {
		BalanceAccountType *pBalanceAccount = LListGetFirstRecordData(pBalanceReport->pListBalanceAccount);
		while (pBalanceAccount) {
			BalanceAccountPeriodType *pAccountPeriod = LListGetFirstRecordData(pBalanceAccount->pListAnnual);
			while (pAccountPeriod) {
				DestroyBalanceAccountPeriod(&pAccountPeriod);
				pAccountPeriod = LListEraseCursorRecord(pBalanceAccount->pListAnnual);
			}
			pAccountPeriod = LListGetFirstRecordData(pBalanceAccount->pListSemi);
			while (pAccountPeriod) {
				DestroyBalanceAccountPeriod(&pAccountPeriod);
				pAccountPeriod = LListEraseCursorRecord(pBalanceAccount->pListSemi);
			}
			pAccountPeriod = LListGetFirstRecordData(pBalanceAccount->pListTertial);
			while (pAccountPeriod) {
				DestroyBalanceAccountPeriod(&pAccountPeriod);
				pAccountPeriod = LListEraseCursorRecord(pBalanceAccount->pListTertial);
			}
			pAccountPeriod = LListGetFirstRecordData(pBalanceAccount->pListQuarter);
			while (pAccountPeriod) {
				DestroyBalanceAccountPeriod(&pAccountPeriod);
				pAccountPeriod = LListEraseCursorRecord(pBalanceAccount->pListQuarter);
			}
			pBalanceAccount = LListGetNextRecordData(pBalanceReport->pListBalanceAccount);
		}
		pBalanceReport = LListGetNextRecordData(pLedger->pListReport);
	}
}

/********************************************************************

static void AddUpBalanceQuarterPeriods(

*********************************************************************/
static void AddUpBalanceQuarterPeriods(
	LedgerType *pLedger)
{
	if (pLedger) {
		BalanceReportType *pBalanceReport = LListGetFirstRecordData(pLedger->pListReport);
		while (pBalanceReport) {
			BalanceAccountType *pBalanceAccount = LListGetFirstRecordData(pBalanceReport->pListBalanceAccount);
			while (pBalanceAccount) {
				BalanceAccountPeriodType *pAccountMonth = LListGetFirstRecordData(pBalanceAccount->pListMonth);
				if (pAccountMonth
					&& (!((LListGetFirstRecordData(pBalanceAccount->pListQuarter))
						&& (LListGetFirstRecordData(pBalanceAccount->pListTertial))
						&& (LListGetFirstRecordData(pBalanceAccount->pListSemi))
						&& (LListGetFirstRecordData(pBalanceAccount->pListAnnual))))) {
					while (pAccountMonth) {
						AddUpAccountPeriod(pBalanceAccount->pStructDocTop, pBalanceAccount, pBalanceAccount->pListQuarter,	pAccountMonth->End, pAccountMonth->Outcome, pAccountMonth->OutcomeQuantity, pAccountMonth->Budget, pAccountMonth->BudgetQuantity, QUARTERPERIOD);
						AddUpAccountPeriod(pBalanceAccount->pStructDocTop, pBalanceAccount, pBalanceAccount->pListTertial,	pAccountMonth->End, pAccountMonth->Outcome, pAccountMonth->OutcomeQuantity, pAccountMonth->Budget, pAccountMonth->BudgetQuantity, TERTIALPERIOD);
						AddUpAccountPeriod(pBalanceAccount->pStructDocTop, pBalanceAccount, pBalanceAccount->pListSemi,		pAccountMonth->End, pAccountMonth->Outcome, pAccountMonth->OutcomeQuantity, pAccountMonth->Budget, pAccountMonth->BudgetQuantity, SEMIPERIOD);
						AddUpAccountPeriod(pBalanceAccount->pStructDocTop, pBalanceAccount, pBalanceAccount->pListAnnual,	pAccountMonth->End, pAccountMonth->Outcome, pAccountMonth->OutcomeQuantity, pAccountMonth->Budget, pAccountMonth->BudgetQuantity, ANNUALPERIOD);
						pAccountMonth = LListGetNextRecordData(pBalanceAccount->pListMonth);
					}
				}
				pBalanceAccount = LListGetNextRecordData(pBalanceReport->pListBalanceAccount);
			}
			pBalanceReport = LListGetNextRecordData(pLedger->pListReport);
		}
	}
}

/********************************************************************

static void ResetReportsOfLedger(

*********************************************************************/
static void ResetReportsOfLedger(
	LedgerType *pLedger)
{
	BalanceReportType *pBalanceReport = LListGetFirstRecordData(pLedger->pListReport);
	while (pBalanceReport) {
		BalanceAccountType *pBalanceAccount = LListGetFirstRecordData(pBalanceReport->pListBalanceAccount);
		while (pBalanceAccount) {
			BalanceAccountPeriodType *pAccountPeriod = LListGetFirstRecordData(pBalanceAccount->pListMonth);
			while (pAccountPeriod) {
				pAccountPeriod->Outcome = 0;
				pAccountPeriod->OutcomeQuantity = 0;
				pAccountPeriod = LListGetNextRecordData(pBalanceAccount->pListMonth);
			}
			pAccountPeriod = LListGetFirstRecordData(pBalanceAccount->pListAnnual);
			while (pAccountPeriod) {
				pAccountPeriod->Outcome = 0;
				pAccountPeriod->OutcomeQuantity = 0;
				pAccountPeriod = LListGetNextRecordData(pBalanceAccount->pListAnnual);
			}
			pAccountPeriod = LListGetFirstRecordData(pBalanceAccount->pListSemi);
			while (pAccountPeriod) {
				pAccountPeriod->Outcome = 0;
				pAccountPeriod->OutcomeQuantity = 0;
				pAccountPeriod = LListGetNextRecordData(pBalanceAccount->pListSemi);
			}
			pAccountPeriod = LListGetFirstRecordData(pBalanceAccount->pListTertial);
			while (pAccountPeriod) {
				pAccountPeriod->Outcome = 0;
				pAccountPeriod->OutcomeQuantity = 0;
				pAccountPeriod = LListGetNextRecordData(pBalanceAccount->pListTertial);
			}
			pAccountPeriod = LListGetFirstRecordData(pBalanceAccount->pListQuarter);
			while (pAccountPeriod) {
				pAccountPeriod->Outcome = 0;
				pAccountPeriod->OutcomeQuantity = 0;
				pAccountPeriod = LListGetNextRecordData(pBalanceAccount->pListQuarter);
			}
			if ((!strcmp(pBalanceAccount->pAccount->pType, ACCOUNT_TYPE_TANGIBLE)) ||
				(!strcmp(pBalanceAccount->pAccount->pType, ACCOUNT_TYPE_STOCK)) ||
				(!strcmp(pBalanceAccount->pAccount->pType, ACCOUNT_TYPE_RECEIVABLES)) ||
				(!strcmp(pBalanceAccount->pAccount->pType, ACCOUNT_TYPE_ASSETS)) ||
				(!strcmp(pBalanceAccount->pAccount->pType, ACCOUNT_TYPE_EQUITIES)) ||
				(!strcmp(pBalanceAccount->pAccount->pType, ACCOUNT_TYPE_PROVISIONS)) ||
				(!strcmp(pBalanceAccount->pAccount->pType, ACCOUNT_TYPE_LIABILITIES))) {
				pBalanceAccount->Forward = pBalanceAccount->Opening;
				pBalanceAccount->ForwardQuantity = pBalanceAccount->OpeningQuantity;
			}
			else {
				pBalanceAccount->Opening = 0;
				pBalanceAccount->OpeningQuantity = 0;
				pBalanceAccount->Forward = 0;
				pBalanceAccount->ForwardQuantity = 0;
			}
			pBalanceAccount = LListGetNextRecordData(pBalanceReport->pListBalanceAccount);
		}
		pBalanceReport = LListGetNextRecordData(pLedger->pListReport);
	}
}

/********************************************************************

static BalanceAccountPeriodType *MergeBalanceAccountQuarterPeriods(

*********************************************************************/
static BalanceAccountPeriodType *MergeBalanceAccountQuarterPeriods(
	AccountingType *pAccounting,
	LedgerType *pLedgerMatch,
	BalanceReportType *pBalanceReport,
	BalanceAccountType *pBalanceAccount,
	BalanceAccountPeriodType *pBalanceAccountPeriod,
	LLIST pListPeriod, char *pPeriodType)
{
	BalanceReportType *pBalanceReportMatch = GetBalanceReport(
		pLedgerMatch,
		pBalanceReport->pObject);
	if (!pBalanceReportMatch) {
		pBalanceReportMatch = CreateBalanceReport(pAccounting, pLedgerMatch);
		pBalanceReportMatch->pObject = pBalanceReport->pObject;
		pBalanceReportMatch = SearchSetBalanceReport(pLedgerMatch->pListReport, pBalanceReportMatch);
	}
	{
		BalanceAccountType *pBalanceAccountMatch = GetBalanceAccount(
			pBalanceReportMatch,
			pBalanceAccount->pAccount);
		if (!pBalanceAccountMatch) {
			pBalanceAccountMatch = CreateBalanceAccount(pAccounting, pBalanceReportMatch);
			pBalanceAccountMatch->pAccount = pBalanceAccount->pAccount;
			pBalanceAccountMatch = SearchSetBalanceAccount(pBalanceReportMatch->pListBalanceAccount, pBalanceAccountMatch);
		}
		{
			BalanceAccountPeriodType *pBalanceAccountPeriodMatch =
				GetBalanceAccountPeriod(pBalanceAccountMatch,
					pBalanceAccountPeriod->End,
					pPeriodType);
			if (pBalanceAccountPeriodMatch) {
				pBalanceAccountPeriodMatch->Budget += pBalanceAccountPeriod->Budget;
				pBalanceAccountPeriodMatch->BudgetQuantity += pBalanceAccountPeriod->BudgetQuantity;
				pBalanceAccountPeriodMatch->Outcome += pBalanceAccountPeriod->Outcome;
				pBalanceAccountPeriodMatch->OutcomeQuantity += pBalanceAccountPeriod->OutcomeQuantity;
				pBalanceAccountPeriod->Budget = pBalanceAccountPeriodMatch->Budget;
				pBalanceAccountPeriod->BudgetQuantity = pBalanceAccountPeriodMatch->BudgetQuantity;
				pBalanceAccountPeriod->Outcome = pBalanceAccountPeriodMatch->Outcome;
				pBalanceAccountPeriod->OutcomeQuantity = pBalanceAccountPeriodMatch->OutcomeQuantity;
			}
			else {
				pBalanceAccountPeriodMatch = CopyBalanceAccountPeriod(pAccounting, pBalanceAccountMatch, pBalanceAccountPeriod);
				pBalanceAccountPeriodMatch = SearchSetBalanceAccountPeriodDynamic(pBalanceAccountMatch, pBalanceAccountPeriodMatch, pPeriodType);
			}
			return pBalanceAccountPeriodMatch;
		}
	}
}

/********************************************************************

static void QuarterPeriodBorderLineHandling(

*********************************************************************/
static void QuarterPeriodBorderLineHandling(
	AccountingType *pAccounting)
{
	if (pAccounting) {
		LedgerType *pLedger
			= LListGetLastRecordData(pAccounting->pLedgersChart->pListLedger);
		if (pLedger && pLedger->Id) { // This is only an issue with multiple Ledgers
			while (pLedger && pLedger->Id) { // Not Ledger 0 == This year
				LedgerType *pLedgerMatch = GetLedger(pAccounting, pLedger->Id + 1);
				if (!pLedgerMatch)
					return; //Nonsense
				{
					BalanceReportType *pBalanceReport
						= LListGetFirstRecordData(pLedger->pListReport);
					while (pBalanceReport) {
						BalanceAccountType *pBalanceAccount
							= LListGetFirstRecordData(pBalanceReport->pListBalanceAccount);
						while (pBalanceAccount) {
							BalanceAccountPeriodType *pBalanceAccountPeriodMatch = 0;
							{
								BalanceAccountPeriodType *pBalanceAccountPeriod
									= LListGetFirstRecordData(pBalanceAccount->pListAnnual);
								while (pBalanceAccountPeriod) {
									if (pLedgerMatch->YearStart < pBalanceAccountPeriod->End)
										pBalanceAccountPeriodMatch = MergeBalanceAccountQuarterPeriods(pAccounting, pLedgerMatch, pBalanceReport, pBalanceAccount, pBalanceAccountPeriod, pBalanceAccount->pListAnnual, "A ");
									pBalanceAccountPeriod = LListGetNextRecordData(pBalanceAccount->pListAnnual);
								}
							}
							{
								BalanceAccountPeriodType *pBalanceAccountPeriod
									= LListGetFirstRecordData(pBalanceAccount->pListSemi);
								while (pBalanceAccountPeriod) {
									if (pLedgerMatch->YearStart < pBalanceAccountPeriod->End)
										pBalanceAccountPeriodMatch = MergeBalanceAccountQuarterPeriods(pAccounting, pLedgerMatch, pBalanceReport, pBalanceAccount, pBalanceAccountPeriod, pBalanceAccount->pListSemi, "S ");
									pBalanceAccountPeriod = LListGetNextRecordData(pBalanceAccount->pListSemi);
								}
							}
							{
								BalanceAccountPeriodType *pBalanceAccountPeriod
									= LListGetFirstRecordData(pBalanceAccount->pListTertial);
								while (pBalanceAccountPeriod) {
									if (pLedgerMatch->YearStart < pBalanceAccountPeriod->End)
										pBalanceAccountPeriodMatch = MergeBalanceAccountQuarterPeriods(pAccounting, pLedgerMatch, pBalanceReport, pBalanceAccount, pBalanceAccountPeriod, pBalanceAccount->pListTertial, "T ");
									pBalanceAccountPeriod = LListGetNextRecordData(pBalanceAccount->pListTertial);
								}
							}
							{
								BalanceAccountPeriodType *pBalanceAccountPeriod
									= LListGetFirstRecordData(pBalanceAccount->pListQuarter);
								while (pBalanceAccountPeriod) {
									if (pLedgerMatch->YearStart < pBalanceAccountPeriod->End)
										pBalanceAccountPeriodMatch = MergeBalanceAccountQuarterPeriods(pAccounting, pLedgerMatch, pBalanceReport, pBalanceAccount, pBalanceAccountPeriod, pBalanceAccount->pListQuarter, "Q ");
									pBalanceAccountPeriod = LListGetNextRecordData(pBalanceAccount->pListQuarter);
								}
							}
							pBalanceAccount = LListGetNextRecordData(pBalanceReport->pListBalanceAccount);
						}
						pBalanceReport = LListGetNextRecordData(pLedger->pListReport);
					}
				}
				pLedger = pLedgerMatch;
			}
		}
	}
}

/********************************************************************

void SumUpAccounting(

*********************************************************************/
void SumUpAccounting(
	AccountingType *pAccounting)
{
	if (pAccounting) {
		LedgerType *pLedger
			= LListGetLastRecordData(pAccounting->pLedgersChart->pListLedger);
		while (pLedger) {
			ResetQuarters(pLedger);
			if (LListGetFirstRecordData(pLedger->pListVoucher)) {
				VoucherType *pVoucher
					= LListGetFirstRecordData(pLedger->pListVoucher);
				ResetReportsOfLedger(pLedger);
				while (pVoucher) {
					VoucherEntryType *pVoucherEntry
						= LListGetFirstRecordData(pVoucher->pListEntry);
					while (pVoucherEntry) {
						if (!pVoucherEntry->pDeprecated) {
							ObjectType *pObject = LListGetFirstRecordData(pVoucherEntry->pListObject);
							while (pObject) {
								AddUpBalanceReport(pLedger, pObject, pVoucherEntry->pAccount, pVoucher->Date, pVoucherEntry->Amount, pVoucherEntry->Quantity);
								pObject = LListGetNextRecordData(pVoucherEntry->pListObject);
							}
							{
								DimensionType *pDimension = CreateDimension(pAccounting, pAccounting->pObjectsChart);
								pDimension->Id = 0;
								pDimension = SearchSetDimension(pAccounting->pObjectsChart->pListDimension, pDimension);
								pObject = CreateObject(pAccounting, pDimension);
								CopyString(&pObject->pId, "0");
								pObject->pDimension = pDimension;
								pObject = SearchSetObject(pAccounting->pObjectsChart->pListDimension, pObject);
								AddUpBalanceReport(pLedger, pObject, pVoucherEntry->pAccount, pVoucher->Date, pVoucherEntry->Amount, pVoucherEntry->Quantity);
							}
						}
						pVoucherEntry = LListGetNextRecordData(pVoucher->pListEntry);
					}
					pVoucher = LListGetNextRecordData(pLedger->pListVoucher);
				}
			}
			AddUpBalanceQuarterPeriods(pLedger);
			pLedger = LListGetPreviousRecordData(pAccounting->pLedgersChart->pListLedger);
		}
		QuarterPeriodBorderLineHandling(pAccounting);
	}
}

/********************************************************************

static void SumUpBalanceTotalPeriods(

*********************************************************************/
static void SumUpBalanceTotalPeriods(
	LLIST pListpBalanceTotalPeriod,
	LLIST pBalanceTotalSectionListMonth,
	void *vpParent,
	char *pPeriodLabel)
{
	if (pListpBalanceTotalPeriod && pBalanceTotalSectionListMonth && vpParent) {
		BalanceTotalPeriodType *pBalanceTotalSectionMonth
			= LListGetFirstRecordData(pBalanceTotalSectionListMonth);
		while (pBalanceTotalSectionMonth) {
			BalanceTotalPeriodType *pBalanceTotalPeriod = CreateBalanceTotalPeriod(pBalanceTotalSectionMonth->pStructDocTop, vpParent);
			pBalanceTotalPeriod->End = GetLastDateInPeriod(pBalanceTotalSectionMonth->End, pPeriodLabel);
			pBalanceTotalPeriod = SearchSetBalanceTotalPeriod(pListpBalanceTotalPeriod, pBalanceTotalPeriod);
			pBalanceTotalPeriod->Budget += pBalanceTotalSectionMonth->Budget;
			pBalanceTotalPeriod->Outcome += pBalanceTotalSectionMonth->Outcome;
			pBalanceTotalSectionMonth = LListGetNextRecordData(pBalanceTotalSectionListMonth);
		}
	}
}

/********************************************************************

static void SumUpBalanceTotalGroupSection(

*********************************************************************/
static void SumUpBalanceTotalGroupSection(
	BalanceTotalType **ppBalanceTotalGroup,
	BalanceTotalType *pBalanceTotalSection,
	void *vpParent)
{
	if (!pBalanceTotalSection)
		return;
	if (!*ppBalanceTotalGroup)
		*ppBalanceTotalGroup = CreateBalanceTotal(pBalanceTotalSection->pStructDocTop, vpParent);

	(*ppBalanceTotalGroup)->Opening += pBalanceTotalSection->Opening;
	(*ppBalanceTotalGroup)->Forward += pBalanceTotalSection->Forward;
	SumUpBalanceTotalPeriods((*ppBalanceTotalGroup)->pListMonth, pBalanceTotalSection->pListMonth, *ppBalanceTotalGroup, MONTHPERIOD);
	SumUpBalanceTotalPeriods((*ppBalanceTotalGroup)->pListQuarter, pBalanceTotalSection->pListQuarter, *ppBalanceTotalGroup, QUARTERPERIOD);
	SumUpBalanceTotalPeriods((*ppBalanceTotalGroup)->pListTertial, pBalanceTotalSection->pListTertial, *ppBalanceTotalGroup, TERTIALPERIOD);
	SumUpBalanceTotalPeriods((*ppBalanceTotalGroup)->pListSemi, pBalanceTotalSection->pListSemi, *ppBalanceTotalGroup, SEMIPERIOD);
	SumUpBalanceTotalPeriods((*ppBalanceTotalGroup)->pListAnnual, pBalanceTotalSection->pListAnnual, *ppBalanceTotalGroup, ANNUALPERIOD);
}

/********************************************************************

static void SumUpBalanceTotalSectionPeriods(

*********************************************************************/
static void SumUpBalanceTotalSectionPeriods(
	LLIST pListBalanceTotalSectionMonth,
	LLIST pBalanceAccountListMonth,
	void *vpParent,
	char *pPeriodLabel)
{
	BalanceTotalType *pBalanceTotal = vpParent;
	if (pListBalanceTotalSectionMonth && pBalanceAccountListMonth && pBalanceTotal) {
		BalanceAccountPeriodType *pBalanceAccountPeriod
			= LListGetFirstRecordData(pBalanceAccountListMonth);
		while (pBalanceAccountPeriod) {
			//BalanceAccountType *pBalanceAccount = pBalanceAccountPeriod->pStructParent;
			//char *pAccountId = pBalanceAccount->pAccount->pId;
			BalanceTotalPeriodType *BalanceTotalSectionMonth = CreateBalanceTotalPeriod(pBalanceAccountPeriod->pStructDocTop, pBalanceTotal);
			BalanceTotalSectionMonth->End = GetLastDateInPeriod(pBalanceAccountPeriod->End, pPeriodLabel);
			BalanceTotalSectionMonth = SearchSetBalanceTotalPeriod(pListBalanceTotalSectionMonth, BalanceTotalSectionMonth);
			BalanceTotalSectionMonth->Budget += pBalanceAccountPeriod->Budget;
			BalanceTotalSectionMonth->Outcome += pBalanceAccountPeriod->Outcome;
			pBalanceAccountPeriod = LListGetNextRecordData(pBalanceAccountListMonth);
		}
	}
}

/********************************************************************

static void SumUpBalanceTotalSection(

*********************************************************************/
static void SumUpBalanceTotalSection(
	BalanceTotalType **ppBalanceTotalSection,
	BalanceAccountType *pBalanceAccount,
	void *vpParent)
{
	if (!pBalanceAccount)
		return;
	if (!*ppBalanceTotalSection)
		*ppBalanceTotalSection = CreateBalanceTotal(pBalanceAccount->pStructDocTop, vpParent);

	if ((!strcmp(pBalanceAccount->pAccount->pType, ACCOUNT_TYPE_TANGIBLE)) ||
		(!strcmp(pBalanceAccount->pAccount->pType, ACCOUNT_TYPE_STOCK)) ||
		(!strcmp(pBalanceAccount->pAccount->pType, ACCOUNT_TYPE_RECEIVABLES)) ||
		(!strcmp(pBalanceAccount->pAccount->pType, ACCOUNT_TYPE_ASSETS)) ||
		(!strcmp(pBalanceAccount->pAccount->pType, ACCOUNT_TYPE_EQUITIES)) ||
		(!strcmp(pBalanceAccount->pAccount->pType, ACCOUNT_TYPE_PROVISIONS)) ||
		(!strcmp(pBalanceAccount->pAccount->pType, ACCOUNT_TYPE_LIABILITIES)))
		(*ppBalanceTotalSection)->Opening += pBalanceAccount->Opening;
	(*ppBalanceTotalSection)->Forward += pBalanceAccount->Forward;
	SumUpBalanceTotalSectionPeriods((*ppBalanceTotalSection)->pListMonth, pBalanceAccount->pListMonth, (*ppBalanceTotalSection), MONTHPERIOD);
	SumUpBalanceTotalSectionPeriods((*ppBalanceTotalSection)->pListQuarter, pBalanceAccount->pListQuarter, *ppBalanceTotalSection, QUARTERPERIOD);
	SumUpBalanceTotalSectionPeriods((*ppBalanceTotalSection)->pListTertial, pBalanceAccount->pListTertial, *ppBalanceTotalSection, TERTIALPERIOD);
	SumUpBalanceTotalSectionPeriods((*ppBalanceTotalSection)->pListSemi, pBalanceAccount->pListSemi, *ppBalanceTotalSection, SEMIPERIOD);
	SumUpBalanceTotalSectionPeriods((*ppBalanceTotalSection)->pListAnnual, pBalanceAccount->pListAnnual, *ppBalanceTotalSection, ANNUALPERIOD);
}


/********************************************************************

static void SumUpBalanceTotalSectionType(

*********************************************************************/
static void SumUpBalanceTotalSectionType(
	char *pType,
	BalanceReportType *pBalanceReport,
	BalanceAccountType *pBalanceAccount)
{
	if ((!pBalanceReport) || (!pBalanceAccount))
		return;

	if (!strcmp(pType, ACCOUNT_TYPE_TANGIBLE))
		SumUpBalanceTotalSection(&pBalanceReport->pTangible, pBalanceAccount, pBalanceReport);
	else if (!strcmp(pType, ACCOUNT_TYPE_STOCK))
		SumUpBalanceTotalSection(&pBalanceReport->pStock, pBalanceAccount, pBalanceReport);
	else if (!strcmp(pType, ACCOUNT_TYPE_RECEIVABLES))
		SumUpBalanceTotalSection(&pBalanceReport->pReceivables, pBalanceAccount, pBalanceReport);
	else if (!strcmp(pType, ACCOUNT_TYPE_ASSETS))
		SumUpBalanceTotalSection(&pBalanceReport->pAssets, pBalanceAccount, pBalanceReport);
	else if (!strcmp(pType, ACCOUNT_TYPE_EQUITIES))
		SumUpBalanceTotalSection(&pBalanceReport->pEquities, pBalanceAccount, pBalanceReport);
	else if (!strcmp(pType, ACCOUNT_TYPE_PROVISIONS))
		SumUpBalanceTotalSection(&pBalanceReport->pProvisions, pBalanceAccount, pBalanceReport);
	else if (!strcmp(pType, ACCOUNT_TYPE_LIABILITIES))
		SumUpBalanceTotalSection(&pBalanceReport->pLiabilities, pBalanceAccount, pBalanceReport);
	else if (!strcmp(pType, ACCOUNT_TYPE_REVENUES))
		SumUpBalanceTotalSection(&pBalanceReport->pRevenues, pBalanceAccount, pBalanceReport);
	else if (!strcmp(pType, ACCOUNT_TYPE_SUPPLIES))
		SumUpBalanceTotalSection(&pBalanceReport->pSupplies, pBalanceAccount, pBalanceReport);
	else if (!strcmp(pType, ACCOUNT_TYPE_EXPENSES))
		SumUpBalanceTotalSection(&pBalanceReport->pExpenses, pBalanceAccount, pBalanceReport);
	else if (!strcmp(pType, ACCOUNT_TYPE_STAFF))
		SumUpBalanceTotalSection(&pBalanceReport->pStaff, pBalanceAccount, pBalanceReport);
	else if (!strcmp(pType, ACCOUNT_TYPE_LOSS_GAINS))
		SumUpBalanceTotalSection(&pBalanceReport->pLossGains, pBalanceAccount, pBalanceReport);
	else if (!strcmp(pType, ACCOUNT_TYPE_FINANCIAL))
		SumUpBalanceTotalSection(&pBalanceReport->pFinancial, pBalanceAccount, pBalanceReport);
	else if (!strcmp(pType, ACCOUNT_TYPE_AUX))
		SumUpBalanceTotalSection(&pBalanceReport->pAuxiliary, pBalanceAccount, pBalanceReport);
}

/********************************************************************

void SumUpBalanceTotals(

*********************************************************************/
void SumUpBalanceTotals(
	AccountingType *pAccounting)
{
	if (pAccounting) {
		LedgerType *pLedger
			= LListGetLastRecordData(pAccounting->pLedgersChart->pListLedger);
		while (pLedger) {
			ResetQuarters(pLedger);
			pLedger = LListGetPreviousRecordData(pAccounting->pLedgersChart->pListLedger);
		}
		pLedger
			= LListGetLastRecordData(pAccounting->pLedgersChart->pListLedger);
		while (pLedger) {
			AddUpBalanceQuarterPeriods(pLedger);
			pLedger = LListGetPreviousRecordData(pAccounting->pLedgersChart->pListLedger);
		}
		QuarterPeriodBorderLineHandling(pAccounting);
	}

	if (pAccounting) {
		LedgerType *pLedger
			= LListGetFirstRecordData(pAccounting->pLedgersChart->pListLedger);
		while (pLedger) {
			BalanceReportType *pBalanceReport
				= LListGetFirstRecordData(pLedger->pListReport);
			while (pBalanceReport) {
				DestroyBalanceTotal(&pBalanceReport->pTangible);
				DestroyBalanceTotal(&pBalanceReport->pStock);
				DestroyBalanceTotal(&pBalanceReport->pReceivables);
				DestroyBalanceTotal(&pBalanceReport->pAssets);
				DestroyBalanceTotal(&pBalanceReport->pEquities);
				DestroyBalanceTotal(&pBalanceReport->pProvisions);
				DestroyBalanceTotal(&pBalanceReport->pLiabilities);
				DestroyBalanceTotal(&pBalanceReport->pRevenues);
				DestroyBalanceTotal(&pBalanceReport->pSupplies);
				DestroyBalanceTotal(&pBalanceReport->pExpenses);
				DestroyBalanceTotal(&pBalanceReport->pStaff);
				DestroyBalanceTotal(&pBalanceReport->pLossGains);
				DestroyBalanceTotal(&pBalanceReport->pFinancial);
				DestroyBalanceTotal(&pBalanceReport->pAuxiliary);
				DestroyBalanceTotal(&pBalanceReport->pActive);
				DestroyBalanceTotal(&pBalanceReport->pPassive);
				DestroyBalanceTotal(&pBalanceReport->pBalance);
				DestroyBalanceTotal(&pBalanceReport->pProfit);
				DestroyBalanceTotal(&pBalanceReport->pEarning);
				DestroyBalanceTotal(&pBalanceReport->pOutcome);
				DestroyBalanceTotal(&pBalanceReport->pResult);
				DestroyBalanceTotal(&pBalanceReport->pTotal);
				{
					BalanceAccountType *pBalanceAccount
						= LListGetFirstRecordData(pBalanceReport->pListBalanceAccount);
					while (pBalanceAccount) {
						SumUpBalanceTotalSectionType(pBalanceAccount->pAccount->pType, pBalanceReport, pBalanceAccount);
						pBalanceAccount = LListGetNextRecordData(pBalanceReport->pListBalanceAccount);
					}
					SumUpBalanceTotalGroupSection(&pBalanceReport->pActive, pBalanceReport->pTangible, pBalanceReport);
					SumUpBalanceTotalGroupSection(&pBalanceReport->pActive, pBalanceReport->pStock, pBalanceReport);
					SumUpBalanceTotalGroupSection(&pBalanceReport->pActive, pBalanceReport->pReceivables, pBalanceReport);
					SumUpBalanceTotalGroupSection(&pBalanceReport->pActive, pBalanceReport->pAssets, pBalanceReport);
					SumUpBalanceTotalGroupSection(&pBalanceReport->pBalance, pBalanceReport->pActive, pBalanceReport);
					SumUpBalanceTotalGroupSection(&pBalanceReport->pPassive, pBalanceReport->pEquities, pBalanceReport);
					SumUpBalanceTotalGroupSection(&pBalanceReport->pPassive, pBalanceReport->pProvisions, pBalanceReport);
					SumUpBalanceTotalGroupSection(&pBalanceReport->pPassive, pBalanceReport->pLiabilities, pBalanceReport);
					SumUpBalanceTotalGroupSection(&pBalanceReport->pBalance, pBalanceReport->pPassive, pBalanceReport);
					SumUpBalanceTotalGroupSection(&pBalanceReport->pTotal, pBalanceReport->pBalance, pBalanceReport);
					SumUpBalanceTotalGroupSection(&pBalanceReport->pProfit, pBalanceReport->pRevenues, pBalanceReport);
					SumUpBalanceTotalGroupSection(&pBalanceReport->pProfit, pBalanceReport->pSupplies, pBalanceReport);
					SumUpBalanceTotalGroupSection(&pBalanceReport->pEarning, pBalanceReport->pProfit, pBalanceReport);
					SumUpBalanceTotalGroupSection(&pBalanceReport->pEarning, pBalanceReport->pExpenses, pBalanceReport);
					SumUpBalanceTotalGroupSection(&pBalanceReport->pEarning, pBalanceReport->pStaff, pBalanceReport);
					SumUpBalanceTotalGroupSection(&pBalanceReport->pOutcome, pBalanceReport->pEarning, pBalanceReport);
					SumUpBalanceTotalGroupSection(&pBalanceReport->pOutcome, pBalanceReport->pLossGains, pBalanceReport);
					SumUpBalanceTotalGroupSection(&pBalanceReport->pOutcome, pBalanceReport->pFinancial, pBalanceReport);
					SumUpBalanceTotalGroupSection(&pBalanceReport->pResult, pBalanceReport->pOutcome, pBalanceReport);
					SumUpBalanceTotalGroupSection(&pBalanceReport->pResult, pBalanceReport->pAuxiliary, pBalanceReport);
					SumUpBalanceTotalGroupSection(&pBalanceReport->pTotal, pBalanceReport->pResult, pBalanceReport);
				}
				pBalanceReport = LListGetNextRecordData(pLedger->pListReport);
			}
			pLedger = LListGetNextRecordData(pAccounting->pLedgersChart->pListLedger);
		}
	}
}

/********************************************************************

Search and get Data structures etc

*********************************************************************/
/********************************************************************

LLIST AddTextListItem(

*********************************************************************/
LLIST AddTextListItem(
	void *pTopStruct,
	void *pParentStruct,
	LLIST pListText,
	char *pString,
	char *plang)
{
	if (!pListText)
		pListText = LListInit();

	{
		TextType *pText = AssembleText(pTopStruct, pParentStruct, pString, plang);
		if (pText) {
			if (RET_OK != LListAddRecord(pListText, pText)) {
				// Error, report it
				return 0; // Fatal Error
			}
		}
		else
			return 0; // Fatal Error
	}

	return pListText;
}

/********************************************************************

char *GetTextListText(

*********************************************************************/
char *GetTextListText(
	LLIST pListText,
	char *pLanguage)
{
	pLanguage = ProgramTextsLanguageSupport(pLanguage);
	if (pListText) {
		TextType *pText
			= LListGetFirstRecordData(pListText);
		if (pText) {
			while (pText) {
				// If none with language is a match 
				if (((!pLanguage) || !(*pLanguage)) && ((!pText->plang) || !(*pText->plang)))
					return pText->pText;
				// If both with same language is a match 
				if ((pLanguage && *pLanguage) && (!strcmp(pText->plang, pLanguage)))
					return pText->pText;

				pText = LListGetNextRecordData(pListText);
			}
			// No match go for plang==""
			pText
				= LListGetFirstRecordData(pListText);
			while (pText) {
				if ((!pText->plang) || !(*pText->plang))
					return pText->pText;
				pText = LListGetNextRecordData(pListText);
			}
			// No match go for plang=="en"
			pText
				= LListGetFirstRecordData(pListText);
			while (pText) {
				if ((pText->plang && *pText->plang) && (!strcmp(pText->plang, "en")))
					return pText->pText;
				pText = LListGetNextRecordData(pListText);
			}
			// If no text without attribute we try what we can get any
			pText = LListGetFirstRecordData(pListText);
			if (pText)
				return pText->pText;
		}
	}
	// If none empty string is retured
	return NulString;
}

/********************************************************************

TextType *AssembleText(

*********************************************************************/
TextType *AssembleText(
	void *pTopStruct,
	void *pParentStruct,
	char *pTextString,
	char *pLang)
{
	// Not Valid without pTextString
	if (!pTextString)
		return 0;
	{
		TextType *pText = CreateText(pTopStruct, pParentStruct);
		CopyString(&pText->pText, pTextString);
		CopyString(&pText->plang, pLang);
		return pText;
	}
}

/********************************************************************

SeriesIdType *AseembleSeriesId(

*********************************************************************/
SeriesIdType *AseembleSeriesId(
	void *pTopStruct,
	void *pParentStruct,
	char *pSeries,
	COUNTER pId)
{
	// Valid without pSeries
	SeriesIdType *pSeriesId = CreateSeriesId(pTopStruct, pParentStruct);
	if (pSeries && *pSeries)
		CopyString(&pSeriesId->pseries, pSeries);
	pSeriesId->SeriesId = pId;
	return pSeriesId;
}

/********************************************************************

OperatorInfoType *AseembleOperatorInfo(

*********************************************************************/
OperatorInfoType *AseembleOperatorInfo(
	void *pTopStruct,
	void *pParentStruct,
	char *pOperatorInfoString,
	DATEINT Date)
{
	// Valid without pOperatorInfoString
	OperatorInfoType *pOperatorInfo = CreateOperatorInfo(pTopStruct, pParentStruct);
	if (pOperatorInfoString && *pOperatorInfoString)
		CopyString(&pOperatorInfo->pOperatorInfo, pOperatorInfoString);
	pOperatorInfo->date = Date;
	return pOperatorInfo;
}

/********************************************************************

char *GetDeprecatedText(

*********************************************************************/
char *GetDeprecatedText(
	OperatorInfoType *pDeprecatedInfo,
	char *pTitle,
	char *pDeprecatedText,
	size_t DeprecatedTextLength)
{
	if (pDeprecatedText && DeprecatedTextLength) {
		memset(pDeprecatedText, '\0', DeprecatedTextLength);
		if (pDeprecatedInfo) {
			if (pDeprecatedInfo->pOperatorInfo && *pDeprecatedInfo->pOperatorInfo) {
				strncat(pDeprecatedText, pDeprecatedInfo->pOperatorInfo, (DeprecatedTextLength - strlen(pDeprecatedText)) - 1);
				strncat(pDeprecatedText, " ", (DeprecatedTextLength - strlen(pDeprecatedText)) - 1);
			}
			DateToStr(pDeprecatedInfo->date, &pDeprecatedText[strlen(pDeprecatedText)]);
			strncat(pDeprecatedText, ": ", DeprecatedTextLength - strlen(pDeprecatedText) - 1);
		}
		if (pTitle && *pTitle)
			strncat(pDeprecatedText, pTitle, DeprecatedTextLength - strlen(pDeprecatedText) - 1);
	}
	return pDeprecatedText;
}

/********************************************************************

AccountCodeType *AseembleAccountCode(

*********************************************************************/
AccountCodeType *AseembleAccountCode(
	void *pTopStruct,
	void *pParentStruct,
	char *pCodeType,
	char *pIssuer,
	char *pApplicableDomain,
	char *pValidYear,
	char *pCode,
	char *pDescription,
	char *pDescriptionLang)
{
	// Not Valid without pCodeType
	if (pCodeType) {
		AccountCodeType *pAccountCode = CreateAccountCode(pTopStruct, pParentStruct);
		CopyString(&pAccountCode->pCodeType, pCodeType);
		CopyString(&pAccountCode->pIssuer, pIssuer);
		if (pApplicableDomain && *pApplicableDomain)
			SearchSetToken(pAccountCode->pListApplicableDomain, CopyString(0, pApplicableDomain));
		CopyString(&pAccountCode->pValidYear, pValidYear);
		CopyString(&pAccountCode->pCode, pCode);
		SearchSetText(pAccountCode->pListDescription, AssembleText(pTopStruct, pAccountCode, CopyString(0, pDescription), CopyString(0, pDescriptionLang)));
		return pAccountCode;
	}
	return 0;
}

/********************************************************************

TextType *SearchSetText(

*********************************************************************/
TextType *SearchSetText(
	LLIST pListText,
	TextType *pSearchText)
{
	if (pSearchText) {
		void *pCursor = pListText->CursorRecord;
		TextType *pText
			= LListGetFirstRecordData(pListText);
		while (pText) {
			if (pSearchText == pText) {
				pListText->CursorRecord = pCursor;
				return pText;
			}
			if ((((!pText->plang) || (!*pText->plang)) && ((!pSearchText->plang) || (!*pSearchText->plang)))
				|| (pText->plang && pSearchText->plang && (!strcmp(pText->plang, pSearchText->plang)))) {
				if ((((!pText->pText) || (!*pText->pText)) && ((!pSearchText->pText) || (!*pSearchText->pText)))
					|| (pText->pText && pSearchText->pText && (!strcmp(pText->pText, pSearchText->pText)))) { // No change of text
					DestroyText(&pSearchText);
					pListText->CursorRecord = pCursor;
					return pText;
				}
				else {
					DestroyText(&pText);
					pListText->CursorRecord->vpData = pSearchText;
					pListText->CursorRecord = pCursor;
					return pSearchText;
				}
			}
			pText = LListGetNextRecordData(pListText);
		}
		pListText->CursorRecord = pCursor;
		LListAddRecord(pListText, pSearchText);
		return pSearchText;
	}
	return 0;

}

/********************************************************************

char *SearchSetToken(

*********************************************************************/
char *SearchSetToken(
	LLIST pListToken,
	char *pSearchToken)
{
	if (pSearchToken && *pSearchToken) {
		void *pCursor = pListToken->CursorRecord;
		char *pToken
			= LListGetFirstRecordData(pListToken);
		while (pToken) {
			if (pSearchToken == pToken) {
				pListToken->CursorRecord = pCursor;
				return pToken;
			}
			if (!strcmp(pSearchToken, pToken)) {
				pListToken->CursorRecord = pCursor;
				FreeString(&pSearchToken);
				return pToken;
			}
			pToken = LListGetNextRecordData(pListToken);
		}
		pListToken->CursorRecord = pCursor;
		LListAddRecord(pListToken, pSearchToken);
		return pSearchToken;
	}
	return 0;
}

/********************************************************************

void *GetEntity(

*********************************************************************/
void *GetEntity(
	void *vpAccounting)
{
	return ((AccountingType *)vpAccounting)->pEntity;
}

/********************************************************************

static ContactType *GetContact(

*********************************************************************/
static ContactType *GetContact(
	void *vpParty,
	char *pName)
{
	PartyType *pParty = vpParty;
	LLISTITEMTYPE *CursorRecord = pParty->pListContact->CursorRecord;

	if (pName) {
		ContactType *pContact
			= LListGetFirstRecordData(pParty->pListContact);
		while (pContact) {
			if ((!pContact->pName)
				|| (!strcmp(pName, pContact->pName))) {
				pParty->pListContact->CursorRecord = CursorRecord;
				return pContact;
			}
			pContact = LListGetNextRecordData(pParty->pListContact);
		}
		pParty->pListContact->CursorRecord = CursorRecord;
	}
	return 0;
}

/********************************************************************

ContactType *AddContact(

*********************************************************************/
ContactType *AddContact(
	void *vpParty,
	char *pName)
{
	PartyType *pParty = vpParty;
	ContactType *pContact = GetContact(pParty, pName);
	if (!pContact) {
		pContact = CreateContact(pParty->pStructDocTop, pParty);
		if (!pContact)
			return 0; // Fatal Error
		if (pName)
			CopyString(&pContact->pName, pName);
		else
			pContact->pName = NulString;
		LListAddRecord(pParty->pListContact, pContact);
	}
	return pContact;
}

/********************************************************************

AccountType *GetAccount(

*********************************************************************/
AccountType *GetAccount(
	void *vpAccounting,
	char *pSearchAccountId)
{
	AccountingType *pAccounting = (AccountingType *)vpAccounting;
	LLISTITEMTYPE *CursorRecord = pAccounting->pAccountsChart->pListAccount->CursorRecord;

	if (pSearchAccountId) {
		AccountType *pAccount
			= LListGetFirstRecordData(pAccounting->pAccountsChart->pListAccount);
		while (pAccount) {
			if (!strcmp(pSearchAccountId, pAccount->pId)) {
				pAccounting->pAccountsChart->pListAccount->CursorRecord = CursorRecord;
				return pAccount;
			}
			pAccount = LListGetNextRecordData(pAccounting->pAccountsChart->pListAccount);
		}
		pAccounting->pAccountsChart->pListAccount->CursorRecord = CursorRecord;
	}
	return 0;
}

/********************************************************************

AccountType *SearchSetAccount(

*********************************************************************/
AccountType *SearchSetAccount(
	LLIST pListAccount,
	AccountType *pSearchAccount)
{
	if (pSearchAccount) {
		void *pCursor = pListAccount->CursorRecord;
		AccountType *pAccount
			= LListGetFirstRecordData(pListAccount);
		while (pAccount) {
			if (pSearchAccount == pAccount) {
				pListAccount->CursorRecord = pCursor;
				return pAccount;
			}
			if (!strcmp(pSearchAccount->pId, pAccount->pId)) {
				pListAccount->CursorRecord = pCursor;
				DestroyAccount(&pSearchAccount);
				return pAccount;
			}
			pAccount = LListGetNextRecordData(pListAccount);
		}
		pListAccount->CursorRecord = pCursor;
		LListAddRecord(pListAccount, pSearchAccount);
		return pSearchAccount;
	}
	return 0;
}

/********************************************************************

AccountType *AddAccount(

*********************************************************************/
AccountType *AddAccount(
	void *vpAccounting,
	char *pAccountId)
{
	if (pAccountId && *pAccountId) {
		AccountingType *pAccounting = vpAccounting;
		AccountType *pAccount = GetAccount(pAccounting, pAccountId);
		if (!pAccount) {
			pAccount = CreateAccount(pAccounting, pAccounting->pObjectsChart);
			if (!pAccount)
				return 0; // Fatal Error
			CopyString(&pAccount->pId, pAccountId);
			LListAddRecord(pAccounting->pAccountsChart->pListAccount, pAccount);
		}
		return pAccount;
	}
	return 0;
}

/********************************************************************

AccountCodeType *SearchSetAccountCode(

*********************************************************************/
AccountCodeType *SearchSetAccountCode(
	LLIST pListAccountCode,
	AccountCodeType *pSearchAccountCode)
{
	if (pSearchAccountCode) {
		void *pCursor = pListAccountCode->CursorRecord;
		AccountCodeType *pAccountCode
			= LListGetFirstRecordData(pListAccountCode);
		while (pAccountCode) {
			if (pSearchAccountCode == pAccountCode) {
				pListAccountCode->CursorRecord = pCursor;
				return pAccountCode;
			}
			if ((!strcmp(pSearchAccountCode->pCodeType, pAccountCode->pCodeType))
				&& (!strcmp(pSearchAccountCode->pCode, pAccountCode->pCode))
				&& (!strcmp(pSearchAccountCode->pValidYear, pAccountCode->pValidYear))) {
				pListAccountCode->CursorRecord = pCursor;
				DestroyAccountCode(&pSearchAccountCode);
				return pAccountCode;
			}
			pAccountCode = LListGetNextRecordData(pListAccountCode);
		}
		pListAccountCode->CursorRecord = pCursor;
		LListAddRecord(pListAccountCode, pSearchAccountCode);
		return pSearchAccountCode;
	}
	return 0;
}

/********************************************************************

LLIST GetObjectsChart(

*********************************************************************/
LLIST GetObjectsChart(
	InstanceDataType *pInstanceData)
{
	AccountingType *pWorkData = pInstanceData->vpWorkData;
	return pWorkData->pObjectsChart->pListDimension;
}

/********************************************************************

DimensionType *GetDimension(

*********************************************************************/
DimensionType *GetDimension(
	LLIST pListDimensions,
	COUNTER cDimensionId)
{
	if (pListDimensions) {
		// Start from where the cursor is (Optimisation)
		DimensionType *pDimension
			= LListGetCursorRecordData(pListDimensions);
		DimensionType *pStartDimension = pDimension;
		while (pDimension) {
			if (pDimension->Id == cDimensionId)
				return pDimension; // OK

			pDimension = LListGetNextRecordData(pListDimensions);
		}

		// Else continue to search from start
		pDimension
			= LListGetFirstRecordData(pListDimensions);
		while (pDimension && (pStartDimension != pDimension)) {
			if (pDimension->Id == cDimensionId)
				return pDimension; // OK

			pDimension = LListGetNextRecordData(pListDimensions);
		}
	}
	return 0; // Not found
}

/********************************************************************

DimensionType *SearchSetDimension(

*********************************************************************/
DimensionType *SearchSetDimension(
	LLIST pListDimension,
	DimensionType *pSearchDimension)
{
	if (pSearchDimension) {
		void *pCursor = pListDimension->CursorRecord;
		DimensionType *pDimension
			= LListGetFirstRecordData(pListDimension);
		while (pDimension) {
			if (pSearchDimension == pDimension) {
				pListDimension->CursorRecord = pCursor;
				return pDimension;
			}
			if (pSearchDimension->Id == pDimension->Id) {
				if (pDimension->Id == 6)
					pDimension->Id = pDimension->Id; 
				{
					TextType *pName
						= LListGetFirstRecordData(pSearchDimension->pListName);
					while (pName) {
						SearchSetText(pDimension->pListName, CopyText(pDimension->pStructDocTop, pDimension, pName));
						pName = LListGetNextRecordData(pSearchDimension->pListName);
					}
				}
				{
					ObjectType *pObject
						= LListGetFirstRecordData(pSearchDimension->pListObject);
					while (pObject) {
						SearchSetObject(pListDimension, CopyObject(pDimension->pStructDocTop, pDimension, pObject));
						pObject = LListGetNextRecordData(pSearchDimension->pListObject);
					}
				}
				if (pSearchDimension->pSuperDimension)
					pDimension->pSuperDimension = pSearchDimension->pSuperDimension;
				pListDimension->CursorRecord = pCursor;
				DestroyDimension(&pSearchDimension);
				return pDimension;
			}
			pDimension = LListGetNextRecordData(pListDimension);
		}
		pListDimension->CursorRecord = pCursor;
		LListAddRecord(pListDimension, pSearchDimension);
		return pSearchDimension;
	}
	return 0;
}

/********************************************************************

DimensionType *AddDimension(

*********************************************************************/
DimensionType *AddDimension(
	void *vpAccounting,
	COUNTER cDimensionId)
{
	if (cDimensionId) {
		AccountingType *pAccounting = vpAccounting;
		DimensionType *pDimension = GetDimension(pAccounting->pObjectsChart->pListDimension, cDimensionId);
		if (!pDimension) {
			pDimension = CreateDimension(pAccounting, pAccounting->pObjectsChart);
			if (!pDimension)
				return 0; // Fatal Error
			pDimension->Id = cDimensionId;
			LListAddRecord(pAccounting->pObjectsChart->pListDimension, pDimension);
		}
		return pDimension;
	}
	return 0;
}

/********************************************************************

void *GetDimObjectAccounting(

*********************************************************************/
void *GetDimObjectAccounting(
	void *vpWorkData,
	COUNTER cDimensionId,
	char *pObjectId)
{
	AccountingType *pAccounting = vpWorkData;
	LLIST pListDimension = pAccounting->pObjectsChart->pListDimension;
	return GetDimObject(pListDimension, cDimensionId, pObjectId);
}

/********************************************************************

ObjectType *GetDimObject(

*********************************************************************/
ObjectType *GetDimObject(
	LLIST pListDimension,
	COUNTER cDimensionId,
	char *pObjectId)
{
	if (pListDimension && pObjectId && *pObjectId) {
		// Find the Dimension 
		DimensionType *pDimension
			= GetDimension(pListDimension, cDimensionId);
		if (pDimension) {
			// Start from where the cursor is (Optimisation)
			ObjectType *pObject
				= LListGetCursorRecordData(pDimension->pListObject);
			ObjectType *pStartObject = pObject;
			while (pObject) {
				if (!strcmp(pObject->pId, pObjectId))
					return pObject; // OK

				pObject = LListGetNextRecordData(pDimension->pListObject);
			}

			// Else continue to search from start
			pObject
				= LListGetFirstRecordData(pDimension->pListObject);
			while (pObject && (pStartObject != pObject)) {
				if (!strcmp(pObject->pId, pObjectId))
					return pObject; // OK

				pObject = LListGetNextRecordData(pDimension->pListObject);
			}
		}
	}
	return 0; // Not found
}

/********************************************************************

ObjectType *SearchSetObject(

*********************************************************************/
ObjectType *SearchSetObject(
	LLIST pListDimension,
	ObjectType *pSearchObject)
{
	if (!pSearchObject)
		return 0;
	if ((!pSearchObject->pDimension) && pSearchObject->pStructParent && ((DimensionType *)pSearchObject->pStructParent)->DataType == SCT_DIMENSION)
		pSearchObject->pDimension = pSearchObject->pStructParent;
	if (!pSearchObject->pDimension)
		return 0;
	if ((!pSearchObject->pId) || (!*pSearchObject->pId))
		CopyString(&pSearchObject->pId, "0");
	{
		void *pCursor = pListDimension->CursorRecord;
		DimensionType *pDimension
			= LListGetFirstRecordData(pListDimension);
		while (pDimension) {
			if (pSearchObject->pDimension->Id == pDimension->Id) {
				void *pCursorObject = pDimension->pListObject->CursorRecord;
				ObjectType *pObject
					= LListGetFirstRecordData(pDimension->pListObject);
				while (pObject) {
					if (pSearchObject == pObject) {
						pListDimension->CursorRecord = pCursor;
						return pObject;
					}
					else if (!strcmp(pSearchObject->pId, pObject->pId)) {
						TextType *pName
							= LListGetFirstRecordData(pSearchObject->pListName);
						while (pName) {
							SearchSetText(pObject->pListName, CopyText(pObject->pStructDocTop, pObject, pName));
							pName = LListGetNextRecordData(pSearchObject->pListName);
						}
						{
							char *pFileUrl
								= LListGetFirstRecordData(pSearchObject->pListFileUrl);
							while (pFileUrl) {
								SearchSetToken(pObject->pListFileUrl, CopyString(0, pFileUrl));
								pFileUrl = LListGetNextRecordData(pSearchObject->pListFileUrl);
							}
						}
						if (pSearchObject->pParty)
							pObject->pParty = pSearchObject->pParty;
						DestroyObject(&pSearchObject);
						pDimension->pListObject->CursorRecord = pCursorObject;
						pListDimension->CursorRecord = pCursor;
						return pObject;
					}
					pObject = LListGetNextRecordData(pDimension->pListObject);
				}
				pSearchObject->pDimension = pDimension;
				pSearchObject->pStructParent = pDimension;
				LListAddRecord(pDimension->pListObject, pSearchObject);
				pDimension->pListObject->CursorRecord = pCursorObject;
				pListDimension->CursorRecord = pCursor;
				return pSearchObject;
			}
			pDimension = LListGetNextRecordData(pListDimension);
		}
		pListDimension->CursorRecord = pCursor;
	}
	{
		DimensionType *pDimension = CreateDimension(pSearchObject->pStructDocTop, ((AccountingType *)pSearchObject->pStructDocTop)->pObjectsChart);
		LListAddRecord(((AccountingType *)pSearchObject->pStructDocTop)->pObjectsChart->pListDimension, pDimension);
		LListAddRecord(pDimension->pListObject, pSearchObject);
	}
	return pSearchObject;
}

/********************************************************************

ObjectType *AddDimObject(

*********************************************************************/
ObjectType *AddDimObject(
	void *vpAccounting,
	COUNTER cDimensionId,
	char *pObjectId)
{
	if (cDimensionId && pObjectId && *pObjectId) {
		AccountingType *pAccounting = vpAccounting;
		DimensionType *pDimension = GetDimension(pAccounting->pObjectsChart->pListDimension, cDimensionId);
		if (!pDimension) {
			pDimension = CreateDimension(pAccounting, pAccounting->pObjectsChart);
			if (!pDimension)
				return 0; // Fatal Error
			pDimension->Id = cDimensionId;
			LListAddRecord(pAccounting->pObjectsChart->pListDimension, pDimension);
		}
		{
			ObjectType *pObject = GetDimObject(pAccounting->pObjectsChart->pListDimension, pDimension->Id, pObjectId);
			if ((!pObject) && pDimension) {
				pObject = CreateObject(pAccounting, pDimension->pListObject);
				if (!pObject)
					return 0; // Fatal Error
				CopyString(&pObject->pId, pObjectId);
				pObject->pDimension = pDimension;
				LListAddRecord(pDimension->pListObject, pObject);
			}
			return pObject;
		}
	}
	return 0;
}

/********************************************************************

void *GetLedger(

*********************************************************************/
void *GetLedger(
	void *vpAccounting,
	NUMERIC nLedgerId)
{
	AccountingType *pAccounting = (AccountingType *)vpAccounting;
	if (pAccounting && pAccounting->pLedgersChart->pListLedger) {
		void* pLedgerCursorRecord = pAccounting->pLedgersChart->pListLedger->CursorRecord;
		LedgerType *pLedger = LListGetFirstRecordData(pAccounting->pLedgersChart->pListLedger);
		while (pLedger) {
			if (pLedger->Id == nLedgerId)
				return pLedger; // OK

			pLedger = LListGetNextRecordData(pAccounting->pLedgersChart->pListLedger);
		}
		pAccounting->pLedgersChart->pListLedger->CursorRecord = pLedgerCursorRecord;
	}
	return 0; // Not found
}

/********************************************************************

LedgerType *AddLedger(

*********************************************************************/
LedgerType *AddLedger(
	void *vpAccounting,
	NUMERIC nLedgerId,
	DATEINT FiscalYearStart,
	DATEINT FiscalYearEnd)
{
	if (FiscalYearStart && FiscalYearEnd) {
		AccountingType *pAccounting = (AccountingType *)vpAccounting;
		LedgerType *pLedger = GetLedger(pAccounting, nLedgerId);
		if (!pLedger)
			pLedger = GetLedgerByDate(pAccounting, FiscalYearStart);
		if (!pLedger)
			pLedger = GetLedgerByDate(pAccounting, FiscalYearEnd);
		if (!pLedger) {
			pLedger = CreateLedger(pAccounting, pAccounting->pLedgersChart);
			if (pLedger)
				LListAddRecord(pAccounting->pLedgersChart->pListLedger, pLedger);
			else
				return 0;	// Fatal Error
		}
		pLedger->Id = nLedgerId;
		if (FiscalYearStart)
			pLedger->YearStart = FiscalYearStart;
		if (FiscalYearEnd)
			pLedger->YearEnd = FiscalYearEnd;
		return pLedger;
	}
	return 0;	// Fatal Error
}

/********************************************************************

LedgerType *GetLedgerByDate(

*********************************************************************/
LedgerType *GetLedgerByDate(
	void *vpAccounting,
	DATEINT Date)
{
	AccountingType *pAccounting = (AccountingType *)vpAccounting;
	if (!Date)
		GetLedger(pAccounting, 0);
	if (pAccounting->pLedgersChart->pListLedger && Date) {
		void* pLedgerCursorRecord = pAccounting->pLedgersChart->pListLedger->CursorRecord;
		LedgerType *pLedger
			= LListGetFirstRecordData(pAccounting->pLedgersChart->pListLedger);
		while (pLedger) {
			if ((Date >= pLedger->YearStart) && (Date <= pLedger->YearEnd))
				return pLedger; // OK

			pLedger = LListGetNextRecordData(pAccounting->pLedgersChart->pListLedger);
		}
		pAccounting->pLedgersChart->pListLedger->CursorRecord = pLedgerCursorRecord;

	}
	return 0; // Not found
}

/********************************************************************

LedgerType *GetLedgerDynamic(

*********************************************************************/
LedgerType *GetLedgerDynamic(
	void *vpAccounting,
	DATEINT Date)
{
	AccountingType *pAccounting = (AccountingType *)vpAccounting;
	LedgerType *pLedger = GetLedgerByDate(pAccounting, Date);
	if (pLedger)
		return pLedger;
	// We are talking about undeclared ledgers
	Sort(pAccounting, VOUCHERS_BY_DATE); //VOUCHERS_BY_SERIES);
	if (pAccounting->pLedgersChart->pListLedger && Date) {
		LedgerType *pLedgerLast = GetLedger(pAccounting, 0);
		LedgerType *pLedgerEldest = LListGetLastRecordData(pAccounting->pLedgersChart->pListLedger);
		pLedger = CreateLedger(pAccounting, pAccounting->pLedgersChart);
		LListAddRecord(pAccounting->pLedgersChart->pListLedger, pLedger);
		if (!pLedgerLast) {
			pLedger->YearStart = Date;
			pLedger->YearStart /= 10000;
			pLedger->YearStart *= 10000;
			pLedger->YearEnd = pLedger->YearStart + 1231;
			pLedger->YearStart += 101;
			pLedger->Id = 0;
			// Else continue to search from start
			return pLedger;
		}
		else if (pLedgerLast->YearEnd < Date) {
			pLedger->YearStart = GetNextDate(pLedgerLast->YearEnd);
			pLedger->YearEnd = GetPrevMonthLastDate(pLedger->YearStart + 10000);
			if (pLedger->YearEnd < Date)
				pLedger->YearEnd = GetLastDateInMonth(Date);
			pLedger->Id = 1;
			{
				LedgerType *pLedgerDo = LListGetFirstRecordData(pAccounting->pLedgersChart->pListLedger);
				while (pLedgerDo) {
					pLedgerDo->Id--;
					pLedgerDo = LListGetNextRecordData(pAccounting->pLedgersChart->pListLedger);
				}
			}
			{
				BalanceReportType *pBalanceReportLast = LListGetFirstRecordData(pLedgerLast->pListReport);
				while (pBalanceReportLast) {
					BalanceReportType *pBalanceReportNew = CreateBalanceReport(pAccounting, pLedger);
					pBalanceReportNew->pObject = pBalanceReportLast->pObject;
					pBalanceReportNew = SearchSetBalanceReport(pLedger->pListReport, pBalanceReportNew);
					{
						BalanceAccountType *pBalanceAccountLast = LListGetFirstRecordData(pBalanceReportLast->pListBalanceAccount);
						while (pBalanceAccountLast) {
							BalanceAccountType *pBalanceAccountNew = CreateBalanceAccount(pAccounting, pBalanceReportNew);
							pBalanceAccountNew->pAccount = pBalanceAccountLast->pAccount;
							if ((!strcmp(pBalanceAccountNew->pAccount->pType, ACCOUNT_TYPE_TANGIBLE)) ||
								(!strcmp(pBalanceAccountNew->pAccount->pType, ACCOUNT_TYPE_STOCK)) ||
								(!strcmp(pBalanceAccountNew->pAccount->pType, ACCOUNT_TYPE_RECEIVABLES)) ||
								(!strcmp(pBalanceAccountNew->pAccount->pType, ACCOUNT_TYPE_ASSETS)) ||
								(!strcmp(pBalanceAccountNew->pAccount->pType, ACCOUNT_TYPE_EQUITIES)) ||
								(!strcmp(pBalanceAccountNew->pAccount->pType, ACCOUNT_TYPE_PROVISIONS)) ||
								(!strcmp(pBalanceAccountNew->pAccount->pType, ACCOUNT_TYPE_LIABILITIES))) {
								pBalanceAccountNew->Opening = pBalanceAccountLast->Forward;
								pBalanceAccountNew->OpeningQuantity = pBalanceAccountLast->ForwardQuantity;
							}
							pBalanceAccountNew = SearchSetBalanceAccount(pBalanceReportNew->pListBalanceAccount, pBalanceAccountNew);
							pBalanceAccountLast = LListGetNextRecordData(pBalanceReportLast->pListBalanceAccount);
						}
						pBalanceReportLast = LListGetNextRecordData(pLedgerLast->pListReport);
					}
				}
			}
			Sort(pAccounting, VOUCHERS_BY_DATE); //VOUCHERS_BY_SERIES);
			return pLedger;
		}
		else if (pLedgerEldest->YearStart > Date) {
			pLedger->YearEnd = GetPrevDate(pLedgerEldest->YearStart);
			pLedger->YearStart = GetNextDate(pLedger->YearEnd - 10000);
			if (pLedger->YearStart > Date)
				pLedger->YearStart = (((Date / 100) * 100) + 1);
			pLedger->Id = pLedgerEldest->Id - 1;
			Sort(pAccounting, VOUCHERS_BY_DATE); //VOUCHERS_BY_SERIES);
			return pLedger;
		}
	}
	return 0; // Not found
}

/********************************************************************

LedgerType *SearchSetLedger(

*********************************************************************/
LedgerType *SearchSetLedger(
	LLIST pListLedger,
	LedgerType *pSearchLedger)
{
	if (pSearchLedger) {
		void *pCursor = pListLedger->CursorRecord;
		LedgerType *pLedger
			= LListGetFirstRecordData(pListLedger);
		while (pLedger) {
			if (pSearchLedger == pLedger) {
				pListLedger->CursorRecord = pCursor;
				return pLedger;
			}
			if (pSearchLedger->Id == pLedger->Id) {
				pListLedger->CursorRecord = pCursor;
				DestroyLedger(&pSearchLedger);
				return pLedger;
			}
			pLedger = LListGetNextRecordData(pListLedger);
		}
		pListLedger->CursorRecord = pCursor;
		LListAddRecord(pListLedger, pSearchLedger);
		return pSearchLedger;
	}
	return 0;
}

/********************************************************************

void *GetBalanceReport(

*********************************************************************/
void *GetBalanceReport(
	void *vpLedger,
	void *vpObject)
{
	LedgerType *pLedger = vpLedger;
	ObjectType *pObject = vpObject;
	if (vpLedger && pLedger->pListReport) {
		void* pBalanceReportCursorRecord = pLedger->pListReport->CursorRecord;
		BalanceReportType *pBalanceReport
			= LListGetFirstRecordData(pLedger->pListReport);
		while (pBalanceReport) {
			if ((!pBalanceReport->pObject) && (!vpObject))
				return pBalanceReport; // OK
			if (!pBalanceReport->pObject && pObject && pObject->pDimension && (pObject->pDimension->Id == 0) && (!strcmp(pObject->pId, "0")))
				return pBalanceReport; // OK
			if (pBalanceReport->pObject == vpObject)
				return pBalanceReport; // OK

			pBalanceReport = LListGetNextRecordData(pLedger->pListReport);
		}
		pLedger->pListReport->CursorRecord = pBalanceReportCursorRecord;
	}
	return 0; // Not found
}

/********************************************************************

BalanceReportType *SearchSetBalanceReport(

*********************************************************************/
BalanceReportType *SearchSetBalanceReport(
	LLIST pBalanceReportList,
	BalanceReportType *pSearchBalanceReport)
{
	if (pSearchBalanceReport) {
		void *pCursor = pBalanceReportList->CursorRecord;
		BalanceReportType *pBalanceReport
			= LListGetFirstRecordData(pBalanceReportList);
		while (pBalanceReport) {
			if (pSearchBalanceReport == pBalanceReport) {
				pBalanceReportList->CursorRecord = pCursor;
				return pBalanceReport;
			}
			if (pSearchBalanceReport->pObject == pBalanceReport->pObject) {
				pBalanceReportList->CursorRecord = pCursor;
				DestroyBalanceReport(&pSearchBalanceReport);
				return pBalanceReport;
			}
			pBalanceReport = LListGetNextRecordData(pBalanceReportList);
		}
		pBalanceReportList->CursorRecord = pCursor;
		LListAddRecord(pBalanceReportList, pSearchBalanceReport);
		return pSearchBalanceReport;
	}
	return 0;
}

/********************************************************************

void *GetBalanceAccount(

*********************************************************************/
void *GetBalanceAccount(
	void *vpBalanceReport,
	void *vpAccount)
{
	if (vpBalanceReport && vpAccount) {
		BalanceReportType *pBalanceReport = vpBalanceReport;
		if (pBalanceReport->pListBalanceAccount) {
			void* pBalanceAccountCursorRecord = pBalanceReport->pListBalanceAccount->CursorRecord;
			BalanceAccountType *pBalanceAccount
				= LListGetFirstRecordData(pBalanceReport->pListBalanceAccount);
			while (pBalanceAccount) {
				if (pBalanceAccount->pAccount == vpAccount)
					return pBalanceAccount; // OK

				pBalanceAccount = LListGetNextRecordData(pBalanceReport->pListBalanceAccount);
			}
			pBalanceReport->pListBalanceAccount->CursorRecord = pBalanceAccountCursorRecord;
		}
	}
	return 0; // Not found
}

/********************************************************************

BalanceAccountType *SearchSetBalanceAccount(

*********************************************************************/
BalanceAccountType *SearchSetBalanceAccount(
	LLIST pListBalanceAccount,
	BalanceAccountType *pSearchBalanceAccount)
{
	if (pSearchBalanceAccount) {
		void *pCursor = pListBalanceAccount->CursorRecord;
		BalanceAccountType *pBalanceAccount
			= LListGetFirstRecordData(pListBalanceAccount);
		while (pBalanceAccount) {
			if (pSearchBalanceAccount == pBalanceAccount) {
				pListBalanceAccount->CursorRecord = pCursor;
				return pBalanceAccount;
			}
			if (!strcmp(pSearchBalanceAccount->pAccount->pId, pBalanceAccount->pAccount->pId)) {
				pListBalanceAccount->CursorRecord = pCursor;
				DestroyBalanceAccount(&pSearchBalanceAccount);
				return pBalanceAccount;
			}
			pBalanceAccount = LListGetNextRecordData(pListBalanceAccount);
		}
		pListBalanceAccount->CursorRecord = pCursor;
		LListAddRecord(pListBalanceAccount, pSearchBalanceAccount);
		return pSearchBalanceAccount;
	}
	return 0;
}

/********************************************************************

void *GetBalanceAccountPeriod(

*********************************************************************/
void *GetBalanceAccountPeriod(
	BalanceAccountType *pBalanceAccount,
	DATEINT PeriodEnd,
	char *pSelTabSubText)
{
	LLIST pListPeriod = pBalanceAccount->pListMonth;
	if (pSelTabSubText
		&& ((pSelTabSubText[1] == ' ')
			|| (pSelTabSubText[1] == '0')
			|| (pSelTabSubText[1] == '1')
			|| (pSelTabSubText[1] == '2')
			|| (pSelTabSubText[1] == '3')
			|| (pSelTabSubText[1] == '4'))) {
		switch (*pSelTabSubText) {
		case ANNUAL:
			pListPeriod = pBalanceAccount->pListAnnual;
			break;
		case SEMI:
			pListPeriod = pBalanceAccount->pListSemi;
			break;
		case TERTIAL:
			pListPeriod = pBalanceAccount->pListTertial;
			break;
		case QUARTER:
			pListPeriod = pBalanceAccount->pListQuarter;
			break;
		case MONTH:
			pListPeriod = pBalanceAccount->pListMonth;
		default:
			break;
		}
		PeriodEnd = GetPeriodDate(PeriodEnd, pSelTabSubText);
	}
	if (pListPeriod && PeriodEnd) {
		void* pAccountPeriodCursorRecord = pListPeriod->CursorRecord;
		BalanceAccountPeriodType *pAccountPeriod
			= LListGetFirstRecordData(pListPeriod);
		while (pAccountPeriod) {
			if ((pAccountPeriod->End / 100) == (PeriodEnd / 100))
				return pAccountPeriod; // OK

			pAccountPeriod = LListGetNextRecordData(pListPeriod);
		}
		pListPeriod->CursorRecord = pAccountPeriodCursorRecord;
	}
	return 0; // Not found
}

/********************************************************************

BalanceAccountPeriodType *SearchSetBalanceAccountPeriodDynamic(

*********************************************************************/
BalanceAccountPeriodType *SearchSetBalanceAccountPeriodDynamic(
	BalanceAccountType *pBalanceAccount,
	BalanceAccountPeriodType *pSearchBalanceAccountPeriod,
	char *pSelTabSubText)
{
	LLIST pListPeriod = pBalanceAccount->pListMonth;
	if (pSelTabSubText
		&& ((pSelTabSubText[1] == ' ')
			|| (pSelTabSubText[1] == '0')
			|| (pSelTabSubText[1] == '1')
			|| (pSelTabSubText[1] == '2')
			|| (pSelTabSubText[1] == '3')
			|| (pSelTabSubText[1] == '4'))) {
		switch (*pSelTabSubText) {
		case ANNUAL:
			pListPeriod = pBalanceAccount->pListAnnual;
			break;
		case SEMI:
			pListPeriod = pBalanceAccount->pListSemi;
			break;
		case TERTIAL:
			pListPeriod = pBalanceAccount->pListTertial;
			break;
		case QUARTER:
			pListPeriod = pBalanceAccount->pListQuarter;
			break;
		case MONTH:
			pListPeriod = pBalanceAccount->pListMonth;
		default:
			break;
		}
	}
	return SearchSetBalanceAccountPeriod(pListPeriod, pSearchBalanceAccountPeriod);
}

/********************************************************************

BalanceAccountPeriodType *SearchSetBalanceAccountPeriod(

*********************************************************************/
BalanceAccountPeriodType *SearchSetBalanceAccountPeriod(
	LLIST pListBalanceAccountPeriod,
	BalanceAccountPeriodType *pSearchBalanceAccountPeriod)
{
	if (pSearchBalanceAccountPeriod) {
		void *pCursor = pListBalanceAccountPeriod->CursorRecord;
		BalanceAccountPeriodType *pBalanceAccountPeriod
			= LListGetFirstRecordData(pListBalanceAccountPeriod);
		while (pBalanceAccountPeriod) {
			if (pSearchBalanceAccountPeriod == pBalanceAccountPeriod) {
				pListBalanceAccountPeriod->CursorRecord = pCursor;
				return pBalanceAccountPeriod;
			}
			if ((pSearchBalanceAccountPeriod->End / 100) == (pBalanceAccountPeriod->End / 100)) {
				if (pSearchBalanceAccountPeriod->Budget)
					pBalanceAccountPeriod->Budget = pSearchBalanceAccountPeriod->Budget;
				if (pSearchBalanceAccountPeriod->BudgetQuantity)
					pBalanceAccountPeriod->BudgetQuantity = pSearchBalanceAccountPeriod->BudgetQuantity;
				if (pSearchBalanceAccountPeriod->Outcome)
					pBalanceAccountPeriod->Outcome = pSearchBalanceAccountPeriod->Outcome;
				if (pSearchBalanceAccountPeriod->OutcomeQuantity)
					pBalanceAccountPeriod->OutcomeQuantity = pSearchBalanceAccountPeriod->OutcomeQuantity;
				pListBalanceAccountPeriod->CursorRecord = pCursor;
				DestroyBalanceAccountPeriod(&pSearchBalanceAccountPeriod);
				return pBalanceAccountPeriod;
			}
			pBalanceAccountPeriod = LListGetNextRecordData(pListBalanceAccountPeriod);
		}
		pListBalanceAccountPeriod->CursorRecord = pCursor;
		LListAddRecord(pListBalanceAccountPeriod, pSearchBalanceAccountPeriod);
		return pSearchBalanceAccountPeriod;
	}
	return 0;
}

/********************************************************************

static BalanceTotalPeriodType *SearchSetBalanceTotalPeriod(

*********************************************************************/
static BalanceTotalPeriodType *SearchSetBalanceTotalPeriod(
	LLIST pListBalanceTotalPeriod,
	BalanceTotalPeriodType *pSearchBalanceTotalPeriod)
{
	if (pSearchBalanceTotalPeriod) {
		void *pCursor = pListBalanceTotalPeriod->CursorRecord;
		BalanceTotalPeriodType *pBalanceTotalPeriod
			= LListGetFirstRecordData(pListBalanceTotalPeriod);
		while (pBalanceTotalPeriod) {
			if (pSearchBalanceTotalPeriod == pBalanceTotalPeriod) {
				pListBalanceTotalPeriod->CursorRecord = pCursor;
				return pBalanceTotalPeriod;
			}
			if ((pSearchBalanceTotalPeriod->End / 100) == (pBalanceTotalPeriod->End / 100)) {
				if (pSearchBalanceTotalPeriod->Budget)
					pBalanceTotalPeriod->Budget = pSearchBalanceTotalPeriod->Budget;
				if (pSearchBalanceTotalPeriod->Outcome)
					pBalanceTotalPeriod->Outcome = pSearchBalanceTotalPeriod->Outcome;
				pListBalanceTotalPeriod->CursorRecord = pCursor;
				DestroyBalanceTotalPeriod(&pSearchBalanceTotalPeriod);
				return pBalanceTotalPeriod;
			}
			pBalanceTotalPeriod = LListGetNextRecordData(pListBalanceTotalPeriod);
		}
		pListBalanceTotalPeriod->CursorRecord = pCursor;
		LListAddRecord(pListBalanceTotalPeriod, pSearchBalanceTotalPeriod);
		return pSearchBalanceTotalPeriod;
	}
	return 0;
}

/********************************************************************

BalanceTotalPeriodType *GetBalanceTotalPeriod(

*********************************************************************/
BalanceTotalPeriodType* GetBalanceTotalPeriod(
	LLIST pListBalanceTotalPeriod,
	DATEINT End)
{
	void* pCursor = pListBalanceTotalPeriod->CursorRecord;
	BalanceTotalPeriodType* pBalanceTotalPeriod
		= LListGetFirstRecordData(pListBalanceTotalPeriod);
	while (pBalanceTotalPeriod) {
		if ((End / 100) == (pBalanceTotalPeriod->End / 100)) {
			pListBalanceTotalPeriod->CursorRecord = pCursor;
			return pBalanceTotalPeriod;
		}
		pBalanceTotalPeriod = LListGetNextRecordData(pListBalanceTotalPeriod);
	}
	pListBalanceTotalPeriod->CursorRecord = pCursor;
	return 0;
}

/********************************************************************

VoucherType *SearchSetVoucher(

*********************************************************************/
VoucherType *SearchSetVoucher(
	LLIST pListVoucher,
	VoucherType *pSearchVoucher)
{
	if (pSearchVoucher) {
		void *pCursor = pListVoucher->CursorRecord;
		VoucherType *pVoucher
			= LListGetFirstRecordData(pListVoucher);
		while (pVoucher) {
			if (pSearchVoucher == pVoucher) {
				pListVoucher->CursorRecord = pCursor;
				return pVoucher;
			}
			if (pSearchVoucher->pId->SeriesId
				&& (pSearchVoucher->pId->SeriesId == pVoucher->pId->SeriesId)
				&& (!strcmp(pSearchVoucher->pId->pseries, pVoucher->pId->pseries))) {
				if (pSearchVoucher->Date)
					pVoucher->Date = pSearchVoucher->Date;
				if (pSearchVoucher->pAccountant)
					pVoucher->pAccountant = CopyOperatorInfo(pVoucher->pStructDocTop, pVoucher, pSearchVoucher->pAccountant);
				{
					TextType *pVoucherTitle
						= LListGetFirstRecordData(pSearchVoucher->pListTitle);
					while (pVoucherTitle) {
						SearchSetText(pVoucher->pListTitle, pVoucherTitle);
						pVoucherTitle = LListGetNextRecordData(pSearchVoucher->pListTitle);
					}
				}
				pListVoucher->CursorRecord = pCursor;
				DestroyVoucher(&pSearchVoucher);
				return pVoucher;
			}
			pVoucher = LListGetNextRecordData(pListVoucher);
		}
		pListVoucher->CursorRecord = pCursor;
		LListAddRecord(pListVoucher, pSearchVoucher);
		return pSearchVoucher;
	}
	return 0;
}

/********************************************************************

void CheckEnum(

*********************************************************************/
void CheckEnum(
	char **ppEnumData,
	EnumType *pEnumList,
	char *pEnumRead)
{
	int i = 0;
	while ((pEnumList + i)->EnumerationID) {
		if (!strcmp((pEnumList + i)->EnumerationID, pEnumRead)) {
			CopyString(ppEnumData, (pEnumList + i)->EnumerationID);
			free(pEnumRead);
			return;
		}
		i++;
	}
	return;
}

/********************************************************************

static InvoiceEntryType *SearchSetInvoiceEntry(

*********************************************************************/
/*
static InvoiceEntryType *SearchSetInvoiceEntry(
	LLIST pListInvoiceEntry,
	InvoiceEntryType *pSearchInvoiceEntry)
{
	if (pSearchInvoiceEntry) {
		void *pCursor = pListInvoiceEntry->CursorRecord;
		InvoiceEntryType *pInvoiceEntry
			= LListGetFirstRecordData(pListInvoiceEntry);
		while (pInvoiceEntry) {
			if (pSearchInvoiceEntry == pInvoiceEntry) {
				pListInvoiceEntry->CursorRecord = pCursor;
				return pInvoiceEntry;
			}
			if ((pSearchInvoiceEntry->pInvoiceId->SeriesId == pInvoiceEntry->pInvoiceId->SeriesId)
				&& (!strcmp(pSearchInvoiceEntry->pInvoiceId->pseries, pInvoiceEntry->pInvoiceId->pseries))) {
				pListInvoiceEntry->CursorRecord = pCursor;
				DestroyInvoiceEntry(&pSearchInvoiceEntry);
				return pInvoiceEntry;
			}
			pInvoiceEntry = LListGetNextRecordData(pListInvoiceEntry);
		}
		pListInvoiceEntry->CursorRecord = pCursor;
		LListAddRecord(pListInvoiceEntry, pSearchInvoiceEntry);
		return pSearchInvoiceEntry;
	}
	return 0;
}
*/
/********************************************************************

Sort functions

*********************************************************************/
/********************************************************************
*
static int CompareSortStrings(
*
*********************************************************************/
static int CompareSortStrings(
	const void *arg1,
	const void *arg2)
{
	if ((!arg1) && (!arg2))
		return 0;
	else if (!arg1)
		return (-1);
	else if (!arg2)
		return 1;
	/* Compare all of both strings: */
	return strcoll(*(char**)arg1, *(char**)arg2);
}

/********************************************************************
*
static int CompareSortNumeric(
*
*********************************************************************/
/*
static int CompareSortNumeric(
	NUMERIC nArg1,
	NUMERIC nArg2)
{
	if (nArg1 < nArg2)
		return (-1);
	else if (nArg1 > nArg2)
		return 1;
	else
		return 0;
}
*/
/********************************************************************
*
static int CompareSortCounter(
*
*********************************************************************/
static int CompareSortCounter(
	COUNTER cArg1,
	COUNTER cArg2)
{
	if (cArg1 < cArg2)
		return (-1);
	else if (cArg1 > cArg2)
		return 1;
	else
		return 0;
}

/********************************************************************
*
static int CompareSortDate(
*
*********************************************************************/
static int CompareSortDate(
	DATEINT dArg1,
	DATEINT dArg2)
{
	if (dArg1 < dArg2)
		return (-1);
	else if (dArg1 > dArg2)
		return 1;
	else
		return 0;
}

/********************************************************************
*
static int CompareSortAccounts(
*
*********************************************************************/
static int CompareSortAccounts(
	const void *arg1,
	const void *arg2)
{
	return CompareSortStrings(arg1, arg2);
}

/********************************************************************
*
static int CompareSortDimensions(
*
*********************************************************************/
static int CompareSortDimensions(
	const void *arg1,
	const void *arg2)
{
	DimensionType *pDimension1 = (*(DimensionType **)arg1);
	DimensionType *pDimension2 = (*(DimensionType **)arg2);

	if ((!arg1) && (!arg2))
		return 0;
	else if (!arg1)
		return (-1);
	else if (!arg2)
		return 1;
	return CompareSortCounter(pDimension1->Id, pDimension2->Id);
}

/********************************************************************
*
static int CompareSortObjects(
*
*********************************************************************/
static int CompareSortObjects(
	const void *arg1,
	const void *arg2)
{
	ObjectType *pObject1 = (*(ObjectType **)arg1);
	ObjectType *pObject2 = (*(ObjectType **)arg2);

	if ((!arg1) && (!arg2))
		return 0;
	else if (!arg1)
		return (-1);
	else if (!arg2)
		return 1;
	else if (!pObject1)
		return (-1);
	else if (!pObject2)
		return 1;
	// Compare dimensions
	else if (CompareSortDimensions(
		&pObject1->pDimension,
		&pObject2->pDimension))
		return CompareSortDimensions(
			&pObject1->pDimension,
			&pObject2->pDimension);
	// Compare object Ids as numeric
	else if (((*pObject1->pId >= '0') && (*pObject1->pId <= '9')) 
		&& (CompareSortCounter(Strtoull(pObject1->pId), Strtoull(pObject2->pId))))
		// If object Ids are numeric strings compare them numerically
		// We do not want 10 to come next to 1, but in numerical order
		// We like highest number on top because it is more likly
		// interesting
		return (0-CompareSortCounter(Strtoull(pObject1->pId), Strtoull(pObject2->pId)));
	// else compare object Ids as text
	return strcoll(pObject1->pId, pObject2->pId);
}


/********************************************************************
*
static int CompareSortVouchersBySeries(
*
*********************************************************************/
static int CompareSortVouchersBySeries(
	const void *arg1,
	const void *arg2)
{
	VoucherType *pVoucher1 = (*(VoucherType **)arg1);
	VoucherType *pVoucher2 = (*(VoucherType **)arg2);

	// To allow int64 VoucherIds to exist
	if ((!arg1) && (!arg2))
		return 0;
	else if (!arg1)
		return (-1);
	else if (!arg2)
		return 1;
	else if (strcoll(pVoucher1->pId->pseries, pVoucher2->pId->pseries))
		return strcoll(pVoucher1->pId->pseries, pVoucher2->pId->pseries);
	if (CompareSortCounter(pVoucher1->pId->SeriesId, pVoucher2->pId->SeriesId))
		return CompareSortCounter(pVoucher1->pId->SeriesId, pVoucher2->pId->SeriesId);
	return CompareSortDate(pVoucher1->Date, pVoucher2->Date); 
}

/********************************************************************
*
static int CompareSortVouchersByDate(
*
*********************************************************************/
static int CompareSortVouchersByDate(
	const void *arg1,
	const void *arg2)
{
	VoucherType *pVoucher1 = (*(VoucherType **)arg1);
	VoucherType *pVoucher2 = (*(VoucherType **)arg2);

	// To allow int64 VoucherDates to exist
	if ((!arg1) && (!arg2))
		return 0;
	else if (!arg1)
		return (-1);
	else if (!arg2)
		return 1;
	else if (CompareSortDate(pVoucher1->Date, pVoucher2->Date))
		return CompareSortDate(pVoucher1->Date, pVoucher2->Date);
	else
		// Same date compare by seires Id
		return (CompareSortVouchersBySeries(arg1, arg2));
}

/********************************************************************
*
static int CompareSortBalanceAccountPeriods(
*
*********************************************************************/
static int CompareSortBalanceAccountPeriods(
	const void *arg1,
	const void *arg2)
{
	BalanceAccountPeriodType *pBalanceAccountPeriod1 = (*(BalanceAccountPeriodType **)arg1);
	BalanceAccountPeriodType *pBalanceAccountPeriod2 = (*(BalanceAccountPeriodType **)arg2);

	if ((!arg1) && (!arg2))
		return 0;
	else if (!arg1)
		return (-1);
	else if (!arg2)
		return 1;
	return CompareSortDate(pBalanceAccountPeriod1->End, pBalanceAccountPeriod2->End);
}

/********************************************************************
*
static int CompareSortBalanceAccounts(
*
*********************************************************************/
static int CompareSortBalanceAccounts(
	const void *arg1,
	const void *arg2)
{
	BalanceAccountType *pBalanceAccount1 = (*(BalanceAccountType **)arg1);
	BalanceAccountType *pBalanceAccount2 = (*(BalanceAccountType **)arg2);

	if ((!arg1) && (!arg2))
		return 0;
	else if (!arg1)
		return (-1);
	else if (!arg2)
		return 1;
	return CompareSortAccounts(
		&pBalanceAccount1->pAccount,
		&pBalanceAccount2->pAccount);
}

/********************************************************************
*
static int CompareSortBalanceTotalPeriods(
*
*********************************************************************/
static int CompareSortBalanceTotalPeriods(
	const void *arg1,
	const void *arg2)
{
	BalanceTotalPeriodType *pBalanceTotalPeriod1 = (*(BalanceTotalPeriodType **)arg1);
	BalanceTotalPeriodType *pBalanceTotalPeriod2 = (*(BalanceTotalPeriodType **)arg2);

	if ((!arg1) && (!arg2))
		return 0;
	else if (!arg1)
		return (-1);
	else if (!arg2)
		return 1;
	return CompareSortDate(pBalanceTotalPeriod1->End, pBalanceTotalPeriod2->End);
}

/********************************************************************
*
static int CompareSortBalanceReports(
*
*********************************************************************/
static int CompareSortBalanceReports(
	const void *arg1,
	const void *arg2)
{
	BalanceReportType *pBalanceReport1 = (*(BalanceReportType **)arg1);
	BalanceReportType *pBalanceReport2 = (*(BalanceReportType **)arg2);

	if ((!arg1) && (!arg2))
		return 0;
	else if (!arg1)
		return (-1);
	else if (!arg2)
		return 1;
	else
		return CompareSortObjects(
			&pBalanceReport1->pObject, 
			&pBalanceReport2->pObject);
}

/********************************************************************
*
static int CompareSortLedgers(
*
*********************************************************************/
static int CompareSortLedgers(
	const void *arg1,
	const void *arg2)
{
	LedgerType *pLedger1 = (*(LedgerType **)arg1);
	LedgerType *pLedger2 = (*(LedgerType **)arg2);

	if ((!arg1) && (!arg2))
		return 0;
	else if (!arg1)
		return (-1);
	else if (!arg2)
		return 1;
	// Last first
	return (0-CompareSortDate(pLedger1->YearStart, pLedger2->YearStart));
}

typedef struct {
	COUNTER DataType;																											/* To make the structure identifiable in void ptr cases*/
} DataTypeStructType;


/********************************************************************
*
LLIST GetSortedDataStructList(
*
*********************************************************************/
LLIST GetSortedDataStructList(
	LLIST pListDataStructs,
	int SortOption)
{
	LLIST pSortedListDataStructs = LListInit();
	DataTypeStructType **ppDataStructs;
	size_t iDataStructs = 0;

	if (SortOption) { // Get the total number of DataStructs
		DataTypeStructType *pDataStruct
			= LListGetFirstRecordData(pListDataStructs);
		while (pDataStruct) {
			iDataStructs++;
			pDataStruct = LListGetNextRecordData(pListDataStructs);
		}

		if (iDataStructs > 1) { // Put them in an array of pDataStruct pointers
			size_t zt = 0;
			pDataStruct
				= LListGetFirstRecordData(pListDataStructs);
			ppDataStructs = calloc((int)iDataStructs, sizeof(DataTypeStructType *));
			for (zt = 0; pDataStruct && (zt < iDataStructs); zt++) {
				ppDataStructs[zt] = pDataStruct;
				pDataStruct = LListGetNextRecordData(pListDataStructs);
			}
			// Sort the array
			{
				void *pFunc = 0;
				switch ((ppDataStructs[0])->DataType) {
				case SCT_ACCOUNT:
					pFunc = &CompareSortAccounts;
					break;
				case SCT_DIMENSION:
					pFunc = &CompareSortDimensions;
					break;
				case SCT_OBJECT:
					pFunc = &CompareSortObjects;
					break;
				case SCT_LEDGER:
					pFunc = &CompareSortLedgers;
					break;
				case SCT_BALANCEREPORT:
					pFunc = &CompareSortBalanceReports;
					break;
				case SCT_BALANCETOTALPERIOD:
					pFunc = &CompareSortBalanceTotalPeriods;
					break;
				case SCT_BALANCEACCOUNT:
					pFunc = &CompareSortBalanceAccounts;
					break;
				case SCT_BALANCEACCOUNTPERIOD:
					pFunc = &CompareSortBalanceAccountPeriods;
					break;
				case SCT_VOUCHER:
					// Sort the array
					switch (SortOption) {
					case VOUCHERS_BY_SERIES:
						pFunc = &CompareSortVouchersBySeries;
						break;
					case VOUCHERS_BY_DATE:
						pFunc = &CompareSortVouchersByDate;
						break;
					case VOUCHERS_BY_ORIGINAL:
					default:
						break;
					}
					break;
				case SCT_ACCOUNTING:
				case SCT_ATTEST:
				case SCT_PARTY:
				case SCT_CONTACT:
				case SCT_ACCOUNTSCHART:
				case SCT_ACCOUNTCODE:
				case SCT_OBJECTSCHART:
				case SCT_LEDGERSCHART:
				case SCT_BALANCETOTAL:
				case SCT_VOUCHERENTRY:
				case SCT_INVOICECHART:
				case SCT_INVOICE:
				case SCT_INVOICEENTRY:
				case SCT_TRADEITEMDATA:
				case SCT_PAY:
				case SCT_DEPRECIATIONS:
				case SCT_DEPRECIATIONENTRY:
				case SCT_CITRADESETTLEMENTPAYMENTMEANS:
				case SCT_CIDEBTORFINANCIALACCOUNT:
				case SCT_CIDEBTORFINANCIALINSTITUTION:
				case SCT_CICREDITORFINANCIALACCOUNT:
				case SCT_CICREDITORFINANCIALINSTITUTION:
				case SCT_TRADESETTLEMENTFINANCIALCARD:
				case SCT_BRANCHFINANCIALINSTITUTION:
				case SCT_FINANCIALINSTITUTIONADDRESS:
				case SCT_AMOUNTCURRENCY:
				case SCT_TEXT:
				case SCT_OPERATORINFO:
				case SCT_PROGRAMINFO:
				case SCT_SERIESID:
				default:
					pFunc = 0;
					break;
				}
				if (pFunc) {
					qsort((DataTypeStructType *)ppDataStructs, (size_t)iDataStructs, sizeof(DataTypeStructType *), pFunc);
					{
						for (zt = 0; ppDataStructs[zt] && (zt < iDataStructs); zt++)
							LListAddRecord(pSortedListDataStructs, ppDataStructs[zt]);
						free(ppDataStructs);
						return pSortedListDataStructs;
					}
				}
				else
					// Just copy the list 
					LListCopyLList(&pSortedListDataStructs, pListDataStructs);

			}
		}
		else
			// Just copy the list 
			LListCopyLList(&pSortedListDataStructs, pListDataStructs);
	}
	else
		// Just copy the list 
		LListCopyLList(&pSortedListDataStructs, pListDataStructs);

	return pSortedListDataStructs;
}


/********************************************************************
*
void Sort(
*
*********************************************************************/
void Sort(AccountingType *pAccounting,
	int VoucherSortOption)
{
	if (pAccounting && pAccounting->pAccountsChart && pAccounting->pAccountsChart->pListAccount) {
		LLIST pListAccount = GetSortedDataStructList(pAccounting->pAccountsChart->pListAccount, 1);
		LListErase(pAccounting->pAccountsChart->pListAccount);
		pAccounting->pAccountsChart->pListAccount = pListAccount;
	}

	if (pAccounting && pAccounting->pObjectsChart && pAccounting->pObjectsChart->pListDimension) {
		DimensionType *pDimension
			= LListGetFirstRecordData(pAccounting->pObjectsChart->pListDimension);
		while (pDimension) {
			LLIST pListObject = GetSortedDataStructList(pDimension->pListObject, 1);
			LListErase(pDimension->pListObject);
			pDimension->pListObject = pListObject;

			pDimension = LListGetNextRecordData(pAccounting->pObjectsChart->pListDimension);
		}
		{
			LLIST pListDimension = GetSortedDataStructList(pAccounting->pObjectsChart->pListDimension, 1);
			LListErase(pAccounting->pObjectsChart->pListDimension);
			pAccounting->pObjectsChart->pListDimension = pListDimension;
		}
	}
	if (pAccounting && pAccounting->pLedgersChart->pListLedger) {
		LedgerType *pLedger
			= LListGetFirstRecordData(pAccounting->pLedgersChart->pListLedger);
		while (pLedger) {
			if (pLedger->pListReport) {
				BalanceReportType *pBalanceReport
					= LListGetFirstRecordData(pLedger->pListReport);
				while (pBalanceReport) {
					if (pBalanceReport->pTangible) {
						LLIST pListMonth = GetSortedDataStructList(pBalanceReport->pTangible->pListMonth, 1);
						LListErase(pBalanceReport->pTangible->pListMonth);
						pBalanceReport->pTangible->pListMonth = pListMonth;
					}
					if (pBalanceReport->pStock) {
						LLIST pListMonth = GetSortedDataStructList(pBalanceReport->pStock->pListMonth, 1);
						LListErase(pBalanceReport->pStock->pListMonth);
						pBalanceReport->pStock->pListMonth = pListMonth;
					}
					if (pBalanceReport->pReceivables) {
						LLIST pListMonth = GetSortedDataStructList(pBalanceReport->pReceivables->pListMonth, 1);
						LListErase(pBalanceReport->pReceivables->pListMonth);
						pBalanceReport->pReceivables->pListMonth = pListMonth;
					}
					if (pBalanceReport->pAssets) {
						LLIST pListMonth = GetSortedDataStructList(pBalanceReport->pAssets->pListMonth, 1);
						LListErase(pBalanceReport->pAssets->pListMonth);
						pBalanceReport->pAssets->pListMonth = pListMonth;
					}
					if (pBalanceReport->pActive) {
						LLIST pListMonth = GetSortedDataStructList(pBalanceReport->pActive->pListMonth, 1);
						LListErase(pBalanceReport->pActive->pListMonth);
						pBalanceReport->pActive->pListMonth = pListMonth;
					}
					if (pBalanceReport->pEquities) {
						LLIST pListMonth = GetSortedDataStructList(pBalanceReport->pEquities->pListMonth, 1);
						LListErase(pBalanceReport->pEquities->pListMonth);
						pBalanceReport->pEquities->pListMonth = pListMonth;
					}
					if (pBalanceReport->pProvisions) {
						LLIST pListMonth = GetSortedDataStructList(pBalanceReport->pProvisions->pListMonth, 1);
						LListErase(pBalanceReport->pProvisions->pListMonth);
						pBalanceReport->pProvisions->pListMonth = pListMonth;
					}
					if (pBalanceReport->pLiabilities) {
						LLIST pListMonth = GetSortedDataStructList(pBalanceReport->pLiabilities->pListMonth, 1);
						LListErase(pBalanceReport->pLiabilities->pListMonth);
						pBalanceReport->pLiabilities->pListMonth = pListMonth;
					}
					if (pBalanceReport->pPassive) {
						LLIST pListMonth = GetSortedDataStructList(pBalanceReport->pPassive->pListMonth, 1);
						LListErase(pBalanceReport->pPassive->pListMonth);
						pBalanceReport->pPassive->pListMonth = pListMonth;
					}
					if (pBalanceReport->pBalance) {
						LLIST pListMonth = GetSortedDataStructList(pBalanceReport->pBalance->pListMonth, 1);
						LListErase(pBalanceReport->pBalance->pListMonth);
						pBalanceReport->pBalance->pListMonth = pListMonth;
					}
					if (pBalanceReport->pRevenues) {
						LLIST pListMonth = GetSortedDataStructList(pBalanceReport->pRevenues->pListMonth, 1);
						LListErase(pBalanceReport->pRevenues->pListMonth);
						pBalanceReport->pRevenues->pListMonth = pListMonth;
					}
					if (pBalanceReport->pSupplies) {
						LLIST pListMonth = GetSortedDataStructList(pBalanceReport->pSupplies->pListMonth, 1);
						LListErase(pBalanceReport->pSupplies->pListMonth);
						pBalanceReport->pSupplies->pListMonth = pListMonth;
					}
					if (pBalanceReport->pProfit) {
						LLIST pListMonth = GetSortedDataStructList(pBalanceReport->pProfit->pListMonth, 1);
						LListErase(pBalanceReport->pProfit->pListMonth);
						pBalanceReport->pProfit->pListMonth = pListMonth;
					}
					if (pBalanceReport->pExpenses) {
						LLIST pListMonth = GetSortedDataStructList(pBalanceReport->pExpenses->pListMonth, 1);
						LListErase(pBalanceReport->pExpenses->pListMonth);
						pBalanceReport->pExpenses->pListMonth = pListMonth;
					}
					if (pBalanceReport->pStaff) {
						LLIST pListMonth = GetSortedDataStructList(pBalanceReport->pStaff->pListMonth, 1);
						LListErase(pBalanceReport->pStaff->pListMonth);
						pBalanceReport->pStaff->pListMonth = pListMonth;
					}
					if (pBalanceReport->pEarning) {
						LLIST pListMonth = GetSortedDataStructList(pBalanceReport->pEarning->pListMonth, 1);
						LListErase(pBalanceReport->pEarning->pListMonth);
						pBalanceReport->pEarning->pListMonth = pListMonth;
					}
					if (pBalanceReport->pLossGains) {
						LLIST pListMonth = GetSortedDataStructList(pBalanceReport->pLossGains->pListMonth, 1);
						LListErase(pBalanceReport->pLossGains->pListMonth);
						pBalanceReport->pLossGains->pListMonth = pListMonth;
					}
					if (pBalanceReport->pFinancial) {
						LLIST pListMonth = GetSortedDataStructList(pBalanceReport->pFinancial->pListMonth, 1);
						LListErase(pBalanceReport->pFinancial->pListMonth);
						pBalanceReport->pFinancial->pListMonth = pListMonth;
					}
					if (pBalanceReport->pOutcome) {
						LLIST pListMonth = GetSortedDataStructList(pBalanceReport->pOutcome->pListMonth, 1);
						LListErase(pBalanceReport->pOutcome->pListMonth);
						pBalanceReport->pOutcome->pListMonth = pListMonth;
					}
					if (pBalanceReport->pAuxiliary) {
						LLIST pListMonth = GetSortedDataStructList(pBalanceReport->pAuxiliary->pListMonth, 1);
						LListErase(pBalanceReport->pAuxiliary->pListMonth);
						pBalanceReport->pAuxiliary->pListMonth = pListMonth;
					}
					if (pBalanceReport->pResult) {
						LLIST pListMonth = GetSortedDataStructList(pBalanceReport->pResult->pListMonth, 1);
						LListErase(pBalanceReport->pResult->pListMonth);
						pBalanceReport->pResult->pListMonth = pListMonth;
					}
					if (pBalanceReport->pTotal) {
						LLIST pListMonth = GetSortedDataStructList(pBalanceReport->pTotal->pListMonth, 1);
						LListErase(pBalanceReport->pTotal->pListMonth);
						pBalanceReport->pTotal->pListMonth = pListMonth;
					}
					{
						BalanceAccountType *pBalanceAccount
							= LListGetFirstRecordData(pBalanceReport->pListBalanceAccount);
						while (pBalanceAccount) {
							LLIST pListMonth = GetSortedDataStructList(pBalanceAccount->pListMonth, 1);
							LListErase(pBalanceAccount->pListMonth);
							pBalanceAccount->pListMonth = pListMonth;

							pBalanceAccount = LListGetNextRecordData(pBalanceReport->pListBalanceAccount);
						}
						{
							LLIST pListBalanceAccount = GetSortedDataStructList(pBalanceReport->pListBalanceAccount, 1);
							LListErase(pBalanceReport->pListBalanceAccount);
							pBalanceReport->pListBalanceAccount = pListBalanceAccount;
						}
					}

					pBalanceReport = LListGetNextRecordData(pLedger->pListReport);
				}
				{
					LLIST pListReport = GetSortedDataStructList(pLedger->pListReport, 1);
					LListErase(pLedger->pListReport);
					pLedger->pListReport = pListReport;
				}
			}

			if (pLedger->pListVoucher) {
				LLIST pListVoucher = GetSortedDataStructList(pLedger->pListVoucher, VoucherSortOption); // VOUCHERS_BY_SERIES VOUCHERS_BY_DATE:
				LListErase(pLedger->pListVoucher);
				pLedger->pListVoucher = pListVoucher;
			}
			pLedger = LListGetNextRecordData(pAccounting->pLedgersChart->pListLedger);
		}
		{
			LLIST pListLedger = GetSortedDataStructList(pAccounting->pLedgersChart->pListLedger, 1);
			LListErase(pAccounting->pLedgersChart->pListLedger);
			pAccounting->pLedgersChart->pListLedger = pListLedger;
		}
	}

}

/********************************************************************

void RefreshAccountingData(void *vpAccounting)

*********************************************************************/
void RefreshAccountingData(void *vpAccounting)
{
	SumUpAccounting(vpAccounting);
	SumUpBalanceTotals(vpAccounting);
	Sort(vpAccounting, VOUCHERS_BY_DATE);
}

/********************************************************************

Interpret, write, copy, merge and destroy the major data set

*********************************************************************/
/********************************************************************

BOOL IsData(

*********************************************************************/
BOOL IsData(
	AccountingType *pAccounting,
	ObjectType *pObject,
	DATEINT PeriodDate,
	char *PeriodText,
	char *pSeries)
{
	LedgerType *pLedger = GetLedger(pAccounting, 0);
	if (pSeries && (((!strcmp(pSeries, ALL_BULLET))) || (!*pSeries)))
		pSeries = 0;
	if (PeriodDate && (PeriodDate < pLedger->YearStart))
		pLedger = GetLedgerByDate(pAccounting, PeriodDate);
	if (!pObject)
		pObject = GetDimObject(pAccounting->pObjectsChart->pListDimension, 0, "0");
	while (pLedger) {
		BalanceReportType *pBalanceReport = GetBalanceReport(pLedger, pObject);
		if (pBalanceReport && (!pSeries)) {
			BalanceAccountType *pBalanceAccount
				= LListGetFirstRecordData(pBalanceReport->pListBalanceAccount);
			while (pBalanceAccount) {
				if (PeriodDate) {
					if (GetBalanceAccountPeriod(pBalanceAccount, PeriodDate, PeriodText))
						return TRUE;
				}
				else {
					if (pBalanceAccount->Opening || pBalanceAccount->OpeningQuantity || pBalanceAccount->Forward || pBalanceAccount->ForwardQuantity)
						return TRUE;
				}
				pBalanceAccount = LListGetNextRecordData(pBalanceReport->pListBalanceAccount);
			}
		}
		{
			VoucherType *pVoucher
				= LListGetFirstRecordData(pLedger->pListVoucher);
			while (pVoucher) {
				if (((!PeriodDate) || (GetLastDateInPeriod(pVoucher->Date, PeriodText) == PeriodDate))
					&& ((!pSeries) || (!strcmp(pSeries, pVoucher->pId->pseries)))) {
					VoucherEntryType *pVoucherEntry
						= LListGetFirstRecordData(pVoucher->pListEntry);
					while (pVoucherEntry) {
						if ((!pObject) || (!pObject->pDimension->Id))
							return TRUE;
						else {
							ObjectType *pEntryObject
								= LListGetFirstRecordData(pVoucherEntry->pListObject);
							while (pEntryObject) {
								if (pEntryObject == pObject)
									return TRUE;
								pEntryObject = LListGetNextRecordData(pVoucherEntry->pListObject);
							}
						}
						pVoucherEntry = LListGetNextRecordData(pVoucher->pListEntry);
					}
				}
				pVoucher = LListGetNextRecordData(pLedger->pListVoucher);
			}
		}
		pLedger = LListGetNextRecordData(pAccounting->pLedgersChart->pListLedger);
	}

	return FALSE; // Not found
}

/********************************************************************

static BOOL IsBalance(

*********************************************************************/
static BOOL IsBalance(
	void *vpAccounting)
{
	AccountingType *pAccounting = (AccountingType *)vpAccounting;
	if (pAccounting->pLedgersChart->pListLedger) {
		// Start from where the cursor is (Optimisation)
		LedgerType *pLedger
			= LListGetFirstRecordData(pAccounting->pLedgersChart->pListLedger);
		while (pLedger) {
			BalanceReportType *pBalanceReport = LListGetFirstRecordData(pLedger->pListReport);
			if (pBalanceReport) {
				BalanceAccountType *pBalanceAccount
					= LListGetFirstRecordData(pBalanceReport->pListBalanceAccount);
				if (pBalanceAccount->Forward
					|| (!DecimalCmp(pBalanceAccount->ForwardQuantity, 0))
					|| pBalanceAccount->Opening
					|| (!DecimalCmp(pBalanceAccount->OpeningQuantity, 0)))
					return TRUE; // OK
			}
			pLedger = LListGetNextRecordData(pAccounting->pLedgersChart->pListLedger);
		}
	}
	return FALSE; // Not found
}

/********************************************************************

BOOL IsPeriodic(

*********************************************************************/
BOOL IsPeriodic(
	void *vpAccounting)
{
	AccountingType *pAccounting = (AccountingType *)vpAccounting;
	if (pAccounting->pLedgersChart->pListLedger) {
		LedgerType *pLedger
			= LListGetFirstRecordData(pAccounting->pLedgersChart->pListLedger);
		while (pLedger) {
			if (IsLedgerPeriodic(pLedger))
				return TRUE;
			pLedger = LListGetNextRecordData(pAccounting->pLedgersChart->pListLedger);
		}
	}
	return FALSE; // Not found
}

/********************************************************************

BOOL IsLedgerPeriodic(

*********************************************************************/
BOOL IsLedgerPeriodic(
	LedgerType *pLedger)
{
	if (pLedger) {
		BalanceReportType *pBalanceReport
			= LListGetFirstRecordData(pLedger->pListReport);
		while (pBalanceReport) {
			BalanceAccountType *pBalanceAccount
				= LListGetFirstRecordData(pBalanceReport->pListBalanceAccount);
			while (pBalanceAccount) {
				if (LListGetFirstRecordData(pBalanceAccount->pListMonth))
					return TRUE;
				pBalanceAccount = LListGetNextRecordData(pBalanceReport->pListBalanceAccount);
			}
			pBalanceReport = LListGetNextRecordData(pLedger->pListReport);
		}
	}
	return FALSE; // Not found
}

/********************************************************************

BOOL IsBudget(

*********************************************************************/
BOOL IsBudget(
	void *vpAccounting,
	AccountType *pAccount,
	char *pAccountType,
	ObjectType *pObject,
	DATEINT PeriodDate,
	char *pPeriodText)
{
	AccountingType *pAccounting = (AccountingType *)vpAccounting;
	ObjectType* pAllObject = GetDimObject(pAccounting->pObjectsChart->pListDimension, 0, "0");
	if (pAccounting->pLedgersChart->pListLedger) {
		LedgerType *pLedger
			= LListGetFirstRecordData(pAccounting->pLedgersChart->pListLedger);
		while (pLedger) {
			BalanceReportType *pBalanceReport
				= LListGetFirstRecordData(pLedger->pListReport);
			while (pBalanceReport) {
				if (((!pObject) && (pBalanceReport->pObject == pAllObject)) 
					|| (pBalanceReport->pObject == pObject)) {
					BalanceAccountType *pBalanceAccount
						= LListGetFirstRecordData(pBalanceReport->pListBalanceAccount);
					while (pBalanceAccount) {
						if (((!pAccount) || (pBalanceAccount->pAccount == pAccount))
							&& AccountTypeCmp(pBalanceAccount->pAccount->pType, pAccountType)) {
							if ((!PeriodDate) || (!((pPeriodText[1] == '1') || (pPeriodText[1] == '2') || (pPeriodText[1] == '3') || (pPeriodText[1] == '4')))) {
								BalanceAccountPeriodType *pBalanceAccountPeriod
									= LListGetFirstRecordData(pBalanceAccount->pListMonth);
								while (pBalanceAccountPeriod) {
									if (((!PeriodDate) || (pBalanceAccountPeriod->End == PeriodDate))
										&& (pBalanceAccountPeriod->Budget || (!DecimalCmp(pBalanceAccountPeriod->BudgetQuantity, 0))))
										return TRUE;
									pBalanceAccountPeriod = LListGetNextRecordData(pBalanceAccount->pListMonth);
								}
							}
							if ((!PeriodDate) || ((*pPeriodText == ANNUAL)
								&& ((pPeriodText[1] == '1') || (pPeriodText[1] == '2') || (pPeriodText[1] == '3') || (pPeriodText[1] == '4')))) {
								DATEINT LabelPeriodDate = GetPeriodDate(PeriodDate, pPeriodText);
								BalanceAccountPeriodType *pBalanceAccountPeriod
									= LListGetFirstRecordData(pBalanceAccount->pListAnnual);
								while (pBalanceAccountPeriod) {
									if (((!LabelPeriodDate) || (pBalanceAccountPeriod->End == LabelPeriodDate))
										&& (pBalanceAccountPeriod->Budget || (!DecimalCmp(pBalanceAccountPeriod->BudgetQuantity, 0))))
										return TRUE;
									pBalanceAccountPeriod = LListGetNextRecordData(pBalanceAccount->pListAnnual);
								}
							}
							if ((!PeriodDate) || ((*pPeriodText == SEMI)
								&& ((pPeriodText[1] == '1') || (pPeriodText[1] == '2') || (pPeriodText[1] == '3') || (pPeriodText[1] == '4')))) {
								DATEINT LabelPeriodDate = GetPeriodDate(PeriodDate, pPeriodText);
								BalanceAccountPeriodType *pBalanceAccountPeriod
									= LListGetFirstRecordData(pBalanceAccount->pListSemi);
								while (pBalanceAccountPeriod) {
									if (((!LabelPeriodDate) || (pBalanceAccountPeriod->End == LabelPeriodDate))
										&& (pBalanceAccountPeriod->Budget || (!DecimalCmp(pBalanceAccountPeriod->BudgetQuantity, 0))))
										return TRUE;
									pBalanceAccountPeriod = LListGetNextRecordData(pBalanceAccount->pListSemi);
								}
							}
							if ((!PeriodDate) || ((*pPeriodText == TERTIAL)
								&& ((pPeriodText[1] == '1') || (pPeriodText[1] == '2') || (pPeriodText[1] == '3') || (pPeriodText[1] == '4')))) {
								DATEINT LabelPeriodDate = GetPeriodDate(PeriodDate, pPeriodText);
								BalanceAccountPeriodType *pBalanceAccountPeriod
									= LListGetFirstRecordData(pBalanceAccount->pListTertial);
								while (pBalanceAccountPeriod) {
									if (((!LabelPeriodDate) || (pBalanceAccountPeriod->End == LabelPeriodDate))
										&& (pBalanceAccountPeriod->Budget || (!DecimalCmp(pBalanceAccountPeriod->BudgetQuantity, 0))))
										return TRUE;
									pBalanceAccountPeriod = LListGetNextRecordData(pBalanceAccount->pListTertial);
								}
							}
							if ((!PeriodDate) || ((*pPeriodText == QUARTER)
								&& ((pPeriodText[1] == '1') || (pPeriodText[1] == '2') || (pPeriodText[1] == '3') || (pPeriodText[1] == '4')))) {
								DATEINT LabelPeriodDate = GetPeriodDate(PeriodDate, pPeriodText);
								BalanceAccountPeriodType *						pBalanceAccountPeriod
									= LListGetFirstRecordData(pBalanceAccount->pListQuarter);
								while (pBalanceAccountPeriod) {
									if (((!LabelPeriodDate) || (pBalanceAccountPeriod->End == LabelPeriodDate))
										&& (pBalanceAccountPeriod->Budget || (!DecimalCmp(pBalanceAccountPeriod->BudgetQuantity, 0))))
										return TRUE;
									pBalanceAccountPeriod = LListGetNextRecordData(pBalanceAccount->pListQuarter);
								}
							}
						}
						pBalanceAccount = LListGetNextRecordData(pBalanceReport->pListBalanceAccount);
					}
				}
				pBalanceReport = LListGetNextRecordData(pLedger->pListReport);
			}
			pLedger = LListGetNextRecordData(pAccounting->pLedgersChart->pListLedger);
		}
	}
	return FALSE; // Not found
}

/********************************************************************

static BOOL IsQuantity(

*********************************************************************/
static BOOL IsQuantity(
	void *vpAccounting)
{
	AccountingType *pAccounting = (AccountingType *)vpAccounting;
	if (pAccounting->pLedgersChart->pListLedger) {
		LedgerType *pLedger
			= LListGetFirstRecordData(pAccounting->pLedgersChart->pListLedger);
		while (pLedger) {
			BalanceReportType *pBalanceReport
				= LListGetFirstRecordData(pLedger->pListReport);
			while (pBalanceReport) {
				BalanceAccountType *pBalanceAccount
					= LListGetFirstRecordData(pBalanceReport->pListBalanceAccount);
				while (pBalanceAccount) {
					BalanceAccountPeriodType *pBalanceAccountPeriod
						= LListGetFirstRecordData(pBalanceAccount->pListMonth);
					while (pBalanceAccountPeriod) {
						if ((!DecimalCmp(pBalanceAccountPeriod->OutcomeQuantity, 0)) || (!DecimalCmp(pBalanceAccountPeriod->BudgetQuantity, 0)))
							return TRUE;
						pBalanceAccountPeriod = LListGetNextRecordData(pBalanceAccount->pListMonth);
					}
					if ((!DecimalCmp(pBalanceAccount->OpeningQuantity, 0)) || (!DecimalCmp(pBalanceAccount->ForwardQuantity, 0)))
						return TRUE;
					pBalanceAccount = LListGetNextRecordData(pBalanceReport->pListBalanceAccount);
				}
				pBalanceReport = LListGetNextRecordData(pLedger->pListReport);
			}
			{
				VoucherType *pVoucher
					= LListGetFirstRecordData(pLedger->pListVoucher);
				while (pVoucher) {
					VoucherEntryType *pVoucherEntry
						= LListGetFirstRecordData(pVoucher->pListEntry);
					while (pVoucherEntry) {
						if (!DecimalCmp(pVoucherEntry->Quantity, 0))
							return TRUE;
						pVoucherEntry = LListGetNextRecordData(pVoucher->pListEntry);
					}
					pVoucher = LListGetNextRecordData(pLedger->pListVoucher);
				}
			}
			pLedger = LListGetNextRecordData(pAccounting->pLedgersChart->pListLedger);
		}
	}
	return FALSE; // Not found
}

/********************************************************************

BOOL IsEntity(

*********************************************************************/
static BOOL IsEntity(
	void *vpAccounting)
{
	AccountingType *pAccounting = (AccountingType *)vpAccounting;
	if (pAccounting->pEntity && (pAccounting->pEntity->pName || pAccounting->pEntity->pLegalRegId))
		return TRUE;
	return FALSE;
}

/********************************************************************

BOOL IsVouchers(

*********************************************************************/
BOOL IsVouchers(
	void *vpAccounting)
{
	AccountingType *pAccounting = (AccountingType *)vpAccounting;
	if (pAccounting->pLedgersChart->pListLedger) {
		// Start from where the cursor is (Optimisation)
		LedgerType *pLedger
			= LListGetFirstRecordData(pAccounting->pLedgersChart->pListLedger);
		while (pLedger) {
			if (LListGetFirstRecordData(pLedger->pListVoucher))
				return TRUE; // OK
			pLedger = LListGetNextRecordData(pAccounting->pLedgersChart->pListLedger);
		}
	}
	return FALSE; // Not found
}

/********************************************************************

BOOL IsDimensionActive(

*********************************************************************/
BOOL IsDimensionActive(
	void *vpAccounting,
	DimensionType *pDimension)
{
	AccountingType *pAccounting = (AccountingType *)vpAccounting;
	if (pDimension && pAccounting->pLedgersChart->pListLedger) {
		// Start from where the cursor is (Optimisation)
		ObjectType *pObject
			= LListGetFirstRecordData(pDimension->pListObject);
		while (pObject) {
			if (!pObject->InActive)
				return TRUE; // OK
			pObject = LListGetNextRecordData(pDimension->pListObject);
		}
	}
	return FALSE; // Not found
}

/********************************************************************

BOOL IsObject(

*********************************************************************/
BOOL IsObject(
	void *vpAccounting)
{
	AccountingType *pAccounting = (AccountingType *)vpAccounting;
	if (pAccounting->pObjectsChart->pListDimension) {
		DimensionType *pDimension
			= LListGetFirstRecordData(pAccounting->pObjectsChart->pListDimension);
		// Alla accounting object 0-"0" does not count
		pDimension = LListGetNextRecordData(pAccounting->pObjectsChart->pListDimension);
		while (pDimension) {
			if (LListGetFirstRecordData(pDimension->pListObject))
				return TRUE; // OK
			pDimension = LListGetNextRecordData(pAccounting->pObjectsChart->pListDimension);
		}
	}
	return FALSE; // Not found
}

/********************************************************************

void CheckActive(

*********************************************************************/
void CheckActive(
	void *vpAccounting,
	void *vpObject,
	DATEINT dPeriod,
	char *pPeriodTabText,
	BOOL bVoucherOnly,
	char *pVoucherSeries)
{
	AccountingType *pAccounting = (AccountingType *)vpAccounting;
	ObjectType *pSelObject = vpObject;
	ObjectType *pAllObject = GetDimObject(pAccounting->pObjectsChart->pListDimension, 0, "0");

	if (pAccounting->pAccountsChart->pListAccount) {
		void *pCursorRecord = pAccounting->pAccountsChart->pListAccount->CursorRecord;
		AccountType *pAccount
			= LListGetFirstRecordData(pAccounting->pAccountsChart->pListAccount);
		while (pAccount) {
			pAccount->InActive = TRUE;
			pAccount = LListGetNextRecordData(pAccounting->pAccountsChart->pListAccount);
		}
		pAccounting->pAccountsChart->pListAccount->CursorRecord = pCursorRecord;
	}
	if (pAccounting->pObjectsChart->pListDimension) {
		void *pCursorRecord = pAccounting->pObjectsChart->pListDimension->CursorRecord;
		DimensionType *pDimension
			= LListGetFirstRecordData(pAccounting->pObjectsChart->pListDimension);
		while (pDimension) {
			void* pObjectCursorRecord = pDimension->pListObject->CursorRecord;
			ObjectType *pObject
				= LListGetFirstRecordData(pDimension->pListObject);
			while (pObject) {
				pObject->InActive = TRUE;
				pObject = LListGetNextRecordData(pDimension->pListObject);
			}
			pDimension->pListObject->CursorRecord = pObjectCursorRecord;
			pDimension->InActive = TRUE;
			pDimension = LListGetNextRecordData(pAccounting->pObjectsChart->pListDimension);
		}
		pAccounting->pObjectsChart->pListDimension->CursorRecord = pCursorRecord;
	}
	pAllObject->InActive = FALSE; // The all object can't be inactive 

	if (pAccounting->pLedgersChart->pListLedger) {
		void *pLedgerCursorRecord = pAccounting->pLedgersChart->pListLedger->CursorRecord;
		LedgerType *pLedger
			= LListGetFirstRecordData(pAccounting->pLedgersChart->pListLedger);
		while (pLedger) {
			if (!bVoucherOnly) {
				void *pBalanceReportCursorRecord = pLedger->pListReport->CursorRecord;
				BalanceReportType *pBalanceReport
					= LListGetFirstRecordData(pLedger->pListReport);
				while (pBalanceReport) {
					ObjectType *pObject = pBalanceReport->pObject;
					if (!pObject)
						pObject = pAllObject;
					{
						void* pBalanceAccountCursorRecord = pBalanceReport->pListBalanceAccount->CursorRecord;
						BalanceAccountType *pBalanceAccount
							= LListGetFirstRecordData(pBalanceReport->pListBalanceAccount);
						while (pBalanceAccount) {
							if (dPeriod) {
								DATEINT TempPeriodDate = GetLastDateInPeriod(dPeriod, pPeriodTabText);
								while (TempPeriodDate >= pLedger->YearStart) {
									BalanceAccountPeriodType *pAccountPeriod = GetBalanceAccountPeriod(pBalanceAccount, TempPeriodDate, pPeriodTabText);
									if (pAccountPeriod && (pAccountPeriod->Outcome || (!DecimalCmp(pAccountPeriod->OutcomeQuantity, 0)) || pAccountPeriod->Budget || (!DecimalCmp(pAccountPeriod->BudgetQuantity, 0)))) {
										if (pObject != pAllObject)
											pAllObject = pAllObject;
										if (pObject->InActive)
											pObject->InActive = FALSE;
										if (pObject->pDimension->InActive)
											pObject->pDimension->InActive = FALSE;
										if (pObject->pDimension->pSuperDimension)
											((DimensionType *)pObject->pDimension->pSuperDimension)->InActive = FALSE;
										if ((!pSelObject) || (pObject == pSelObject))
											pBalanceAccount->pAccount->InActive = FALSE;
									}
									TempPeriodDate -= 10000;
								}
							}
							else if (pBalanceAccount->Opening || (!DecimalCmp(pBalanceAccount->OpeningQuantity, 0)) || pBalanceAccount->Forward || (!DecimalCmp(pBalanceAccount->ForwardQuantity, 0))) {
								if (pObject->InActive)
									pObject->InActive = FALSE;
								if (pObject->pDimension->InActive)
									pObject->pDimension->InActive = FALSE;
								if (pObject->pDimension->pSuperDimension)
									((DimensionType *)pObject->pDimension->pSuperDimension)->InActive = FALSE;
								if ((!pSelObject) || (pObject == pSelObject))
									pBalanceAccount->pAccount->InActive = FALSE;
							}
							else { // In some cases there can be period data wothout Account object data
								void* pAccountPeriodCursorRecord = pBalanceAccount->pListMonth->CursorRecord;
								BalanceAccountPeriodType *pAccountPeriod
									= LListGetFirstRecordData(pBalanceAccount->pListMonth);
								while (pAccountPeriod) {
									if (pAccountPeriod && (pAccountPeriod->Outcome || (!DecimalCmp(pAccountPeriod->OutcomeQuantity, 0)) || pAccountPeriod->Budget || (!DecimalCmp(pAccountPeriod->BudgetQuantity, 0)))) {
										if (pObject != pAllObject)
											pAllObject = pAllObject;
										if (pObject->InActive)
											pObject->InActive = FALSE;
										if (pObject->pDimension->InActive)
											pObject->pDimension->InActive = FALSE;
										if (pObject->pDimension->pSuperDimension)
											((DimensionType *)pObject->pDimension->pSuperDimension)->InActive = FALSE;
										if ((!pSelObject) || (pObject == pSelObject))
											pBalanceAccount->pAccount->InActive = FALSE;
									}
									pAccountPeriod = LListGetNextRecordData(pBalanceAccount->pListMonth);
								}
								pBalanceAccount->pListMonth->CursorRecord = pAccountPeriodCursorRecord;
							}
							pBalanceAccount = LListGetNextRecordData(pBalanceReport->pListBalanceAccount);
						}
						pBalanceReport->pListBalanceAccount->CursorRecord = pBalanceAccountCursorRecord;
					}
					pBalanceReport = LListGetNextRecordData(pLedger->pListReport);
				}
				pLedger->pListReport->CursorRecord = pBalanceReportCursorRecord;
			}
			if (pVoucherSeries && *pVoucherSeries && (!strcmp(pVoucherSeries, ALL_BULLET)))
				pVoucherSeries = 0; // If All no Series
			{
				void* pVoucherCursorRecord = pLedger->pListVoucher->CursorRecord;
				VoucherType *pVoucher
					= LListGetFirstRecordData(pLedger->pListVoucher);
				while (pVoucher) {
					if (((!dPeriod) || (GetLastDateInPeriod(dPeriod, pPeriodTabText) == GetLastDateInPeriod(pVoucher->Date, pPeriodTabText)))
						&& ((!bVoucherOnly) || (!(pVoucherSeries && *pVoucherSeries)) || (!strcmp(pVoucherSeries, pVoucher->pId->pseries)))) {
						void* pEntryCursorRecord = pVoucher->pListEntry->CursorRecord;
						VoucherEntryType *pEntry
							= LListGetFirstRecordData(pVoucher->pListEntry);
						while (pEntry) {
							if (!pEntry->pDeprecated) {
								if ((!pSelObject) || (pSelObject == pAllObject))
									pEntry->pAccount->InActive = FALSE;
								{
									void* pObjectCursorRecord = pEntry->pListObject->CursorRecord;
									ObjectType *pObject
										= LListGetFirstRecordData(pEntry->pListObject);
									while (pObject) {
										if (pObject->InActive)
											pObject->InActive = FALSE;
										if (pObject->pDimension->InActive)
											pObject->pDimension->InActive = FALSE;
										if (pObject->pDimension->pSuperDimension)
											((DimensionType *)pObject->pDimension->pSuperDimension)->InActive = FALSE;
										if (pSelObject == pObject)
											pEntry->pAccount->InActive = FALSE;
										pObject = LListGetNextRecordData(pEntry->pListObject);
									}
									pEntry->pListObject->CursorRecord = pObjectCursorRecord;
								}
							}
							pEntry = LListGetNextRecordData(pVoucher->pListEntry);
						}
						pVoucher->pListEntry->CursorRecord = pEntryCursorRecord;
					}
					pVoucher = LListGetNextRecordData(pLedger->pListVoucher);
				}
				pLedger->pListVoucher->CursorRecord = pVoucherCursorRecord;
			}
			pLedger = LListGetNextRecordData(pAccounting->pLedgersChart->pListLedger);
		}
		pAccounting->pLedgersChart->pListLedger->CursorRecord = pLedgerCursorRecord;
	}
}

/********************************************************************

static void CheckPeriodRange(

*********************************************************************/
static void CheckPeriodRange(
	AccountingType *pAccounting)
{
	// Clear fatal field error Period data out of Ledger rage errors, in read file data
	if (pAccounting && pAccounting->pLedgersChart) {
		LedgerType *pLedger = LListGetFirstRecordData(pAccounting->pLedgersChart->pListLedger);
		while (pLedger) {
			BalanceReportType *pBalanceReport = LListGetFirstRecordData(pLedger->pListReport);
			while (pBalanceReport) {
				BalanceAccountType *pBalanceAccount = LListGetFirstRecordData(pBalanceReport->pListBalanceAccount);
				while (pBalanceAccount) {
					BalanceAccountPeriodType *pAccountPeriod = LListGetFirstRecordData(pBalanceAccount->pListMonth);
					while (pAccountPeriod) {
						if (!((pLedger->YearStart <= pAccountPeriod->End) && (pLedger->YearEnd >= pAccountPeriod->End))) {
							DestroyBalanceAccountPeriod(&pAccountPeriod);
							LListEraseCursorRecord(pBalanceAccount->pListMonth);
						}
						pAccountPeriod = LListGetNextRecordData(pBalanceAccount->pListMonth);
					}
					pAccountPeriod = LListGetFirstRecordData(pBalanceAccount->pListAnnual);
					while (pAccountPeriod) {
						if (!((pLedger->YearStart <= pAccountPeriod->End) || (pLedger->YearEnd >= (GetLastDateInMonth(pAccountPeriod->End - (ANNUAL_LENGTH-100)))))) {
							DestroyBalanceAccountPeriod(&pAccountPeriod);
							LListEraseCursorRecord(pBalanceAccount->pListAnnual);
						}
						pAccountPeriod = LListGetNextRecordData(pBalanceAccount->pListAnnual);
					}
					pAccountPeriod = LListGetFirstRecordData(pBalanceAccount->pListSemi);
					while (pAccountPeriod) {
						if (!((pLedger->YearStart <= pAccountPeriod->End) || (pLedger->YearEnd >= (GetLastDateInMonth(pAccountPeriod->End - (SEMI_LENGTH - 100)))))) {
							DestroyBalanceAccountPeriod(&pAccountPeriod);
							LListEraseCursorRecord(pBalanceAccount->pListSemi);
						}
						pAccountPeriod = LListGetNextRecordData(pBalanceAccount->pListSemi);
					}
					pAccountPeriod = LListGetFirstRecordData(pBalanceAccount->pListTertial);
					while (pAccountPeriod) {
						if (!((pLedger->YearStart <= pAccountPeriod->End) || (pLedger->YearEnd >= (GetLastDateInMonth(pAccountPeriod->End - (TERTIAL_LENGTH - 100)))))) {
							DestroyBalanceAccountPeriod(&pAccountPeriod);
							LListEraseCursorRecord(pBalanceAccount->pListTertial);
						}
						pAccountPeriod = LListGetNextRecordData(pBalanceAccount->pListTertial);
					}
					pAccountPeriod = LListGetFirstRecordData(pBalanceAccount->pListQuarter);
					while (pAccountPeriod) {
						if (!((pLedger->YearStart <= pAccountPeriod->End) || (pLedger->YearEnd >= (GetLastDateInMonth(pAccountPeriod->End - (QUARTER_LENGTH - 100)))))) {
							DestroyBalanceAccountPeriod(&pAccountPeriod);
							LListEraseCursorRecord(pBalanceAccount->pListQuarter);
						}
						pAccountPeriod = LListGetNextRecordData(pBalanceAccount->pListQuarter);
					}
					pBalanceAccount = LListGetNextRecordData(pBalanceReport->pListBalanceAccount);
				}
				pBalanceReport = LListGetNextRecordData(pLedger->pListReport);
			}
			pLedger = LListGetNextRecordData(pAccounting->pLedgersChart->pListLedger);
		}
	}
}

/********************************************************************

static void CheckAccounting(

*********************************************************************/
static void CheckAccounting(
	InstanceDataType *pInstanceData)
{
	AccountingType *pAccounting = pInstanceData->vpWorkData;
	CheckPeriodRange(pAccounting);
	Sort(pAccounting, VOUCHERS_BY_DATE); //VOUCHERS_BY_SERIES);
	if (IsVouchers(pAccounting))
		CheckActive(pAccounting, 0, 0, 0, TRUE, 0);
	else
		CheckActive(pAccounting, 0, 0, 0, FALSE, 0);
	pInstanceData->bIsEntity = IsEntity(pAccounting);
	pInstanceData->bIsBalance = IsBalance(pAccounting);
	pInstanceData->bIsQuantity = IsQuantity(pAccounting);
	pInstanceData->bIsObject = IsObject(pAccounting);
	pInstanceData->bIsBudget = IsBudget(pAccounting, 0, 0, 0, 0, 0);
	pInstanceData->bIsVouchers = IsVouchers(pAccounting);
	pInstanceData->iAccountLength = AccountLength(pAccounting);
	pInstanceData->iThousandSets = ThousandSets(pAccounting);
	if (pInstanceData->iThousandSets == 0)
		pInstanceData->iThousandSets = 1;
	/*		if(pInstanceData->iThousandSets>3)
	pInstanceData->bIsQuantity = FALSE;
	if(pInstanceData->iThousandSets > 4) // No screenspace
	pInstanceData->iThousandSets = 4;
	*/
	if (pInstanceData->bIsBalance) {
		if (pInstanceData->bIsVouchers)
			CopyString(&pAccounting->pContent, CONTENT_LABEL_LEDGER);
		else
			CopyString(&pAccounting->pContent, CONTENT_LABEL_BALANCE);
	}
	else if (pInstanceData->bIsVouchers)
		CopyString(&pAccounting->pContent, CONTENT_LABEL_VOUCHER);
	else
		CopyString(&pAccounting->pContent, CONTENT_LABEL_CHARTS);
}

/********************************************************************

BOOL AccountTypeCmp(

*********************************************************************/
BOOL AccountTypeCmp(
	char *pAccountType,
	char *pAccountTypeCmp)
{
	if (!pAccountTypeCmp)
		return TRUE;
	if (!strcmp(pAccountTypeCmp, ACCOUNT_TYPE_ACTIVE)) {
		if (!strcmp(pAccountType, ACCOUNT_TYPE_TANGIBLE))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_STOCK))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_RECEIVABLES))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_ASSETS))
			return TRUE;
		else
			return FALSE;
	}
	else if (!strcmp(pAccountTypeCmp, ACCOUNT_TYPE_PASSIVE)) {
		if (!strcmp(pAccountType, ACCOUNT_TYPE_EQUITIES))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_PROVISIONS))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_LIABILITIES))
			return TRUE;
		else
			return FALSE;
	}
	else if (!strcmp(pAccountTypeCmp, ACCOUNT_TYPE_BALANCE)) {
		if (!strcmp(pAccountType, ACCOUNT_TYPE_TANGIBLE))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_STOCK))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_RECEIVABLES))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_ASSETS))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_EQUITIES))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_PROVISIONS))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_LIABILITIES))
			return TRUE;
		else
			return FALSE;
	}
	else if (!strcmp(pAccountTypeCmp, ACCOUNT_TYPE_PROFIT)) {
		if (!strcmp(pAccountType, ACCOUNT_TYPE_REVENUES))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_SUPPLIES))
			return TRUE;
		else
			return FALSE;
	}
	else if (!strcmp(pAccountTypeCmp, ACCOUNT_TYPE_EARNING)) {
		if (!strcmp(pAccountType, ACCOUNT_TYPE_REVENUES))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_SUPPLIES))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_EXPENSES))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_STAFF))
			return TRUE;
		else
			return FALSE;
	}
	else if (!strcmp(pAccountTypeCmp, ACCOUNT_TYPE_OUTCOME)) {
		if (!strcmp(pAccountType, ACCOUNT_TYPE_REVENUES))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_SUPPLIES))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_EXPENSES))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_STAFF))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_LOSS_GAINS))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_FINANCIAL))
			return TRUE;
		else
			return FALSE;
	}
	else if (!strcmp(pAccountTypeCmp, ACCOUNT_TYPE_RESULT)) {
		if (!strcmp(pAccountType, ACCOUNT_TYPE_REVENUES))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_SUPPLIES))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_EXPENSES))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_STAFF))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_LOSS_GAINS))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_FINANCIAL))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_AUX))
			return TRUE;
		else
			return FALSE;
	}
	else if (!strcmp(pAccountTypeCmp, ACCOUNT_TYPE_TOTAL)) {
		if (!strcmp(pAccountType, ACCOUNT_TYPE_TANGIBLE))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_STOCK))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_RECEIVABLES))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_ASSETS))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_EQUITIES))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_PROVISIONS))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_LIABILITIES))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_REVENUES))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_SUPPLIES))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_EXPENSES))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_STAFF))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_LOSS_GAINS))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_FINANCIAL))
			return TRUE;
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_AUX))
			return TRUE;
		else
			return FALSE;
	}
	else if (!strcmp(pAccountType, pAccountTypeCmp))
		return TRUE;
	else
		return FALSE;
}

/********************************************************************

static int AccountLength(

*********************************************************************/
static int AccountLength(
	void* vpAccounting)
{
	size_t iAccountLength = 0;
	AccountingType* pAccounting = (AccountingType*)vpAccounting;
	if (pAccounting->pAccountsChart->pListAccount) {
		AccountType* pAccount
			= LListGetFirstRecordData(pAccounting->pAccountsChart->pListAccount);
		while (pAccount) {
			if (iAccountLength < strlen(pAccount->pId))
				iAccountLength = strlen(pAccount->pId);
			pAccount = LListGetNextRecordData(pAccounting->pAccountsChart->pListAccount);
		}
	}
	return (int)iAccountLength;
}

/********************************************************************

static int CalcThousandSets(

*********************************************************************/
static int CalcThousandSets(
	AMOUNT Amount)
{
	int iThousandSets = 0;
	Amount /= 100;	// No decimals
	Amount /= 1000;	// First set is default
	while (Amount) {
		Amount /= 1000;
		iThousandSets++;
	}
	if (iThousandSets < 1)
		iThousandSets = 1;
	return iThousandSets;
}

/********************************************************************

static int ThousandSets(

*********************************************************************/
static int ThousandSets(
	void *vpAccounting)
{
	int iThousandSets = 0;
	AccountingType *pAccounting = (AccountingType *)vpAccounting;
	if (pAccounting->pLedgersChart->pListLedger) {
		LedgerType *pLedger
			= LListGetFirstRecordData(pAccounting->pLedgersChart->pListLedger);
		while (pLedger) {
			BalanceReportType *pBalanceReport
				= LListGetFirstRecordData(pLedger->pListReport);
			while (pBalanceReport) {
				BalanceAccountType *pBalanceAccount
					= LListGetFirstRecordData(pBalanceReport->pListBalanceAccount);
				while (pBalanceAccount) {
					BalanceAccountPeriodType *pBalanceAccountPeriod
						= LListGetFirstRecordData(pBalanceAccount->pListMonth);
					while (pBalanceAccountPeriod) {
						if (CalcThousandSets(llabs(pBalanceAccountPeriod->Outcome)) > iThousandSets)
							iThousandSets = CalcThousandSets(llabs(pBalanceAccountPeriod->Outcome));
						if (CalcThousandSets(llabs(pBalanceAccountPeriod->Budget)) > iThousandSets)
							iThousandSets = CalcThousandSets(llabs(pBalanceAccountPeriod->Budget));
						pBalanceAccountPeriod = LListGetNextRecordData(pBalanceAccount->pListMonth);
					}
					if (CalcThousandSets(llabs(pBalanceAccount->Opening)) > iThousandSets)
						iThousandSets = CalcThousandSets(llabs(pBalanceAccount->Opening));
					if (CalcThousandSets(llabs(pBalanceAccount->Forward)) > iThousandSets)
						iThousandSets = CalcThousandSets(llabs(pBalanceAccount->Forward));
					pBalanceAccount = LListGetNextRecordData(pBalanceReport->pListBalanceAccount);
				}
				pBalanceReport = LListGetNextRecordData(pLedger->pListReport);
			}
			{
				VoucherType *pVoucher
					= LListGetFirstRecordData(pLedger->pListVoucher);
				while (pVoucher) {
					VoucherEntryType *pVoucherEntry
						= LListGetFirstRecordData(pVoucher->pListEntry);
					while (pVoucherEntry) {
						if (CalcThousandSets(llabs(pVoucherEntry->Amount)) > iThousandSets)
							iThousandSets = CalcThousandSets(llabs(pVoucherEntry->Amount));
						pVoucherEntry = LListGetNextRecordData(pVoucher->pListEntry);
					}
					pVoucher = LListGetNextRecordData(pLedger->pListVoucher);
				}
			}
			pLedger = LListGetNextRecordData(pAccounting->pLedgersChart->pListLedger);
		}
	}
	return iThousandSets;
}

/********************************************************************

InstanceDataType *DestroyInstanceData(

*********************************************************************/
InstanceDataType *DestroyInstanceData(
	InstanceDataType *pInstanceData)
{
	if (pInstanceData) {
		DestroyWorkData(&pInstanceData->vpWorkData);
		FreeString(&pInstanceData->ProgramFilePath);
		FreeString(&pInstanceData->ProgramFilePathName);
		FreeString(&pInstanceData->pWorkFileName);
		FreeString(&pInstanceData->pWorkFilePath);
		FreeString(&pInstanceData->pWorkFilePathName);
		LListDestroy(pInstanceData->plErrorList);
		free(pInstanceData);
		pInstanceData = 0;
	}
	return pInstanceData;
}

/********************************************************************

static InstanceDataType *CopyInstanceDataBody(

*********************************************************************/
static InstanceDataType *CopyInstanceDataBody(
	InstanceDataType *pInstanceData,
	void *vpAccounting)
{
	if (pInstanceData) {
		InstanceDataType *pNewInstanceData = calloc(1, sizeof(InstanceDataType));
		if (pNewInstanceData) {
			pNewInstanceData->vpWorkData = vpAccounting;
			pNewInstanceData->plErrorList = LListInit();
			pNewInstanceData->iFiletype = FILE_TYPE_STD;
			WrapUpAccounting(vpAccounting);

			CopyString(&pNewInstanceData->pWorkFilePathName, pInstanceData->pWorkFilePathName);
			CopyString(&pNewInstanceData->pWorkFilePath, pInstanceData->pWorkFilePath);
			{
				char *Ptr = strrchr(pNewInstanceData->pWorkFilePath, '/');
				if (!Ptr)
					Ptr = strrchr(pNewInstanceData->pWorkFilePath, '\\');
				if (Ptr) {
					*Ptr = 0;
					Ptr++;
				}
				else
					Ptr = pNewInstanceData->pWorkFilePath;
				CopyString(&pNewInstanceData->pWorkFileName, Ptr);
			}
			CheckAccounting(pNewInstanceData);
			pNewInstanceData->pLocaleData = pInstanceData->pLocaleData;
			return pNewInstanceData;
		}
	}
	return 0;
}

/********************************************************************

LLIST GetSeriesList(

*********************************************************************/
LLIST GetSeriesList(
	void *vpAccounting,
	TabDataType *pTabData,
	NUMERIC nLedgerYearId)
{
	AccountingType *pAccounting = vpAccounting;
	LLIST pListSeries = LListInit();

	// Ledger Vouchers
	LedgerType *pLedger
		= LListGetFirstRecordData(pAccounting->pLedgersChart->pListLedger);
	while (pLedger) {
		void *LedgerCursorRecord = pAccounting->pLedgersChart->pListLedger->CursorRecord;
		if ((nLedgerYearId == pLedger->Id) || (nLedgerYearId == (-1))) {
			VoucherType *pVoucher
				= LListGetFirstRecordData(pLedger->pListVoucher);
			while (pVoucher) {
				void *CursorRecord = pLedger->pListVoucher->CursorRecord;
				if (((!pTabData) || (IsData(pAccounting, pTabData->vpObject, pTabData->dPeriod, pTabData->pPeriodLabel, pVoucher->pId->pseries)))
					&& (RET_ERR == LListSearchText(pListSeries, pVoucher->pId->pseries)))
					LListAddRecord(pListSeries, pVoucher->pId->pseries);
				pLedger->pListVoucher->CursorRecord = CursorRecord;
				pVoucher = LListGetNextRecordData(pLedger->pListVoucher);
			}
		}
		pAccounting->pLedgersChart->pListLedger->CursorRecord = LedgerCursorRecord;
		pLedger = LListGetNextRecordData(pAccounting->pLedgersChart->pListLedger);
	}

	{ // Sort the Voucher list
		size_t iSeriesCounter = 0;
		char *Series
			= LListGetFirstRecordData(pListSeries);
		while (Series) {
			iSeriesCounter++;
			Series = LListGetNextRecordData(pListSeries);
		}

		if (iSeriesCounter > 1) { // Put them in an array of Account pointers
			char **ppSeries;
			size_t iSeries = iSeriesCounter;
			Series
				= LListGetFirstRecordData(pListSeries);
			ppSeries = calloc(((size_t)iSeriesCounter) + 1, sizeof(char *));
			while (Series && iSeriesCounter) {
				iSeriesCounter--;
				ppSeries[iSeriesCounter] = Series;
				Series = LListGetNextRecordData(pListSeries);
			}
			LListClear(pListSeries);

			// Sort the array
			qsort((void *)ppSeries, (size_t)iSeries, sizeof(char *), CompareSortStrings);

			for (iSeriesCounter = 0; iSeriesCounter < iSeries; iSeriesCounter++)
				LListAddRecord(pListSeries, ppSeries[iSeriesCounter]);
			free(ppSeries);
			return pListSeries;
		}
		else
			return pListSeries;
	}

}

/********************************************************************

size_t MergeAccountingInstanceData(

*********************************************************************/
size_t MergeAccountingInstanceData(
	size_t ztOldInstanceData,
	size_t ztNewInstanceData)
{
	int iRetval = ERR_FILE_CONTENT;
	InstanceDataType *pOldInstance = (InstanceDataType *)ztOldInstanceData;
	InstanceDataType *pNewInstance = (InstanceDataType *)ztNewInstanceData;
	if((!strcmp(GetWorkDataEnityName(pOldInstance->vpWorkData), GetWorkDataEnityName(pNewInstance->vpWorkData)))
		|| (!strcmp(GetWorkDataEnityLegalRegId(pOldInstance->vpWorkData), GetWorkDataEnityLegalRegId(pNewInstance->vpWorkData)))) {
		AccountingType *pOldAccounting = pOldInstance->vpWorkData;
		AccountingType *pUpdatedAccounting = CopyAccounting(0, 0, pOldAccounting);
		AccountingType *pNewAccounting = (AccountingType *)pNewInstance->vpWorkData;
		if (!strcmp(pNewAccounting->pContent, CONTENT_LABEL_VOUCHER)) { // Voucher list import (normally from presystems) SIE4i
			LedgerType *pNewLedger
				= LListGetFirstRecordData(pNewAccounting->pLedgersChart->pListLedger);
			while (pNewLedger) {
				VoucherType *pNewVoucher
					= LListGetFirstRecordData(pNewLedger->pListVoucher);
				while (pNewVoucher) {
					LedgerType *pOldLedger
						= LListGetFirstRecordData(pUpdatedAccounting->pLedgersChart->pListLedger);
					while (pOldLedger) {
						if ((pOldLedger->YearStart <= pNewVoucher->Date)
							&& (pOldLedger->YearEnd >= pNewVoucher->Date)) {
							VoucherType *pVoucher = CopyVoucher(pUpdatedAccounting, pOldLedger, pNewVoucher);
							if (!pVoucher->pId->pseries)
								pVoucher->pId->pseries = CopyString(0, TXT_MENU_DO_IMPORT);
							if (!pVoucher->pId->SeriesId)
								pVoucher->pId->SeriesId = GetLastVoucherId(pUpdatedAccounting, pVoucher->pId->pseries) + 1;
							LListAddRecord(pOldLedger->pListVoucher, pVoucher);
							iRetval = ERR_OK;
							{
								VoucherEntryType *pNewVoucherEntry
									= LListGetFirstRecordData(pNewVoucher->pListEntry);
								while (pNewVoucherEntry) {
									SearchSetAccount(pUpdatedAccounting->pAccountsChart->pListAccount, CopyAccount(pUpdatedAccounting, pUpdatedAccounting->pAccountsChart, pNewVoucherEntry->pAccount));
									ObjectType *pNewObject
										= LListGetFirstRecordData(pNewVoucherEntry->pListObject);
									while (pNewObject) {
										DimensionType *pUpdatedDimension = SearchSetDimension(pUpdatedAccounting->pObjectsChart->pListDimension, CopyDimension(pUpdatedAccounting, pUpdatedAccounting->pObjectsChart, pNewObject->pDimension));
										SearchSetObject(pUpdatedAccounting->pObjectsChart->pListDimension, CopyObject(pUpdatedAccounting, pUpdatedDimension, pNewObject));
										pNewObject = LListGetNextRecordData(pNewVoucherEntry->pListObject);
									}
									pNewVoucherEntry = LListGetNextRecordData(pNewVoucher->pListEntry);
								}
							}
						}
						pOldLedger = LListGetNextRecordData(pUpdatedAccounting->pLedgersChart->pListLedger);
					}
					pNewVoucher = LListGetNextRecordData(pNewLedger->pListVoucher);
				}
				pNewLedger = LListGetNextRecordData(pNewAccounting->pLedgersChart->pListLedger);
			}
		}
		else { // Import of complete Balance and Ledger contents
			LedgerType *pNewLedger
				= LListGetFirstRecordData(pNewAccounting->pLedgersChart->pListLedger);
			while (pNewLedger) {
				LedgerType *pUpdateLedger
					= LListGetFirstRecordData(pUpdatedAccounting->pLedgersChart->pListLedger);
				while (pUpdateLedger) {
					// Check match of Accounting years for update
					if (pNewLedger->YearEnd == pUpdateLedger->YearEnd) { // Update Ledger
						if (pNewLedger->Id > pUpdateLedger->Id) {
							// later balances are normallt right
							BalanceReportType *pUpdateBalanceReport
								= LListGetFirstRecordData(pUpdateLedger->pListReport);
							while (pUpdateBalanceReport) {
								DestroyBalanceReport(&pUpdateBalanceReport);
								pUpdateBalanceReport = LListGetNextRecordData(pUpdateLedger->pListReport);
							}
							LListClear(pUpdateLedger->pListReport);
							{
								// later balances are normallt right
								BalanceReportType *pNewBalanceReport
									= LListGetFirstRecordData(pNewLedger->pListReport);
								while (pNewBalanceReport) {
									SearchSetBalanceReport(pUpdateLedger->pListReport, CopyBalanceReport(pUpdatedAccounting, pUpdateLedger, pNewBalanceReport));
									pNewBalanceReport = LListGetNextRecordData(pNewLedger->pListReport);
								}
							}
							iRetval = ERR_OK;
						}
						{
							VoucherType *pNewVoucher
								= LListGetFirstRecordData(pNewLedger->pListVoucher);
							while (pNewVoucher) {
								SearchSetVoucher(pUpdateLedger->pListVoucher, CopyVoucher(pUpdatedAccounting, pUpdateLedger, pNewVoucher));
								iRetval = ERR_OK;
								pNewVoucher = LListGetNextRecordData(pNewLedger->pListVoucher);
							}
						}
					}
					pUpdateLedger = LListGetNextRecordData(pUpdatedAccounting->pLedgersChart->pListLedger);
				}
				pNewLedger = LListGetNextRecordData(pNewAccounting->pLedgersChart->pListLedger);
			}
			// Check match of Accounting years for supplements of Ledgers
			// Add at end if it fits the legder date chains
			pNewLedger
				= LListGetFirstRecordData(pNewAccounting->pLedgersChart->pListLedger);
			while (pNewLedger) {
				BOOL bFound = FALSE;
				LedgerType *pOldLedger
					= LListGetFirstRecordData(pOldAccounting->pLedgersChart->pListLedger);
				while (pOldLedger && (!bFound)) {
					if (pNewLedger->YearEnd == pOldLedger->YearEnd)
						// Ledger already exists
						bFound = TRUE;
					pOldLedger = LListGetNextRecordData(pOldAccounting->pLedgersChart->pListLedger);
				}
				if (!bFound) {
					pOldLedger
						= LListGetFirstRecordData(pOldAccounting->pLedgersChart->pListLedger);
					while (pOldLedger && (!bFound)) {
						if (pOldLedger->YearStart == GetNextDate(pNewLedger->YearEnd)) {
							// Add Ledger
							LedgerType *pAddLedger = CopyLedger(pUpdatedAccounting, pUpdatedAccounting->pLedgersChart, pNewLedger);
							pAddLedger->Id = (pOldLedger->Id - 1);
							LListAddRecord(pUpdatedAccounting->pLedgersChart->pListLedger, pAddLedger);
							iRetval = ERR_OK;
							bFound = TRUE;
						}
						pOldLedger = LListGetNextRecordData(pOldAccounting->pLedgersChart->pListLedger);
					}
				}
				pNewLedger = LListGetNextRecordData(pNewAccounting->pLedgersChart->pListLedger);
			}
			// Add at beginning if it fits the legder date chains
			pNewLedger
				= LListGetLastRecordData(pNewAccounting->pLedgersChart->pListLedger);
			while (pNewLedger) {
				BOOL bFound = FALSE;
				LedgerType *pOldLedger
					= LListGetLastRecordData(pOldAccounting->pLedgersChart->pListLedger);
				while (pOldLedger && (!bFound)) {
					if (pNewLedger->YearEnd == pOldLedger->YearEnd)
						// Ledger already exists
						bFound = TRUE;
					pOldLedger = LListGetPreviousRecordData(pOldAccounting->pLedgersChart->pListLedger);
				}
				if (!bFound) {
					pOldLedger
						= LListGetLastRecordData(pOldAccounting->pLedgersChart->pListLedger);
					while (pOldLedger && (!bFound)) {
						if (GetNextDate(pOldLedger->YearEnd) == pNewLedger->YearStart) {
							// Add Ledger
							LedgerType *pAddLedger = CopyLedger(pUpdatedAccounting, pUpdatedAccounting->pLedgersChart, pNewLedger);
							pAddLedger->Id = (pOldLedger->Id + 1);
							LListAddRecord(pUpdatedAccounting->pLedgersChart->pListLedger, pAddLedger);
							iRetval = ERR_OK;
							bFound = TRUE;
							{
								LedgerType *pUpdatedledger
									= LListGetLastRecordData(pUpdatedAccounting->pLedgersChart->pListLedger);
								while (pUpdatedledger) {
									pUpdatedledger->Id--;
									pUpdatedledger = LListGetPreviousRecordData(pUpdatedAccounting->pLedgersChart->pListLedger);
								}
							}
						}
						pOldLedger = LListGetPreviousRecordData(pOldAccounting->pLedgersChart->pListLedger);
					}
				}
				pNewLedger = LListGetPreviousRecordData(pNewAccounting->pLedgersChart->pListLedger);
			}
			Sort(pUpdatedAccounting, VOUCHERS_BY_DATE); //VOUCHERS_BY_SERIES);
			pUpdatedAccounting->LastEntryDate = GetLastEntryDate(pUpdatedAccounting);
		}
		{
			BITMASK bmContent = 0;
			if (LListGetFirstRecordData(pUpdatedAccounting->pAccountsChart->pListAccount))
				bmContent = CONTENT_CHARTS;
			if (LListGetFirstRecordData(pUpdatedAccounting->pLedgersChart->pListLedger)) {
				LedgerType *pUpdateLedger
					= LListGetFirstRecordData(pUpdatedAccounting->pLedgersChart->pListLedger);
				if (LListGetFirstRecordData(pUpdateLedger->pListReport)) {
					bmContent = CONTENT_BALANCE;
					if (LListGetFirstRecordData(pUpdateLedger->pListVoucher))
						bmContent = CONTENT_LEDGER;
				}
				else if (LListGetFirstRecordData(pUpdateLedger->pListVoucher))
					bmContent = CONTENT_VOUCHERS;
			}
			CopyString(&pUpdatedAccounting->pContent, AppGetContentFromBm(bmContent));

			/*		if(LListGetFirstRecordData(pUpdatedAccounting->pLedgersChart->pListLedger))
			return CONTENT_INVOICE;
			if(LListGetFirstRecordData(pUpdatedAccounting->pLedgersChart->pListLedger))
			return CONTENT_BANKACS;
			else
			return CONTENT_NOTHING;
			*/
		}
		// New instance data should always be destroyed
		DestroyInstanceData((InstanceDataType *)ztNewInstanceData);
		if (iRetval == ERR_FILE_CONTENT) {
			// Failure, destroy the new merged data
			DestroyAccounting(&pUpdatedAccounting);
			return 0;
		}
		{
			// Success s wrap the new merged data
			InstanceDataType *pInstance = CopyInstanceDataBody(pOldInstance, pUpdatedAccounting);
			return (size_t)pInstance;
		}
		// Old instance data is destroyed with the old virtual tabs, when new tabs are created ShowOpenedFileData()
	}
	DestroyInstanceData((InstanceDataType *)ztNewInstanceData);
	return 0;
}
/********************************************************************

int ExternalWriteWorkData(

*********************************************************************/
int ExternalWriteWorkData(
	InstanceDataType *pInstanceData,
	BITMASK bmCommand,
	char *pFilePathName)
{
	if ((!pFilePathName) || (!*pFilePathName))
		return ERR_FILE_NAME;
	if (!pInstanceData)
		return ERR_FILE_CONTENT;

	switch (bmCommand & FILE_TYPE_MASK) {
#if(PROGRAM_FEATURES_MASK & PROGRAM_FEATURES_SIE)
	case FILE_TYPE_SIE:
		return FileWriteSie(
			pFilePathName,
			pInstanceData->vpWorkData,
			AppGetContentFromBm(bmCommand),
			pInstanceData->pLocaleData->pOperator, //char *pGenUserName,
			TRUE, // BOOL CompactFileOption,
			FALSE, // LeanChartsOption,
			TRUE, // QuantityOption,
			TRUE, // PeriodOption,
			TRUE, // ObjectOption,
			TRUE, // AccountSortOption,
			TRUE, // DimensionSortOption,
			TRUE, // ObjectSortOption,
			TRUE, // VoucherSortOption,
			0, // pInitialDate,
			0, // pFinalDate,
			0, // FilterEnteredBy,
			BRAND_TEXT_LONG,
			PROGRAM_VERSION,
			pInstanceData->pLocaleData->pLocaleLanguage,
			FALSE);
#endif
#if(PROGRAM_FEATURES_MASK & PROGRAM_FEATURES_XGL)
	case FILE_TYPE_XGL:
		return FileWriteXbrlGl(
			pFilePathName,
			pInstanceData->vpWorkData,
			AppGetContentFromBm(bmCommand),
			pInstanceData->pLocaleData->pOperator,
//			"C:\\Users\\Jan\\Desktop\\xgl.xsl", // Stylesheet 
			"http://www.alphabet.se/XBRL/xsl/xgl.xsl", // Stylesheet 
			TRUE, // ChartsOption,
			TRUE, // ObjectOption,
			TRUE, // BalanceOption,
			TRUE, // PeriodOption,
			TRUE, // VouchersOption,
			TRUE, // QuantityOption,
			TRUE, // LeanChartsOption,
			TRUE, // CompactFileOption,
			TRUE, // ExtendedOption,
			0, // pInitialDate,
			0, // pFinalDate,
			0, // FilterEnteredBy,
			BRAND_TEXT_LONG,
			PROGRAM_VERSION,
			pInstanceData->pLocaleData);
#endif
	case FILE_TYPE_TXT:
	case FILE_TYPE_CSV:
	default:
		return ERR_APP_REJECT;
	}
}

/********************************************************************

static size_t ExternalInterpretFile(

*********************************************************************/
static size_t ExternalInterpretFile(
	char *pFilePathName,
	InstanceDataType **ppInstanceData,
	char *pBuffer)
{
	// Interpret the Buffer App specific
#if(PROGRAM_FEATURES_MASK & PROGRAM_FEATURES_SIE)
	if (strstr(pBuffer, "#KONTO") || strstr(pBuffer, "#VER")) { // SIE file
		LLIST plErrorList = LListInit();
		void *vpAccounting = InterpretSie(pFilePathName, pBuffer, plErrorList);
		NUMERIC nAccounting = (NUMERIC)vpAccounting;
		free(pBuffer);
		if ((nAccounting > 0) || (nAccounting < (-100))) {
			(*ppInstanceData) = calloc(1, sizeof(InstanceDataType));
			if (*ppInstanceData) {
				(*ppInstanceData)->vpWorkData = vpAccounting;
				(*ppInstanceData)->plErrorList = plErrorList;
				(*ppInstanceData)->iFiletype = FILE_TYPE_SIE;
				WrapUpAccounting(vpAccounting);
				return (size_t)*ppInstanceData;
			}
			else {
				LListErase(plErrorList);
				return ERR_TECHNICAL; // Done nothing, read default type
			}
		}
		else {
			LListErase(plErrorList);
			return ERR_FILE_CONTENT; // Done nothing, read default type
		}
	}
	else 
#endif
#if(PROGRAM_FEATURES_MASK & PROGRAM_FEATURES_XGL)
		if (strstr(pBuffer, "gl-bus")) { //  XBRL/GL file
		LLIST plErrorList = LListInit();
		void *vpAccounting = InterpretXbrlGl(pBuffer, plErrorList);
		free(pBuffer);
		if ((((intptr_t)vpAccounting) > 0) || (((intptr_t)vpAccounting) < (-100))) {
			(*ppInstanceData) = calloc(1, sizeof(InstanceDataType));
			if (*ppInstanceData) {
				(*ppInstanceData)->vpWorkData = vpAccounting;
				(*ppInstanceData)->plErrorList = plErrorList;
				(*ppInstanceData)->iFiletype = FILE_TYPE_XGL;
				WrapUpAccounting(vpAccounting);
				return (size_t)*ppInstanceData;
			}
			else {
				LListErase(plErrorList);
				return ERR_TECHNICAL; // Done nothing, read default type
			}
		}
		else {
			LListErase(plErrorList);
			return ERR_FILE_CONTENT; // Done nothing, read default type
		}
	}
	else
#endif
		return ERR_APP_REJECT; // Done nothing, read default type
}

/********************************************************************

size_t InterpretFile(

*********************************************************************/
size_t InterpretFile(
	char *pBuffer,
	char *pFilePathName,
	LocaleDataType *pLocaleData)
{
	// Interpret the Buffer 
	InstanceDataType *pInstanceData = 0;
	// Try App specials
	size_t ztRetval = ExternalInterpretFile(pFilePathName, &pInstanceData, pBuffer);
#if(PROGRAM_FEATURES_MASK & PROGRAM_FEATURES_ROSETTA)
	if (ztRetval == ERR_APP_REJECT) { // Not processed app specific 
		// Else the general routine
		void *vpAccounting = ReadWorkData(pBuffer);
		free(pBuffer);
		if (!vpAccounting)
			return ERR_FILE_CONTENT; // Failure
		if ((vpAccounting > (void *)0) || (vpAccounting < (void *)(-100))) {
			pInstanceData = calloc(1, sizeof(InstanceDataType));
			if (pInstanceData) {
				pInstanceData->vpWorkData = vpAccounting;
				pInstanceData->plErrorList = LListInit();
				pInstanceData->iFiletype = FILE_TYPE_STD;
				WrapUpAccounting(vpAccounting);
				ztRetval = (size_t)pInstanceData;
			}
			else
				return ERR_TECHNICAL; // Done nothing, read default type
		}
		else
			return ERR_FILE_CONTENT; // Error file content not accepted
	}
#endif
	// Test the Interpreted File Open File data
	if ((((int)ztRetval) > 0) || (((int)ztRetval) < (-100))) { // processed non-fio specific 
		if (GetLedgerStart(pInstanceData->vpWorkData, 0) && (GetLedgerStart(pInstanceData->vpWorkData, 0) > EXPIRE_DATE))
			return ERR_EXPIRED;
		CheckAccounting(pInstanceData);
		// Only sumup of Vouchers, else supposed to be OK
		if (CONTENT_VOUCHERS == GetContentParameters(pInstanceData->vpWorkData, 0)) 
			RefreshAccountingData(pInstanceData->vpWorkData);
		CopyString(&pInstanceData->pWorkFilePathName, pFilePathName);
		CopyString(&pInstanceData->pWorkFilePath, pFilePathName);
		{
			char *Ptr = strrchr(pInstanceData->pWorkFilePath, '/');
			if (!Ptr)
				Ptr = strrchr(pInstanceData->pWorkFilePath, '\\');
			if (Ptr) {
				*Ptr = 0;
				Ptr++;
			}
			else
				Ptr = pInstanceData->pWorkFilePath;
			CopyString(&pInstanceData->pWorkFileName, Ptr);
		}
		pInstanceData->pLocaleData = pLocaleData;
		return (size_t)pInstanceData; // OK
	}
	return ztRetval; // Failure if nothing else ERR_APP_REJECT
}

