/*********************************************************************************

	RosettaFioXbrlGl.c

	Alphabet Rosetta

	(c)Copyright 2017-2020  Alphabet AB, Sweden (Jan Bergstrom)

	http://www.alphabet.se

	Licensed under the Apache License, Version 2.0 (the "License");
	you may not use this file except in compliance with the License.
	You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.

	We ask for attribution to us, when used.

******************************************************************************************

The RosettaFioXbrlGl.c file contains functions to interpret and write XBRL/GL-files
(www.xbrl.org/the-standard/what/global-ledger/) into the Alphabet Rosetta Accounting
basic data logics and database system.

The RosettaFioXbrlGl.h file contains the external declarations and definitions.

Content of RosettaFioXbrlGl.c
1.) Interpret XBRL/GL file data
2.) Create XBRL/GL file data

**********************************************************************************/
#include "RosettaFioTypes.h"

/********************************************************************

Interpret XBRL/GL file data

*********************************************************************/

#define BALANCE_ACCOUNTS				((!strcmp(pAccount->pType, ACCOUNT_TYPE_TANGIBLE) || (!strcmp(pAccount->pType, ACCOUNT_TYPE_STOCK))) || (!strcmp(pAccount->pType, ACCOUNT_TYPE_RECEIVABLES)) || (!strcmp(pAccount->pType, ACCOUNT_TYPE_ASSETS)) || (!strcmp(pAccount->pType, ACCOUNT_TYPE_EQUITIES)) || (!strcmp(pAccount->pType, ACCOUNT_TYPE_PROVISIONS)) || (!strcmp(pAccount->pType, ACCOUNT_TYPE_LIABILITIES)))

#define MAX_COMMAND_FIELDS				8 // Number of parameters of a SIE-line except the Command tag

#define MOVE_MACHINE					FALSE

#define XBRL_ID_PREFIX_STRING			"id-"
#define XBRL_ID_PREFIX_STRING_CHART		"id-Chart"
#define XBRL_ID_ID_STRING				"id"

#define LENGTH_ISO639_LANGUAGE			2
#define LENGTH_ISO3166_COUNTRY			2
#define LENGTH_ISO4217_CURRENCY			8 // unitRef="NotUsed"

#define BALANCE_BEGINNING				0
#define BALANCE_ENDING					1
#define PERIOD_CHANGE					2
#define BUDGET							3

extern char *CopyString(char **ppTarget, const char *pOriginal);
extern char *ClearXmlEsc(char *pBuffer);

#define INTERPRET_SYNTAX_OK				0

#define INTERPRET_SYNTAX_ERR_FATAL		(BITMASK) (-1) // System error no memory

#define	XT_NONE							0
#define XT_DOCINFO						1
#define XT_ENTITY						2
#define XT_ACCOUNT						3
#define XT_IDENTIFIER					4
#define XT_BALANCE						5
#define XT_MAPPING						6 // Object balances
#define XT_VOUCHER						7

#define XBRLGL_SYNTAX_ERR_FATAL			(BITMASK) (-1) // System error no memory
#define XBRLGL_SYNTAX_NOT_USED			(BITMASK) (-10) // System error no memory

#define PREFIX_ISO639_LANG				"iso639:"
#define PREFIX_ISO4217_CURR				"iso4217:"
#define ATTRIBUTE_CONTEXTREF			"contextRef"
#define ATTRIBUTE_UNITREF				"unitRef"

typedef struct {
	BOOL FLAGGA;
	BOOL PROGRAM;
	BOOL FORMAT;
	BOOL GEN;
	BOOL FNAMN;
	BOOL RAR;
	BOOL OMFATTN;
	BOOL KONTO;
	BOOL SRU;
	BOOL OBJEKT;
	BOOL IB;
	BOOL UB;
	BOOL RES;
	BOOL OIB;
	BOOL OUB;
	BOOL PBUDGET;
	BOOL PSALDO;
} XBRLGLMANDATORYTYPE;


typedef struct {
	char *EntriesType;
	char *UniqueID;
	char *Language;
	char *CreationDate;
	char *Creator;
	char *EntriesComment;
	char *pProgram;
	char *TargetApplication;
	char *PeriodCoveredStart;
	char *PeriodCoveredEnd;
	char *DefaultCurrency;
} XbrlGlDocumentInfoType;

typedef struct {
	char *OrganizationIdentifier;				// char *pLegalRegId;
	char *BusinessDescription;					// char *pStatisticalClass;
	char *VatRegId;								// char *pVatRegId;
	char *VatCurrency;							// char *pVatCurrency;
	char *CashAccounting;						// BOOL *pCashAccounting;
	char *OrganizationDescription;				// char *pName;
	char *PhoneNumber;							// char *pEntityPhone;
	char *EntityEmailAddress;					// char *pEntityEmail;
	char *OrganizationAddressName;				// char *pName;
	char *OrganizationBuildingNumber;			// char *pAddressLineOne;
	char *OrganizationAddressStreet;			// char *pAddressLineTwo;
	char *OrganizationAddressStreet2;			// char *pAddressLineThree;
	char *OrganizationAddressZipOrPostalCode;	// char *pAddressLineFour;
	char *OrganizationAddressStateOrProvince;	// char *pAddressLineFive;
	char *OrganizationAddressCity;				// char *pPostalAddress;
	char *OrganizationAddressCountry;			// char *pCountry;
	char *ContactFirstName;						// char *pFiscalContact;
	char *ContactLastName;						// char *pFiscalContact;
	char *ContactPhoneNumber;					// char *pFiscalContactPhone;
	char *ContactEMail;							// char *pFiscalContactEmail;
	char *FiscalYearStart;
	char *FiscalYearEnd;
	char *AccountantAddressName;
	char *AccountantBuildingNumber;
	char *AccountantAddressStreet;
	char *AccountantAddressStreet2;
	char *AccountantAddressStateOrProvince;
	char *AccountantAddressZipOrPostalCode;
	char *AccountantAddressCity;
	char *AccountantAddressCountry;
	char *AccountantContactFirstName;
	char *AccountantContactLastName;
	char *AccountantContactPhoneNumber;
	char *AccountantContactEMail;
} XbrlGlEntityInformationType;

typedef struct {
	char *PostedDate;
	char *EnteredBy;
	char *EnteredDate;
	char *EntryType;
	char *EntryNumber;
	char *EntryComment;
	char *QualifierEntry;
	char *BatchID;
	AMOUNT BalanceAmount;
	VoucherType *pVoucher;
} XbrlGlEntriesHeaderType;

typedef struct {
	char *LineNumber;
	char *LineNumberCounter;
	char *Amount;
	char *AmountUnit;
	char *AmountDecimals;
	char *SignOfAmount;
	char *DebitCreditCode;
	char *PostingDate;
	char *AmountMemo;
	char *AllocationCode;
	char *DocumentType;
	char *DocumentTypeDescription;
	char *InvoiceType;
	char *DocumentNumber;
	char *DocumentApplyToNumber;
	char *DocumentReference;
	char *DocumentDate;
	char *DocumentReceivedDate;
	char *DocumentChargeReimb;
	char *DocumentLocation;
	char *PaymentMethod;
	char *PostingStatus;
	char *PostingStatusDescription;
	char *XbrlInclude;
	char *DetailComment;
	char *DateAcknowledged;
	char *ConfirmedDate;
	char *ShipFrom;
	char *ShipReceivedDate;
	char *MaturityDate;
	char *Terms;
	LLIST pListIdentifiers;
} XbrlGlEntriesDetailsType;

typedef struct {
	char *IdentifierCode;
	char *IdentifierDescription;
	char *IdentifierCategory;
	char *IdentifierPurpose;
	char *IdentifierAuthorityCode;
	char *IdentifierAuthority;
} XbrlGlEntriesIdentifierType;

typedef struct {
	char *AccountMainID;
	char *AccountMainDescription;
	char *MainAccountType;
	//	char *MainAccountTypeDescription;
	//	char *ParentAccountMainID;
	//	char *AccountPurposeCode;
	char *AccountPurposeDescription;
	//	char *AccountType;
	//	char *AccountTypeDescription;
	//	char *EntryAccountingMethod;
	//	char *EntryAccountingMethodDescription;
	//	char *EntryAccountingMethodPurpose;
	//	char *EntryAccountingMethodPurposeDescription;
	//	char *AccountSub;
	//	char *AccountActive;
	LLIST pListAccountCodes;
} XbrlGlEntriesAccountType;

typedef struct {
	char *TaxAuthority;
	char *TaxTableCode;
	char *TaxDescription;
	char *TaxAmount;
	char *TaxBasis;
	char *TaxExchangeRate;
	char *TaxPercentageRate;
	char *TaxCode;
	char *TaxCommentExemption;
} XbrlGlTaxesType;

typedef struct {
	char *MeasurableCode;
	char *MeasurableCodeDescription;
	char *MeasurableCategory;
	char *MeasurableID;
	char *MeasurableIDSchema;
	char *MeasurableIDOther;
	char *MeasurableIDOtherSchema;
	char *MeasurableDescription;
	char *MeasurableQuantity;
	char *MeasurableQualifier;
	char *MeasurableUnitOfMeasure;
	char *MeasurableCostPerUnit;
	char *MeasurableStartDateTime;
	char *MeasurableEndDateTime;
	char *MeasurableActive;
} XbrlGlMeasurableType;


typedef struct {
	char *XbrlCurrency;
	char *Iso4217Currency;
} XbrlGlCurrenciesType;

typedef struct {
	AccountingType *pAccounting;

	int XbrlGlItemType;

	XbrlGlDocumentInfoType			XbrlGlDocumentInfo;
	XbrlGlEntityInformationType		XbrlGlEntityInformation;
	XbrlGlEntriesAccountType		XbrlGlEntriesAccount;
	XbrlGlMeasurableType			XbrlGlMeasurable;
	XbrlGlTaxesType					XbrlGlTax;
	XbrlGlEntriesIdentifierType		XbrlGlEntriesIdentifier;
	XbrlGlEntriesDetailsType		XbrlGlEntriesDetails;
	XbrlGlEntriesHeaderType			XbrlGlEntriesHeader;

	char *Message[MAX_COMMAND_FIELDS];
	//	XBRLGLMANDATORYTYPE *XbrlGlMandatory;
	LLIST pXmlPath;
	LLIST pCurrencies;
	char *XbrliContext;
} XBRLGLPARAMTYPE;

#ifndef FALSE
#define FALSE				0
#endif

#ifndef TRUE
#define TRUE				1
#endif

static BITMASK  XbrlGlStoreTags(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagData);
// Note that the unctions are invoked after the XML end-lable
static BITMASK	XbrlGlTagAccountingEntries(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);	// Clearing Data after section is made

// Unit
static BITMASK	XbrlGlTagXbrliUnitMeasure(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);	// XBRL Curr unit list
static BITMASK	XbrlGlTagXbrliContext(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);	// #FNR
static BITMASK	XbrlGlTagXbrliContextPeriodInstant(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);	// #GEN date
//static BITMASK	XbrlGlTagXbrliContextScenarioIncomeTaxRegistration(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);
//static BITMASK	XbrlGlTagXbrliContextScenarioVatRegistration(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);
//static BITMASK	XbrlGlTagXbrliContextScenarioLogotype(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);
//static BITMASK	XbrlGlTagXbrliContextScenarioLegalRegistrationFileUrl(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);
//static BITMASK	XbrlGlTagXbrliContextScenarioVatRegistrationFileUrl(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);
//static BITMASK	XbrlGlTagXbrliContextScenarioIncomeTaxRegistrationFileUrl(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);
//static BITMASK	XbrlGlTagXbrliContextScenarioReferenceFileUrl(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);
//static BITMASK	XbrlGlTagXbrliContextScenarioAuditAttestant(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);
//static BITMASK	XbrlGlTagXbrliContextScenarioAuditAttestPkiFileUrl(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);
//static BITMASK	XbrlGlTagXbrliContextScenarioPurpose(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);	// #SIETYP
static BITMASK	XbrlGlTagXbrliwebSiteURL(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);

// DocumentInfo
static BITMASK	XbrlGlTagDocumentInfoDocumentInfo(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);	// Process general data
static BITMASK	XbrlGlTagDocumentInfoLanguage(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);	// #XLAND

// EntityInformation
static BITMASK	XbrlGlTagEntityInformationEntityInformation(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);	// Process general data
static BITMASK	XbrlGlTagEntityInformationOrganizationIdentifiers(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);	// Process general data
static BITMASK	XbrlGlTagEntityInformationAccountantInformation(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);	// Process general data
static BITMASK	XbrlGlTagEntityInformationAuditInformation(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);	// Process general data

static BITMASK	XbrlGlTagGeneralGlcorEntryHeader(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);	// End of entry header, Storing units
static BITMASK	XbrlGlTagGeneralGlcorEntryDetail(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);	// End of entry detail, Storing units
static BITMASK	XbrlGlTagGeneralGlcorIdentifierReference(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);	// End of entry detail, Storing units
static BITMASK	XbrlGlTagGeneralGlcorAccountTaxCodeSpecifications(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);	// End of entry detail, Storing units


// Balances
static BITMASK	XbrlGlTagAmount(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);	// Unknown command

static BITMASK	XbrlGlTagNotInterpret(XBRLGLPARAMTYPE *psXbrlGlParam, char *pTagMainData, char *pTagData);	// Unknown command

static char *GetXbrlCurrency(XBRLGLPARAMTYPE *psXbrlGlParam, char *XbrlCurrencyString);	// Curr unit
static void XbrlGlTagGeneralGlcorEntryDetailReset(XBRLGLPARAMTYPE *psXbrlGlParam);
static void XbrlGlTagGeneralGlcorEntryHeaderReset(XBRLGLPARAMTYPE *psXbrlGlParam);
static char *XbrlGlParamNameUnification(char *FirstName, char *LastName);
static char *XbrlMainAccountType2AccountType(char *XbrlMainAccountType);

void *InterpretXbrlGl(char *pBuffer, LLIST plErrorList);

static int WriteXbrlGlAccountTaxCodeSpecifications(FILE *pStream, char *pContextRef, AccountType *pAccount, AccountingType *pAccounting, BOOL bCompactFileOption, int iTabPos);
static int WriteXbrlGlAccountSpecifications(FILE *pStream, AccountingType *pAccounting, char *AccountChartType, char *pContextRef, char *pAccountID, char *pAccountName, char *pAccountType, char *pAccountTypeDescription, char *pAccountActive, BOOL bQuantityOption, BOOL bCompactFileOption, int iTabPos);
static int WriteXbrlGlAccountStatements(FILE *pStream, char *pContextRef, AccountingType *pAccounting, BOOL bQuantityOption, BOOL bLeanChartsOption, DATEINT InitialDate, DATEINT FinalDate, char * FilterEnteredBy, char *pContent, char *pGenUserName, int iTabPos, char *pLocaleLanguage, char *pProgramName, char *pProgramVersion);
static int WriteXbrlGlQuantitySpecifications(FILE *pStream, char *pQuantity, char *pQuantityUnit, char *pContextRef, int iTabPos);
//static BOOL WriteXbrlGlDimObjStatements(FILE *pStream, char *pContextRef, AccountingType *pAccounting, BOOL bLeanChartsOption, DATEINT InitialDate, DATEINT FinalDate, char * FilterEnteredBy, char *pContent, char *pGenUserName, int iTabPos, char *pLocaleLanguage, char *pProgramName, char *pProgramVersion);
static int WriteXbrlGlDimObjSpecificationObject(FILE *pStream, char *pDimensionId, char *pDimensionDescription, char *pDimensionSuperId, char *pDimensionSuperDescription, char *pObjectId, char *pObjectDescription, char *pFileUrl, char *pContextRef, BOOL bCompactFileOption, BOOL bActiveObject, int iTabPos);
static int WriteXbrlGlBalances(FILE *pStream, char *pContextRef, AccountingType *pAccounting, BOOL bCompactFileOption, BOOL bQuantityOption, BOOL bObjectOption, BOOL bPeriodOption, BOOL bLeanChartsOption, DATEINT InitialDate, DATEINT FinalDate, char *pContent, char *pGenUserName, BOOL *pbChartsDeclared, int iTabPos, char *pLocaleLanguage, char **Month, char *pProgramName, char *pProgramVersion);
static int WriteXbrlGlBalanceReports(FILE *pStream, char *pContextRef, AccountingType *pAccounting, LedgerType *pLedger, BalanceReportType *pReport, BOOL bCompactFileOption, BOOL bQuantityOption, BOOL bObjectOption, BOOL bPeriodOption, BOOL bBudgetOption, BOOL bLeanChartsOption, DATEINT InitialDate, DATEINT FinalDate, DATEINT LastEntryDate, char *pContent, char *pGenUserName, int iTabPos, char *pLocaleLanguage, char **ppMonth, char *pProgramName, char *pProgramVersion);
static int	WriteXbrlGlBalanceReportLines(FILE *pStream, char *pContextRef, AccountingType *pAccounting, LedgerType *pLedger, BalanceReportType *pReport, AccountType *pAccount, char *pAccountTypeDescription, AMOUNT Amount, QUANT Quant, BOOL bCompactFileOption, BOOL bLeanChartsOption, BOOL bQuantityOption, char *pContent, BOOL BalanceType, DATEINT FinalDate, DATEINT EndDate, DATEINT LastEntryDate, char *pGenUserName, char *pTitle, char *pEntryOrigin, int *piTabPos, int iInitiated, char *pLocaleLanguage, char **ppMonth, char *pProgramName, char *pProgramVersion);
static BOOL WriteXbrlGlLedgerVouchers(FILE *pStream, char *pContextRef, AccountingType *pAccounting, BOOL bCompactFileOption, BOOL bQuantityOption, BOOL bObjectOption, DATEINT InitialDate, DATEINT FinalDate, char * FilterEnteredBy, char *pContent, char *pGenUserName, BOOL *pbChartsDeclared, int iTabPos, char *pLocaleLanguage, char *pProgramName, char *pProgramVersion);
static int WriteXbrlGlVoucherDimObjSpecificationList(FILE *pStream, AccountingType *pAccounting, LLIST pListObject, char *pContextRef, BOOL bCompactFileOption, int iTabPos, char *pLocaleLanguage);
static BOOL WriteXbrlGlInformationHeader(FILE *pStream, char *pContextRef, AccountingType *pAccounting, char *EntriesType, char *pHeaderTitle, char *UniqueID, char *pContent, char *pGenUserName, DATEINT PeriodCoveredEnd, DATEINT PeriodCoveredStart, DATEINT FiscalYearEnd, DATEINT FiscalYearStart, char *Iso4217Currency, int iTabPos, char *pLocaleLanguage, char *pProgramName, char *pProgramVersion);
static int WriteXbrlGlOrganizationIdentifiers(FILE *pStream, char *pContextRef, int iTabPos, char *OrganizationIdentifier, char *OrganizationDescription);
static int WriteXbrlGlOrganizationAddress(FILE *pStream, char *pContextRef, int iTabPos, AccountingType *pAccounting);
static int WriteXbrlGlOrganizationWebSite(FILE *pStream, char *pContextRef, int iTabPos, AccountingType *pAccounting);
static int WriteXbrlGlContactInformation(FILE *pStream, char *pContextRef, int iTabPos, AccountingType *pAccounting);
static int WriteXbrlGlAccountantInformation(FILE *pStream, char *pContextRef, int iTabPos, AccountingType *pAccounting);


/********************************************************************

void *OpenXbrlGlFile(

*********************************************************************/
void *OpenXbrlGlFile(
	char*pFilePathName, 
	LLIST plErrorList)
{

	char	*pBuffer = FileRead(pFilePathName);
	if (pBuffer) {
		void *vpRet = InterpretXbrlGl(pBuffer, plErrorList);
		free(pBuffer);
		return vpRet;
	}
	return (void *)-2;
}


/********************************************************************

void *InterpretXbrlGl(

*********************************************************************/
void *InterpretXbrlGl(
	char *pBuffer, 
	LLIST plErrorList)
{
	XBRLGLPARAMTYPE sXbrlGlParam;
#define TAG_BEGIN	'<'
#define TAG_END		'>'

	memset(&sXbrlGlParam, 0, sizeof(XBRLGLPARAMTYPE));
	if (pBuffer) {
		sXbrlGlParam.pXmlPath = LListInit();
		sXbrlGlParam.pCurrencies = LListInit();
		sXbrlGlParam.pAccounting = CreateAccounting(0, 0);
		memset(&sXbrlGlParam.XbrlGlDocumentInfo, 0, sizeof(XbrlGlDocumentInfoType));
		memset(&sXbrlGlParam.XbrlGlEntityInformation, 0, sizeof(XbrlGlEntityInformationType));
		XbrlGlTagGeneralGlcorEntryHeaderReset(&sXbrlGlParam);
		XbrlGlTagGeneralGlcorEntryDetailReset(&sXbrlGlParam);

		sXbrlGlParam.XbrlGlItemType = XT_NONE;
		{
			char *StartPtr = strchr(pBuffer, TAG_BEGIN);
			char *EndPtr = strchr(StartPtr, TAG_END);
			while (StartPtr && EndPtr) {
				if ((StartPtr[1] != '?')
					&& (StartPtr[1] != '!')
					&& (StartPtr[1] != '-')
					&& (StartPtr[1] != '/')
					&& (EndPtr[-1] != '/')) {
					LListAddRecord(sXbrlGlParam.pXmlPath, StartPtr);
				}
				else if (StartPtr[1] == '/') {
					char *cTagBeginTag = LListGetLastRecordData(sXbrlGlParam.pXmlPath);
					*EndPtr = 0; // Terminate string
					if (cTagBeginTag && (strncmp(cTagBeginTag, StartPtr, strlen(StartPtr)))) {
						XbrlGlStoreTags(&sXbrlGlParam, cTagBeginTag);
						cTagBeginTag = LListGetLastRecordData(sXbrlGlParam.pXmlPath);
						LListEraseCursorRecord(sXbrlGlParam.pXmlPath);
					}
					else
						sXbrlGlParam.pXmlPath = sXbrlGlParam.pXmlPath; // Error

					*EndPtr = TAG_END; // UnTerminate string
				}

				StartPtr = strchr(EndPtr + 1, TAG_BEGIN);
				if (StartPtr)
					EndPtr = strchr(StartPtr, TAG_END);
			}
		}
		//		SumUpErrors(sXbrlGlParam.pAccounting, &dwErrorLines, TRUE, TRUE);
		if (sXbrlGlParam.pAccounting
			&& sXbrlGlParam.pAccounting->pGaap
			&& (!strncmp(sXbrlGlParam.pAccounting->pGaap, "BAS20", 5)))
			CopyString(&sXbrlGlParam.pAccounting->pGaap, BAS2000);
		AccountingType *pSourceAccounting = ExpandSieAccountTypes(sXbrlGlParam.pAccounting, FILE_TYPE_STD);
		if (pSourceAccounting != sXbrlGlParam.pAccounting) {
			DestroyAccounting(&sXbrlGlParam.pAccounting);
			sXbrlGlParam.pAccounting = pSourceAccounting;
		}
	}
	{
		XbrlGlCurrenciesType *XbrlGlCurrency = LListGetFirstRecordData(sXbrlGlParam.pCurrencies);
		while (XbrlGlCurrency) {
			FreeString(&XbrlGlCurrency->XbrlCurrency);
			FreeString(&XbrlGlCurrency->Iso4217Currency);
			free(XbrlGlCurrency);
			XbrlGlCurrency = LListGetNextRecordData(sXbrlGlParam.pCurrencies);
		}
		LListErase(sXbrlGlParam.pCurrencies);
	}
	LListErase(sXbrlGlParam.pXmlPath);
	LListErase(sXbrlGlParam.XbrlGlEntriesDetails.pListIdentifiers);
	return sXbrlGlParam.pAccounting; // There is no error analyse on file interpretation
}

/********************************************************************

static BOOL XbrlGlIdentifyTags(

*********************************************************************/
static BOOL XbrlGlIdentifyTags(
	char **ppSearch, 
	char **ppSearchEnd, 
	char *pKey)
{
	char *ppCheck = strchr(*ppSearch, '>');
	char *ppCheck2 = strchr(*ppSearch, ' ');

	// This is to avoid tags, where the text of the tag is a part of another tag
	if (ppCheck2 && (ppCheck > ppCheck2))
		ppCheck = ppCheck2;	// The first '>' or ' ' is what counts (If there is not hit on ppCheck2 it should not be counted for)

	if (((size_t)(ppCheck - *ppSearch)) != strlen(pKey)) // If the tag text length is not eq it is different tags
		return FALSE;		// Not found

	if (!strncmp(*ppSearch, pKey, strlen(pKey))) {
		*ppSearch = strchr(*ppSearch, '>');
		*ppSearch += 1;
		*ppSearchEnd = strchr(*ppSearch, '<');
		if (*ppSearchEnd)
			**ppSearchEnd = 0;	// Terminate string (String is unterminated by the XbrlGlStoreTags()
		ClearXmlEsc(*ppSearch);
		return TRUE;		// Found
	}
	return FALSE;			// Not found
}

/********************************************************************

static BITMASK XbrlGlStoreTags(

*********************************************************************/
static BITMASK XbrlGlStoreTags(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagData)
{
	char *pTagMainData = pTagData;
	char *pSearchEnd = 0;
	char *pContextRef = strstr(pTagData, ATTRIBUTE_CONTEXTREF);

	if (pContextRef) {
		pContextRef = strstr(pContextRef, XBRL_ID_PREFIX_STRING);

		if (pContextRef && psXbrlGlParam->XbrliContext) {
			pContextRef += strlen(XBRL_ID_PREFIX_STRING);
			//			while (*pContextRef && ((*pContextRef <= ' ') || (*pContextRef == '"') || (*pContextRef == '=')))
			//				pContextRef++;
			if (strncmp(pContextRef, psXbrlGlParam->XbrliContext, strlen(psXbrlGlParam->XbrliContext)))
				//				return XBRLGL_SYNTAX_ERR_FATAL; // Syntax Error
				psXbrlGlParam->Message[0] = ATTRIBUTE_CONTEXTREF" is not correct, XBRL/GL demands it is";
		}
	}
	switch (psXbrlGlParam->XbrlGlItemType) {
	case XT_MAPPING:
		psXbrlGlParam->XbrlGlItemType = psXbrlGlParam->XbrlGlItemType;
	case XT_ACCOUNT:
	case XT_IDENTIFIER:
	case XT_BALANCE:
	case XT_VOUCHER:
		// Accounting entries
		if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:accountingEntries"))						// End Data Group Switch
			XbrlGlTagAccountingEntries(psXbrlGlParam, pTagMainData, pTagData);
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:entryHeader"))							// End of entry header, Storing data
			XbrlGlTagGeneralGlcorEntryHeader(psXbrlGlParam, pTagMainData, pTagData);
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:entryDetail"))							// End of entry detail, Storing data
			XbrlGlTagGeneralGlcorEntryDetail(psXbrlGlParam, pTagMainData, pTagData);
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:identifierReference"))					// End of entry detail, Storing data
			XbrlGlTagGeneralGlcorIdentifierReference(psXbrlGlParam, pTagMainData, pTagData);
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:taxes"))								// End of taxes, Storing data
			XbrlGlTagGeneralGlcorAccountTaxCodeSpecifications(psXbrlGlParam, pTagMainData, pTagData);
		// Account
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:accountMainID"))						// KONTO nr		
			psXbrlGlParam->XbrlGlEntriesAccount.AccountMainID = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:accountMainDescription"))				// KONTO namn
			psXbrlGlParam->XbrlGlEntriesAccount.AccountMainDescription = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:mainAccountType"))						// KTYP
			psXbrlGlParam->XbrlGlEntriesAccount.MainAccountType = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:accountPurposeDescription"))			// KPTYP
			psXbrlGlParam->XbrlGlEntriesAccount.AccountPurposeDescription = pTagMainData;
		// Meassure
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:measurableCode"))						// ENHET
			psXbrlGlParam->XbrlGlMeasurable.MeasurableCode = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:measurableCodeDescription"))			// ENHET
			psXbrlGlParam->XbrlGlMeasurable.MeasurableCodeDescription = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:measurableCategory"))					// ENHET
			psXbrlGlParam->XbrlGlMeasurable.MeasurableCategory = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:measurableID"))						// ENHET
			psXbrlGlParam->XbrlGlMeasurable.MeasurableID = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:measurableIDSchema"))					// ENHET
			psXbrlGlParam->XbrlGlMeasurable.MeasurableIDSchema = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:measurableIDOther"))					// ENHET
			psXbrlGlParam->XbrlGlMeasurable.MeasurableIDOther = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:measurableIDOtherSchema"))				// ENHET
			psXbrlGlParam->XbrlGlMeasurable.MeasurableIDOtherSchema = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:measurableDescription"))				// ENHET
			psXbrlGlParam->XbrlGlMeasurable.MeasurableDescription = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:measurableQuantity"))					// ENHET
			psXbrlGlParam->XbrlGlMeasurable.MeasurableQuantity = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:measurableQualifier"))					// ENHET
			psXbrlGlParam->XbrlGlMeasurable.MeasurableQualifier = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:measurableUnitOfMeasure"))				// ENHET
			psXbrlGlParam->XbrlGlMeasurable.MeasurableUnitOfMeasure = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:measurableCostPerUnit"))				// ENHET
			psXbrlGlParam->XbrlGlMeasurable.MeasurableCostPerUnit = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:measurableStartDateTime"))				// ENHET
			psXbrlGlParam->XbrlGlMeasurable.MeasurableStartDateTime = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:measurableEndDateTime"))				// ENHET
			psXbrlGlParam->XbrlGlMeasurable.MeasurableEndDateTime = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:measurableActive"))					// ENHET
			psXbrlGlParam->XbrlGlMeasurable.MeasurableActive = pTagMainData;
		// Tax
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:taxAuthority"))						// SRU
			psXbrlGlParam->XbrlGlTax.TaxAuthority = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:taxTableCode"))						// TAXAR
			psXbrlGlParam->XbrlGlTax.TaxTableCode = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:taxDescription"))						// SRU
			psXbrlGlParam->XbrlGlTax.TaxDescription = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:taxAmount"))							// SRU
			psXbrlGlParam->XbrlGlTax.TaxAmount = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:taxBasis"))							// SRU
			psXbrlGlParam->XbrlGlTax.TaxBasis = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:taxExchangeRate"))						// SRU
			psXbrlGlParam->XbrlGlTax.TaxExchangeRate = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:taxPercentageRate"))					// SRU
			psXbrlGlParam->XbrlGlTax.TaxPercentageRate = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:taxCode"))								// SRU
			psXbrlGlParam->XbrlGlTax.TaxCode = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:taxCommentExemption"))					// SRU
			psXbrlGlParam->XbrlGlTax.TaxCommentExemption = pTagMainData;
		// Identifier
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:identifierCode"))						// #OBJEKT
			psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierCode = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:identifierDescription"))				// #OBJEKT
			psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierDescription = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:identifierCategory"))					// #DIM
			psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierCategory = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:identifierPurpose"))					// #DIM
			psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierPurpose = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:identifierAuthorityCode"))				// #DIM super ID
			psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierAuthorityCode = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:identifierAuthority"))					// #DIM super Title
			psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierAuthority = pTagMainData;
		// Detail
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:lineNumber"))							// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.LineNumberCounter = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:lineNumberCounter"))					// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.LineNumberCounter = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:amount"))								// TRANS Belopp // Filtrera ut valuta
			XbrlGlTagAmount(psXbrlGlParam, pTagMainData, pTagData);
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:signOfAmount"))						// Flip Amount sign ?
			psXbrlGlParam->XbrlGlEntriesDetails.SignOfAmount = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:debitCreditCode"))						// Flip Amount sign ?
			psXbrlGlParam->XbrlGlEntriesDetails.DebitCreditCode = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:postingDate"))							// Balansdatum
			psXbrlGlParam->XbrlGlEntriesDetails.PostingDate = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:amountMemo"))							// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.AmountMemo = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:allocationCode"))						// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.AllocationCode = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:documentType"))						// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.DocumentType = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:documentTypeDescription"))				// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.DocumentTypeDescription = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:invoiceType"))							// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.InvoiceType = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:documentNumber"))						// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.DocumentNumber = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:documentApplyToNumber"))				// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.DocumentApplyToNumber = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:documentReference"))					// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.DocumentReference = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:documentDate"))						// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.DocumentDate = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:documentReceivedDate"))				// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.DocumentReceivedDate = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:documentChargeReimb"))					// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.DocumentChargeReimb = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:documentLocation"))					// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.DocumentLocation = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:paymentMethod"))						// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.PaymentMethod = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:postingStatus"))						// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.PostingStatus = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:postingStatusDescription"))			// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.PostingStatusDescription = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:xbrlInclude"))							// beginning_balance, ending_balance
			psXbrlGlParam->XbrlGlEntriesDetails.XbrlInclude = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:detailComment"))						// TRANS title
			psXbrlGlParam->XbrlGlEntriesDetails.DetailComment = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:dateAcknowledged"))					// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.DateAcknowledged = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:confirmedDate"))						// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.ConfirmedDate = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:shipFrom"))							// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.ShipFrom = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:shipReceivedDate"))					// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.ShipReceivedDate = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:maturityDate"))						// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.MaturityDate = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:terms"))								// Not used
			psXbrlGlParam->XbrlGlEntriesDetails.Terms = pTagMainData;	

		// Header
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:postedDate"))							// 
			psXbrlGlParam->XbrlGlEntriesHeader.PostedDate = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:enteredBy"))							// 
			psXbrlGlParam->XbrlGlEntriesHeader.EnteredBy = pTagMainData;																																								
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:enteredDate"))							// 
			psXbrlGlParam->XbrlGlEntriesHeader.EnteredDate = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:entryType"))							// 
			psXbrlGlParam->XbrlGlEntriesHeader.EntryType = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:entryNumber"))							// 
			psXbrlGlParam->XbrlGlEntriesHeader.EntryNumber = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:entryComment"))						// 
			psXbrlGlParam->XbrlGlEntriesHeader.EntryComment = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:qualifierEntry"))						// 
			psXbrlGlParam->XbrlGlEntriesHeader.QualifierEntry = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:batchID"))								// #DIM
			psXbrlGlParam->XbrlGlEntriesHeader.BatchID = pTagMainData;


		// Entity info  - Entity information is much more rare and at the end here
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:entityInformation"))					// Process Entity informatiom	
			XbrlGlTagEntityInformationEntityInformation(psXbrlGlParam, pTagMainData, pTagData);
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:organizationIdentifiers"))				// Process Entity organizationIdentifiers
			XbrlGlTagEntityInformationOrganizationIdentifiers(psXbrlGlParam, pTagMainData, pTagData);
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:organizationIdentifier"))				// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.OrganizationIdentifier = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:organizationDescription"))				// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.OrganizationDescription = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:phoneNumber"))							// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.PhoneNumber = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:entityEmailAddress"))					// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.EntityEmailAddress = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:organizationAddressName"))				// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressName = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:organizationBuildingNumber"))			// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.OrganizationBuildingNumber = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:organizationAddressStreet"))			// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStreet = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:organizationAddressStreet2"))			// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStreet2 = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:organizationAddressStateOrProvince"))	// ADRESS		
			psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStateOrProvince = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:organizationAddressZipOrPostalCode"))	// ADRESS		
			psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressZipOrPostalCode = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:organizationAddressCity"))				// ADRESS		
			psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressCity = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:organizationAddressCountry"))			// ADRESS		
			psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressCountry = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:contactFirstName"))					// ADRESS		
			psXbrlGlParam->XbrlGlEntityInformation.ContactFirstName = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:contactLastName"))						// ADRESS		
			psXbrlGlParam->XbrlGlEntityInformation.ContactLastName = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:contactPhoneNumber"))					// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.ContactPhoneNumber = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:contactEmailAddress"))					// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.ContactEMail = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:webSiteURL"))							// Entity
			XbrlGlTagXbrliwebSiteURL(psXbrlGlParam, pTagMainData, pTagData);
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:businessDescription"))					// SNI
			psXbrlGlParam->XbrlGlEntityInformation.BusinessDescription = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:fiscalYearStart"))						// RAR	
			psXbrlGlParam->XbrlGlEntityInformation.FiscalYearStart = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:fiscalYearEnd"))						// RAR	
			psXbrlGlParam->XbrlGlEntityInformation.FiscalYearEnd = pTagMainData;

		// Entity info  - Entity information is much more rare and at the end here								// Accountant
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:accountantInformation"))				// Process Entity informatiom	
			XbrlGlTagEntityInformationAccountantInformation(psXbrlGlParam, pTagMainData, pTagData);
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:accountantName"))						// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.AccountantAddressName = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:accountantAddressName"))				// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.AccountantAddressName = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:accountantBuildingNumber"))			// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.AccountantBuildingNumber = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:accountantStreet"))					// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.AccountantAddressStreet = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:accountantAddressStreet2"))			// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.AccountantAddressStreet2 = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:accountantStateOrProvince"))			// ADRESS		
			psXbrlGlParam->XbrlGlEntityInformation.AccountantAddressStateOrProvince = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:accountantZipOrPostalCode"))			// ADRESS		
			psXbrlGlParam->XbrlGlEntityInformation.AccountantAddressZipOrPostalCode = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:accountantCity"))						// ADRESS		
			psXbrlGlParam->XbrlGlEntityInformation.AccountantAddressCity = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:accountantCountry"))					// ADRESS		
			psXbrlGlParam->XbrlGlEntityInformation.AccountantAddressCountry = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:accountantContactFirstName"))			// ADRESS		
			psXbrlGlParam->XbrlGlEntityInformation.AccountantContactLastName = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:accountantContactLastName"))			// ADRESS		
			psXbrlGlParam->XbrlGlEntityInformation.AccountantContactLastName = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:accountantContactPhoneNumber"))		// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.AccountantContactPhoneNumber = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:accountantContactEmailAddress"))		// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.AccountantContactEMail = pTagMainData;

		// Entity info  - Entity information is much more rare and at the end here								// Accountant
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:auditInformation"))					// Process Entity informatiom	
			XbrlGlTagEntityInformationAuditInformation(psXbrlGlParam, pTagMainData, pTagData);
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:auditName"))							// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.AccountantAddressName = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:auditAddressName"))					// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.AccountantAddressName = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:auditBuildingNumber"))					// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.AccountantBuildingNumber = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:auditStreet"))							// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.AccountantAddressStreet = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:auditAddressStreet2"))					// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.AccountantAddressStreet2 = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:auditStateOrProvince"))				// ADRESS		
			psXbrlGlParam->XbrlGlEntityInformation.AccountantAddressStateOrProvince = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:auditZipOrPostalCode"))				// ADRESS		
			psXbrlGlParam->XbrlGlEntityInformation.AccountantAddressZipOrPostalCode = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:auditCity"))							// ADRESS		
			psXbrlGlParam->XbrlGlEntityInformation.AccountantAddressCity = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:auditCountry"))						// ADRESS		
			psXbrlGlParam->XbrlGlEntityInformation.AccountantAddressCountry = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:auditContactFirstName"))				// ADRESS		
			psXbrlGlParam->XbrlGlEntityInformation.AccountantContactLastName = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:auditContactLastName"))				// ADRESS		
			psXbrlGlParam->XbrlGlEntityInformation.AccountantContactLastName = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:auditContactPhoneNumber"))				// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.AccountantContactPhoneNumber = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:auditContactEmailAddress"))			// ADRESS	
			psXbrlGlParam->XbrlGlEntityInformation.AccountantContactEMail = pTagMainData;
		// Nonsupported info
		else
			XbrlGlTagNotInterpret(psXbrlGlParam, pTagMainData, pTagData);										// Non supported command
		break;
	default: // XT_NONE
		// Document info
		if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:documentInfo"))								// Process Document informatiom	
			XbrlGlTagDocumentInfoDocumentInfo(psXbrlGlParam, pTagMainData, pTagData);
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:uniqueID"))							// Accounting year identifier
			psXbrlGlParam->XbrlGlDocumentInfo.UniqueID = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:language"))							// Language
			XbrlGlTagDocumentInfoLanguage(psXbrlGlParam, pTagMainData, pTagData);
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:entriesType"))							// Data Type Switch
			psXbrlGlParam->XbrlGlDocumentInfo.EntriesType = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:creationDate"))						// GEN date
			psXbrlGlParam->XbrlGlDocumentInfo.CreationDate = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:creator"))								// GEN sign
			psXbrlGlParam->XbrlGlDocumentInfo.Creator = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:entriesComment"))						// Not used, only for XSL
			psXbrlGlParam->XbrlGlDocumentInfo.EntriesComment = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:pProgram"))							// PROGRAM
			psXbrlGlParam->XbrlGlDocumentInfo.pProgram = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-bus:targetApplication"))					// PROGRAM
			psXbrlGlParam->XbrlGlDocumentInfo.TargetApplication = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:periodCoveredStart"))					// RAR start
			psXbrlGlParam->XbrlGlDocumentInfo.PeriodCoveredStart = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-cor:periodCoveredEnd"))					// RAR end
			psXbrlGlParam->XbrlGlDocumentInfo.PeriodCoveredEnd = pTagMainData;
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<gl-muc:defaultCurrency"))						// VALUTA
			psXbrlGlParam->XbrlGlDocumentInfo.DefaultCurrency = GetXbrlCurrency(psXbrlGlParam, pTagMainData);

		// Context info - Context information is much more rare and at the end here
		if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<xbrli:measure"))									// VALUTA
			XbrlGlTagXbrliUnitMeasure(psXbrlGlParam, pTagMainData, pTagData);
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<xbrli:context"))								// 
			XbrlGlTagXbrliContext(psXbrlGlParam, pTagMainData, pTagData);
		else if (XbrlGlIdentifyTags(&pTagMainData, &pSearchEnd, "<xbrli:instant"))								// 
			XbrlGlTagXbrliContextPeriodInstant(psXbrlGlParam, pTagMainData, pTagData);							// Nonsupported info
			else
				XbrlGlTagNotInterpret(psXbrlGlParam, pTagMainData, pTagData);	// Unknown command
			break;
		}
		return INTERPRET_SYNTAX_OK;
}

/********************************************************************

static BITMASK	XbrlGlTagXbrliUnitMeasure(

*********************************************************************/
static BITMASK	XbrlGlTagXbrliUnitMeasure(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)	
{
	// Quantity unit
	char *psTagPreviousTag = 0;
	char *psXbrlStart = 0;
	char *psXbrlEnd = 0;
	char *psIso4217Start = 0;
	char *psIso4217End = 0;

	psTagPreviousTag = LListGetPreviousRecordData(psXbrlGlParam->pXmlPath);
	if (!psTagPreviousTag)
		return XBRLGL_SYNTAX_ERR_FATAL; // Syntax Error
	psXbrlStart = strstr(psTagPreviousTag, "<xbrli:unit");
	if (!psXbrlStart)
		return XBRLGL_SYNTAX_ERR_FATAL; // Syntax Error
	psXbrlStart += strlen("<xbrli:unit");
	psXbrlStart = strstr(psXbrlStart, "id");
	if (!psXbrlStart)
		return XBRLGL_SYNTAX_ERR_FATAL; // Syntax Error
	psXbrlStart++;
	psXbrlStart++;
	while (*psXbrlStart && ((*psXbrlStart <= ' ') || (*psXbrlStart == '"') || (*psXbrlStart == '=')))
		psXbrlStart++;
	psXbrlEnd = strstr(psXbrlStart, ">");
	if (!psXbrlEnd)
		return XBRLGL_SYNTAX_ERR_FATAL; // Syntax Error
	psXbrlEnd--;
	while ((*psXbrlEnd == ' ') || (*psXbrlEnd == '"'))
		psXbrlEnd--;
	psXbrlEnd++;
	*psXbrlEnd = 0;

	if (!pTagMainData)
		return XBRLGL_SYNTAX_ERR_FATAL; // Syntax Error
	psIso4217Start = strstr(pTagMainData, PREFIX_ISO4217_CURR);
	if (!psIso4217Start)
		return XBRLGL_SYNTAX_NOT_USED; // Syntax Error
	psIso4217Start += strlen(PREFIX_ISO4217_CURR);
	while ((*psIso4217Start <= ' ') || (*psIso4217Start == '"'))
		psIso4217Start++;
	psIso4217End = psIso4217Start;
	while ((*psIso4217End > ' ') && (*psIso4217End != '"'))
		psIso4217End++;
	*psIso4217End = 0;

	{
		XbrlGlCurrenciesType *XbrlGlCurrency = calloc(1, sizeof(XbrlGlCurrenciesType));
		if (XbrlGlCurrency) {
			CopyString(&XbrlGlCurrency->XbrlCurrency, psXbrlStart);
			CopyString(&XbrlGlCurrency->Iso4217Currency, psIso4217Start);

			if (RET_OK != LListAddRecord(psXbrlGlParam->pCurrencies, XbrlGlCurrency)) {
				// realse the last data when we can't go on, due to memory fuzz
				free(XbrlGlCurrency);
				// Error, report it
				return XBRLGL_SYNTAX_ERR_FATAL; // Fatal Error
			}
		}
		else
			return XBRLGL_SYNTAX_ERR_FATAL;
	}
	return INTERPRET_SYNTAX_OK;
}

/********************************************************************

static char *GetXbrlCurrency(

*********************************************************************/
static char *GetXbrlCurrency(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *XbrlCurrencyString)	
{
	// Curr unit
	XbrlGlCurrenciesType TargetXbrlGlCurrency;
	char *psIso4217 = 0;
	memset(&TargetXbrlGlCurrency, 0, sizeof(XbrlGlCurrenciesType));
	if (!XbrlCurrencyString)
		return 0;  // Syntax Error
	psIso4217 = strstr(XbrlCurrencyString, PREFIX_ISO4217_CURR);
	if (psIso4217)
		psIso4217 += strlen(PREFIX_ISO4217_CURR);
	else // No psIso4217 prefix
		psIso4217 = XbrlCurrencyString;

	{
		XbrlGlCurrenciesType *XbrlGlCurrency = LListGetFirstRecordData(psXbrlGlParam->pCurrencies);
		while (XbrlGlCurrency) {
			if (!strcmp(XbrlGlCurrency->XbrlCurrency, psIso4217))
				return XbrlGlCurrency->Iso4217Currency;
			XbrlGlCurrency = LListGetNextRecordData(psXbrlGlParam->pCurrencies);
		}
	}

	return 0;
}

/********************************************************************

static BITMASK	XbrlGlTagXbrliContext(

*********************************************************************/
static BITMASK	XbrlGlTagXbrliContext(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)	
{
	char *psXbrlStart = 0;
	char *psXbrlEnd = 0;

	if (!pTagData)
		return XBRLGL_SYNTAX_ERR_FATAL; // Syntax Error
	psXbrlStart = strstr(pTagData, "<xbrli:context");
	if (!psXbrlStart)
		return XBRLGL_SYNTAX_ERR_FATAL; // Syntax Error
	psXbrlStart += strlen("<xbrli:context");
	psXbrlStart = strstr(psXbrlStart, XBRL_ID_ID_STRING);
	if (!psXbrlStart)
		return XBRLGL_SYNTAX_ERR_FATAL; // Syntax Error
	psXbrlStart += strlen(XBRL_ID_ID_STRING);
	psXbrlStart = strstr(psXbrlStart, XBRL_ID_PREFIX_STRING);
	if (!psXbrlStart)
		return XBRLGL_SYNTAX_ERR_FATAL; // Syntax Error
	psXbrlStart += strlen(XBRL_ID_PREFIX_STRING);
	psXbrlEnd = strstr(psXbrlStart, ">");
	if (!psXbrlEnd)
		return XBRLGL_SYNTAX_ERR_FATAL; // Syntax Error
	psXbrlEnd--;
	while ((*psXbrlEnd == ' ') || (*psXbrlEnd == '"'))
		psXbrlEnd--;
	psXbrlEnd++;
	*psXbrlEnd = 0;

	psXbrlGlParam->XbrliContext = psXbrlStart;
#if 0
	// Store data
	if (!psXbrlGlParam->pAccounting->pEntity)
		psXbrlGlParam->pAccounting->pEntity = CreateParty(psXbrlGlParam->pAccounting, psXbrlGlParam->pAccounting);
	CopyString(&psXbrlGlParam->pAccounting->pEntity->pLegalRegId, psXbrlStart);
#endif
	return INTERPRET_SYNTAX_OK;
}

/********************************************************************

static BITMASK	XbrlGlTagXbrliContextPeriodInstant(

*********************************************************************/
static BITMASK	XbrlGlTagXbrliContextPeriodInstant(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)	
{
	psXbrlGlParam->pAccounting->LastEntryDate = StringToDate(pTagMainData);

	return INTERPRET_SYNTAX_OK;
}

/********************************************************************

static BITMASK	XbrlGlTagXbrliContextScenarioVatRegistration(

*********************************************************************/
/*static BITMASK	XbrlGlTagXbrliContextScenarioVatRegistration(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)
{
	// Store data
	if (pTagMainData && *pTagMainData) {
		if (!psXbrlGlParam->pAccounting->pEntity)
			psXbrlGlParam->pAccounting->pEntity = CreateParty(psXbrlGlParam->pAccounting, psXbrlGlParam->pAccounting);
		CopyString(&psXbrlGlParam->pAccounting->pEntity->pVatRegId, pTagMainData);
	}
	return INTERPRET_SYNTAX_OK;
}
*/
/********************************************************************

static BITMASK	XbrlGlTagXbrliContextScenarioIncomeTaxRegistration(

*********************************************************************/
/*
static BITMASK	XbrlGlTagXbrliContextScenarioIncomeTaxRegistration(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)
{
	// Store data
	if (pTagMainData && *pTagMainData) {
		if (!psXbrlGlParam->pAccounting->pEntity)
			psXbrlGlParam->pAccounting->pEntity = CreateParty(psXbrlGlParam->pAccounting, psXbrlGlParam->pAccounting);
		CopyString(&psXbrlGlParam->pAccounting->pEntity->pLegalRegId, pTagMainData);
	}
	return INTERPRET_SYNTAX_OK;
}
*/

/********************************************************************

static BITMASK	XbrlGlTagXbrliContextScenarioLogotype(

*********************************************************************/
/*
static BITMASK	XbrlGlTagXbrliContextScenarioLogotype(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)
{
	// Store data
	if (pTagMainData && *pTagMainData) {
		if (!psXbrlGlParam->pAccounting->pEntity)
			psXbrlGlParam->pAccounting->pEntity = CreateParty(psXbrlGlParam->pAccounting, psXbrlGlParam->pAccounting);
		if (pTagMainData && *pTagMainData)
			SearchSetToken(psXbrlGlParam->pAccounting->pEntity->pListLogotypeFileUrl, CopyString(0, pTagMainData));
	}
	return INTERPRET_SYNTAX_OK;
}
*/

/********************************************************************

static BITMASK	XbrlGlTagXbrliwebSiteURL(

*********************************************************************/
static BITMASK	XbrlGlTagXbrliwebSiteURL(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)
{
	// Store data
	if (pTagMainData && *pTagMainData) {
		if (!psXbrlGlParam->pAccounting->pEntity)
			psXbrlGlParam->pAccounting->pEntity = CreateParty(psXbrlGlParam->pAccounting, psXbrlGlParam->pAccounting);
		if (pTagMainData && *pTagMainData)
			SearchSetToken(psXbrlGlParam->pAccounting->pEntity->pListWebPage, CopyString(0, pTagMainData));
	}
	return INTERPRET_SYNTAX_OK;
}


/********************************************************************

static BITMASK	XbrlGlTagXbrliContextScenarioLegalRegistrationFileUrl(

*********************************************************************/
/*static BITMASK	XbrlGlTagXbrliContextScenarioLegalRegistrationFileUrl(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)
{
	// Store data
	if (pTagMainData && *pTagMainData) {
		if (!psXbrlGlParam->pAccounting->pEntity)
			psXbrlGlParam->pAccounting->pEntity = CreateParty(psXbrlGlParam->pAccounting, psXbrlGlParam->pAccounting);
		if (pTagMainData && *pTagMainData)
			SearchSetToken(psXbrlGlParam->pAccounting->pEntity->pListLegalFileUrl, CopyString(0, pTagMainData));
	}
	return INTERPRET_SYNTAX_OK;
}
*/

/********************************************************************

static BITMASK	XbrlGlTagXbrliContextScenarioVatRegistrationFileUrl(

*********************************************************************/
/*
static BITMASK	XbrlGlTagXbrliContextScenarioVatRegistrationFileUrl(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)
{
	// Store data
	if (pTagMainData && *pTagMainData) {
		if (!psXbrlGlParam->pAccounting->pEntity)
			psXbrlGlParam->pAccounting->pEntity = CreateParty(psXbrlGlParam->pAccounting, psXbrlGlParam->pAccounting);
		if (pTagMainData && *pTagMainData)
			SearchSetToken(psXbrlGlParam->pAccounting->pEntity->pListVatFileUrl, CopyString(0, pTagMainData));
	}
	return INTERPRET_SYNTAX_OK;
}
*/

/********************************************************************

static BITMASK	XbrlGlTagXbrliContextScenarioIncomeTaxRegistrationFileUrl(

*********************************************************************/
/*
static BITMASK	XbrlGlTagXbrliContextScenarioIncomeTaxRegistrationFileUrl(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)
{
	// Store data
	if (pTagMainData && *pTagMainData) {
		if (!psXbrlGlParam->pAccounting->pEntity)
			psXbrlGlParam->pAccounting->pEntity = CreateParty(psXbrlGlParam->pAccounting, psXbrlGlParam->pAccounting);
		if (pTagMainData && *pTagMainData)
			SearchSetToken(psXbrlGlParam->pAccounting->pEntity->pListIncomeTaxFileUrl, CopyString(0, pTagMainData));
	}
	return INTERPRET_SYNTAX_OK;
}
*/

/********************************************************************

static BITMASK	XbrlGlTagXbrliContextScenarioReferenceFileUrl(

*********************************************************************/
/*static BITMASK	XbrlGlTagXbrliContextScenarioReferenceFileUrl(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)
{
	// Store data
	if (pTagMainData && *pTagMainData) {
		if (!psXbrlGlParam->pAccounting->pEntity)
			psXbrlGlParam->pAccounting->pEntity = CreateParty(psXbrlGlParam->pAccounting, psXbrlGlParam->pAccounting);
		if (pTagMainData && *pTagMainData)
			SearchSetToken(psXbrlGlParam->pAccounting->pEntity->pListFileUrl, CopyString(0, pTagMainData));
	}
	return INTERPRET_SYNTAX_OK;
}
*/

/********************************************************************

static BITMASK	XbrlGlTagXbrliContextScenarioAuditAttestant(

*********************************************************************/
/*
static BITMASK	XbrlGlTagXbrliContextScenarioAuditAttestant(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)
{
	// Store data
	if (pTagMainData && *pTagMainData) {
		if (!psXbrlGlParam->pAccounting->pAudit)
			psXbrlGlParam->pAccounting->pAudit = CreateParty(psXbrlGlParam->pAccounting, psXbrlGlParam->pAccounting);
		CopyString(&psXbrlGlParam->pAccounting->pAudit->pName, pTagMainData);
	}
	return INTERPRET_SYNTAX_OK;
}
*/

/********************************************************************

static BITMASK	XbrlGlTagXbrliContextScenarioAuditAttestPkiFileUrl(

*********************************************************************/
/*
static BITMASK	XbrlGlTagXbrliContextScenarioAuditAttestPkiFileUrl(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)
{
	// Store data
	if (pTagMainData && *pTagMainData) {
		AttestType *pAttest = CreateAttest(psXbrlGlParam->pAccounting, psXbrlGlParam->pAccounting);
		//		if(psXbrlGlParam->pAccounting->pAudit)
		//			pAttest->pAttestant = psXbrlGlParam->pAccounting->pAudit->pName;
		if (pTagMainData && *pTagMainData)
			SearchSetToken(pAttest->pListPkiNote, pTagMainData);
		LListAddRecord(psXbrlGlParam->pAccounting->pListAuditApproved, pAttest);
	}
	return INTERPRET_SYNTAX_OK;
}
*/

/********************************************************************

static BITMASK	XbrlGlTagXbrliContextScenarioPurpose(

*********************************************************************/
/*
static BITMASK	XbrlGlTagXbrliContextScenarioPurpose(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, char *pTagData)	// SIE Type 
{
	// Store data
	if (pTagMainData && *pTagMainData) {
		CopyString(&psXbrlGlParam->pAccounting->pContent, pTagMainData);
	}
	return INTERPRET_SYNTAX_OK;
}
*/

// XBRL GL DocumentInfo

/********************************************************************

static BITMASK	XbrlGlTagDocumentInfoDocumentInfo(

*********************************************************************/
static BITMASK	XbrlGlTagDocumentInfoDocumentInfo(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)	// Info of XML-data
{
	if (!psXbrlGlParam->XbrlGlDocumentInfo.EntriesType)
		return XBRLGL_SYNTAX_ERR_FATAL; // Syntax Error
	if (!strcmp(psXbrlGlParam->XbrlGlDocumentInfo.EntriesType, "account"))
		psXbrlGlParam->XbrlGlItemType = XT_ACCOUNT;
	else if (!strcmp(psXbrlGlParam->XbrlGlDocumentInfo.EntriesType, "other"))
		psXbrlGlParam->XbrlGlItemType = XT_IDENTIFIER;
	else if (!strcmp(psXbrlGlParam->XbrlGlDocumentInfo.EntriesType, "mapping"))
		psXbrlGlParam->XbrlGlItemType = XT_MAPPING;
	else if (!strcmp(psXbrlGlParam->XbrlGlDocumentInfo.EntriesType, "balance"))
		psXbrlGlParam->XbrlGlItemType = XT_BALANCE;
	else if (!strcmp(psXbrlGlParam->XbrlGlDocumentInfo.EntriesType, "trialbalance"))
		psXbrlGlParam->XbrlGlItemType = XT_BALANCE;
	else if (!strcmp(psXbrlGlParam->XbrlGlDocumentInfo.EntriesType, "journal"))
		psXbrlGlParam->XbrlGlItemType = XT_VOUCHER;
	else
		return XBRLGL_SYNTAX_ERR_FATAL; // Syntax Error

	if (psXbrlGlParam->XbrlGlDocumentInfo.Language && *psXbrlGlParam->XbrlGlDocumentInfo.Language)
		CopyString(&psXbrlGlParam->pAccounting->pDefaultLanguage, psXbrlGlParam->XbrlGlDocumentInfo.Language);
	// #GEN
	if ((psXbrlGlParam->XbrlGlDocumentInfo.Creator && *psXbrlGlParam->XbrlGlDocumentInfo.Creator)
		|| (psXbrlGlParam->XbrlGlDocumentInfo.CreationDate && *psXbrlGlParam->XbrlGlDocumentInfo.CreationDate))
		psXbrlGlParam->pAccounting->pGeneratingUser
		= AseembleOperatorInfo(psXbrlGlParam->pAccounting, psXbrlGlParam->pAccounting,
			psXbrlGlParam->XbrlGlDocumentInfo.Creator,
			StringToDate(psXbrlGlParam->XbrlGlDocumentInfo.CreationDate));
	// #PROGRAM
	if (psXbrlGlParam->XbrlGlDocumentInfo.pProgram && *psXbrlGlParam->XbrlGlDocumentInfo.pProgram) {
		if (!psXbrlGlParam->pAccounting->pProgram)
			psXbrlGlParam->pAccounting->pProgram = CreateProgramInfo(psXbrlGlParam->pAccounting, psXbrlGlParam->pAccounting);
		CopyString(&psXbrlGlParam->pAccounting->pProgram->pProgramInfo, psXbrlGlParam->XbrlGlDocumentInfo.pProgram);
	}

	//	if(psXbrlGlParam->XbrlGlDocumentInfo.TargetApplication)
	//		CopyString(&psXbrlGlParam->pAccounting->TargetApplication, psXbrlGlParam->XbrlGlDocumentInfo.TargetApplication);

	if (psXbrlGlParam->XbrlGlDocumentInfo.DefaultCurrency && *psXbrlGlParam->XbrlGlDocumentInfo.DefaultCurrency)
		CopyString(&psXbrlGlParam->pAccounting->pCurrency, psXbrlGlParam->XbrlGlDocumentInfo.DefaultCurrency);

	XbrlGlTagGeneralGlcorEntryHeaderReset(psXbrlGlParam);
	XbrlGlTagGeneralGlcorEntryDetailReset(psXbrlGlParam);
	return INTERPRET_SYNTAX_OK;
}


// XBRL GL DocumentInfo

/********************************************************************

static BITMASK	XbrlGlTagEntityInformationEntityInformation(

*********************************************************************/
static BITMASK	XbrlGlTagEntityInformationEntityInformation(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)	// Info of XML-data
{
	if (!psXbrlGlParam->pAccounting->pEntity)
		psXbrlGlParam->pAccounting->pEntity = CreateParty(psXbrlGlParam->pAccounting, psXbrlGlParam->pAccounting);
	if (!psXbrlGlParam->pAccounting->pEntity)
		return INTERPRET_SYNTAX_ERR_FATAL; // Fatal Error
	psXbrlGlParam->pAccounting->pEntity->CashAccounting = (psXbrlGlParam->XbrlGlEntityInformation.CashAccounting && (*psXbrlGlParam->XbrlGlEntityInformation.CashAccounting == '1'));
	if (psXbrlGlParam->XbrlGlEntityInformation.VatCurrency  && *psXbrlGlParam->XbrlGlEntityInformation.VatCurrency)
		CopyString(&psXbrlGlParam->pAccounting->pEntity->pVatCurrency, psXbrlGlParam->XbrlGlEntityInformation.VatCurrency);
	if (psXbrlGlParam->XbrlGlEntityInformation.BusinessDescription  && *psXbrlGlParam->XbrlGlEntityInformation.BusinessDescription)
		CopyString(&psXbrlGlParam->pAccounting->pEntity->pEntityType, psXbrlGlParam->XbrlGlEntityInformation.BusinessDescription);
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressName && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressName)
		CopyString(&psXbrlGlParam->pAccounting->pEntity->pName, psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressName);
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationBuildingNumber && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationBuildingNumber)
		SearchSetToken(psXbrlGlParam->pAccounting->pEntity->pListAddress, CopyString(0, psXbrlGlParam->XbrlGlEntityInformation.OrganizationBuildingNumber));
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStreet && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStreet)
		SearchSetToken(psXbrlGlParam->pAccounting->pEntity->pListAddress, CopyString(0, psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStreet));
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStreet2 && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStreet2)
		SearchSetToken(psXbrlGlParam->pAccounting->pEntity->pListAddress, CopyString(0, psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStreet2));
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStateOrProvince && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStateOrProvince)
		SearchSetToken(psXbrlGlParam->pAccounting->pEntity->pListAddress, CopyString(0, psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStateOrProvince));
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressZipOrPostalCode && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressZipOrPostalCode)
		CopyString(&psXbrlGlParam->pAccounting->pEntity->pPostalCode, psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressZipOrPostalCode);
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressCity && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressCity)
		CopyString(&psXbrlGlParam->pAccounting->pEntity->pPostalAddress, psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressCity);
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressCountry && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressCountry)
		CopyString(&psXbrlGlParam->pAccounting->pEntity->pPostalCountry, psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressCountry);
	if (psXbrlGlParam->XbrlGlEntityInformation.PhoneNumber && *psXbrlGlParam->XbrlGlEntityInformation.PhoneNumber)
		CopyString(&psXbrlGlParam->pAccounting->pEntity->pEntityPhone, psXbrlGlParam->XbrlGlEntityInformation.PhoneNumber);
	if (psXbrlGlParam->XbrlGlEntityInformation.EntityEmailAddress && *psXbrlGlParam->XbrlGlEntityInformation.EntityEmailAddress)
		CopyString(&psXbrlGlParam->pAccounting->pEntity->pEntityEmail, psXbrlGlParam->XbrlGlEntityInformation.EntityEmailAddress);
	if (((psXbrlGlParam->XbrlGlEntityInformation.ContactFirstName && *psXbrlGlParam->XbrlGlEntityInformation.ContactFirstName)
		|| (psXbrlGlParam->XbrlGlEntityInformation.ContactLastName && *psXbrlGlParam->XbrlGlEntityInformation.ContactLastName))
		|| (psXbrlGlParam->XbrlGlEntityInformation.ContactPhoneNumber && *psXbrlGlParam->XbrlGlEntityInformation.ContactPhoneNumber)
		|| (psXbrlGlParam->XbrlGlEntityInformation.ContactEMail && *psXbrlGlParam->XbrlGlEntityInformation.ContactEMail)) {
		ContactType *pContact = AddContact(psXbrlGlParam->pAccounting->pEntity, XbrlGlParamNameUnification(
			psXbrlGlParam->XbrlGlEntityInformation.ContactFirstName,
			psXbrlGlParam->XbrlGlEntityInformation.ContactLastName));
		if (pContact) {
			if (psXbrlGlParam->XbrlGlEntityInformation.ContactPhoneNumber && *psXbrlGlParam->XbrlGlEntityInformation.ContactPhoneNumber)
				SearchSetToken(pContact->pListPhone, CopyString(0, psXbrlGlParam->XbrlGlEntityInformation.ContactPhoneNumber));
			if (psXbrlGlParam->XbrlGlEntityInformation.ContactEMail && *psXbrlGlParam->XbrlGlEntityInformation.ContactEMail)
				SearchSetToken(pContact->pListEmail, CopyString(0, psXbrlGlParam->XbrlGlEntityInformation.ContactEMail));
		}
		else
			return INTERPRET_SYNTAX_ERR_FATAL; // Fatal Error
	}
	switch (psXbrlGlParam->XbrlGlItemType) {
	case XT_BALANCE:
	case XT_MAPPING:
		// #RAR
	{
		if ((psXbrlGlParam->XbrlGlEntityInformation.FiscalYearStart &&*psXbrlGlParam->XbrlGlEntityInformation.FiscalYearStart)
			&& (psXbrlGlParam->XbrlGlEntityInformation.FiscalYearEnd && *psXbrlGlParam->XbrlGlEntityInformation.FiscalYearEnd)) {
			LedgerType *pLedger = GetLedgerByDate(psXbrlGlParam->pAccounting, StringToDate(psXbrlGlParam->XbrlGlEntityInformation.FiscalYearStart));
			if (!pLedger) {
				pLedger = CreateLedger(psXbrlGlParam->pAccounting, psXbrlGlParam->pAccounting->pLedgersChart);
				if (pLedger)
					LListAddRecord(psXbrlGlParam->pAccounting->pLedgersChart->pListLedger, pLedger);
				else
					return INTERPRET_SYNTAX_ERR_FATAL; // Fatal Error
			}
			if (psXbrlGlParam->XbrlGlDocumentInfo.UniqueID)
				pLedger->Id = StringToNumeric(psXbrlGlParam->XbrlGlDocumentInfo.UniqueID);
			// If UniqueID is nothing not need to set 0, is
			// OMFATTN date
			if (!psXbrlGlParam->pAccounting->LastEntryDate)
				psXbrlGlParam->pAccounting->LastEntryDate = StringToDate(psXbrlGlParam->XbrlGlEntityInformation.FiscalYearEnd);
			if (psXbrlGlParam->XbrlGlEntityInformation.FiscalYearStart)
				pLedger->YearStart = StringToDate(psXbrlGlParam->XbrlGlEntityInformation.FiscalYearStart);
			if (psXbrlGlParam->XbrlGlEntityInformation.FiscalYearEnd)
				pLedger->YearEnd = StringToDate(psXbrlGlParam->XbrlGlEntityInformation.FiscalYearEnd);
		}
	} // RAR
	break;
	case XT_ACCOUNT:
	case XT_IDENTIFIER:
	case XT_VOUCHER:
	default:
		break;
	}
	XbrlGlTagGeneralGlcorEntryHeaderReset(psXbrlGlParam);
	XbrlGlTagGeneralGlcorEntryDetailReset(psXbrlGlParam);
	return INTERPRET_SYNTAX_OK;
}


/********************************************************************

static BITMASK	XbrlGlTagEntityInformationOrganizationIdentifiers(

*********************************************************************/
static BITMASK	XbrlGlTagEntityInformationOrganizationIdentifiers(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)	// Info of XML-data
{
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationIdentifier && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationIdentifier) {
		if (!strcmp(psXbrlGlParam->XbrlGlEntityInformation.OrganizationDescription, "Vat")) {
			if (!psXbrlGlParam->pAccounting->pEntity)
				psXbrlGlParam->pAccounting->pEntity = CreateParty(psXbrlGlParam->pAccounting, psXbrlGlParam->pAccounting);
			if (!psXbrlGlParam->pAccounting->pEntity)
				return INTERPRET_SYNTAX_ERR_FATAL; // Fatal Error
			CopyString(&psXbrlGlParam->pAccounting->pEntity->pVatRegId, psXbrlGlParam->XbrlGlEntityInformation.OrganizationIdentifier);
		}
		else if (!strcmp(psXbrlGlParam->XbrlGlEntityInformation.OrganizationDescription, "Statistical")) {
			if (!psXbrlGlParam->pAccounting->pEntity)
				psXbrlGlParam->pAccounting->pEntity = CreateParty(psXbrlGlParam->pAccounting, psXbrlGlParam->pAccounting);
			if (!psXbrlGlParam->pAccounting->pEntity)
				return INTERPRET_SYNTAX_ERR_FATAL; // Fatal Error
			CopyString(&psXbrlGlParam->pAccounting->pEntity->pStatisticalClass, psXbrlGlParam->XbrlGlEntityInformation.OrganizationIdentifier);
		}
		else if (!strcmp(psXbrlGlParam->XbrlGlEntityInformation.OrganizationDescription, "Jurisdiction")) {
			if (!psXbrlGlParam->pAccounting->pEntity)
				psXbrlGlParam->pAccounting->pEntity = CreateParty(psXbrlGlParam->pAccounting, psXbrlGlParam->pAccounting);
			if (!psXbrlGlParam->pAccounting->pEntity)
				return INTERPRET_SYNTAX_ERR_FATAL; // Fatal Error
			CopyString(&psXbrlGlParam->pAccounting->pEntity->pJurisdiction, psXbrlGlParam->XbrlGlEntityInformation.OrganizationIdentifier);
		}
		else {
			if (!psXbrlGlParam->pAccounting->pEntity)
				psXbrlGlParam->pAccounting->pEntity = CreateParty(psXbrlGlParam->pAccounting, psXbrlGlParam->pAccounting);
			if (!psXbrlGlParam->pAccounting->pEntity)
				return INTERPRET_SYNTAX_ERR_FATAL; // Fatal Error
			CopyString(&psXbrlGlParam->pAccounting->pEntity->pLegalRegId, psXbrlGlParam->XbrlGlEntityInformation.OrganizationIdentifier);
		}
	}
	return INTERPRET_SYNTAX_OK;
}


/********************************************************************

static BITMASK	XbrlGlTagEntityInformationAccountantInformation(

*********************************************************************/
static BITMASK	XbrlGlTagEntityInformationAccountantInformation(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)	// Info of XML-data
{
	if (!psXbrlGlParam->pAccounting->pAccountant)
		psXbrlGlParam->pAccounting->pAccountant = CreateParty(psXbrlGlParam->pAccounting, psXbrlGlParam->pAccounting);
//	if (!psXbrlGlParam->pAccounting->pAccountant)
//		return INTERPRET_SYNTAX_ERR_FATAL; // Fatal Error
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressName && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressName)
		CopyString(&psXbrlGlParam->pAccounting->pAccountant->pName, psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressName);
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationBuildingNumber && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationBuildingNumber)
		SearchSetToken(psXbrlGlParam->pAccounting->pAccountant->pListAddress, CopyString(0, psXbrlGlParam->XbrlGlEntityInformation.OrganizationBuildingNumber));
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStreet && * psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStreet)
		SearchSetToken(psXbrlGlParam->pAccounting->pAccountant->pListAddress, CopyString(0, psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStreet));
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStreet2 && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStreet2)
		SearchSetToken(psXbrlGlParam->pAccounting->pAccountant->pListAddress, CopyString(0, psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStreet2));
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStateOrProvince && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStateOrProvince)
		SearchSetToken(psXbrlGlParam->pAccounting->pAccountant->pListAddress, CopyString(0, psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStateOrProvince));
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressZipOrPostalCode && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressZipOrPostalCode)
		CopyString(&psXbrlGlParam->pAccounting->pAccountant->pPostalCode, psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressZipOrPostalCode);
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressCity && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressCity)
		CopyString(&psXbrlGlParam->pAccounting->pAccountant->pPostalAddress, psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressCity);
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressCountry && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressCountry)
		CopyString(&psXbrlGlParam->pAccounting->pAccountant->pPostalCountry, psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressCountry);
	if (psXbrlGlParam->XbrlGlEntityInformation.PhoneNumber && *psXbrlGlParam->XbrlGlEntityInformation.PhoneNumber)
		CopyString(&psXbrlGlParam->pAccounting->pAccountant->pEntityPhone, psXbrlGlParam->XbrlGlEntityInformation.PhoneNumber);
	if (psXbrlGlParam->XbrlGlEntityInformation.EntityEmailAddress && *psXbrlGlParam->XbrlGlEntityInformation.EntityEmailAddress)
		CopyString(&psXbrlGlParam->pAccounting->pAccountant->pEntityEmail, psXbrlGlParam->XbrlGlEntityInformation.EntityEmailAddress);
	if (((psXbrlGlParam->XbrlGlEntityInformation.ContactFirstName && *psXbrlGlParam->XbrlGlEntityInformation.ContactFirstName)
		|| (psXbrlGlParam->XbrlGlEntityInformation.ContactLastName && *psXbrlGlParam->XbrlGlEntityInformation.ContactLastName))
		|| (psXbrlGlParam->XbrlGlEntityInformation.ContactPhoneNumber && *psXbrlGlParam->XbrlGlEntityInformation.ContactPhoneNumber)
		|| (psXbrlGlParam->XbrlGlEntityInformation.ContactEMail && *psXbrlGlParam->XbrlGlEntityInformation.ContactEMail)) {
		ContactType *pContact = AddContact(psXbrlGlParam->pAccounting->pEntity, XbrlGlParamNameUnification(
			psXbrlGlParam->XbrlGlEntityInformation.ContactFirstName,
			psXbrlGlParam->XbrlGlEntityInformation.ContactLastName));
		if (pContact) {
			return INTERPRET_SYNTAX_ERR_FATAL; // Fatal Error
/*			if ((psXbrlGlParam->XbrlGlEntityInformation.ContactFirstName && *psXbrlGlParam->XbrlGlEntityInformation.ContactFirstName)
				|| (psXbrlGlParam->XbrlGlEntityInformation.ContactLastName && *psXbrlGlParam->XbrlGlEntityInformation.ContactLastName))
				CopyString(&pContact->pName, XbrlGlParamNameUnification(
					psXbrlGlParam->XbrlGlEntityInformation.ContactFirstName,
					psXbrlGlParam->XbrlGlEntityInformation.ContactLastName));
			if (psXbrlGlParam->XbrlGlEntityInformation.ContactPhoneNumber && *psXbrlGlParam->XbrlGlEntityInformation.ContactPhoneNumber)
				SearchSetToken(pContact->pListPhone, CopyString(0, psXbrlGlParam->XbrlGlEntityInformation.ContactPhoneNumber));
			if (psXbrlGlParam->XbrlGlEntityInformation.ContactEMail && *psXbrlGlParam->XbrlGlEntityInformation.ContactEMail)
				SearchSetToken(pContact->pListEmail, CopyString(0, psXbrlGlParam->XbrlGlEntityInformation.ContactEMail));
 */
		}
	}
	return INTERPRET_SYNTAX_OK;
}

/********************************************************************

static BITMASK	XbrlGlTagEntityInformationAuditInformation(

*********************************************************************/
static BITMASK	XbrlGlTagEntityInformationAuditInformation(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)	// Info of XML-data
{
	if (!psXbrlGlParam->pAccounting->pAudit)
		psXbrlGlParam->pAccounting->pAudit = CreateParty(psXbrlGlParam->pAccounting, psXbrlGlParam->pAccounting);
	if (!psXbrlGlParam->pAccounting->pAudit)
		return INTERPRET_SYNTAX_ERR_FATAL; // Fatal Error
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressName && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressName)
		CopyString(&psXbrlGlParam->pAccounting->pAudit->pName, psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressName);
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationBuildingNumber && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationBuildingNumber)
		SearchSetToken(psXbrlGlParam->pAccounting->pAudit->pListAddress, CopyString(0, psXbrlGlParam->XbrlGlEntityInformation.OrganizationBuildingNumber));
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStreet && * psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStreet)
		SearchSetToken(psXbrlGlParam->pAccounting->pAudit->pListAddress, CopyString(0, psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStreet));
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStreet2 && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStreet2)
		SearchSetToken(psXbrlGlParam->pAccounting->pAudit->pListAddress, CopyString(0, psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStreet2));
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStateOrProvince && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStateOrProvince)
		SearchSetToken(psXbrlGlParam->pAccounting->pAudit->pListAddress, CopyString(0, psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressStateOrProvince));
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressZipOrPostalCode && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressZipOrPostalCode)
		CopyString(&psXbrlGlParam->pAccounting->pAudit->pPostalCode, psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressZipOrPostalCode);
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressCity && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressCity)
		CopyString(&psXbrlGlParam->pAccounting->pAudit->pPostalAddress, psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressCity);
	if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressCountry && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressCountry)
		CopyString(&psXbrlGlParam->pAccounting->pAudit->pPostalCountry, psXbrlGlParam->XbrlGlEntityInformation.OrganizationAddressCountry);
	if (psXbrlGlParam->XbrlGlEntityInformation.PhoneNumber && *psXbrlGlParam->XbrlGlEntityInformation.PhoneNumber)
		CopyString(&psXbrlGlParam->pAccounting->pAudit->pEntityPhone, psXbrlGlParam->XbrlGlEntityInformation.PhoneNumber);
	if (psXbrlGlParam->XbrlGlEntityInformation.EntityEmailAddress && *psXbrlGlParam->XbrlGlEntityInformation.EntityEmailAddress)
		CopyString(&psXbrlGlParam->pAccounting->pAudit->pEntityEmail, psXbrlGlParam->XbrlGlEntityInformation.EntityEmailAddress);
	if (((psXbrlGlParam->XbrlGlEntityInformation.ContactFirstName && *psXbrlGlParam->XbrlGlEntityInformation.ContactFirstName)
		|| (psXbrlGlParam->XbrlGlEntityInformation.ContactLastName && *psXbrlGlParam->XbrlGlEntityInformation.ContactLastName))
		|| (psXbrlGlParam->XbrlGlEntityInformation.ContactPhoneNumber && *psXbrlGlParam->XbrlGlEntityInformation.ContactPhoneNumber)
		|| (psXbrlGlParam->XbrlGlEntityInformation.ContactEMail && *psXbrlGlParam->XbrlGlEntityInformation.ContactEMail)) {
		ContactType *pContact = AddContact(psXbrlGlParam->pAccounting->pEntity, XbrlGlParamNameUnification(
			psXbrlGlParam->XbrlGlEntityInformation.ContactFirstName,
			psXbrlGlParam->XbrlGlEntityInformation.ContactLastName));
		if (pContact) {
			return INTERPRET_SYNTAX_ERR_FATAL; // Fatal Error
/*			if ((psXbrlGlParam->XbrlGlEntityInformation.ContactFirstName && *psXbrlGlParam->XbrlGlEntityInformation.ContactFirstName)
				|| (psXbrlGlParam->XbrlGlEntityInformation.ContactLastName && *psXbrlGlParam->XbrlGlEntityInformation.ContactLastName))
				CopyString(&pContact->pName, XbrlGlParamNameUnification(
					psXbrlGlParam->XbrlGlEntityInformation.ContactFirstName,
					psXbrlGlParam->XbrlGlEntityInformation.ContactLastName));
			if (psXbrlGlParam->XbrlGlEntityInformation.ContactPhoneNumber && *psXbrlGlParam->XbrlGlEntityInformation.ContactPhoneNumber)
				SearchSetToken(pContact->pListPhone, CopyString(0, psXbrlGlParam->XbrlGlEntityInformation.ContactPhoneNumber));
			if (psXbrlGlParam->XbrlGlEntityInformation.ContactEMail && *psXbrlGlParam->XbrlGlEntityInformation.ContactEMail)
				SearchSetToken(pContact->pListEmail, CopyString(0, psXbrlGlParam->XbrlGlEntityInformation.ContactEMail));
 */
		}
	}
	return INTERPRET_SYNTAX_OK;
}

/********************************************************************

static BITMASK	XbrlGlTagDocumentInfoLanguage(

*********************************************************************/
static BITMASK	XbrlGlTagDocumentInfoLanguage(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)	// Switch of XML-data
{
	char *pLang = strstr(pTagMainData, PREFIX_ISO639_LANG);
	if (pLang) {
		pLang += strlen(PREFIX_ISO639_LANG);
		psXbrlGlParam->XbrlGlDocumentInfo.Language = pLang;
	}
	else
		return INTERPRET_SYNTAX_ERR_FATAL; // Fatal Error

	return INTERPRET_SYNTAX_OK;
}


/********************************************************************

static BITMASK	XbrlGlTagAccountingEntries(

*********************************************************************/
static BITMASK	XbrlGlTagAccountingEntries(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)	// Switch of XML-data
{
	psXbrlGlParam->XbrlGlDocumentInfo.EntriesType = 0;
	psXbrlGlParam->XbrlGlDocumentInfo.UniqueID = 0;
	psXbrlGlParam->XbrlGlDocumentInfo.PeriodCoveredStart = 0;
	psXbrlGlParam->XbrlGlDocumentInfo.PeriodCoveredEnd = 0;

	XbrlGlTagGeneralGlcorEntryHeaderReset(psXbrlGlParam);
	XbrlGlTagGeneralGlcorEntryDetailReset(psXbrlGlParam);

	psXbrlGlParam->XbrlGlItemType = XT_NONE;

	return INTERPRET_SYNTAX_OK;
}


/********************************************************************

static BITMASK	XbrlGlTagNotInterpret(

*********************************************************************/
static BITMASK	XbrlGlTagNotInterpret(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)	// Unknown command
{
	return INTERPRET_SYNTAX_OK;
}


// XBRL GL Data interpret

/********************************************************************

static BITMASK	XbrlGlTagGeneralGlcorEntryHeader(

*********************************************************************/
static BITMASK	XbrlGlTagGeneralGlcorEntryHeader(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)	// End of entry detail, Storing units
{
	switch (psXbrlGlParam->XbrlGlItemType) {
	case XT_ACCOUNT:
		break;
	case XT_IDENTIFIER:
		if (psXbrlGlParam->XbrlGlEntriesHeader.EntryNumber && *psXbrlGlParam->XbrlGlEntriesHeader.EntryNumber) {
			char *pDimensionId = psXbrlGlParam->XbrlGlEntriesHeader.EntryNumber;
			char *pDimensionSuperId = psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierDescription;
			char *pDimensionName = psXbrlGlParam->XbrlGlEntriesHeader.EntryComment;
			char *pDimensionNameLang = psXbrlGlParam->pAccounting->pDefaultLanguage;
			COUNTER	cDimensionId = StringToCounter(pDimensionId);
			COUNTER	cDimensionSuperId = StringToCounter(pDimensionSuperId);
			DimensionType *pDimension = AddDimension(psXbrlGlParam->pAccounting, cDimensionId);
			if (!pDimension) {
				XbrlGlTagGeneralGlcorEntryDetailReset(psXbrlGlParam);
				return XBRLGL_SYNTAX_ERR_FATAL; // Fatal Error
			}
			if (cDimensionSuperId)
				pDimension->pSuperDimension = AddDimension(psXbrlGlParam->pAccounting, cDimensionSuperId);
			if (pDimensionName && *pDimensionName)
				SearchSetText(pDimension->pListName, AssembleText(psXbrlGlParam->pAccounting, pDimension, pDimensionName, pDimensionNameLang));
		}
		else
			return XBRLGL_SYNTAX_ERR_FATAL; // Fatal Error
		break;
	case XT_BALANCE:
	case XT_MAPPING:
		break;
	case XT_VOUCHER:
		break;
	default:
		// Error
		break;
	}

	XbrlGlTagGeneralGlcorEntryHeaderReset(psXbrlGlParam);
	XbrlGlTagGeneralGlcorEntryDetailReset(psXbrlGlParam);
	return INTERPRET_SYNTAX_OK;
}

/********************************************************************

static BITMASK	XbrlGlTagGeneralGlcorEntryDetail(

*********************************************************************/
static BITMASK	XbrlGlTagGeneralGlcorEntryDetail(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)	// End of entry detail, Storing units
{
	if ((psXbrlGlParam->XbrlGlEntriesAccount.AccountPurposeDescription)
		&& ((!psXbrlGlParam->pAccounting->pGaap) || strcmp(psXbrlGlParam->pAccounting->pGaap, psXbrlGlParam->XbrlGlEntriesAccount.AccountPurposeDescription)))
		CopyString(&psXbrlGlParam->pAccounting->pGaap, psXbrlGlParam->XbrlGlEntriesAccount.AccountPurposeDescription);
	// Store data
	switch (psXbrlGlParam->XbrlGlItemType) {
	case XT_ACCOUNT:
		// There must be a numerc account number
		if (psXbrlGlParam->XbrlGlEntriesAccount.AccountMainID && *psXbrlGlParam->XbrlGlEntriesAccount.AccountMainID) {
			char *pAccountId = psXbrlGlParam->XbrlGlEntriesAccount.AccountMainID;
			char *pAccountName = psXbrlGlParam->XbrlGlEntriesAccount.AccountMainDescription;
			char *pAccountNameLang = psXbrlGlParam->pAccounting->pDefaultLanguage;
			char *pQuantityUnit = psXbrlGlParam->XbrlGlMeasurable.MeasurableUnitOfMeasure;
			char *pQuantityUnitLang = psXbrlGlParam->pAccounting->pDefaultLanguage;
			char *pAccountType = psXbrlGlParam->XbrlGlEntriesAccount.MainAccountType;
			AccountType *pAccount = AddAccount(psXbrlGlParam->pAccounting, pAccountId);
			if (!pAccount) {
				XbrlGlTagGeneralGlcorEntryDetailReset(psXbrlGlParam);
				return XBRLGL_SYNTAX_ERR_FATAL; // Fatal Error
			}
			if (pAccountName && *pAccountName)
				SearchSetText(pAccount->pListName, AssembleText(psXbrlGlParam->pAccounting, pAccount, pAccountName, pAccountNameLang));
			if (pQuantityUnit && *pQuantityUnit)
				SearchSetText(pAccount->pListQuantityUnit, AssembleText(psXbrlGlParam->pAccounting, pAccount, pQuantityUnit, pQuantityUnitLang));
			if (pAccountType)
				CopyString(&pAccount->pType, XbrlMainAccountType2AccountType(pAccountType));
		}
		else
			return XBRLGL_SYNTAX_ERR_FATAL; // Fatal Error
		if (psXbrlGlParam->XbrlGlEntityInformation.OrganizationIdentifier && *psXbrlGlParam->XbrlGlEntityInformation.OrganizationIdentifier) {
			char *pRegId = psXbrlGlParam->XbrlGlEntityInformation.OrganizationIdentifier;
			if (!psXbrlGlParam->pAccounting->pEntity)
				psXbrlGlParam->pAccounting->pEntity = CreateParty(psXbrlGlParam->pAccounting, psXbrlGlParam->pAccounting);
			if (!psXbrlGlParam->pAccounting->pEntity)
				return INTERPRET_SYNTAX_ERR_FATAL; // Fatal Error
			CopyString(&psXbrlGlParam->pAccounting->pEntity->pLegalRegId, pRegId);
		}
		break;
	case XT_IDENTIFIER:
		if (psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierCategory && psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierCode) {
			COUNTER cDimensionId = StringToCounter(psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierCategory);
			char *pObjectId = psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierCode;
			DimensionType *pDimension = 0;
			ObjectType *pObject = 0;
			if (cDimensionId)
				pDimension = AddDimension(psXbrlGlParam->pAccounting, cDimensionId);
			if (pDimension && pObjectId && *pObjectId)
				pObject = AddDimObject(psXbrlGlParam->pAccounting, cDimensionId, pObjectId);
			if (pObject) {
				if (psXbrlGlParam->XbrlGlEntriesDetails.DocumentReference && *psXbrlGlParam->XbrlGlEntriesDetails.DocumentReference)
					SearchSetToken(pObject->pListFileUrl, CopyString(0, psXbrlGlParam->XbrlGlEntriesDetails.DocumentReference));
			}
			else {
				XbrlGlTagGeneralGlcorEntryDetailReset(psXbrlGlParam);
				return XBRLGL_SYNTAX_ERR_FATAL; // Fatal Error
			}
		}
		else
			return XBRLGL_SYNTAX_ERR_FATAL; // Fatal Error
		break;
	case XT_BALANCE:
	case XT_MAPPING:
		// There must be a account number noted
		if (!psXbrlGlParam->XbrlGlEntriesAccount.AccountMainID) {
			XbrlGlTagGeneralGlcorEntryDetailReset(psXbrlGlParam);
			return XBRLGL_SYNTAX_ERR_FATAL; // Fatal Error
		}
		// There must be a numerc amount
		if (!psXbrlGlParam->XbrlGlEntriesDetails.Amount) {	// LedgerVoucher->pSeries
			XbrlGlTagGeneralGlcorEntryDetailReset(psXbrlGlParam);
			return XBRLGL_SYNTAX_ERR_FATAL; // Fatal Error
		}
		// If no object number it is 0-0 (all)
		if (!psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierCategory)
			psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierCategory = NulString;
		if (!psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierCode)
			psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierCode = NulString;
		if (!psXbrlGlParam->XbrlGlDocumentInfo.UniqueID)
			psXbrlGlParam->XbrlGlDocumentInfo.UniqueID = NulString;
		{
			AccountingType *pAccounting = psXbrlGlParam->pAccounting;
			NUMERIC nLedgerId = StringToNumeric(psXbrlGlParam->XbrlGlDocumentInfo.UniqueID);
			char *pEntryType = psXbrlGlParam->XbrlGlEntriesHeader.EntryType;
			char *pQualifierEntry = psXbrlGlParam->XbrlGlEntriesHeader.QualifierEntry;
			char *pAccountId = psXbrlGlParam->XbrlGlEntriesAccount.AccountMainID;
			char *pAccountName = psXbrlGlParam->XbrlGlEntriesAccount.AccountMainDescription;
			char *pAccountNameLang = psXbrlGlParam->pAccounting->pDefaultLanguage;
			QUANT Quantity = StringToDecimal(psXbrlGlParam->XbrlGlMeasurable.MeasurableQuantity);
			char *pQuantityUnit = psXbrlGlParam->XbrlGlMeasurable.MeasurableUnitOfMeasure;
			char *pQuantityUnitLang = psXbrlGlParam->pAccounting->pDefaultLanguage;
			char *pAccountType = psXbrlGlParam->XbrlGlEntriesAccount.MainAccountType;
			AMOUNT Amount = StringToAmount(psXbrlGlParam->XbrlGlEntriesDetails.Amount);
			char *pAmountUnit = psXbrlGlParam->XbrlGlEntriesDetails.AmountUnit;
			char *pXbrlGlDebitCreditCode = psXbrlGlParam->XbrlGlEntriesDetails.DebitCreditCode;
			char *pSignOfAmount = psXbrlGlParam->XbrlGlEntriesDetails.SignOfAmount;
			//			char *pAmountDecimals			= psXbrlGlParam->XbrlGlEntriesDetails.AmountDecimals;
			char *pXbrlInclude = psXbrlGlParam->XbrlGlEntriesDetails.XbrlInclude;
			//			char *pPostingStatus			= psXbrlGlParam->XbrlGlEntriesDetails.PostingStatus;
			//			char *pDetailComment			= psXbrlGlParam->XbrlGlEntriesDetails.DetailComment;
			DATEINT FiscalYearStart = StringToDate(psXbrlGlParam->XbrlGlEntityInformation.FiscalYearStart);
			DATEINT FiscalYearEnd = StringToDate(psXbrlGlParam->XbrlGlEntityInformation.FiscalYearEnd);
			DATEINT PostingDate = StringToDate(psXbrlGlParam->XbrlGlEntriesDetails.PostingDate);
			DATEINT PeriodCoveredStart = StringToDate(psXbrlGlParam->XbrlGlDocumentInfo.PeriodCoveredStart);
			DATEINT PeriodCoveredEnd = StringToDate(psXbrlGlParam->XbrlGlDocumentInfo.PeriodCoveredEnd);
			//			int	iLedgerYearId				= 1;
			LedgerType *pLedger = AddLedger(psXbrlGlParam->pAccounting, nLedgerId, FiscalYearStart, FiscalYearEnd);
			AccountType *pAccount = AddAccount(psXbrlGlParam->pAccounting, pAccountId);
			// We get pDimension and pObject from the XBRL identifer section, can only be one
			COUNTER cDimensionId = StringToCounter(psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierCategory);
			COUNTER cDimensionSuperId = StringToCounter(psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierAuthorityCode);
			char *pDimensionName = psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierPurpose;
			char *pDimensionNameLang = psXbrlGlParam->pAccounting->pDefaultLanguage;
			char *pDimensionSuperName = psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierAuthority;
			char *pDimensionSuperNameLang = psXbrlGlParam->pAccounting->pDefaultLanguage;
			char *pObjectId = psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierCode;
			char *pObjectName = psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierDescription;
			char *pObjectNameLang = psXbrlGlParam->pAccounting->pDefaultLanguage;
			DimensionType *pDimension = 0;
			DimensionType *pDimensionSuper = 0;
			ObjectType *pObject = 0;
			if (cDimensionId)
				pDimension = AddDimension(psXbrlGlParam->pAccounting, cDimensionId);
			if (cDimensionSuperId)
				pDimensionSuper = AddDimension(psXbrlGlParam->pAccounting, cDimensionSuperId);
			if (pDimension && pObjectId && *pObjectId)
				pObject = AddDimObject(psXbrlGlParam->pAccounting, cDimensionId, pObjectId);
			if (!pLedger)
				return INTERPRET_SYNTAX_ERR_FATAL;	// Fatal Error
			if (pAccount) {
				SearchSetText(pAccount->pListName, AssembleText(psXbrlGlParam->pAccounting, pAccount, pAccountName, pAccountNameLang));
				SearchSetText(pAccount->pListQuantityUnit, AssembleText(psXbrlGlParam->pAccounting, pAccount, pQuantityUnit, pQuantityUnitLang));
				if (pAccountType)
					CopyString(&pAccount->pType, XbrlMainAccountType2AccountType(pAccountType));
			}
			else
				return INTERPRET_SYNTAX_ERR_FATAL;	// Fatal Error
			if (pDimension)
				SearchSetText(pDimension->pListName, AssembleText(psXbrlGlParam->pAccounting, pDimension, pDimensionName, pDimensionNameLang));
			if (pDimensionSuper)
				SearchSetText(pDimensionSuper->pListName, AssembleText(psXbrlGlParam->pAccounting, pDimensionSuper, pDimensionSuperName, pDimensionSuperNameLang));
			if (pObject)
				SearchSetText(pObject->pListName, AssembleText(psXbrlGlParam->pAccounting, pObject, pObjectName, pObjectNameLang));

			if (!PostingDate)
				PostingDate = PeriodCoveredEnd;
			if (!PostingDate)
				PostingDate = PeriodCoveredStart;
			if (!PostingDate)
				PostingDate = FiscalYearEnd;
			if (!PostingDate)
				PostingDate = FiscalYearEnd;

			// Check the amount stuff
			// Negative DebitCreditCode flips sign on Amount
			if (pXbrlGlDebitCreditCode && pXbrlGlDebitCreditCode[0] &&
				(!((pXbrlGlDebitCreditCode[0] == 'D') || (pXbrlGlDebitCreditCode[0] == 'd'))))
				Amount = 0 - Amount;

			// Negative DebitCreditCode flips sign on Amount
			if (pSignOfAmount && pSignOfAmount[0] &&
				((pSignOfAmount[0] == '-') || (!strcmp(pSignOfAmount, "minus"))))
				Amount = 0 - Amount;

			// Check the currency code ZZZ
			if (psXbrlGlParam->XbrlGlDocumentInfo.DefaultCurrency && strcmp(psXbrlGlParam->XbrlGlDocumentInfo.DefaultCurrency, GetXbrlCurrency(psXbrlGlParam, pAmountUnit)))
				psXbrlGlParam->Message[0] = "Not the default currency, measurableUnitOfMeasure should be used for other currencies";

			// Make the report input
			if (pAccount && pLedger) {
				int pReportingType = BALANCE_BEGINNING;
				BalanceReportType *pBalanceReport = GetBalanceReport(pLedger, pObject);
				BalanceAccountType *pBalanceAccount = 0;
				if (!pBalanceReport) {
					pBalanceReport = CreateBalanceReport(pAccounting, pLedger);
					if (!pBalanceReport)
						return INTERPRET_SYNTAX_ERR_FATAL;	// Fatal Error
					pBalanceReport->pObject = pObject;
					LListAddRecord(pLedger->pListReport, pBalanceReport);
				}

				pBalanceAccount = GetBalanceAccount(pBalanceReport, pAccount);
				if (!pBalanceAccount) {
					pBalanceAccount = CreateBalanceAccount(pAccounting, pBalanceReport);
					if (!pBalanceAccount)
						return INTERPRET_SYNTAX_ERR_FATAL;	// Fatal Error
					pBalanceAccount->pAccount = pAccount;
					LListAddRecord(pBalanceReport->pListBalanceAccount, pBalanceAccount);
				}

				if ((!strcmp(pEntryType, "standard"))
					&& (!strcmp(pQualifierEntry, "balance-brought-forward"))
					&& (!strcmp(pXbrlInclude, "beginning_balance")))	// Opening balance
					pReportingType = BALANCE_BEGINNING;
				else if ((!strcmp(pEntryType, "standard"))
					&& (!strcmp(pQualifierEntry, "balance-brought-forward"))
					&& (!strcmp(pXbrlInclude, "ending_balance")))		// Forward balance
					pReportingType = BALANCE_ENDING;
				else if ((!strcmp(pEntryType, "budget"))
					&& (!strcmp(pQualifierEntry, "standard"))
					&& (!strcmp(pXbrlInclude, "period_change")))		// Budget monthly
					pReportingType = BUDGET;
				else if ((!strcmp(pEntryType, "standard"))
					&& (!strcmp(pQualifierEntry, "standard"))
					&& (!strcmp(pXbrlInclude, "period_change")))		// Outcome monthly
					pReportingType = PERIOD_CHANGE;

				if ((pReportingType == BALANCE_BEGINNING) || (pReportingType == BALANCE_ENDING)) {
					if ((pReportingType == BALANCE_BEGINNING) && Amount)
						pBalanceAccount->Opening = Amount;
					if ((pReportingType == BALANCE_BEGINNING) && (!DecimalCmp(Quantity, 0)))
						pBalanceAccount->OpeningQuantity = Quantity;
					if ((pReportingType == BALANCE_ENDING) && Amount)
						pBalanceAccount->Forward = Amount;
					if ((pReportingType == BALANCE_ENDING) && (!DecimalCmp(Quantity, 0)))
						pBalanceAccount->ForwardQuantity = Quantity;
				}
				else { 	// PERIOD_CHANGE BUDGET
					BalanceAccountPeriodType *pBalanceAccountPeriod = GetBalanceAccountPeriod(pBalanceAccount, PostingDate, 0);
					if (!pBalanceAccountPeriod) {
						pBalanceAccountPeriod = CreateBalanceAccountPeriod(pAccounting, pBalanceReport);
						if ((!pBalanceAccountPeriod)
							|| (PostingDate && (!((pLedger->YearStart <= PostingDate) && (pLedger->YearEnd >= PostingDate)))))
							return INTERPRET_SYNTAX_ERR_FATAL; // Fatal field error Month data out of Ledger rage
						pBalanceAccountPeriod->End = PostingDate;
						LListAddRecord(pBalanceAccount->pListMonth, pBalanceAccountPeriod);
					}
					if ((pReportingType == BUDGET) && Amount)
						pBalanceAccountPeriod->Budget = Amount;
					if ((pReportingType == BUDGET) && (!DecimalCmp(Quantity, 0)))
						pBalanceAccountPeriod->BudgetQuantity = Quantity;
					if ((pReportingType == PERIOD_CHANGE) && Amount)
						pBalanceAccountPeriod->Outcome = Amount;
					if ((pReportingType == PERIOD_CHANGE) && (!DecimalCmp(Quantity, 0)))
						pBalanceAccountPeriod->OutcomeQuantity = Quantity;
				}
			}
		}
		break;
	case XT_VOUCHER:
		// There must be a account number noted
		if (!psXbrlGlParam->XbrlGlEntriesAccount.AccountMainID) {
			XbrlGlTagGeneralGlcorEntryDetailReset(psXbrlGlParam);
			return XBRLGL_SYNTAX_ERR_FATAL; // Fatal Error
		}
		// There must be a numerc amount
		if (!psXbrlGlParam->XbrlGlEntriesDetails.Amount) { 	// LedgerVoucher->pSeries
			XbrlGlTagGeneralGlcorEntryDetailReset(psXbrlGlParam);
			return XBRLGL_SYNTAX_ERR_FATAL; // Fatal Error
		}
		// If no object number it is 0-0 (all)
		if (!psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierCategory)
			psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierCategory = NulString;
		if (!psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierCode)
			psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierCode = NulString;
		if (!psXbrlGlParam->XbrlGlDocumentInfo.UniqueID)
			psXbrlGlParam->XbrlGlDocumentInfo.UniqueID = NulString;
		{
			//AccountingType *pAccounting = psXbrlGlParam->pAccounting;
			char *pVoucherSeries = psXbrlGlParam->XbrlGlDocumentInfo.UniqueID;
			COUNTER cVouchertId = StringToCounter(psXbrlGlParam->XbrlGlEntriesHeader.EntryNumber);
			DATEINT VoucherDate = StringToDate(psXbrlGlParam->XbrlGlEntriesHeader.PostedDate);
			char *pVoucherTitle = psXbrlGlParam->XbrlGlEntriesHeader.EntryComment;
			char *pVoucherTitleLang = psXbrlGlParam->pAccounting->pDefaultLanguage;
			char *pVoucherEntryTitle = psXbrlGlParam->XbrlGlEntriesDetails.DetailComment;
			char *pVoucherEntryTitleLang = psXbrlGlParam->pAccounting->pDefaultLanguage;
			char *pAccountant = psXbrlGlParam->XbrlGlEntriesHeader.EnteredBy;
			DATEINT EditDate = StringToDate(psXbrlGlParam->XbrlGlEntriesHeader.EnteredDate);
			//char *pEntryType = psXbrlGlParam->XbrlGlEntriesHeader.EntryType;
			//char *pQualifierEntry = psXbrlGlParam->XbrlGlEntriesHeader.QualifierEntry;
			char *pAccountId = psXbrlGlParam->XbrlGlEntriesAccount.AccountMainID;
			char *pAccountName = psXbrlGlParam->XbrlGlEntriesAccount.AccountMainDescription;
			char *pAccountNameLang = psXbrlGlParam->pAccounting->pDefaultLanguage;
			QUANT Quantity = StringToDecimal(psXbrlGlParam->XbrlGlMeasurable.MeasurableQuantity);
			char *pQuantityUnit = psXbrlGlParam->XbrlGlMeasurable.MeasurableUnitOfMeasure;
			char *pQuantityUnitLang = psXbrlGlParam->pAccounting->pDefaultLanguage;
			char *pAccountType = psXbrlGlParam->XbrlGlEntriesAccount.MainAccountType;
			AMOUNT Amount = StringToAmount(psXbrlGlParam->XbrlGlEntriesDetails.Amount);
			char *pAmountUnit = psXbrlGlParam->XbrlGlEntriesDetails.AmountUnit;
			char *pXbrlGlDebitCreditCode = psXbrlGlParam->XbrlGlEntriesDetails.DebitCreditCode;
			char *pSignOfAmount = psXbrlGlParam->XbrlGlEntriesDetails.SignOfAmount;
			//char *pXbrlInclude = psXbrlGlParam->XbrlGlEntriesDetails.XbrlInclude;
			char *pPostingStatus = psXbrlGlParam->XbrlGlEntriesDetails.PostingStatus;
			char *pDocumentReference = psXbrlGlParam->XbrlGlEntriesDetails.DocumentReference;

			DATEINT FiscalYearStart = StringToDate(psXbrlGlParam->XbrlGlEntityInformation.FiscalYearStart);
			DATEINT FiscalYearEnd = StringToDate(psXbrlGlParam->XbrlGlEntityInformation.FiscalYearEnd);
			DATEINT PostingDate = StringToDate(psXbrlGlParam->XbrlGlEntriesDetails.PostingDate);
			//DATEINT PeriodCoveredStart = StringToDate(psXbrlGlParam->XbrlGlDocumentInfo.PeriodCoveredStart);
			//DATEINT PeriodCoveredEnd = StringToDate(psXbrlGlParam->XbrlGlDocumentInfo.PeriodCoveredEnd);
			LedgerType *pLedger = GetLedgerDynamic(psXbrlGlParam->pAccounting, FiscalYearStart);
			AccountType *pAccount = AddAccount(psXbrlGlParam->pAccounting, pAccountId);
			VoucherType *pVoucher = psXbrlGlParam->XbrlGlEntriesHeader.pVoucher;
			VoucherEntryType *pVoucherEntry = CreateVoucherEntry(psXbrlGlParam->pAccounting, pVoucher);
			if (pLedger) {
				if (FiscalYearStart)
					pLedger->YearStart = FiscalYearStart;
				if (FiscalYearEnd)
					pLedger->YearEnd = FiscalYearEnd;
			}
			else
				return INTERPRET_SYNTAX_ERR_FATAL;	// Fatal Error
													// Check the amount stuff
													// Negative DebitCreditCode flips sign on Amount
			if (pXbrlGlDebitCreditCode && pXbrlGlDebitCreditCode[0] &&
				(!((pXbrlGlDebitCreditCode[0] == 'D') || (pXbrlGlDebitCreditCode[0] == 'd'))))
				Amount = 0 - Amount;

			// Negative DebitCreditCode flips sign on Amount
			if (pSignOfAmount && pSignOfAmount[0] &&
				((pSignOfAmount[0] == '-') || (!strcmp(pSignOfAmount, "minus"))))
				Amount = 0 - Amount;

			// Check the currency code ZZZ
			if (psXbrlGlParam->XbrlGlDocumentInfo.DefaultCurrency && strcmp(psXbrlGlParam->XbrlGlDocumentInfo.DefaultCurrency, GetXbrlCurrency(psXbrlGlParam, pAmountUnit)))
				psXbrlGlParam->Message[0] = "Not the default currency, measurableUnitOfMeasure should be used for other currencies";

			if (pAccount) {
				SearchSetText(pAccount->pListName, AssembleText(psXbrlGlParam->pAccounting, pAccount, pAccountName, pAccountNameLang));
				SearchSetText(pAccount->pListQuantityUnit, AssembleText(psXbrlGlParam->pAccounting, pAccount, pQuantityUnit, pQuantityUnitLang));
				if (pAccountType)
					CopyString(&pAccount->pType, XbrlMainAccountType2AccountType(pAccountType));
			}
			else
				return INTERPRET_SYNTAX_ERR_FATAL;	// Fatal Error
			if (!pVoucher) { // New (header been reset since last)
				pVoucher
					= pVoucherEntry->pStructParent
					= psXbrlGlParam->XbrlGlEntriesHeader.pVoucher
					= CreateVoucher(psXbrlGlParam->pAccounting, pLedger);
				LListAddRecord(pLedger->pListVoucher, pVoucher);
			}
			if (pVoucher) {
				if ((!pVoucher->pId) && ((pVoucherSeries && *pVoucherSeries) || cVouchertId))
					pVoucher->pId = AseembleSeriesId(psXbrlGlParam->pAccounting, pVoucher, pVoucherSeries, cVouchertId);
				pVoucher->Date = VoucherDate;
				SearchSetText(pVoucher->pListTitle, AssembleText(psXbrlGlParam->pAccounting, pVoucher, pVoucherTitle, pVoucherTitleLang));
				if ((pAccountant && *pAccountant) || EditDate) {
					if (!pVoucher->pAccountant)
						pVoucher->pAccountant = AseembleOperatorInfo(psXbrlGlParam->pAccounting, pVoucher, pAccountant, EditDate);
					else if ((pAccountant && *pAccountant) && strcmp(pVoucher->pAccountant->pOperatorInfo, pAccountant)) {
						CopyString(&pVoucher->pAccountant->pOperatorInfo, pAccountant);
						if (EditDate)
							pVoucher->pAccountant->date = EditDate;
					}
					else if (EditDate)
						pVoucher->pAccountant->date = EditDate;

				}
				// pVoucherEntry->pInvoice->pListOriginalInvoiceFileUrl;
			}
			else
				return INTERPRET_SYNTAX_ERR_FATAL;	// Fatal Error
			if (pVoucherEntry) {
				if (pPostingStatus && (!strcmp(pPostingStatus, "cancelled")))
					pVoucherEntry->pDeprecated = AseembleOperatorInfo(psXbrlGlParam->pAccounting, pVoucherEntry, "XbrlGl", GetSystemDate());
				pVoucherEntry->Amount = Amount;
				pVoucherEntry->Quantity = Quantity;
				SearchSetText(pVoucherEntry->pListTitle, AssembleText(psXbrlGlParam->pAccounting, pVoucherEntry, pVoucherEntryTitle, pVoucherEntryTitleLang));
				pVoucherEntry->AuxDate = PostingDate;
				pVoucherEntry->pAccount = pAccount;
				//pVoucherEntry->pInvoice;
				if ((pAccountant && *pAccountant) || EditDate)
					pVoucherEntry->pAccountant = AseembleOperatorInfo(psXbrlGlParam->pAccounting, pVoucherEntry, pAccountant, EditDate);
				if (pDocumentReference && *pDocumentReference) {
					if (!pVoucherEntry->pInvoice)
						pVoucherEntry->pInvoice = CreateInvoice(psXbrlGlParam->pAccounting, pVoucherEntry);
					SearchSetToken(pVoucherEntry->pInvoice->pListOriginalInvoiceFileUrl, CopyString(0, pDocumentReference));
				}
				// We get pDimension and pObject from the psXbrlGlParam->XbrlGlEntriesDetails.pListIdentifiers list, can be multiple
				LListCopyLList(&pVoucherEntry->pListObject, psXbrlGlParam->XbrlGlEntriesDetails.pListIdentifiers);
				LListAddRecord(pVoucher->pListEntry, pVoucherEntry);
			}
			else
				return INTERPRET_SYNTAX_ERR_FATAL;	// Fatal Error
		}
		break;
	default:
		// Error
		break;

	}	// End - switch(psXbrlGlParam->XbrlGlItemType)
	XbrlGlTagGeneralGlcorEntryDetailReset(psXbrlGlParam);
	return INTERPRET_SYNTAX_OK;
}


/********************************************************************

static BITMASK	XbrlGlTagGeneralGlcorIdentifierReference(

*********************************************************************/
static BITMASK	XbrlGlTagGeneralGlcorIdentifierReference(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)	// End of entry detail, Storing units
{
	switch (psXbrlGlParam->XbrlGlItemType) {
	case XT_ACCOUNT:
		break;

	case XT_IDENTIFIER:		// Direct declaration in XBRL GL is not necessary
	case XT_MAPPING:	// Indirect declaration in XBRL GL is allowed
	case XT_BALANCE:		// Indirect declaration in XBRL GL is allowed
	case XT_VOUCHER:	// Indirect declaration in XBRL GL is allowed
		// In XT_IDENTIFIER multiple identifiers can be listed, else only single and taken care of in entry details
	{
		COUNTER cDimensionId = StringToCounter(psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierCategory);
		COUNTER cDimensionSuperId = StringToCounter(psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierAuthorityCode);
		char *pDimensionName = psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierPurpose;
		char *pDimensionNameLang = psXbrlGlParam->pAccounting->pDefaultLanguage;
		char *pDimensionSuperName = psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierAuthority;
		char *pDimensionSuperNameLang = psXbrlGlParam->pAccounting->pDefaultLanguage;
		char *pObjectId = psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierCode;
		char *pObjectName = psXbrlGlParam->XbrlGlEntriesIdentifier.IdentifierDescription;
		char *pObjectNameLang = psXbrlGlParam->pAccounting->pDefaultLanguage;
		DimensionType *pDimension = 0;
		DimensionType *pDimensionSuper = 0;
		ObjectType *pObject = 0;
		if (cDimensionId)
			pDimension = AddDimension(psXbrlGlParam->pAccounting, cDimensionId);
		if (cDimensionSuperId)
			pDimensionSuper = AddDimension(psXbrlGlParam->pAccounting, cDimensionSuperId);
		if (pDimension && pObjectId && *pObjectId)
			pObject = AddDimObject(psXbrlGlParam->pAccounting, cDimensionId, pObjectId);
		if (pDimension)
			SearchSetText(pDimension->pListName, AssembleText(psXbrlGlParam->pAccounting, pDimension, pDimensionName, pDimensionNameLang));
		if (pDimensionSuper)
			SearchSetText(pDimensionSuper->pListName, AssembleText(psXbrlGlParam->pAccounting, pDimensionSuper, pDimensionSuperName, pDimensionSuperNameLang));
		if (pObject)
			SearchSetText(pObject->pListName, AssembleText(psXbrlGlParam->pAccounting, pObject, pObjectName, pObjectNameLang));
		// Check if voucher record object is registered in the voucher record llist

		if ((psXbrlGlParam->XbrlGlItemType == XT_VOUCHER) && (RET_OK != LListAddRecord(psXbrlGlParam->XbrlGlEntriesDetails.pListIdentifiers, pObject)))
			return INTERPRET_SYNTAX_ERR_FATAL; // Fatal Error
	}
	break;
	default:
		// Error
		break;

	}	// End - switch(psXbrlGlParam->XbrlGlItemType)
	return INTERPRET_SYNTAX_OK;
}

/********************************************************************

static BITMASK	XbrlGlTagAmount(

*********************************************************************/
static BITMASK	XbrlGlTagAmount(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)	// Unknown command
{
	// Amount
	psXbrlGlParam->XbrlGlEntriesDetails.Amount = pTagMainData;

	// Amount (currency) unit
	{
		char *pPtrStart = strstr(pTagData, ATTRIBUTE_UNITREF);
		char *pPtrEnd = pTagData;
		if (pPtrStart) {
			while (*pPtrStart && ((*pPtrStart <= ' ') || (*pPtrStart == '"') || (*pPtrStart == '=')))
				pPtrStart++;
			pPtrStart = strchr(pPtrStart, '\"');
			if (pPtrStart) {
				pPtrStart++;
				pPtrEnd = strchr(pPtrStart, '\"');
				if (pPtrEnd) {
					*pPtrEnd = 0;
					if(pPtrStart && /*not*/ strcmp(pPtrStart, "NotUsed"))
						psXbrlGlParam->XbrlGlEntriesDetails.AmountUnit = pPtrStart;
					pPtrEnd++;
				}
				else
					pPtrEnd = pTagData;
			}
		}

		// Amount decimals
		pPtrStart = strstr(pPtrEnd, "decimals");
		if (!pPtrStart)
			pPtrStart = strstr(pTagData, "decimals");

		if (pPtrStart) {
			pPtrStart = strchr(pPtrStart, '\"');
			if (pPtrStart) {
				pPtrStart++;
				char *pPtrEndDecimals = strchr(pPtrStart + 1, '\"');
				if (pPtrEndDecimals) {
					*pPtrEndDecimals = 0;
					psXbrlGlParam->XbrlGlEntriesDetails.AmountDecimals = pPtrStart;
				}
			}
		}
	}
	return INTERPRET_SYNTAX_OK;
}

/********************************************************************

static BITMASK	XbrlGlTagGeneralGlcorAccountTaxCodeSpecifications(

*********************************************************************/
static BITMASK	XbrlGlTagGeneralGlcorAccountTaxCodeSpecifications(
	XBRLGLPARAMTYPE *psXbrlGlParam, 
	char *pTagMainData, 
	char *pTagData)	// End of entry detail, Storing units
{
	switch (psXbrlGlParam->XbrlGlItemType) {
	case XT_ACCOUNT:
	case XT_MAPPING:
	case XT_BALANCE:
	case XT_VOUCHER:
		// In XT_IDENTIFIER multiple identifiers can be listed, else only single and taken care of in entry details
	{
		// We get pDimension and pObject from the XBRL identifer section
		//AccountingType *pAccounting = psXbrlGlParam->pAccounting;
		//char *pEntryType = psXbrlGlParam->XbrlGlEntriesHeader.EntryType;
		//char *pQualifierEntry = psXbrlGlParam->XbrlGlEntriesHeader.QualifierEntry;
		char *pAccountId = psXbrlGlParam->XbrlGlEntriesAccount.AccountMainID;
		char *pAccountName = psXbrlGlParam->XbrlGlEntriesAccount.AccountMainDescription;
		char *pAccountNameLang = psXbrlGlParam->pAccounting->pDefaultLanguage;
		char *pQuantityUnit = psXbrlGlParam->XbrlGlMeasurable.MeasurableUnitOfMeasure;
		char *pQuantityUnitLang = psXbrlGlParam->pAccounting->pDefaultLanguage;
		char *pAccountType = psXbrlGlParam->XbrlGlEntriesAccount.MainAccountType;

		char *pAccountCodeIssuer = psXbrlGlParam->XbrlGlTax.TaxAuthority;
		char *pAccountCodepValidYear = psXbrlGlParam->XbrlGlTax.TaxTableCode;
		char *pAccountCodeType = psXbrlGlParam->XbrlGlTax.TaxDescription;
		char *pAccountCode = psXbrlGlParam->XbrlGlTax.TaxCode;
		char *pApplicableDomain = psXbrlGlParam->XbrlGlTax.TaxCommentExemption;
		AccountType *pAccount = AddAccount(psXbrlGlParam->pAccounting, pAccountId);

		if (pAccount) {
			SearchSetText(pAccount->pListName, AssembleText(psXbrlGlParam->pAccounting, pAccount, pAccountName, pAccountNameLang));
			SearchSetText(pAccount->pListQuantityUnit, AssembleText(psXbrlGlParam->pAccounting, pAccount, pQuantityUnit, pQuantityUnitLang));
			if (pAccountType)
				CopyString(&pAccount->pType, XbrlMainAccountType2AccountType(pAccountType));
			SearchSetAccountCode(pAccount->pListAccountCode,
				AseembleAccountCode(
					psXbrlGlParam->pAccounting,
					pAccount,
					pAccountCodeType,
					pAccountCodeIssuer,
					pApplicableDomain,
					pAccountCodepValidYear,
					pAccountCode,
					0,
					0));
		}
		else
			return INTERPRET_SYNTAX_ERR_FATAL;	// Fatal Error
	}
	break;
	case XT_IDENTIFIER:
	default:
		// Error
		break;

	}	// End - switch(psXbrlGlParam->XbrlGlItemType)
	return INTERPRET_SYNTAX_OK;
}


/********************************************************************

static void XbrlGlTagGeneralGlcorEntryDetailReset(

*********************************************************************/
static void XbrlGlTagGeneralGlcorEntryDetailReset(
	XBRLGLPARAMTYPE *psXbrlGlParam)
{
	LListErase(psXbrlGlParam->XbrlGlEntriesDetails.pListIdentifiers);
	memset(&psXbrlGlParam->XbrlGlEntriesDetails, 0, sizeof(XbrlGlEntriesDetailsType));
	psXbrlGlParam->XbrlGlEntriesDetails.pListIdentifiers = LListInit();
	memset(&psXbrlGlParam->XbrlGlEntriesIdentifier, 0, sizeof(XbrlGlEntriesIdentifierType));
	memset(&psXbrlGlParam->XbrlGlMeasurable, 0, sizeof(XbrlGlMeasurableType));
	memset(&psXbrlGlParam->XbrlGlEntriesAccount, 0, sizeof(XbrlGlEntriesAccountType));
	memset(&psXbrlGlParam->XbrlGlTax, 0, sizeof(XbrlGlTaxesType));
}

/********************************************************************

static void XbrlGlTagGeneralGlcorEntryHeaderReset(

*********************************************************************/
static void XbrlGlTagGeneralGlcorEntryHeaderReset(
	XBRLGLPARAMTYPE *psXbrlGlParam)
{
	memset(&psXbrlGlParam->XbrlGlEntriesHeader, 0, sizeof(XbrlGlEntriesHeaderType));
}

/********************************************************************

static char *XbrlGlParamNameUnification(

*********************************************************************/
static char *XbrlGlParamNameUnification(
	char *pFirstName, 
	char *pLastName)
{
	static char Name[1024];
	Name[0] = 0;
	if (pFirstName && *pFirstName && pLastName && *pLastName)
		snprintf(Name, 1024, "%s %s", pFirstName, pLastName);
	else if (pFirstName)
		strcpy(Name, pFirstName);
	else if (pLastName)
		strcpy(Name, pLastName);
	return Name;
}


/********************************************************************

static char *XbrlMainAccountType2AccountType(

*********************************************************************/
static char *XbrlMainAccountType2AccountType(
	char *XbrlMainAccountType)
{
	if (XbrlMainAccountType && (!strcmp("equity", XbrlMainAccountType)))
		return ACCOUNT_TYPE_EQUITIES;
	else if (XbrlMainAccountType && (!strcmp("contr-to-equity", XbrlMainAccountType)))
		return ACCOUNT_TYPE_EQUITIES;
	else if (XbrlMainAccountType && (!strcmp("distr-from-equity", XbrlMainAccountType)))
		return ACCOUNT_TYPE_EQUITIES;
	else if (XbrlMainAccountType && (!strcmp("liability", XbrlMainAccountType)))
		return ACCOUNT_TYPE_LIABILITIES;
	else if (XbrlMainAccountType && (!strcmp("income", XbrlMainAccountType)))
		return ACCOUNT_TYPE_REVENUES;
	else if (XbrlMainAccountType && (!strcmp("expense", XbrlMainAccountType)))
		return ACCOUNT_TYPE_EXPENSES;
	else if (XbrlMainAccountType && (!strcmp("gain", XbrlMainAccountType)))
		return ACCOUNT_TYPE_LOSS_GAINS;
	else if (XbrlMainAccountType && (!strcmp("loss", XbrlMainAccountType)))
		return ACCOUNT_TYPE_LOSS_GAINS;
	else if (XbrlMainAccountType && (!strcmp("comprehensive-income", XbrlMainAccountType)))
		return ACCOUNT_TYPE_FINANCIAL;
	else if (XbrlMainAccountType && (!strcmp("other", XbrlMainAccountType)))
		return ACCOUNT_TYPE_AUX;
	else if (XbrlMainAccountType && (!strcmp("asset", XbrlMainAccountType)))
		return ACCOUNT_TYPE_ASSETS;
	else
		return 0;
}

/********************************************************************

Create XBRL/GL file data

*********************************************************************/
/*************************************************************

int FileWriteXbrlGl(

*************************************************************/
int FileWriteXbrlGl(
	char *pFilePathName,
	void *vpAccounting,
	char *pContent,
	char *pGenUserName,
	char *pStylesheetFilePathName,
	BOOL ChartsOption,
	BOOL bObjectOption,
	BOOL BalanceOption,
	BOOL bPeriodOption,
	BOOL VouchersOption,
	BOOL bQuantityOption,
	BOOL bLeanChartsOption,
	BOOL bCompactFileOption,
	BOOL ExtendedOption,
	char *pInitialDate,
	char *pFinalDate,
	char *FilterEnteredBy,
	char *pProgramName,
	char *pProgramVersion,
	LocaleDataType *pLocaleData)
{
	// Initiate a new file
	if (!pFilePathName)
		return (-1);

	// Open the file 
	{
		FILE *pStream = FileOpen(pFilePathName, "wb");
		if (!pStream)
			return (-2); // Bad

		// Write the XBRL GL file
		{
			int iReturn = StreamWriteXbrlGl(
				pStream,
				vpAccounting,
				pContent,
				pGenUserName,
				pStylesheetFilePathName,
				ChartsOption,
				bObjectOption,
				BalanceOption,
				bPeriodOption,
				VouchersOption,
				bQuantityOption,
				bLeanChartsOption,
				bCompactFileOption,
				ExtendedOption,
				pInitialDate,
				pFinalDate,
				FilterEnteredBy,
				pProgramName,
				pProgramVersion,
				pLocaleData);

			// Close the file 
			if (fclose(pStream) && (!iReturn))
				iReturn = (-2);

			// If bad delete the file 
			if (iReturn) // Bad
				remove(pFilePathName);
			return iReturn;
		}
	}
}


/*************************************************************

int StreamWriteXbrlGl(

*************************************************************/
int StreamWriteXbrlGl(
	FILE *pStream,
	void *vpAccounting,
	char *pContent,
	char *pGenUserName,
	char *pStylesheetFilePathName,
	BOOL ChartsOption,
	BOOL bObjectOption,
	BOOL BalanceOption,
	BOOL bPeriodOption,
	BOOL VouchersOption,
	BOOL bQuantityOption,
	BOOL bLeanChartsOption,
	BOOL bCompactFileOption,
	BOOL ExtendedOption,
	char *pInitialDate,
	char *pFinalDate,
	char *FilterEnteredBy,
	char *pProgramName,
	char *pProgramVersion,
	LocaleDataType *pLocaleData)
{
	if (!vpAccounting)
		return ERR_TECHNICAL;	// Nothing to save
	if (!pStream)
		return ERR_FILE_NAME;	// File not found or opened
	{
		DATEINT InitialDate = 0;
		DATEINT LedgerPeriodStart = 0;
		DATEINT FinalDate = 0;
		int iTabPos = 0;
		char *pXbrlIdentifier = 0;
		LedgerType* pDefaultLedger = 0;
		char *pContextId = 0;
		char *pContextRef = 0;
		AccountingType* pXglAccounting = ConvertAccountCoding(vpAccounting, FILE_TYPE_XGL);
		// Need to make a copy of Accounting to recode the accounts for a SIE-file (has fewer account types)
		if(!pXglAccounting)
			return ERR_TECHNICAL;	// File not found or opened

		Sort(pXglAccounting, VOUCHERS_BY_SERIES);

		pDefaultLedger = GetLedger(pXglAccounting, 0);

		if (!strcmp(pContent, TXT_VOUCHERS))
			bLeanChartsOption = TRUE;
		else
			FilterEnteredBy = 0;

		if (!IsPeriodic(pXglAccounting))
			bPeriodOption = FALSE;

		if (!IsObject(pXglAccounting))
			bObjectOption = FALSE;

		// OMFATTN as switch??

		if ((!pXglAccounting->pEntity) || (!pXglAccounting->pEntity->pName) || (!pXglAccounting->pEntity->pName[0])) {
			// Company name iss mandatory
			DestroyAccounting(&pXglAccounting);
			return ERR_FILE_CONTENT;
		}

		if ((!pDefaultLedger)
			|| (((!strcmp(pContent, TXT_BALANCE)) || (!strcmp(pContent, TXT_LEDGER)))
				&& ((!pDefaultLedger->YearStart) || (!pDefaultLedger->YearEnd)))) {
			DestroyAccounting(&pXglAccounting);
			return ERR_FILE_CONTENT;
		}

		if (pDefaultLedger && pInitialDate && *pInitialDate) {
			InitialDate = StringToDate(pInitialDate);
			LedgerPeriodStart = pDefaultLedger->YearStart;

			if (InitialDate > pDefaultLedger->YearEnd) {
				DestroyAccounting(&pXglAccounting);
				return ERR_FILE_CONTENT;
			}
			else if (InitialDate < pDefaultLedger->YearStart) {
				// Find the beginning of balances (that the InitialDate is not before it
				DATEINT TempInitialDate = 0;
				// Find the AccountingYear 
				LedgerType *pLedger
					= LListGetFirstRecordData(pXglAccounting->pLedgersChart->pListLedger);
				while (pLedger) {
					if (TempInitialDate > pLedger->YearStart)
						TempInitialDate = pLedger->YearStart;
					pLedger = LListGetNextRecordData(pXglAccounting->pLedgersChart->pListLedger);
				} // End Dimension 0 Object 0 = All, containing IB, UB, RESULT, 

				if (InitialDate < TempInitialDate)
					InitialDate = TempInitialDate;
			}
			// We shall not work with Ledger entries before the InitialDate
			if (LedgerPeriodStart < InitialDate)
				LedgerPeriodStart = InitialDate;
		}
		else if (pDefaultLedger) {
			// Find the beginning of balances
			DATEINT TempInitialDate = pDefaultLedger->YearStart;
			// Find the AccountingYear 
			LedgerType *pLedger
				= LListGetFirstRecordData(pXglAccounting->pLedgersChart->pListLedger);
			while (pLedger) {
				if (TempInitialDate > pLedger->YearStart)
					TempInitialDate = pLedger->YearStart;
				pLedger = LListGetNextRecordData(pXglAccounting->pLedgersChart->pListLedger);
			} // End Dimension 0 Object 0 = All, containing IB, UB, RESULT, 

			LedgerPeriodStart =	// Must start with the beginning of this year
				InitialDate =	// Start with the beginning of this year
				TempInitialDate;
		}

		if (pDefaultLedger && pFinalDate && *pFinalDate) { // The complete year
			FinalDate = StringToDate(pFinalDate);
			if (pDefaultLedger->YearEnd && pDefaultLedger && (FinalDate > pDefaultLedger->YearEnd))
				FinalDate = pDefaultLedger->YearEnd;
		}
		else if (pDefaultLedger && pDefaultLedger->YearEnd)
			FinalDate = pDefaultLedger->YearEnd;
		else
			FinalDate = (DATEINT)(-1);

		if (FinalDate < InitialDate) {
			DestroyAccounting(&pXglAccounting);
			return ERR_FILE_CONTENT;
		}

		if (!strcmp(pContent, TXT_CHART)) {
			pXbrlIdentifier = calloc(strlen(XBRL_ID_PREFIX_STRING_CHART) + 1, 1);
			strcat(pXbrlIdentifier, XBRL_ID_PREFIX_STRING_CHART);
		}
		else if (pXglAccounting->pEntity && pXglAccounting->pEntity->pLegalRegId && *pXglAccounting->pEntity->pLegalRegId) {
			pXbrlIdentifier = calloc(strlen(pXglAccounting->pEntity->pLegalRegId) + strlen(XBRL_ID_PREFIX_STRING) + 1, sizeof(char));
			strcat(pXbrlIdentifier, XBRL_ID_PREFIX_STRING);
			strcat(pXbrlIdentifier, pXglAccounting->pEntity->pLegalRegId);
			{
				int i = 0;
				while (pXbrlIdentifier[i]) {
					if (!(((pXbrlIdentifier[i] >= '0') && (pXbrlIdentifier[i] <= '9'))
						|| ((pXbrlIdentifier[i] >= 'A') && (pXbrlIdentifier[i] <= 'Z'))
						|| ((pXbrlIdentifier[i] >= 'a') && (pXbrlIdentifier[i] <= 'z'))
						|| (pXbrlIdentifier[i] == '-')))
						memmove(&pXbrlIdentifier[i], &pXbrlIdentifier[i + 1], strlen(&pXbrlIdentifier[i]));
					else
						i++;
				}
			}
		}
		else {
			pXbrlIdentifier = calloc(strlen(XBRL_ID_PREFIX_STRING) + 1, 1);
			strcat(pXbrlIdentifier, XBRL_ID_PREFIX_STRING);
		}
		{
			char szInstantDateString[LENGTH_LONG_DATE];
			memset(szInstantDateString, 0, LENGTH_LONG_DATE);

			// Todays date preparation
			DateToStr(GetSystemDate(), szInstantDateString);

			// XBRL FILE HEADER

			// UTF8
			ufWrite(pStream, "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\r\n");

			if (pStylesheetFilePathName && (*pStylesheetFilePathName)) {
				ufWrite(pStream, "<?xml-stylesheet type=\"text/xsl\" href=\"");
				ufWrite(pStream, pStylesheetFilePathName);
				ufWrite(pStream, "\"?>\r\n");
			}
			// Creation #PROGRAM
			ufWrite(pStream, "<!-- Created by ");
			ufWrite(pStream, pProgramName);
			ufWrite(pStream, " version ");
			ufWrite(pStream, pProgramVersion);
			ufWrite(pStream, ", ");
			ufWrite(pStream, szInstantDateString);
			ufWrite(pStream, "    -->\r\n");

			ufWrite(pStream, "<!-- Alphabet XBRL GL interpretation of SIE type ");
			ufWrite(pStream, pContent);
			ufWrite(pStream, "       -->\r\n");

			ufWrite(pStream, "<!-- See: www.alphabet.se/XBRL                           -->\r\n");
			ufWrite(pStream, "<!-- This XBRL GL file is a converted Swedish SIE file   -->\r\n");
			ufWrite(pStream, "<!-- It means its taxonomy is based on the SIE-logics    -->\r\n");
			ufWrite(pStream, "<!-- About SIE see http://www.sie.se/english/english.asp -->\r\n");

				// Links
			iTabPos++;
			ufWrite(pStream, "<xbrli:xbrl xmlns:xbrli=\"http://www.xbrl.org/2003/instance\" \r\n");
			ufWrite(pStream, "\t\txmlns:xbrll=\"http://www.xbrl.org/2003/linkbase\" \r\n");
			ufWrite(pStream, "\t\txmlns:xlink=\"http://www.w3.org/1999/xlink\" \r\n");
			ufWrite(pStream, "\t\txmlns:xsl=\"http://www.w3.org/2001/XMLSchema-instance\" \r\n");
			ufWrite(pStream, "\t\txmlns:gl-cor=\"http://www.xbrl.org/int/gl/cor/2016-12-01\" \r\n");
			ufWrite(pStream, "\t\txmlns:gl-muc=\"http://www.xbrl.org/int/gl/muc/2016-12-01\" \r\n");
			ufWrite(pStream, "\t\txmlns:gl-bus=\"http://www.xbrl.org/int/gl/bus/2016-12-01\" \r\n");
			ufWrite(pStream, "\t\txmlns:gl-plt=\"http://www.xbrl.org/int/gl/plt/2016-12-01\" \r\n");
			ufWrite(pStream, "\t\txmlns:iso4217=\"http://www.xbrl.org/2003/iso4217\" \r\n");
			ufWrite(pStream, "\t\txmlns:iso639=\"http://www.xbrl.org/2005/iso639\" \r\n");
			ufWrite(pStream, "\t\txsl:schemaLocation=\"http://www.xbrl.org/int/gl/plt/2016-12-01 http://www.xbrl.org/taxonomy/int/gl/2016-12-01/plt/case-c-b-m-u/gl-plt-all-2016-12-01.xsd\">\r\n");
			ufWrite(pStream, "\t<xbrll:schemaRef xlink:type=\"simple\" xlink:arcrole=\"http://www.w3.org/1999/xlink/properties/linkbase\" xlink:href=\"http://www.xbrl.org/taxonomy/int/gl/2016-12-01/plt/case-c-b-m-u/gl-plt-all-2016-12-01.xsd\" />\r\n");

			// MAIN HEADER

			// Unit #VALUTA 
			int xmlWriteLine(FILE *pStream, int iTab, char *attr, char *pPrefix, char *pData, char *pSuffix);
			if (strcmp(pContent, TXT_CHART)) { // Not chart
				xmlWriteLine(pStream, iTabPos++, "id=\"NotUsed\"", "xbrli:unit", 0, 0);
				xmlWriteLine(pStream, iTabPos, 0, "xbrli:measure", "pure", "xbrli:measure");
				xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "xbrli:unit");

				if (pXglAccounting->pCurrency && *pXglAccounting->pCurrency) {
					char XbrlCurrencyData[LENGTH_ISO4217_CURRENCY + 1 + 8];
					char XbrlCurrency[LENGTH_ISO4217_CURRENCY + 1];
					char XbrlCurrencyAttr[LENGTH_ISO4217_CURRENCY + 1 + 16];
					memset(XbrlCurrencyData, 0, LENGTH_ISO4217_CURRENCY + 1 + 8);
					memset(XbrlCurrency, 0, LENGTH_ISO4217_CURRENCY + 1);
					memset(XbrlCurrencyAttr, 0, LENGTH_ISO4217_CURRENCY + 16);
					strcpy(XbrlCurrencyAttr, "id=\"");
					strncat(XbrlCurrencyAttr, pXglAccounting->pCurrency, LENGTH_ISO4217_CURRENCY);
					strcat(XbrlCurrencyAttr, "\"");
					StrLwr((unsigned char *)XbrlCurrencyAttr);
					strncpy(XbrlCurrency, pXglAccounting->pCurrency, LENGTH_ISO4217_CURRENCY);
					StrUpper((UCHAR *)XbrlCurrency);
					strcpy(XbrlCurrencyData, PREFIX_ISO4217_CURR);
					strcat(XbrlCurrencyData, XbrlCurrency);

					xmlWriteLine(pStream, iTabPos++, XbrlCurrencyAttr, "xbrli:unit", 0, 0);
					xmlWriteLine(pStream, iTabPos, 0, "xbrli:measure", XbrlCurrencyData, "xbrli:measure");
					xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "xbrli:unit");
				}
			}

			// xbrli:identifier
			pContextId = calloc(strlen(pXbrlIdentifier) + 10, sizeof(char));
			strcpy(pContextId, "id=\"");
			strcat(pContextId, pXbrlIdentifier);
			strcat(pContextId, "\"");
			xmlWriteLine(pStream, iTabPos++, pContextId, "xbrli:context", 0, 0);
			free(pContextId);
			xmlWriteLine(pStream, iTabPos++, 0, "xbrli:entity", 0, 0);
			if ((pXglAccounting->pEntity && pXglAccounting->pEntity->pName && *pXglAccounting->pEntity->pName)
				&& ((!strcmp(pContent, TXT_BALANCE)) || (!strcmp(pContent, TXT_LEDGER)) || (!strcmp(pContent, TXT_VOUCHERS))))
				xmlWriteLine(pStream, iTabPos, "scheme=\"http://www.alphabet.se/XBRL\"", "xbrli:identifier", pXglAccounting->pEntity->pName, "xbrli:identifier");
			else
				xmlWriteLine(pStream, iTabPos, "scheme=\"http://www.alphabet.se/XBRL\"", "xbrli:identifier", "void", "xbrli:identifier");
			xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "xbrli:entity");
			xmlWriteLine(pStream, iTabPos++, 0, "xbrli:period", 0, 0);
			if (!strcmp(pContent, TXT_CHART)) { // Chart
				xmlWriteLine(pStream, iTabPos, 0, "xbrli:instant", DateToStr(GetSystemDate(), szInstantDateString), "xbrli:instant");
			}
			else {
				if (!strcmp(pContent, TXT_VOUCHERS)) {
					LedgerType *pLedger
						= LListGetFirstRecordData(pXglAccounting->pLedgersChart->pListLedger);
					LedgerPeriodStart = pLedger->YearStart;
					while (pLedger) {
						if (LListGetFirstRecordData(pLedger->pListVoucher))
							LedgerPeriodStart = pLedger->YearStart;
						pLedger = LListGetNextRecordData(pXglAccounting->pLedgersChart->pListLedger);
					}
				}
				xmlWriteLine(pStream, iTabPos, 0, "xbrli:startDate", DateToStr(LedgerPeriodStart, szInstantDateString), "xbrli:startDate");
				xmlWriteLine(pStream, iTabPos, 0, "xbrli:endDate", DateToStr(FinalDate, szInstantDateString), "xbrli:endDate");
			}
			xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "xbrli:period");
			xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "xbrli:context");
			pContextRef = calloc(strlen(pXbrlIdentifier) + 20, sizeof(char));
			if (pContextRef) {
				strcpy(pContextRef, ATTRIBUTE_CONTEXTREF"=\"");
				strcat(pContextRef, pXbrlIdentifier);
				strcat(pContextRef, "\"");
			}
			free(pXbrlIdentifier);
		}

		// ACCOUNT HEADER
		if (pContextRef) {
			char *pLocaleLanguage = 0;
			char **pMonth = 0;
			if (pLocaleData) {
				pLocaleLanguage = pLocaleData->pLocaleLanguage;
				pMonth = pLocaleData->pMonth;
			}
			BOOL bChartsDeclared = FALSE;
			// #KONTO kontonr kontonamn
			// #KTYP kontonr kontotyp
			// #SRU konto SRU-kod
			// #ENHET kontonr enhet
			// #KPTYP
			if (!strcmp(pContent, TXT_CHART)) {
				bLeanChartsOption = FALSE;

				if (WriteXbrlGlAccountStatements(pStream, pContextRef, pXglAccounting, bQuantityOption, bLeanChartsOption, InitialDate, FinalDate, FilterEnteredBy, pContent, pGenUserName, iTabPos, pLocaleLanguage, pProgramName, pProgramVersion)) {
					DestroyAccounting(&pXglAccounting);
					return ERR_TECHNICAL;
				}
				bChartsDeclared = TRUE;
			}
			/*
			// #DIM dimensionsnr namn
			// #UNDERDIM dimensionsnr namn superdimension
			// #OBJEKT dimensionsnr objektnr objektnamn
			if (bObjectOption) {
				if (WriteXbrlGlDimObjStatements(pStream, pContextRef, pXglAccounting, bLeanChartsOption, InitialDate, FinalDate, FilterEnteredBy, pContent, pGenUserName, iTabPos, pLocaleLanguage, pProgramName, pProgramVersion)) {
				DestroyAccounting(&pXglAccounting);
				return ERR_TECHNICAL;
				}
				bChartsDeclared = TRUE;
				}
				*/
				// #IB årsnr konto saldo kvantitet
				// #UB årsnr konto saldo kvantitet
				// #RES årsnr konto saldo kvantitet
				// #PSALDO årsnr period konto {dimensionsnr objektnr} saldo kvantitet 
				// #PBUDGET årsnr period konto {dimensionsnr objektnr} saldo kvantitet
			if (pDefaultLedger && ((!strcmp(pContent, TXT_BALANCE)) || (!strcmp(pContent, TXT_LEDGER)))) {
				if (WriteXbrlGlBalances(pStream, pContextRef, pXglAccounting, bCompactFileOption, bQuantityOption, bObjectOption, bPeriodOption, bLeanChartsOption, InitialDate, FinalDate, pContent, pGenUserName, &bChartsDeclared, iTabPos, pLocaleLanguage, pMonth, pProgramName, pProgramVersion)) {
					DestroyAccounting(&pXglAccounting);
					return ERR_TECHNICAL;
				}
			}

			// #VER serie vernr verdatum vertext regdatum sign
			// #TRANS kontonr {objektlista} belopp transdat transtext kvantitet sign			
			// #BTRANS kontonr {objektlista} belopp transdat transtext kvantitet sign			
			// #RTRANS kontonr {objektlista} belopp transdat transtext kvantitet sign			
			if (pDefaultLedger && ((!strcmp(pContent, TXT_VOUCHERS)) || (!strcmp(pContent, TXT_LEDGER)))) {
				if (GetLastEntryDate(pXglAccounting) < FinalDate)
					FinalDate = GetLastEntryDate(pXglAccounting);

				if (WriteXbrlGlLedgerVouchers(pStream, pContextRef, pXglAccounting, bCompactFileOption, bQuantityOption, bObjectOption, LedgerPeriodStart, FinalDate, FilterEnteredBy, pContent, pGenUserName, &bChartsDeclared, iTabPos, pLocaleLanguage, pProgramName, pProgramVersion)) {
					DestroyAccounting(&pXglAccounting);
					return ERR_TECHNICAL;
				}
			}

			// End
			xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "xbrli:xbrl");
			free(pContextRef);
		}
		iTabPos = 0;
		DestroyAccounting(&pXglAccounting);
		return ERR_OK; // Good
	}
}


/********************************************************************
*
static int WriteXbrlGlAccountTaxCodeSpecifications(
*
*********************************************************************/
static int WriteXbrlGlAccountTaxCodeSpecifications(
	FILE *pStream, 
	char *pContextRef, 
	AccountType *pAccount, 
	AccountingType *pAccounting, 
	BOOL bCompactFileOption, 
	int iTabPos)
{
	{
		BOOL TaxcodesNoted = FALSE;
		AccountCodeType *pAccountCode
			= LListGetFirstRecordData(pAccount->pListAccountCode);
		while (pAccountCode) {
			if (!TaxcodesNoted) {
				TaxcodesNoted = TRUE;
				xmlWriteLine(pStream, iTabPos++, 0, "gl-cor:taxes", 0, 0);
			}
			if (pAccountCode->pValidYear
				&& (pAccountCode->pCodeType && *pAccountCode->pCodeType)
				&& (pAccountCode->pCode && *pAccountCode->pCode)
				&& LListGetFirstRecordData(pAccountCode->pListApplicableDomain)) {
				if (pAccountCode->pIssuer)
					xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:taxAuthority", pAccountCode->pIssuer, "gl-cor:taxAuthority");
				if (pAccountCode->pValidYear)
					xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:taxTableCode", pAccountCode->pValidYear, "gl-cor:taxTableCode");
				xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:taxDescription", pAccountCode->pCodeType, "gl-cor:taxDescription");
				xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:taxCode", pAccountCode->pCode, "gl-cor:taxCode");
				if (LListGetFirstRecordData(pAccounting->pEntity->pListFileUrl)) {
					char *pApplicableDomain = LListGetFirstRecordData(pAccountCode->pListApplicableDomain);
					while (pApplicableDomain) {
						xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:taxCommentExemption", pApplicableDomain, "gl-cor:taxCommentExemption");
						pApplicableDomain = LListGetNextRecordData(pAccountCode->pListApplicableDomain);
					}
				}
			}
			pAccountCode = LListGetNextRecordData(pAccount->pListAccountCode);
		}
		if (TaxcodesNoted)
			xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:taxes");
	}
	return 0;
}


/********************************************************************
*
static int WriteXbrlGlAccountSpecifications(
*
*********************************************************************/
static int WriteXbrlGlAccountSpecifications(
	FILE *pStream, 
	AccountingType *pAccounting, 
	char *AccountChartType, 
	char *pContextRef, 
	char *pAccountID, 
	char *pAccountName, 
	char *pAccountType, 
	char *pAccountTypeDescription, 
	char *pAccountActive, 
	BOOL bQuantityOption, 
	BOOL bCompactFileOption, 
	int iTabPos)
{
	xmlWriteLine(pStream, iTabPos++, 0, "gl-cor:account", 0, 0);
	if (pAccountID && *pAccountID)
		xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:accountMainID", pAccountID, "gl-cor:accountMainID");
	if (pAccountName && *pAccountName)
		xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:accountMainDescription", pAccountName, "gl-cor:accountMainDescription");
	if (pAccountType) {
		if (!strcmp(pAccountType, ACCOUNT_TYPE_ASSETS))
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:mainAccountType", "asset", "gl-cor:mainAccountType");
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_EQUITIES))
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:mainAccountType", "equity", "gl-cor:mainAccountType");
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_LIABILITIES))
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:mainAccountType", "liability", "gl-cor:mainAccountType");
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_REVENUES))
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:mainAccountType", "income", "gl-cor:mainAccountType");
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_EXPENSES))
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:mainAccountType", "expense", "gl-cor:mainAccountType");
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_LOSS_GAINS))
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:mainAccountType", "gain", "gl-cor:mainAccountType");
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_FINANCIAL))
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:mainAccountType", "comprehensive-income", "gl-cor:mainAccountType");
		else if (!strcmp(pAccountType, ACCOUNT_TYPE_AUX))
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:mainAccountType", "other", "gl-cor:mainAccountType");
	}
	if (pAccountTypeDescription && *pAccountTypeDescription)
		xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:mainAccountTypeDescription", pAccountTypeDescription, "gl-cor:mainAccountTypeDescription");

	if (!bCompactFileOption && AccountChartType && *AccountChartType && pAccountID && *pAccountID) {
		if (!strncmp(AccountChartType, "BAS20", 5)) {
			//			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:accountPurposeCode", "primary", "gl-cor:accountPurposeCode");
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:accountPurposeDescription", BAS2000, "gl-cor:accountPurposeDescription");
		}
		else {
			//			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:accountPurposeCode", "primary", "gl-cor:accountPurposeCode");
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:accountPurposeDescription", AccountChartType, "gl-cor:accountPurposeDescription");
		}
	}
	if (pAccountActive)
		xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:accountActive", pAccountActive, "gl-cor:accountActive");
	xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:account");

	return 0; // Good
}


/********************************************************************
*
static int WriteXbrlGlAccountStatements(
*
*********************************************************************/
static int WriteXbrlGlAccountStatements(
	FILE *pStream, 
	char *pContextRef, 
	AccountingType *pAccounting, 
	BOOL bQuantityOption, 
	BOOL bLeanChartsOption, 
	DATEINT InitialDate, 
	DATEINT FinalDate, 
	char *FilterEnteredBy, 
	char *pContent, 
	char *pGenUserName, 
	int iTabPos, 
	char *pLocaleLanguage, 
	char *pProgramName, 
	char *pProgramVersion)
{
	// #KONTO kontonr kontonamn
	// #KTYP kontonr kontotyp
	// #SRU konto SRU-kod
	// #ENHET kontonr enhet
	// #KPTYP
	if (!pAccounting->pAccountsChart)
		return 0; // NOthing to write

	xmlWriteLine(pStream, iTabPos++, 0, "gl-cor:accountingEntries", 0, 0);

	if (WriteXbrlGlInformationHeader(pStream, pContextRef, pAccounting, "account", GetEnumFolderTypeText(CONTENT_LABEL_CHARTS, pLocaleLanguage), 0, pContent, pGenUserName, 0, 0, 0, 0, 0, iTabPos, pLocaleLanguage, pProgramName, pProgramVersion))
		return (-2);

	{
		BOOL bHeader = FALSE;
		AccountType *pAccount
			= LListGetFirstRecordData(pAccounting->pAccountsChart->pListAccount);

		if (pAccount) {
			xmlWriteLine(pStream, iTabPos++, 0, "gl-cor:entryHeader", 0, 0);
			bHeader = TRUE;
		}

		while (pAccount) {
			if ((!(bLeanChartsOption)) || (!pAccount->InActive)) {
				char *pAccountActive = 0;
				if (pAccount->InActive)
					pAccountActive = "false";

				xmlWriteLine(pStream, iTabPos++, 0, "gl-cor:entryDetail", 0, 0);

				// #KONTO
				if (WriteXbrlGlAccountSpecifications(pStream, pAccounting, pAccounting->pGaap, pContextRef, pAccount->pId, GetTextListText(pAccount->pListName, pLocaleLanguage), pAccount->pType, 0, pAccountActive, bQuantityOption, FALSE, iTabPos))
					return (-2);

				// #ENHET			
				if (bQuantityOption) {
					if (GetTextListText(pAccount->pListQuantityUnit, pLocaleLanguage)) {
						if (WriteXbrlGlQuantitySpecifications(pStream, 0, GetTextListText(pAccount->pListQuantityUnit, pLocaleLanguage), pContextRef, iTabPos))
							return (-2);
					}
				}

				// #SRU
				if (WriteXbrlGlAccountTaxCodeSpecifications(pStream, pContextRef, pAccount, pAccounting, FALSE, iTabPos))
					return (-2);

				xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:entryDetail");
			}

			pAccount = LListGetNextRecordData(pAccounting->pAccountsChart->pListAccount);
		}
		if (bHeader)
			xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:entryHeader");
	}
	xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:accountingEntries");
	return 0;
}


/********************************************************************
*
static int WriteXbrlGlQuantitySpecifications(
*
*********************************************************************/
static int WriteXbrlGlQuantitySpecifications(
	FILE *pStream, 
	char *pQuantity, 
	char *pQuantityUnit, 
	char *pContextRef, 
	int iTabPos)
{
	char ExtraAttr[] = { ATTRIBUTE_UNITREF" = \"NotUsed\" decimals=\"INF\" " };
	char *pContextRefExtra = calloc(strlen(ExtraAttr) + strlen(pContextRef) + 1, sizeof(char));
	if (pContextRefExtra) {
		strcpy(pContextRefExtra, ExtraAttr);
		strcat(pContextRefExtra, pContextRef);
		if ((pQuantity && *pQuantity) || (pQuantityUnit && *pQuantityUnit)) {
			xmlWriteLine(pStream, iTabPos++, 0, "gl-bus:measurable", 0, 0);
			if (pQuantity && *pQuantity)
				xmlWriteLine(pStream, iTabPos, pContextRefExtra, "gl-bus:measurableQuantity", pQuantity, "gl-bus:measurableQuantity");

			if (pQuantityUnit && *pQuantityUnit)
				xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:measurableUnitOfMeasure", pQuantityUnit, "gl-bus:measurableUnitOfMeasure");

			xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-bus:measurable");
		}
		free(pContextRefExtra);
		return 0;
	}
	else if ((pQuantity && *pQuantity) || (pQuantityUnit && *pQuantityUnit)) {
		xmlWriteLine(pStream, iTabPos++, 0, "gl-bus:measurable", 0, 0);
		if (pQuantity && *pQuantity)
			xmlWriteLine(pStream, iTabPos, "Error memory", "gl-bus:measurableQuantity", pQuantity, "gl-bus:measurableQuantity");

		if (pQuantityUnit && *pQuantityUnit)
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:measurableUnitOfMeasure", pQuantityUnit, "gl-bus:measurableUnitOfMeasure");

		xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-bus:measurable");
	}
	return -1;
}

#if XBRL_DIM_OBJ
/********************************************************************
*
static BOOL WriteXbrlGlDimObjStatements(
*
*********************************************************************/
static BOOL WriteXbrlGlDimObjStatements(
	FILE *pStream, 
	char *pContextRef, 
	AccountingType *pAccounting, 
	BOOL bLeanChartsOption, 
	DATEINT InitialDate, 
	DATEINT FinalDate, 
	char * FilterEnteredBy, 
	char *pContent, 
	char *pGenUserName, 
	int iTabPos, 
	char *pLocaleLanguage, 
	char *pProgramName, 
	char *pProgramVersion)
{
	// #DIM dimensionsnr namn
	// #UNDERDIM dimensionsnr namn superdimension
	// #OBJEKT dimensionsnr objektnr objektnamn
	if (!pAccounting->pObjectsChart)
		return 0; // Nothoing to write about

	xmlWriteLine(pStream, iTabPos++, 0, "gl-cor:accountingEntries", 0, 0);

	if (WriteXbrlGlInformationHeader(pStream, pContextRef, pAccounting, "other", GetEnumFolderTypeText(CONTENT_LABEL_OBJECTS, pLocaleLanguage), 0, pContent, pGenUserName, 0, 0, 0, 0, 0, iTabPos, pLocaleLanguage, pProgramName, pProgramVersion))
		return (-2);

	{
		char szDimId[LENGTH_NUMERIC_STR_MAX];
		DimensionType *pDimension
			= LListGetFirstRecordData(pAccounting->pObjectsChart->pListDimension);
		pDimension = LListGetNextRecordData(pAccounting->pObjectsChart->pListDimension);
		while (pDimension) {
			if ((!bLeanChartsOption) || IsDimensionActive(pAccounting, pDimension)) {
				char *pDimSuperTitle = NulString;
				char szDimSuper[LENGTH_NUMERIC_STR_MAX] = { "" };

				xmlWriteLine(pStream, iTabPos++, 0, "gl-cor:entryHeader", 0, 0);
				memset(szDimId, 0, LENGTH_NUMERIC_STR_MAX);
				xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:entryNumber", CounterToStr(pDimension->Id, szDimId), "gl-cor:entryNumber");
				if (GetTextListText(pDimension->pListName, pLocaleLanguage))
					xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:entryComment", GetTextListText(pDimension->pListName, pLocaleLanguage), "gl-cor:entryComment");
				memset(szDimId, 0, LENGTH_NUMERIC_STR_MAX);

				if ((pDimension->pSuperDimension) && ((DimensionType *)pDimension->pSuperDimension)->Id)
					xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:batchID", CounterToStr(((DimensionType *)pDimension->pSuperDimension)->Id, szDimSuper), "gl-bus:batchID");
				// Objects in the dimension
				{
					ObjectType *pObject
						= LListGetFirstRecordData(pDimension->pListObject);
					while (pObject) {
						if ((!bLeanChartsOption) || (!pObject->InActive)) {
							xmlWriteLine(pStream, iTabPos++, 0, "gl-cor:entryDetail", 0, 0);
							// gl-cor:identifierReference
							// SIE #DIM #OBJ info
							memset(szDimSuper, 0, LENGTH_NUMERIC_STR_MAX);
							if (pDimension->pSuperDimension && ((DimensionType *)pDimension->pSuperDimension)->Id)
								CounterToStr(((DimensionType *)pDimension->pSuperDimension)->Id, szDimSuper);
							if (WriteXbrlGlDimObjSpecificationObject(
								pStream,
								CounterToStr(pDimension->Id, szDimId),
								GetTextListText(pDimension->pListName, pLocaleLanguage),
								szDimSuper,
								pDimSuperTitle,
								pObject->pId,
								GetTextListText(pObject->pListName, pLocaleLanguage),
								LListGetFirstRecordData(pObject->pListFileUrl),
								pContextRef,
								FALSE,
								(!pObject->InActive),
								iTabPos))
								return (-2);

							xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:entryDetail");
						}
						pObject = LListGetNextRecordData(pDimension->pListObject);
					}
				}
				xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:entryHeader");
			}

			pDimension = LListGetNextRecordData(pAccounting->pObjectsChart->pListDimension);
		}
		xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:accountingEntries");
	}
	return 0;
}
#endif

/********************************************************************
*
static int WriteXbrlGlDimObjSpecificationObject(
*
*********************************************************************/
static int WriteXbrlGlDimObjSpecificationObject(
	FILE *pStream, 
	char *pDimensionId, 
	char *pDimensionDescription, 
	char *pDimensionSuperId, 
	char *pDimensionSuperDescription, 
	char *pObjectId, 
	char *pObjectDescription, 
	char *pFileUrl, 
	char *pContextRef, 
	BOOL bCompactFileOption, 
	BOOL bActiveObject, 
	int iTabPos)
{
	xmlWriteLine(pStream, iTabPos++, 0, "gl-cor:identifierReference", 0, 0);
	xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:identifierCode", pObjectId, "gl-cor:identifierCode");
	if (pDimensionSuperId && *pDimensionSuperId && (!bCompactFileOption)) {
		xmlWriteLine(pStream, iTabPos++, 0, "gl-cor:identifierExternalReference", 0, 0);
		xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:identifierAuthorityCode", pDimensionSuperId, "gl-cor:identifierAuthorityCode");
		if ((!bCompactFileOption) && (pDimensionSuperDescription && *pDimensionSuperDescription))
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:identifierAuthority", pDimensionSuperDescription, "gl-cor:identifierAuthority");
		xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:identifierExternalReference");
	}

	if (pObjectDescription && *pObjectDescription)
		xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:identifierDescription", pObjectDescription, "gl-cor:identifierDescription");

	if (!bCompactFileOption) {
		switch (atoll(pDimensionId)) {
		case 8:
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:identifierType", "customer", "gl-cor:identifierType");
			break;
		case 7:
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:identifierType", "employee", "gl-cor:identifierType");
			break;
		case 9:
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:identifierType", "vendor", "gl-cor:identifierType");
			break;
		default:
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:identifierType", "other", "gl-cor:identifierType");
			break;
		}
	}
	xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:identifierCategory", pDimensionId, "gl-cor:identifierCategory");
	xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:identifierPurpose", pDimensionDescription, "gl-bus:identifierPurpose");

	if (!bActiveObject)
		xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:identifierActive", "false", "gl-cor:identifierActive");

	xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:identifierReference");
	//	if(pFileUrl)
	//		xmlWriteLine(pStream,			iTabPos,	"gl-cor",	"gl-cor",	ATTRIBUTE_CONTEXTREF,	pContextRef,		"documentReference",			pFileUrl);
	return 0;
}


/********************************************************************
*
static int WriteXbrlGlBalances(
*
*********************************************************************/
static int WriteXbrlGlBalances(
	FILE *pStream,
	char *pContextRef,
	AccountingType *pAccounting,
	BOOL bCompactFileOption,
	BOOL bQuantityOption,
	BOOL bObjectOption,
	BOOL bPeriodOption,
	BOOL bLeanChartsOption,
	DATEINT InitialDate,
	DATEINT FinalDate,
	char *pContent,
	char *pGenUserName,
	BOOL *pbChartsDeclared,
	int iTabPos,
	char *pLocaleLanguage, 
	char **ppMonth,
	char *pProgramName,
	char *pProgramVersion)
{
	// #IB årsnr konto saldo kvantitet
	// #UB årsnr konto saldo kvantitet
	// #RES årsnr konto saldo kvantitet
	// #PSALDO årsnr period konto {dimensionsnr objektnr} saldo kvantitet 
	// #PBUDGET årsnr period konto {dimensionsnr objektnr} saldo kvantitet
	//LLIST SortedListpLedgers = GetSortedAccountingYearList(pAccounting->pLedgersChart->pListLedger, TRUE);
	DATEINT LastEntryDate = GetLastEntryDate(pAccounting);

	BOOL bBudgetOption = IsBudget(pAccounting, 0, 0, 0, 0, 0);
	
	if (!pAccounting->pAccountsChart)
		return 0; // OK, No accounts to report

	if (!strcmp(pContent, TXT_VOUCHERS))
		return 0; // No balances for SIE4I

	if (FinalDate < InitialDate)
		return ERR_SIE_PERIOD;

	if (pbChartsDeclared) {
		bCompactFileOption = FALSE;
		*pbChartsDeclared = TRUE;
	}

	{
		LedgerType *pLedger
			= LListGetFirstRecordData(pAccounting->pLedgersChart->pListLedger);
		while (pLedger) {
			BalanceReportType *pReport
				= LListGetFirstRecordData(pLedger->pListReport);
			while (pReport) {
				if (FinalDate > pLedger->YearEnd)
					FinalDate = pLedger->YearEnd;

				if ((FinalDate > InitialDate)
					&& ((!(pReport->pObject && pReport->pObject->pDimension->Id))
						|| ((!strcmp(pContent, TXT_BALANCE)) || (!strcmp(pContent, TXT_LEDGER))))) {
					int iResult =
						WriteXbrlGlBalanceReports(
							pStream,
							pContextRef,
							pAccounting,
							pLedger,
							pReport,
							bCompactFileOption,
							bQuantityOption,
							bObjectOption,
							bPeriodOption,
							bBudgetOption,
							bLeanChartsOption,
							InitialDate,
							FinalDate,
							LastEntryDate,
							pContent,
							pGenUserName,
							iTabPos,
							pLocaleLanguage,
							ppMonth,
							pProgramName,
							pProgramVersion);
					if (iResult)
						return iResult;
				}
				pReport = LListGetNextRecordData(pLedger->pListReport);
			}
			pLedger = LListGetNextRecordData(pAccounting->pLedgersChart->pListLedger);
		}
	}
	return 0;
}


/********************************************************************
*
static int WriteXbrlGlBalanceReports(
*
*********************************************************************/
static int WriteXbrlGlBalanceReports(
	FILE *pStream,
	char *pContextRef,
	AccountingType *pAccounting,
	LedgerType *pLedger,
	BalanceReportType *pReport,
	BOOL bCompactFileOption,
	BOOL bQuantityOption,
	BOOL bObjectOption,
	BOOL bPeriodOption,
	BOOL bBudgetOption,
	BOOL bLeanChartsOption,
	DATEINT InitialDate,
	DATEINT FinalDate,
	DATEINT LastEntryDate,
	char *pContent,
	char *pGenUserName,
	int iTabPos,
	char *pLocaleLanguage,
	char **ppMonth,
	char *pProgramName,
	char *pProgramVersion)
{
	int iInitiated = 0;
    extern EnumType EnumAccountTypeType[];
	BOOL bAccountTypes[ENUM_ACCOUNTTYPETYPE];

	BalanceAccountType* pBalanceAccount
		= LListGetFirstRecordData(pReport->pListBalanceAccount);

	if(!pBalanceAccount)
		return 0;

	{
		int i = 0;
		for (i = 0; i < ENUM_ACCOUNTTYPETYPE; i++)
			bAccountTypes[i] = FALSE;
	}
	{
		int i = 0;
		CheckActive(pAccounting, pReport->pObject, 0, 0, FALSE, 0);
		for (i = 0; i < ENUM_ACCOUNTTYPETYPE; i++) {
			// To generate a statement for each active account at the last accounting year representation for HTML/XSL scripts to relate to
			AccountType* pAccount
				= LListGetFirstRecordData(pAccounting->pAccountsChart->pListAccount);
			while (pAccount) {
				if ((!strcmp((EnumAccountTypeType + i)->EnumerationID, pAccount->pType))
					&& ((!(bLeanChartsOption)) || (!pAccount->InActive))) {
					BalanceAccountType *pAccountStatement = GetBalanceAccount(pReport, pAccount);
					AMOUNT	Opening = 0;
					QUANT	OpeningQuantity = 0;
					AMOUNT	Forward = 0;
					QUANT	ForwardQuantity = 0;
					if (pAccountStatement) {
						Opening = pAccountStatement->Opening;
						OpeningQuantity = pAccountStatement->OpeningQuantity;
						Forward = pAccountStatement->Forward;
						ForwardQuantity = pAccountStatement->ForwardQuantity;
						bAccountTypes[i] = TRUE;
					}
						iInitiated = WriteXbrlGlBalanceReportLines(
							pStream,
							pContextRef,
							pAccounting,
							pLedger,
							pReport,
							pAccount,
							GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
							Forward,
							ForwardQuantity,
							bCompactFileOption,
							bLeanChartsOption,
							bQuantityOption,
							pContent,
							BALANCE_ENDING,
							FinalDate,
							0,
							LastEntryDate,
							pGenUserName,
							GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
							GetEnumDepreciationPeriodsType(TXT_ANNUALLY, pLocaleLanguage),
							&iTabPos,
							iInitiated,
							pLocaleLanguage,
							ppMonth,
							pProgramName,
							pProgramVersion);

						if (BALANCE_ACCOUNTS) {
							iInitiated = WriteXbrlGlBalanceReportLines(
								pStream,
								pContextRef,
								pAccounting,
								pLedger,
								pReport,
								pAccount,
								GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
								Opening,
								OpeningQuantity,
								bCompactFileOption,
								bLeanChartsOption,
								bQuantityOption,
								pContent,
								BALANCE_BEGINNING,
								FinalDate,
								0,
								LastEntryDate,
								pGenUserName,
								GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
								GetEnumDepreciationPeriodsType(TXT_ANNUALLY, pLocaleLanguage),
								&iTabPos,
								iInitiated,
								pLocaleLanguage,
								ppMonth,
								pProgramName,
								pProgramVersion);
							if (iInitiated < 0)
								return iInitiated;
						}
						if (iInitiated < 0)
							return iInitiated;
				}
				pAccount = LListGetNextRecordData(pAccounting->pAccountsChart->pListAccount);
			}
			{
				AMOUNT	Opening = 0;
				AMOUNT	Forward = 0;
				AccountType LabelAccount;
				AccountType *pAccount = &LabelAccount; // For BALANCE_ACCOUNTS
				memset(&LabelAccount, 0, sizeof(AccountType));
				LabelAccount.pType = (EnumAccountTypeType + i)->EnumerationID;
				if (pReport->pAssets) {
					Opening = pReport->pAssets->Opening;
					Forward = pReport->pAssets->Forward;
				}

				if (!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_ASSETS)) {
					if (bAccountTypes[0] || bAccountTypes[1] || bAccountTypes[2] || bAccountTypes[3]) {
						iInitiated = WriteXbrlGlBalanceReportLines(
							pStream,
							pContextRef,
							pAccounting,
							pLedger,
							pReport,
							pAccount,
							GetEnumAccountTitleTypeText(ACCOUNT_TYPE_ACTIVE, pLocaleLanguage),
							Forward,
							0,
							bCompactFileOption,
							bLeanChartsOption,
							bQuantityOption,
							pContent,
							BALANCE_ENDING,
							FinalDate,
							0,
							LastEntryDate,
							pGenUserName,
							GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
							GetEnumDepreciationPeriodsType(TXT_ANNUALLY, pLocaleLanguage),
							&iTabPos,
							iInitiated,
							pLocaleLanguage,
							ppMonth,
							pProgramName,
							pProgramVersion);

						if (iInitiated < 0)
							return iInitiated;

						iInitiated = WriteXbrlGlBalanceReportLines(
							pStream,
							pContextRef,
							pAccounting,
							pLedger,
							pReport,
							pAccount,
							GetEnumAccountTitleTypeText(ACCOUNT_TYPE_ACTIVE, pLocaleLanguage),
							Opening,
							0,
							bCompactFileOption,
							bLeanChartsOption,
							bQuantityOption,
							pContent,
							BALANCE_BEGINNING,
							FinalDate,
							0,
							LastEntryDate,
							pGenUserName,
							GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
							GetEnumDepreciationPeriodsType(TXT_ANNUALLY, pLocaleLanguage),
							&iTabPos,
							iInitiated,
							pLocaleLanguage,
							ppMonth,
							pProgramName,
							pProgramVersion);

						if (iInitiated < 0)
							return iInitiated;
					}
				}
				if(!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_EQUITIES)) {
					if (bAccountTypes[4]) {
						AMOUNT	Opening = 0;
						AMOUNT	Forward = 0;
						if (pReport->pEquities) {
							Opening = pReport->pEquities->Opening;
							Forward = pReport->pEquities->Forward;
						}
						iInitiated = WriteXbrlGlBalanceReportLines(
							pStream,
							pContextRef,
							pAccounting,
							pLedger,
							pReport,
							pAccount,
							GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
							Forward,
							0,
							bCompactFileOption,
							bLeanChartsOption,
							bQuantityOption,
							pContent,
							BALANCE_ENDING,
							FinalDate,
							0,
							LastEntryDate,
							pGenUserName,
							GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
							GetEnumDepreciationPeriodsType(TXT_ANNUALLY, pLocaleLanguage),
							&iTabPos,
							iInitiated,
							pLocaleLanguage,
							ppMonth,
							pProgramName,
							pProgramVersion);

						if (iInitiated < 0)
							return iInitiated;

						iInitiated = WriteXbrlGlBalanceReportLines(
							pStream,
							pContextRef,
							pAccounting,
							pLedger,
							pReport,
							pAccount,
							GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
							Opening,
							0,
							bCompactFileOption,
							bLeanChartsOption,
							bQuantityOption,
							pContent,
							BALANCE_BEGINNING,
							FinalDate,
							0,
							LastEntryDate,
							pGenUserName,
							GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
							GetEnumDepreciationPeriodsType(TXT_ANNUALLY, pLocaleLanguage),
							&iTabPos,
							iInitiated,
							pLocaleLanguage,
							ppMonth,
							pProgramName,
							pProgramVersion);

						if (iInitiated < 0)
							return iInitiated;
					}
				}
				if (!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_LIABILITIES)) {
					if (bAccountTypes[6]) {
						AMOUNT	Opening = 0;
						AMOUNT	Forward = 0;
						if (pReport->pLiabilities) {
							Opening = pReport->pLiabilities->Opening;
							Forward = pReport->pLiabilities->Forward;
						}
						iInitiated = WriteXbrlGlBalanceReportLines(
							pStream,
							pContextRef,
							pAccounting,
							pLedger,
							pReport,
							pAccount,
							GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
							Forward,
							0,
							bCompactFileOption,
							bLeanChartsOption,
							bQuantityOption,
							pContent,
							BALANCE_ENDING,
							FinalDate,
							0,
							LastEntryDate,
							pGenUserName,
							GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
							GetEnumDepreciationPeriodsType(TXT_ANNUALLY, pLocaleLanguage),
							&iTabPos,
							iInitiated,
							pLocaleLanguage,
							ppMonth,
							pProgramName,
							pProgramVersion);

						if (iInitiated < 0)
							return iInitiated;

						iInitiated = WriteXbrlGlBalanceReportLines(
							pStream,
							pContextRef,
							pAccounting,
							pLedger,
							pReport,
							pAccount,
							GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
							Opening,
							0,
							bCompactFileOption,
							bLeanChartsOption,
							bQuantityOption,
							pContent,
							BALANCE_BEGINNING,
							FinalDate,
							0,
							LastEntryDate,
							pGenUserName,
							GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
							GetEnumDepreciationPeriodsType(TXT_ANNUALLY, pLocaleLanguage),
							&iTabPos,
							iInitiated,
							pLocaleLanguage,
							ppMonth,
							pProgramName,
							pProgramVersion);

						if (iInitiated < 0)
							return iInitiated;
					}
					if ((bAccountTypes[4] && (bAccountTypes[5] || bAccountTypes[6]))
						|| (bAccountTypes[5] && (bAccountTypes[4] || bAccountTypes[6]))
						|| (bAccountTypes[6] && (bAccountTypes[5] || bAccountTypes[4])))
					{
						Opening = 0;
						Forward = 0;
						if (pReport->pPassive) {
							Opening = pReport->pPassive->Opening;
							Forward = pReport->pPassive->Forward;
						}
						LabelAccount.pType = NulString;
						iInitiated = WriteXbrlGlBalanceReportLines(
							pStream,
							pContextRef,
							pAccounting,
							pLedger,
							pReport,
							pAccount,
							GetEnumAccountTitleTypeText(ACCOUNT_TYPE_PASSIVE, pLocaleLanguage),
							Forward,
							0,
							bCompactFileOption,
							bLeanChartsOption,
							bQuantityOption,
							pContent,
							BALANCE_ENDING,
							FinalDate,
							0,
							LastEntryDate,
							pGenUserName,
							GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
							GetEnumDepreciationPeriodsType(TXT_ANNUALLY, pLocaleLanguage),
							&iTabPos,
							iInitiated,
							pLocaleLanguage,
							ppMonth,
							pProgramName,
							pProgramVersion);

						if (iInitiated < 0)
							return iInitiated;

						iInitiated = WriteXbrlGlBalanceReportLines(
							pStream,
							pContextRef,
							pAccounting,
							pLedger,
							pReport,
							pAccount,
							GetEnumAccountTitleTypeText(ACCOUNT_TYPE_PASSIVE, pLocaleLanguage),
							Opening,
							0,
							bCompactFileOption,
							bLeanChartsOption,
							bQuantityOption,
							pContent,
							BALANCE_BEGINNING,
							FinalDate,
							0,
							LastEntryDate,
							pGenUserName,
							GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
							GetEnumDepreciationPeriodsType(TXT_ANNUALLY, pLocaleLanguage),
							&iTabPos,
							iInitiated,
							pLocaleLanguage,
							ppMonth,
							pProgramName,
							pProgramVersion);

						if (iInitiated < 0)
							return iInitiated;
					}
					if ((bAccountTypes[0] || bAccountTypes[1] || bAccountTypes[2] || bAccountTypes[3]) 
						&& (bAccountTypes[4] || bAccountTypes[5] || bAccountTypes[6])) {
						Opening = 0;
						Forward = 0;
						if (pReport->pBalance) {
							Opening = pReport->pBalance->Opening;
							Forward = pReport->pBalance->Forward;
						}
						LabelAccount.pType = NulString;
						iInitiated = WriteXbrlGlBalanceReportLines(
							pStream,
							pContextRef,
							pAccounting,
							pLedger,
							pReport,
							pAccount,
							GetEnumAccountTitleTypeText(ACCOUNT_TYPE_BALANCE, pLocaleLanguage),
							Forward,
							0,
							bCompactFileOption,
							bLeanChartsOption,
							bQuantityOption,
							pContent,
							BALANCE_ENDING,
							FinalDate,
							0,
							LastEntryDate,
							pGenUserName,
							GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
							GetEnumDepreciationPeriodsType(TXT_ANNUALLY, pLocaleLanguage),
							&iTabPos,
							iInitiated,
							pLocaleLanguage,
							ppMonth,
							pProgramName,
							pProgramVersion);

						if (iInitiated < 0)
							return iInitiated;

						iInitiated = WriteXbrlGlBalanceReportLines(
							pStream,
							pContextRef,
							pAccounting,
							pLedger,
							pReport,
							pAccount,
							GetEnumAccountTitleTypeText(ACCOUNT_TYPE_BALANCE, pLocaleLanguage),
							Opening,
							0,
							bCompactFileOption,
							bLeanChartsOption,
							bQuantityOption,
							pContent,
							BALANCE_BEGINNING,
							FinalDate,
							0,
							LastEntryDate,
							pGenUserName,
							GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
							GetEnumDepreciationPeriodsType(TXT_ANNUALLY, pLocaleLanguage),
							&iTabPos,
							iInitiated,
							pLocaleLanguage,
							ppMonth,
							pProgramName,
							pProgramVersion);

						if (iInitiated < 0)
							return iInitiated;
					}
				}
				if (!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_REVENUES)) {
					if (bAccountTypes[7]) {
						AMOUNT	Forward = 0;
						if (pReport->pRevenues) {
							Forward = pReport->pRevenues->Forward;
						}
						iInitiated = WriteXbrlGlBalanceReportLines(
							pStream,
							pContextRef,
							pAccounting,
							pLedger,
							pReport,
							pAccount,
							GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
							Forward,
							0,
							bCompactFileOption,
							bLeanChartsOption,
							bQuantityOption,
							pContent,
							BALANCE_ENDING,
							FinalDate,
							0,
							LastEntryDate,
							pGenUserName,
							GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
							GetEnumDepreciationPeriodsType(TXT_ANNUALLY, pLocaleLanguage),
							&iTabPos,
							iInitiated,
							pLocaleLanguage,
							ppMonth,
							pProgramName,
							pProgramVersion);

						if (iInitiated < 0)
							return iInitiated;
					}
				}
				if (!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_EXPENSES)) {
					if (bAccountTypes[9]) {
						AMOUNT	Forward = 0;
						if (pReport->pExpenses) {
							Forward = pReport->pExpenses->Forward;
						}
						iInitiated = WriteXbrlGlBalanceReportLines(
							pStream,
							pContextRef,
							pAccounting,
							pLedger,
							pReport,
							pAccount,
							GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
							Forward,
							0,
							bCompactFileOption,
							bLeanChartsOption,
							bQuantityOption,
							pContent,
							BALANCE_ENDING,
							FinalDate,
							0,
							LastEntryDate,
							pGenUserName,
							GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
							GetEnumDepreciationPeriodsType(TXT_ANNUALLY, pLocaleLanguage),
							&iTabPos,
							iInitiated,
							pLocaleLanguage,
							ppMonth,
							pProgramName,
							pProgramVersion);

						if (iInitiated < 0)
							return iInitiated;
					}
					if (bAccountTypes[7] && bAccountTypes[9]) {
						Forward = 0;
						if (pReport->pEarning) {
							Forward = pReport->pEarning->Forward;
						}
						LabelAccount.pType = NulString;
						iInitiated = WriteXbrlGlBalanceReportLines(
							pStream,
							pContextRef,
							pAccounting,
							pLedger,
							pReport,
							pAccount,
							GetEnumAccountTitleTypeText(ACCOUNT_TYPE_EARNING, pLocaleLanguage),
							0-Forward,
							0,
							bCompactFileOption,
							bLeanChartsOption,
							bQuantityOption,
							pContent,
							BALANCE_ENDING,
							FinalDate,
							0,
							LastEntryDate,
							pGenUserName,
							GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
							GetEnumDepreciationPeriodsType(TXT_ANNUALLY, pLocaleLanguage),
							&iTabPos,
							iInitiated,
							pLocaleLanguage,
							ppMonth,
							pProgramName,
							pProgramVersion);

						if (iInitiated < 0)
							return iInitiated;
					}
				}
				if(!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_LOSS_GAINS)) {
					if (bAccountTypes[11]) {
						AMOUNT	Forward = 0;
						if (pReport->pLossGains) {
							Forward = pReport->pLossGains->Forward;
						}
						iInitiated = WriteXbrlGlBalanceReportLines(
							pStream,
							pContextRef,
							pAccounting,
							pLedger,
							pReport,
							pAccount,
							GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
							Forward,
							0,
							bCompactFileOption,
							bLeanChartsOption,
							bQuantityOption,
							pContent,
							BALANCE_ENDING,
							FinalDate,
							0,
							LastEntryDate,
							pGenUserName,
							GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
							GetEnumDepreciationPeriodsType(TXT_ANNUALLY, pLocaleLanguage),
							&iTabPos,
							iInitiated,
							pLocaleLanguage,
							ppMonth,
							pProgramName,
							pProgramVersion);

						if (iInitiated < 0)
							return iInitiated;
					}
				}
				if (!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_FINANCIAL)) {
					if (bAccountTypes[12]) {
						AMOUNT	Forward = 0;
						if (pReport->pFinancial) {
							Forward = pReport->pFinancial->Forward;
						}
						iInitiated = WriteXbrlGlBalanceReportLines(
							pStream,
							pContextRef,
							pAccounting,
							pLedger,
							pReport,
							pAccount,
							GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
							Forward,
							0,
							bCompactFileOption,
							bLeanChartsOption,
							bQuantityOption,
							pContent,
							BALANCE_ENDING,
							FinalDate,
							0,
							LastEntryDate,
							pGenUserName,
							GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
							GetEnumDepreciationPeriodsType(TXT_ANNUALLY, pLocaleLanguage),
							&iTabPos,
							iInitiated,
							pLocaleLanguage,
							ppMonth,
							pProgramName,
							pProgramVersion);

						if (iInitiated < 0)
							return iInitiated;
					}
					if ((bAccountTypes[7] || bAccountTypes[9]) && ((bAccountTypes[11] || bAccountTypes[12]))) {
						Forward = 0;
						if (pReport->pOutcome) {
							Forward = pReport->pOutcome->Forward;
						}
						LabelAccount.pType = NulString;
						iInitiated = WriteXbrlGlBalanceReportLines(
							pStream,
							pContextRef,
							pAccounting,
							pLedger,
							pReport,
							pAccount,
							GetEnumAccountTitleTypeText(ACCOUNT_TYPE_OUTCOME, pLocaleLanguage),
							0-Forward,
							0,
							bCompactFileOption,
							bLeanChartsOption,
							bQuantityOption,
							pContent,
							BALANCE_ENDING,
							FinalDate,
							0,
							LastEntryDate,
							pGenUserName,
							GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
							GetEnumDepreciationPeriodsType(TXT_ANNUALLY, pLocaleLanguage),
							&iTabPos,
							iInitiated,
							pLocaleLanguage,
							ppMonth,
							pProgramName,
							pProgramVersion);

						if (iInitiated < 0)
							return iInitiated;
					}
				}
				if (!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_AUX)) {
					if (bAccountTypes[13]) {
						AMOUNT	Forward = 0;
						if (pReport->pAuxiliary) {
							Forward = pReport->pAuxiliary->Forward;
						}
						iInitiated = WriteXbrlGlBalanceReportLines(
							pStream,
							pContextRef,
							pAccounting,
							pLedger,
							pReport,
							pAccount,
							GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
							Forward,
							0,
							bCompactFileOption,
							bLeanChartsOption,
							bQuantityOption,
							pContent,
							BALANCE_ENDING,
							FinalDate,
							0,
							LastEntryDate,
							pGenUserName,
							GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
							GetEnumDepreciationPeriodsType(TXT_ANNUALLY, pLocaleLanguage),
							&iTabPos,
							iInitiated,
							pLocaleLanguage,
							ppMonth,
							pProgramName,
							pProgramVersion);

						if (iInitiated < 0)
							return iInitiated;
					}
					if (bAccountTypes[7]
						&& (bAccountTypes[9] || bAccountTypes[11] || bAccountTypes[12] || bAccountTypes[13])) {
						Forward = 0;
						if (pReport->pResult) {
							Forward = pReport->pResult->Forward;
						}
						LabelAccount.pType = NulString;
						iInitiated = WriteXbrlGlBalanceReportLines(
							pStream,
							pContextRef,
							pAccounting,
							pLedger,
							pReport,
							pAccount,
							GetEnumAccountTitleTypeText(ACCOUNT_TYPE_RESULT, pLocaleLanguage),
							0-Forward,
							0,
							bCompactFileOption,
							bLeanChartsOption,
							bQuantityOption,
							pContent,
							BALANCE_ENDING,
							FinalDate,
							0,
							LastEntryDate,
							pGenUserName,
							GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
							GetEnumDepreciationPeriodsType(TXT_ANNUALLY, pLocaleLanguage),
							&iTabPos,
							iInitiated,
							pLocaleLanguage,
							ppMonth,
							pProgramName,
							pProgramVersion);

						if (iInitiated < 0)
							return iInitiated;

						if (((bAccountTypes[0] || bAccountTypes[1] || bAccountTypes[2] || bAccountTypes[3] || bAccountTypes[4] || bAccountTypes[5] || bAccountTypes[6]))
							&& (bAccountTypes[7] || bAccountTypes[9] || bAccountTypes[11] || bAccountTypes[12] || bAccountTypes[13])) {
							Forward = 0;
							if (pReport->pTotal) {
								Forward = pReport->pTotal->Forward;
							}
							LabelAccount.pType = NulString;
							iInitiated = WriteXbrlGlBalanceReportLines(
								pStream,
								pContextRef,
								pAccounting,
								pLedger,
								pReport,
								pAccount,
								GetEnumAccountTitleTypeText(ACCOUNT_TYPE_TOTAL, pLocaleLanguage),
								0-Forward,
								0,
								bCompactFileOption,
								bLeanChartsOption,
								bQuantityOption,
								pContent,
								BALANCE_ENDING,
								FinalDate,
								0,
								LastEntryDate,
								pGenUserName,
								GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
								GetEnumDepreciationPeriodsType(TXT_ANNUALLY, pLocaleLanguage),
								&iTabPos,
								iInitiated,
								pLocaleLanguage,
								ppMonth,
								pProgramName,
								pProgramVersion);

							if (iInitiated < 0)
								return iInitiated;
						}
					}
				}
			}
		}
		if (iInitiated == 2) {
			xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:entryHeader");
			iInitiated = 1;
		}
	}

	if (bPeriodOption) {
		int i = 0;
		for (i = 0; i < ENUM_ACCOUNTTYPETYPE; i++)
			bAccountTypes[i] = FALSE;
		{ // Period
			BOOL bFirst = TRUE;
			for (i = 0; i < ENUM_ACCOUNTTYPETYPE; i++) {
				AccountType* pAccount
					= LListGetFirstRecordData(pAccounting->pAccountsChart->pListAccount);
				while (pAccount) {
					if ((!strcmp((EnumAccountTypeType + i)->EnumerationID, pAccount->pType))
						&& ((!bLeanChartsOption) || (!pAccount->InActive))) {
						BalanceAccountType* pAccountStatement = GetBalanceAccount(pReport, pAccount);
						AMOUNT	Outcome = 0;
						QUANT	OutcomeQuantity = 0;
						DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));

						if (pAccountStatement && LListGetFirstRecordData(pAccountStatement->pListMonth)) {
							while (BalanceDate <= pLedger->YearEnd) {
								// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
								BalanceAccountPeriodType* pMonth = GetBalanceAccountPeriod(pAccountStatement, BalanceDate, 0);
								if (pMonth) {
									// Update with the report data
									Outcome = pMonth->Outcome;
									OutcomeQuantity = pMonth->OutcomeQuantity;
									bAccountTypes[i] = TRUE;
								}
								if (pMonth
									|| bFirst // First account to make the time frame, or first date to server account info
									|| (BalanceDate == GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart)))) {
									iInitiated = WriteXbrlGlBalanceReportLines(
										pStream,
										pContextRef,
										pAccounting,
										pLedger,
										pReport,
										pAccountStatement->pAccount,
										GetEnumAccountTitleTypeText(pAccountStatement->pAccount->pType, pLocaleLanguage),
										Outcome,
										OutcomeQuantity,
										bCompactFileOption,
										FALSE,
										bQuantityOption,
										pContent,
										PERIOD_CHANGE,
										FinalDate,
										BalanceDate,
										LastEntryDate,
										pGenUserName,
										GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
										GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
										&iTabPos,
										iInitiated,
										pLocaleLanguage,
										ppMonth,
										pProgramName,
										pProgramVersion);
									if (iInitiated < 0)
										return iInitiated;
								}
								// To generate a Ledger account representation for HTML/XSL scripts to relate to, for the first account 
								Outcome = 0;
								OutcomeQuantity = 0;
								BalanceDate = GetNextMonthLastDate(BalanceDate);
							}
							bFirst = FALSE;
						}
					}
					pAccount = LListGetNextRecordData(pAccounting->pAccountsChart->pListAccount);
				}
				{
					AccountType LabelAccount;
					AccountType *pAccount = &LabelAccount; // For BALANCE_ACCOUNTS
					memset(&LabelAccount, 0, sizeof(AccountType));
					LabelAccount.pType = (EnumAccountTypeType + i)->EnumerationID;

					if (!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_ASSETS)) {
						if (bAccountTypes[0] || bAccountTypes[1] || bAccountTypes[2] || bAccountTypes[3]) {
							AMOUNT	Outcome = 0;
							DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
							while (BalanceDate <= pLedger->YearEnd) {
								// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
								BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pAssets->pListMonth, BalanceDate);
								if (pMonth) {
									// Update with the report data
									Outcome = pMonth->Outcome;
								}
								if (((FinalDate / 100) >= (BalanceDate / 100)) && (Outcome
									|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
									iInitiated = WriteXbrlGlBalanceReportLines(
										pStream,
										pContextRef,
										pAccounting,
										pLedger,
										pReport,
										pAccount,
										GetEnumAccountTitleTypeText(ACCOUNT_TYPE_ACTIVE, pLocaleLanguage),
										Outcome,
										0,
										bCompactFileOption,
										bLeanChartsOption,
										bQuantityOption,
										pContent,
										PERIOD_CHANGE,
										FinalDate,
										BalanceDate,
										LastEntryDate,
										pGenUserName,
										GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
										GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
										&iTabPos,
										iInitiated,
										pLocaleLanguage,
										ppMonth,
										pProgramName,
										pProgramVersion);

									if (iInitiated < 0)
										return iInitiated;
								}
								Outcome = 0;
								BalanceDate = GetNextMonthLastDate(BalanceDate);
							}
						}
					}
					else if (!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_EQUITIES)) {
						if (bAccountTypes[4]) {
							AMOUNT	Outcome = 0;
							DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
							while (BalanceDate <= pLedger->YearEnd) {
								// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
								BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pEquities->pListMonth, BalanceDate);
								if (pMonth) {
									// Update with the report data
									Outcome = pMonth->Outcome;
								}
								if (((FinalDate / 100) >= (BalanceDate / 100)) && (Outcome
									|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
									iInitiated = WriteXbrlGlBalanceReportLines(
										pStream,
										pContextRef,
										pAccounting,
										pLedger,
										pReport,
										pAccount,
										GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
										Outcome,
										0,
										bCompactFileOption,
										bLeanChartsOption,
										bQuantityOption,
										pContent,
										PERIOD_CHANGE,
										FinalDate,
										BalanceDate,
										LastEntryDate,
										pGenUserName,
										GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
										GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
										&iTabPos,
										iInitiated,
										pLocaleLanguage,
										ppMonth,
										pProgramName,
										pProgramVersion);

									if (iInitiated < 0)
										return iInitiated;
								}
								Outcome = 0;
								BalanceDate = GetNextMonthLastDate(BalanceDate);
							}
						}
					}
					else if (!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_LIABILITIES)) {
						if (bAccountTypes[6]) {
							AMOUNT	Outcome = 0;
							DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
							while (BalanceDate <= pLedger->YearEnd) {
								// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
								BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pLiabilities->pListMonth, BalanceDate);
								if (pMonth) {
									// Update with the report data
									Outcome = pMonth->Outcome;
								}
								if (((FinalDate / 100) >= (BalanceDate / 100)) && (Outcome
									|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
									iInitiated = WriteXbrlGlBalanceReportLines(
										pStream,
										pContextRef,
										pAccounting,
										pLedger,
										pReport,
										pAccount,
										GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
										Outcome,
										0,
										bCompactFileOption,
										bLeanChartsOption,
										bQuantityOption,
										pContent,
										PERIOD_CHANGE,
										FinalDate,
										BalanceDate,
										LastEntryDate,
										pGenUserName,
										GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
										GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
										&iTabPos,
										iInitiated,
										pLocaleLanguage,
										ppMonth,
										pProgramName,
										pProgramVersion);

									if (iInitiated < 0)
										return iInitiated;
								}
								Outcome = 0;
								BalanceDate = GetNextMonthLastDate(BalanceDate);
							}
						}
						if ((bAccountTypes[4] && (bAccountTypes[5] || bAccountTypes[6]))
							|| (bAccountTypes[5] && (bAccountTypes[4] || bAccountTypes[6]))
							|| (bAccountTypes[6] && (bAccountTypes[5] || bAccountTypes[4]))) {
							LabelAccount.pType = NulString;
							{
								AMOUNT	Outcome = 0;
								DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
								while (BalanceDate <= pLedger->YearEnd) {
									// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
									BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pPassive->pListMonth, BalanceDate);
									if (pMonth) {
										// Update with the report data
										Outcome = pMonth->Outcome;
									}
									if (((FinalDate / 100) >= (BalanceDate / 100)) && (Outcome
										|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
										iInitiated = WriteXbrlGlBalanceReportLines(
											pStream,
											pContextRef,
											pAccounting,
											pLedger,
											pReport,
											pAccount,
											GetEnumAccountTitleTypeText(ACCOUNT_TYPE_PASSIVE, pLocaleLanguage),
											Outcome,
											0,
											bCompactFileOption,
											bLeanChartsOption,
											bQuantityOption,
											pContent,
											PERIOD_CHANGE,
											FinalDate,
											BalanceDate,
											LastEntryDate,
											pGenUserName,
											GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
											GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
											&iTabPos,
											iInitiated,
											pLocaleLanguage,
											ppMonth,
											pProgramName,
											pProgramVersion);

										if (iInitiated < 0)
											return iInitiated;
									}
									Outcome = 0;
									BalanceDate = GetNextMonthLastDate(BalanceDate);
								}
							}
						}
						if ((bAccountTypes[0] || bAccountTypes[1] || bAccountTypes[2] || bAccountTypes[3])
							&& (bAccountTypes[4] || bAccountTypes[5] || bAccountTypes[6])) {
							LabelAccount.pType = NulString;
							{
								AMOUNT	Outcome = 0;
								DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
								while (BalanceDate <= pLedger->YearEnd) {
									// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
									BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pBalance->pListMonth, BalanceDate);
									if (pMonth) {
										// Update with the report data
										Outcome = pMonth->Outcome;
									}
									if (((FinalDate / 100) >= (BalanceDate / 100)) && (Outcome
										|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
										iInitiated = WriteXbrlGlBalanceReportLines(
											pStream,
											pContextRef,
											pAccounting,
											pLedger,
											pReport,
											pAccount,
											GetEnumAccountTitleTypeText(ACCOUNT_TYPE_BALANCE, pLocaleLanguage),
											Outcome,
											0,
											bCompactFileOption,
											bLeanChartsOption,
											bQuantityOption,
											pContent,
											PERIOD_CHANGE,
											FinalDate,
											BalanceDate,
											LastEntryDate,
											pGenUserName,
											GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
											GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
											&iTabPos,
											iInitiated,
											pLocaleLanguage,
											ppMonth,
											pProgramName,
											pProgramVersion);

										if (iInitiated < 0)
											return iInitiated;
									}
									Outcome = 0;
									BalanceDate = GetNextMonthLastDate(BalanceDate);
								}
							}
						}
					}
					else if (!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_REVENUES)) {
						if (bAccountTypes[7]) {
							AMOUNT	Outcome = 0;
							DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
							while (BalanceDate <= pLedger->YearEnd) {
								// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
								BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pRevenues->pListMonth, BalanceDate);
								if (pMonth) {
									// Update with the report data
									Outcome = pMonth->Outcome;
								}
								if (((FinalDate / 100) >= (BalanceDate / 100)) && (Outcome
									|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
									iInitiated = WriteXbrlGlBalanceReportLines(
										pStream,
										pContextRef,
										pAccounting,
										pLedger,
										pReport,
										pAccount,
										GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
										Outcome,
										0,
										bCompactFileOption,
										bLeanChartsOption,
										bQuantityOption,
										pContent,
										PERIOD_CHANGE,
										FinalDate,
										BalanceDate,
										LastEntryDate,
										pGenUserName,
										GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
										GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
										&iTabPos,
										iInitiated,
										pLocaleLanguage,
										ppMonth,
										pProgramName,
										pProgramVersion);

									if (iInitiated < 0)
										return iInitiated;
								}
								Outcome = 0;
								BalanceDate = GetNextMonthLastDate(BalanceDate);
							}
						}
					}
					if (!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_EXPENSES)) {
						if (bAccountTypes[9]) {
							AMOUNT	Outcome = 0;
							DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
							while (BalanceDate <= pLedger->YearEnd) {
								// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
								BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pExpenses->pListMonth, BalanceDate);
								if (pMonth) {
									// Update with the report data
									Outcome = pMonth->Outcome;
								}
								if (((FinalDate / 100) >= (BalanceDate / 100)) && (Outcome
									|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
									iInitiated = WriteXbrlGlBalanceReportLines(
										pStream,
										pContextRef,
										pAccounting,
										pLedger,
										pReport,
										pAccount,
										GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
										Outcome,
										0,
										bCompactFileOption,
										bLeanChartsOption,
										bQuantityOption,
										pContent,
										PERIOD_CHANGE,
										FinalDate,
										BalanceDate,
										LastEntryDate,
										pGenUserName,
										GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
										GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
										&iTabPos,
										iInitiated,
										pLocaleLanguage,
										ppMonth,
										pProgramName,
										pProgramVersion);

									if (iInitiated < 0)
										return iInitiated;
								}
								Outcome = 0;
								BalanceDate = GetNextMonthLastDate(BalanceDate);
							}
						}
						if (bAccountTypes[7] && bAccountTypes[9]) {
							LabelAccount.pType = NulString;
							{
								AMOUNT	Outcome = 0;
								DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
								while (BalanceDate <= pLedger->YearEnd) {
									// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
									BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pEarning->pListMonth, BalanceDate);
									if (pMonth) {
										// Update with the report data
										Outcome = pMonth->Outcome;
									}
									if (((FinalDate / 100) >= (BalanceDate / 100)) && (Outcome
										|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
										iInitiated = WriteXbrlGlBalanceReportLines(
											pStream,
											pContextRef,
											pAccounting,
											pLedger,
											pReport,
											pAccount,
											GetEnumAccountTitleTypeText(ACCOUNT_TYPE_EARNING, pLocaleLanguage),
											0-Outcome,
											0,
											bCompactFileOption,
											bLeanChartsOption,
											bQuantityOption,
											pContent,
											PERIOD_CHANGE,
											FinalDate,
											BalanceDate,
											LastEntryDate,
											pGenUserName,
											GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
											GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
											&iTabPos,
											iInitiated,
											pLocaleLanguage,
											ppMonth,
											pProgramName,
											pProgramVersion);

										if (iInitiated < 0)
											return iInitiated;
									}
									Outcome = 0;
									BalanceDate = GetNextMonthLastDate(BalanceDate);
								}
							}
						}
					}
					else if (!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_LOSS_GAINS)) {
						if (bAccountTypes[11]) {
							AMOUNT	Outcome = 0;
							DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
							while (BalanceDate <= pLedger->YearEnd) {
								// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
								BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pLossGains->pListMonth, BalanceDate);
								if (pMonth) {
									// Update with the report data
									Outcome = pMonth->Outcome;
								}
								if (((FinalDate / 100) >= (BalanceDate / 100)) && (Outcome
									|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
									iInitiated = WriteXbrlGlBalanceReportLines(
										pStream,
										pContextRef,
										pAccounting,
										pLedger,
										pReport,
										pAccount,
										GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
										Outcome,
										0,
										bCompactFileOption,
										bLeanChartsOption,
										bQuantityOption,
										pContent,
										PERIOD_CHANGE,
										FinalDate,
										BalanceDate,
										LastEntryDate,
										pGenUserName,
										GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
										GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
										&iTabPos,
										iInitiated,
										pLocaleLanguage,
										ppMonth,
										pProgramName,
										pProgramVersion);

									if (iInitiated < 0)
										return iInitiated;
								}
								Outcome = 0;
								BalanceDate = GetNextMonthLastDate(BalanceDate);
							}
						}
					}
					else if (!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_FINANCIAL)) {
						if (bAccountTypes[12]) {
							AMOUNT	Outcome = 0;
							DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
							while (BalanceDate <= pLedger->YearEnd) {
								// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
								BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pFinancial->pListMonth, BalanceDate);
								if (pMonth) {
									// Update with the report data
									Outcome = pMonth->Outcome;
								}
								if (((FinalDate / 100) >= (BalanceDate / 100)) && (Outcome
									|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
									iInitiated = WriteXbrlGlBalanceReportLines(
										pStream,
										pContextRef,
										pAccounting,
										pLedger,
										pReport,
										pAccount,
										GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
										Outcome,
										0,
										bCompactFileOption,
										bLeanChartsOption,
										bQuantityOption,
										pContent,
										PERIOD_CHANGE,
										FinalDate,
										BalanceDate,
										LastEntryDate,
										pGenUserName,
										GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
										GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
										&iTabPos,
										iInitiated,
										pLocaleLanguage,
										ppMonth,
										pProgramName,
										pProgramVersion);

									if (iInitiated < 0)
										return iInitiated;
								}
								Outcome = 0;
								BalanceDate = GetNextMonthLastDate(BalanceDate);
							}
						}
						if ((bAccountTypes[7] || bAccountTypes[9]) && ((bAccountTypes[11] || bAccountTypes[12]))) {
							LabelAccount.pType = NulString;
							{
								AMOUNT	Outcome = 0;
								DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
								while (BalanceDate <= pLedger->YearEnd) {
									// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
									BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pOutcome->pListMonth, BalanceDate);
									if (pMonth) {
										// Update with the report data
										Outcome = pMonth->Outcome;
									}
									if (((FinalDate / 100) >= (BalanceDate / 100)) && (Outcome
										|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
										iInitiated = WriteXbrlGlBalanceReportLines(
											pStream,
											pContextRef,
											pAccounting,
											pLedger,
											pReport,
											pAccount,
											GetEnumAccountTitleTypeText(ACCOUNT_TYPE_OUTCOME, pLocaleLanguage),
											0-Outcome,
											0,
											bCompactFileOption,
											bLeanChartsOption,
											bQuantityOption,
											pContent,
											PERIOD_CHANGE,
											FinalDate,
											BalanceDate,
											LastEntryDate,
											pGenUserName,
											GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
											GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
											&iTabPos,
											iInitiated,
											pLocaleLanguage,
											ppMonth,
											pProgramName,
											pProgramVersion);

										if (iInitiated < 0)
											return iInitiated;
									}
									Outcome = 0;
									BalanceDate = GetNextMonthLastDate(BalanceDate);
								}
							}
						}
					}
					if (!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_AUX)) {
						if (bAccountTypes[13]) {
							AMOUNT	Outcome = 0;
							DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
							while (BalanceDate <= pLedger->YearEnd) {
								// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
								BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pAuxiliary->pListMonth, BalanceDate);
								if (pMonth) {
									// Update with the report data
									Outcome = pMonth->Outcome;
								}
								if (((FinalDate / 100) >= (BalanceDate / 100)) && (Outcome
									|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
									iInitiated = WriteXbrlGlBalanceReportLines(
										pStream,
										pContextRef,
										pAccounting,
										pLedger,
										pReport,
										pAccount,
										GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
										Outcome,
										0,
										bCompactFileOption,
										bLeanChartsOption,
										bQuantityOption,
										pContent,
										PERIOD_CHANGE,
										FinalDate,
										BalanceDate,
										LastEntryDate,
										pGenUserName,
										GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
										GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
										&iTabPos,
										iInitiated,
										pLocaleLanguage,
										ppMonth,
										pProgramName,
										pProgramVersion);

									if (iInitiated < 0)
										return iInitiated;
								}
								Outcome = 0;
								BalanceDate = GetNextMonthLastDate(BalanceDate);
							}
						}
						if (bAccountTypes[7] && (bAccountTypes[9] || bAccountTypes[11] || bAccountTypes[12] || bAccountTypes[13])) {
							LabelAccount.pType = NulString;
							{
								AMOUNT	Outcome = 0;
								DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
								while (BalanceDate <= pLedger->YearEnd) {
									// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
									BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pResult->pListMonth, BalanceDate);
									if (pMonth) {
										// Update with the report data
										Outcome = pMonth->Outcome;
									}
									if (((FinalDate / 100) >= (BalanceDate / 100)) && (Outcome
										|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
										iInitiated = WriteXbrlGlBalanceReportLines(
											pStream,
											pContextRef,
											pAccounting,
											pLedger,
											pReport,
											pAccount,
											GetEnumAccountTitleTypeText(ACCOUNT_TYPE_RESULT, pLocaleLanguage),
											0-Outcome,
											0,
											bCompactFileOption,
											bLeanChartsOption,
											bQuantityOption,
											pContent,
											PERIOD_CHANGE,
											FinalDate,
											BalanceDate,
											LastEntryDate,
											pGenUserName,
											GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
											GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
											&iTabPos,
											iInitiated,
											pLocaleLanguage,
											ppMonth,
											pProgramName,
											pProgramVersion);

										if (iInitiated < 0)
											return iInitiated;
									}
									Outcome = 0;
									BalanceDate = GetNextMonthLastDate(BalanceDate);
								}
							}
						}
						if (((bAccountTypes[0] || bAccountTypes[1] || bAccountTypes[2] || bAccountTypes[3] || bAccountTypes[4] || bAccountTypes[5] || bAccountTypes[6]))
							&& (bAccountTypes[7] || bAccountTypes[9] || bAccountTypes[11] || bAccountTypes[12] || bAccountTypes[13])) {
							LabelAccount.pType = NulString;
							{
								AMOUNT	Outcome = 0;
								DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
								while (BalanceDate <= pLedger->YearEnd) {
									// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
									BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pTotal->pListMonth, BalanceDate);
									if (pMonth) {
										// Update with the report data
										Outcome = pMonth->Outcome;
									}
									if (((FinalDate / 100) >= (BalanceDate / 100)) && (Outcome
										|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
										iInitiated = WriteXbrlGlBalanceReportLines(
											pStream,
											pContextRef,
											pAccounting,
											pLedger,
											pReport,
											pAccount,
											GetEnumAccountTitleTypeText(ACCOUNT_TYPE_TOTAL, pLocaleLanguage),
											0-Outcome,
											0,
											bCompactFileOption,
											bLeanChartsOption,
											bQuantityOption,
											pContent,
											PERIOD_CHANGE,
											FinalDate,
											BalanceDate,
											LastEntryDate,
											pGenUserName,
											GetEnumFolderTypeText(CONTENT_LABEL_BALANCE, pLocaleLanguage),
											GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
											&iTabPos,
											iInitiated,
											pLocaleLanguage,
											ppMonth,
											pProgramName,
											pProgramVersion);

										if (iInitiated < 0)
											return iInitiated;
									}
									Outcome = 0;
									BalanceDate = GetNextMonthLastDate(BalanceDate);
								}
							}
						}
					}
				}
			}
			if (iInitiated == 2) {
				xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:entryHeader");
				iInitiated = 1;
			}
		}
	if (bBudgetOption) { 
		BOOL bFirst = TRUE;
		for (i = 0; i < ENUM_ACCOUNTTYPETYPE; i++) {
			AccountType* pAccount
				= LListGetFirstRecordData(pAccounting->pAccountsChart->pListAccount);
			while (pAccount) {
				if ((!strcmp((EnumAccountTypeType + i)->EnumerationID, pAccount->pType))
					&& ((!bLeanChartsOption) || (!pAccount->InActive))) {
					BalanceAccountType* pAccountStatement = GetBalanceAccount(pReport, pAccount);
					AMOUNT	Budget = 0;
					QUANT	BudgetQuantity = 0;
					DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));

					if (pAccountStatement && LListGetFirstRecordData(pAccountStatement->pListMonth)) {
						while (BalanceDate <= pLedger->YearEnd) {
							// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
							BalanceAccountPeriodType* pMonth = GetBalanceAccountPeriod(pAccountStatement, BalanceDate, 0);
							if (pMonth) {
								// Update with the report data
								Budget = pMonth->Budget;
								BudgetQuantity = pMonth->BudgetQuantity;
								bAccountTypes[i] = TRUE;
							}
							if (pMonth
								|| bFirst // First account to make the time frame, or first date to server account info
								|| (BalanceDate == GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart)))) {
								iInitiated = WriteXbrlGlBalanceReportLines(
									pStream,
									pContextRef,
									pAccounting,
									pLedger,
									pReport,
									pAccountStatement->pAccount,
									GetEnumAccountTitleTypeText(pAccountStatement->pAccount->pType, pLocaleLanguage),
									Budget,
									BudgetQuantity,
									bCompactFileOption,
									FALSE,
									bQuantityOption,
									pContent,
									BUDGET,
									FinalDate,
									BalanceDate,
									LastEntryDate,
									pGenUserName,
									GetEnumApplicationTextType(TXT_BUDGET, pLocaleLanguage),
									GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
									&iTabPos,
									iInitiated,
									pLocaleLanguage,
									ppMonth,
									pProgramName,
									pProgramVersion);
								if (iInitiated < 0)
									return iInitiated;
							}
							// To generate a Ledger account representation for HTML/XSL scripts to relate to, for the first account 
							Budget = 0;
							BudgetQuantity = 0;
							BalanceDate = GetNextMonthLastDate(BalanceDate);
						}
						bFirst = FALSE;
					}
				}
				pAccount = LListGetNextRecordData(pAccounting->pAccountsChart->pListAccount);
			}
			{
				AccountType LabelAccount;
				AccountType* pAccount = &LabelAccount; // For BALANCE_ACCOUNTS
				memset(&LabelAccount, 0, sizeof(AccountType));
				LabelAccount.pType = (EnumAccountTypeType + i)->EnumerationID;

				if (!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_ASSETS)) {
					if (bAccountTypes[0] || bAccountTypes[1] || bAccountTypes[2] || bAccountTypes[3]) {
						AMOUNT	Budget = 0;
						DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
						while (BalanceDate <= pLedger->YearEnd) {
							// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
							BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pAssets->pListMonth, BalanceDate);
							if (pMonth) {
								// Update with the report data
								Budget = pMonth->Budget;
							}
							if (((FinalDate / 100) >= (BalanceDate / 100)) && (Budget
								|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
								iInitiated = WriteXbrlGlBalanceReportLines(
									pStream,
									pContextRef,
									pAccounting,
									pLedger,
									pReport,
									pAccount,
									GetEnumAccountTitleTypeText(ACCOUNT_TYPE_ACTIVE, pLocaleLanguage),
									Budget,
									0,
									bCompactFileOption,
									bLeanChartsOption,
									bQuantityOption,
									pContent,
									BUDGET,
									FinalDate,
									BalanceDate,
									LastEntryDate,
									pGenUserName,
									GetEnumApplicationTextType(TXT_BUDGET, pLocaleLanguage),
									GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
									&iTabPos,
									iInitiated,
									pLocaleLanguage,
									ppMonth,
									pProgramName,
									pProgramVersion);

								if (iInitiated < 0)
									return iInitiated;
							}
							Budget = 0;
							BalanceDate = GetNextMonthLastDate(BalanceDate);
						}
					}
				}
				else if (!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_EQUITIES)) {
					if (bAccountTypes[4]) {
						AMOUNT	Budget = 0;
						DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
						while (BalanceDate <= pLedger->YearEnd) {
							// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
							BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pEquities->pListMonth, BalanceDate);
							if (pMonth) {
								// Update with the report data
								Budget = pMonth->Budget;
							}
							if (((FinalDate / 100) >= (BalanceDate / 100)) && (Budget
								|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
								iInitiated = WriteXbrlGlBalanceReportLines(
									pStream,
									pContextRef,
									pAccounting,
									pLedger,
									pReport,
									pAccount,
									GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
									Budget,
									0,
									bCompactFileOption,
									bLeanChartsOption,
									bQuantityOption,
									pContent,
									BUDGET,
									FinalDate,
									BalanceDate,
									LastEntryDate,
									pGenUserName,
									GetEnumApplicationTextType(TXT_BUDGET, pLocaleLanguage),
									GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
									&iTabPos,
									iInitiated,
									pLocaleLanguage,
									ppMonth,
									pProgramName,
									pProgramVersion);

								if (iInitiated < 0)
									return iInitiated;
							}
							Budget = 0;
							BalanceDate = GetNextMonthLastDate(BalanceDate);
						}
					}
				}
				else if (!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_LIABILITIES)) {
					if (bAccountTypes[6]) {
						AMOUNT	Budget = 0;
						DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
						while (BalanceDate <= pLedger->YearEnd) {
							// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
							BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pLiabilities->pListMonth, BalanceDate);
							if (pMonth) {
								// Update with the report data
								Budget = pMonth->Budget;
							}
							if (((FinalDate / 100) >= (BalanceDate / 100)) && (Budget
								|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
								iInitiated = WriteXbrlGlBalanceReportLines(
									pStream,
									pContextRef,
									pAccounting,
									pLedger,
									pReport,
									pAccount,
									GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
									Budget,
									0,
									bCompactFileOption,
									bLeanChartsOption,
									bQuantityOption,
									pContent,
									BUDGET,
									FinalDate,
									BalanceDate,
									LastEntryDate,
									pGenUserName,
									GetEnumApplicationTextType(TXT_BUDGET, pLocaleLanguage),
									GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
									&iTabPos,
									iInitiated,
									pLocaleLanguage,
									ppMonth,
									pProgramName,
									pProgramVersion);

								if (iInitiated < 0)
									return iInitiated;
							}
							Budget = 0;
							BalanceDate = GetNextMonthLastDate(BalanceDate);
						}
					}
					if ((bAccountTypes[4] && (bAccountTypes[5] || bAccountTypes[6]))
						|| (bAccountTypes[5] && (bAccountTypes[4] || bAccountTypes[6]))
						|| (bAccountTypes[6] && (bAccountTypes[5] || bAccountTypes[4]))) {
						LabelAccount.pType = NulString;
						{
							AMOUNT	Budget = 0;
							DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
							while (BalanceDate <= pLedger->YearEnd) {
								// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
								BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pPassive->pListMonth, BalanceDate);
								if (pMonth) {
									// Update with the report data
									Budget = pMonth->Budget;
								}
								if (((FinalDate / 100) >= (BalanceDate / 100)) && (Budget
									|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
									iInitiated = WriteXbrlGlBalanceReportLines(
										pStream,
										pContextRef,
										pAccounting,
										pLedger,
										pReport,
										pAccount,
										GetEnumAccountTitleTypeText(ACCOUNT_TYPE_PASSIVE, pLocaleLanguage),
										Budget,
										0,
										bCompactFileOption,
										bLeanChartsOption,
										bQuantityOption,
										pContent,
										BUDGET,
										FinalDate,
										BalanceDate,
										LastEntryDate,
										pGenUserName,
										GetEnumApplicationTextType(TXT_BUDGET, pLocaleLanguage),
										GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
										&iTabPos,
										iInitiated,
										pLocaleLanguage,
										ppMonth,
										pProgramName,
										pProgramVersion);

									if (iInitiated < 0)
										return iInitiated;
								}
								Budget = 0;
								BalanceDate = GetNextMonthLastDate(BalanceDate);
							}
						}
					}
					if ((bAccountTypes[0] || bAccountTypes[1] || bAccountTypes[2] || bAccountTypes[3])
						&& (bAccountTypes[4] || bAccountTypes[5] || bAccountTypes[6])) {
						LabelAccount.pType = NulString;
						{
							AMOUNT	Budget = 0;
							DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
							while (BalanceDate <= pLedger->YearEnd) {
								// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
								BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pBalance->pListMonth, BalanceDate);
								if (pMonth) {
									// Update with the report data
									Budget = pMonth->Budget;
								}
								if (((FinalDate / 100) >= (BalanceDate / 100)) && (Budget
									|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
									iInitiated = WriteXbrlGlBalanceReportLines(
										pStream,
										pContextRef,
										pAccounting,
										pLedger,
										pReport,
										pAccount,
										GetEnumAccountTitleTypeText(ACCOUNT_TYPE_BALANCE, pLocaleLanguage),
										Budget,
										0,
										bCompactFileOption,
										bLeanChartsOption,
										bQuantityOption,
										pContent,
										BUDGET,
										FinalDate,
										BalanceDate,
										LastEntryDate,
										pGenUserName,
										GetEnumApplicationTextType(TXT_BUDGET, pLocaleLanguage),
										GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
										&iTabPos,
										iInitiated,
										pLocaleLanguage,
										ppMonth,
										pProgramName,
										pProgramVersion);

									if (iInitiated < 0)
										return iInitiated;
								}
								Budget = 0;
								BalanceDate = GetNextMonthLastDate(BalanceDate);
							}
						}
					}
				}
				else if (!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_REVENUES)) {
					if (bAccountTypes[7]) {
						AMOUNT	Budget = 0;
						DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
						while (BalanceDate <= pLedger->YearEnd) {
							// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
							BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pRevenues->pListMonth, BalanceDate);
							if (pMonth) {
								// Update with the report data
								Budget = pMonth->Budget;
							}
							if (((FinalDate / 100) >= (BalanceDate / 100)) && (Budget
								|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
								iInitiated = WriteXbrlGlBalanceReportLines(
									pStream,
									pContextRef,
									pAccounting,
									pLedger,
									pReport,
									pAccount,
									GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
									Budget,
									0,
									bCompactFileOption,
									bLeanChartsOption,
									bQuantityOption,
									pContent,
									BUDGET,
									FinalDate,
									BalanceDate,
									LastEntryDate,
									pGenUserName,
									GetEnumApplicationTextType(TXT_BUDGET, pLocaleLanguage),
									GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
									&iTabPos,
									iInitiated,
									pLocaleLanguage,
									ppMonth,
									pProgramName,
									pProgramVersion);

								if (iInitiated < 0)
									return iInitiated;
							}
							Budget = 0;
							BalanceDate = GetNextMonthLastDate(BalanceDate);
						}
					}
				}
				if (!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_EXPENSES)) {
					if (bAccountTypes[9]) {
						AMOUNT	Budget = 0;
						DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
						while (BalanceDate <= pLedger->YearEnd) {
							// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
							BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pExpenses->pListMonth, BalanceDate);
							if (pMonth) {
								// Update with the report data
								Budget = pMonth->Budget;
							}
							if (((FinalDate / 100) >= (BalanceDate / 100)) && (Budget
								|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
								iInitiated = WriteXbrlGlBalanceReportLines(
									pStream,
									pContextRef,
									pAccounting,
									pLedger,
									pReport,
									pAccount,
									GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
									Budget,
									0,
									bCompactFileOption,
									bLeanChartsOption,
									bQuantityOption,
									pContent,
									BUDGET,
									FinalDate,
									BalanceDate,
									LastEntryDate,
									pGenUserName,
									GetEnumApplicationTextType(TXT_BUDGET, pLocaleLanguage),
									GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
									&iTabPos,
									iInitiated,
									pLocaleLanguage,
									ppMonth,
									pProgramName,
									pProgramVersion);

								if (iInitiated < 0)
									return iInitiated;
							}
							Budget = 0;
							BalanceDate = GetNextMonthLastDate(BalanceDate);
						}
					}
					if (bAccountTypes[7] && bAccountTypes[9]) {
						LabelAccount.pType = NulString;
						{
							AMOUNT	Budget = 0;
							DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
							while (BalanceDate <= pLedger->YearEnd) {
								// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
								BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pEarning->pListMonth, BalanceDate);
								if (pMonth) {
									// Update with the report data
									Budget = pMonth->Budget;
								}
								if (((FinalDate / 100) >= (BalanceDate / 100)) && (Budget
									|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
									iInitiated = WriteXbrlGlBalanceReportLines(
										pStream,
										pContextRef,
										pAccounting,
										pLedger,
										pReport,
										pAccount,
										GetEnumAccountTitleTypeText(ACCOUNT_TYPE_EARNING, pLocaleLanguage),
										0-Budget,
										0,
										bCompactFileOption,
										bLeanChartsOption,
										bQuantityOption,
										pContent,
										BUDGET,
										FinalDate,
										BalanceDate,
										LastEntryDate,
										pGenUserName,
										GetEnumApplicationTextType(TXT_BUDGET, pLocaleLanguage),
										GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
										&iTabPos,
										iInitiated,
										pLocaleLanguage,
										ppMonth,
										pProgramName,
										pProgramVersion);

									if (iInitiated < 0)
										return iInitiated;
								}
								Budget = 0;
								BalanceDate = GetNextMonthLastDate(BalanceDate);
							}
						}
					}
				}
				else if (!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_LOSS_GAINS)) {
					if (bAccountTypes[11]) {
						AMOUNT	Budget = 0;
						DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
						while (BalanceDate <= pLedger->YearEnd) {
							// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
							BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pLossGains->pListMonth, BalanceDate);
							if (pMonth) {
								// Update with the report data
								Budget = pMonth->Budget;
							}
							if (((FinalDate / 100) >= (BalanceDate / 100)) && (Budget
								|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
								iInitiated = WriteXbrlGlBalanceReportLines(
									pStream,
									pContextRef,
									pAccounting,
									pLedger,
									pReport,
									pAccount,
									GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
									Budget,
									0,
									bCompactFileOption,
									bLeanChartsOption,
									bQuantityOption,
									pContent,
									BUDGET,
									FinalDate,
									BalanceDate,
									LastEntryDate,
									pGenUserName,
									GetEnumApplicationTextType(TXT_BUDGET, pLocaleLanguage),
									GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
									&iTabPos,
									iInitiated,
									pLocaleLanguage,
									ppMonth,
									pProgramName,
									pProgramVersion);

								if (iInitiated < 0)
									return iInitiated;
							}
							Budget = 0;
							BalanceDate = GetNextMonthLastDate(BalanceDate);
						}
					}
				}
				else if (!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_FINANCIAL)) {
					if (bAccountTypes[12]) {
						AMOUNT	Budget = 0;
						DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
						while (BalanceDate <= pLedger->YearEnd) {
							// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
							BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pFinancial->pListMonth, BalanceDate);
							if (pMonth) {
								// Update with the report data
								Budget = pMonth->Budget;
							}
							if (((FinalDate / 100) >= (BalanceDate / 100)) && (Budget
								|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
								iInitiated = WriteXbrlGlBalanceReportLines(
									pStream,
									pContextRef,
									pAccounting,
									pLedger,
									pReport,
									pAccount,
									GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
									Budget,
									0,
									bCompactFileOption,
									bLeanChartsOption,
									bQuantityOption,
									pContent,
									BUDGET,
									FinalDate,
									BalanceDate,
									LastEntryDate,
									pGenUserName,
									GetEnumApplicationTextType(TXT_BUDGET, pLocaleLanguage),
									GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
									&iTabPos,
									iInitiated,
									pLocaleLanguage,
									ppMonth,
									pProgramName,
									pProgramVersion);

								if (iInitiated < 0)
									return iInitiated;
							}
							Budget = 0;
							BalanceDate = GetNextMonthLastDate(BalanceDate);
						}
					}
					if ((bAccountTypes[7] || bAccountTypes[9]) && ((bAccountTypes[11] || bAccountTypes[12]))) {
						LabelAccount.pType = NulString;
						{
							AMOUNT	Budget = 0;
							DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
							while (BalanceDate <= pLedger->YearEnd) {
								// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
								BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pOutcome->pListMonth, BalanceDate);
								if (pMonth) {
									// Update with the report data
									Budget = pMonth->Budget;
								}
								if (((FinalDate / 100) >= (BalanceDate / 100)) && (Budget
									|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
									iInitiated = WriteXbrlGlBalanceReportLines(
										pStream,
										pContextRef,
										pAccounting,
										pLedger,
										pReport,
										pAccount,
										GetEnumAccountTitleTypeText(ACCOUNT_TYPE_OUTCOME, pLocaleLanguage),
										0-Budget,
										0,
										bCompactFileOption,
										bLeanChartsOption,
										bQuantityOption,
										pContent,
										BUDGET,
										FinalDate,
										BalanceDate,
										LastEntryDate,
										pGenUserName,
										GetEnumApplicationTextType(TXT_BUDGET, pLocaleLanguage),
										GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
										&iTabPos,
										iInitiated,
										pLocaleLanguage,
										ppMonth,
										pProgramName,
										pProgramVersion);

									if (iInitiated < 0)
										return iInitiated;
								}
								Budget = 0;
								BalanceDate = GetNextMonthLastDate(BalanceDate);
							}
						}
					}
				}
				if (!strcmp((EnumAccountTypeType + i)->EnumerationID, ACCOUNT_TYPE_AUX)) {
					if (bAccountTypes[13]) {
						AMOUNT	Budget = 0;
						DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
						while (BalanceDate <= pLedger->YearEnd) {
							// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
							BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pAuxiliary->pListMonth, BalanceDate);
							if (pMonth) {
								// Update with the report data
								Budget = pMonth->Budget;
							}
							if (((FinalDate / 100) >= (BalanceDate / 100)) && (Budget
								|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
								iInitiated = WriteXbrlGlBalanceReportLines(
									pStream,
									pContextRef,
									pAccounting,
									pLedger,
									pReport,
									pAccount,
									GetEnumAccountTitleTypeText(pAccount->pType, pLocaleLanguage),
									Budget,
									0,
									bCompactFileOption,
									bLeanChartsOption,
									bQuantityOption,
									pContent,
									BUDGET,
									FinalDate,
									BalanceDate,
									LastEntryDate,
									pGenUserName,
									GetEnumApplicationTextType(TXT_BUDGET, pLocaleLanguage),
									GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
									&iTabPos,
									iInitiated,
									pLocaleLanguage,
									ppMonth,
									pProgramName,
									pProgramVersion);

								if (iInitiated < 0)
									return iInitiated;
							}
							Budget = 0;
							BalanceDate = GetNextMonthLastDate(BalanceDate);
						}
					}
					if (bAccountTypes[7] && (bAccountTypes[9] || bAccountTypes[11] || bAccountTypes[12] || bAccountTypes[13])) {
						LabelAccount.pType = NulString;
						{
							AMOUNT	Budget = 0;
							DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
							while (BalanceDate <= pLedger->YearEnd) {
								// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
								BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pResult->pListMonth, BalanceDate);
								if (pMonth) {
									// Update with the report data
									Budget = pMonth->Budget;
								}
								if (((FinalDate / 100) >= (BalanceDate / 100)) && (Budget
									|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
									iInitiated = WriteXbrlGlBalanceReportLines(
										pStream,
										pContextRef,
										pAccounting,
										pLedger,
										pReport,
										pAccount,
										GetEnumAccountTitleTypeText(ACCOUNT_TYPE_RESULT, pLocaleLanguage),
										0-Budget,
										0,
										bCompactFileOption,
										bLeanChartsOption,
										bQuantityOption,
										pContent,
										BUDGET,
										FinalDate,
										BalanceDate,
										LastEntryDate,
										pGenUserName,
										GetEnumApplicationTextType(TXT_BUDGET, pLocaleLanguage),
										GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
										&iTabPos,
										iInitiated,
										pLocaleLanguage,
										ppMonth,
										pProgramName,
										pProgramVersion);

									if (iInitiated < 0)
										return iInitiated;
								}
								Budget = 0;
								BalanceDate = GetNextMonthLastDate(BalanceDate);
							}
						}
					}
					if (((bAccountTypes[0] || bAccountTypes[1] || bAccountTypes[2] || bAccountTypes[3] || bAccountTypes[4] || bAccountTypes[5] || bAccountTypes[6]))
						&& (bAccountTypes[7] || bAccountTypes[9] || bAccountTypes[11] || bAccountTypes[12] || bAccountTypes[13])) {
						LabelAccount.pType = NulString;
						{
							AMOUNT	Budget = 0;
							DATEINT BalanceDate = GetNextMonthLastDate(GetPrevMonthLastDate(pLedger->YearStart));
							while (BalanceDate <= pLedger->YearEnd) {
								// To generate a Ledger month period representation for HTML/XSL scripts to relate to, for the first account 
								BalanceTotalPeriodType* pMonth = GetBalanceTotalPeriod(pReport->pTotal->pListMonth, BalanceDate);
								if (pMonth) {
									// Update with the report data
									Budget = pMonth->Budget;
								}
								if (((FinalDate / 100) >= (BalanceDate / 100)) && (Budget
									|| ((pLedger->YearStart / 100) == (BalanceDate / 100)))) {
									iInitiated = WriteXbrlGlBalanceReportLines(
										pStream,
										pContextRef,
										pAccounting,
										pLedger,
										pReport,
										pAccount,
										GetEnumAccountTitleTypeText(ACCOUNT_TYPE_TOTAL, pLocaleLanguage),
										0-Budget,
										0,
										bCompactFileOption,
										bLeanChartsOption,
										bQuantityOption,
										pContent,
										BUDGET,
										FinalDate,
										BalanceDate,
										LastEntryDate,
										pGenUserName,
										GetEnumApplicationTextType(TXT_BUDGET, pLocaleLanguage),
										GetEnumDepreciationPeriodsType(TXT_MONTHLY, pLocaleLanguage),
										&iTabPos,
										iInitiated,
										pLocaleLanguage,
										ppMonth,
										pProgramName,
										pProgramVersion);

									if (iInitiated < 0)
										return iInitiated;
								}
								Budget = 0;
								BalanceDate = GetNextMonthLastDate(BalanceDate);
							}
						}
					}
				}
			}
		}
		if (iInitiated == 2) {
			xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:entryHeader");
			iInitiated = 1;
		}
	}
	
		if (iInitiated == 2) {
			xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:entryHeader");
			iInitiated = 1;
		}
	}
	if (iInitiated)
		xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:accountingEntries");
	return 0;
}


/********************************************************************
*
static int	WriteXbrlGlBalanceReportLines(
	xmlWriteLine(pStream, iTabPos++,	"gl-cor",	0,			0,				0,					"accountingEntries",		0);
	xmlWriteLine(pStream, iTabPos,		"gl-cor",	"gl-cor",	ATTRIBUTE_CONTEXTREF,	ATTRIBUTE_CONTEXTREF,		"accountMainID",			pAccount->pId);
	xmlWriteLine(pStream, --iTabPos,	0,			"gl-cor",	0,				0,					"accountingEntries",					0);
*
*********************************************************************/
static int WriteXbrlGlBalanceReportLines(
	FILE *pStream,
	char *pContextRef,
	AccountingType *pAccounting,
	LedgerType *pLedger,
	BalanceReportType *pReport,
	AccountType *pAccount,
	char *pAccountTypeDescription,
	AMOUNT Amount,
	QUANT Quant,
	BOOL bCompactFileOption,
	BOOL bLeanChartsOption,
	BOOL bQuantityOption,
	char *pContent,
	BOOL BalanceType,
	DATEINT FinalDate,
	DATEINT EndDate,
	DATEINT LastEntryDate,
	char *pGenUserName,
	char *pTitle,
	char *pEntryOrigin,
	int *piTabPos,
	int iInitiated,
	char *pLocaleLanguage,
	char **ppMonth,
	char *pProgramName,
	char *pProgramVersion)
{
	// #IB årsnr konto saldo kvantitet
	// #UB årsnr konto saldo kvantitet
	// #RES årsnr konto saldo kvantitet
	// #PSALDO årsnr period konto {dimensionsnr objektnr} saldo kvantitet 
	// #PBUDGET årsnr period konto {dimensionsnr objektnr} saldo kvantitet
	char szDim[LENGTH_NUMERIC_STR_MAX];
	char szDimSuper[LENGTH_NUMERIC_STR_MAX];
	char *pDimSuperTitle = NulString;
	char szAmount[LENGTH_AMOUNT_STRING];
	DimensionType *pDimension = 0;
	ObjectType *pObject = 0;

	if (!iInitiated) {
		char NumericString[LENGTH_NUMERIC_STR_MAX];

		xmlWriteLine(pStream, (*piTabPos)++, 0, "gl-cor:accountingEntries", 0, 0);

		if (pReport->pObject && pReport->pObject->pDimension->Id) {
			// Objects mapping
			char szTitle[1024];
			snprintf(szTitle, 1024, "%s %llu-%s %s",
				GetEnumApplicationTextType(TXT_MENU_OBJECT, pLocaleLanguage),
				pReport->pObject->pDimension->Id,
				pReport->pObject->pId,
				GetTextListText(pReport->pObject->pListName, pLocaleLanguage));
			if (WriteXbrlGlInformationHeader(
				pStream,
				pContextRef,
				pAccounting,
				"mapping",
				szTitle,
				NumericToStr(pLedger->Id, NumericString),
				pContent,
				pGenUserName,
				pLedger->YearStart,
				FinalDate,
				pLedger->YearStart,
				pLedger->YearEnd,
				pAccounting->pCurrency,
				*piTabPos,
				pLocaleLanguage,
				pProgramName,
				pProgramVersion))
				return (-2);
			else
				iInitiated = 1;
		}
		else {
			// Balances
			if (WriteXbrlGlInformationHeader(
				pStream,
				pContextRef,
				pAccounting,
				"balance",
				GetEnumFolderTypeText(TXT_BALANCE, pLocaleLanguage),
				NumericToStr(pLedger->Id, NumericString),
				pContent,
				pGenUserName,
				pLedger->YearStart,
				FinalDate,
				pLedger->YearStart,
				pLedger->YearEnd,
				pAccounting->pCurrency,
				*piTabPos,
				pLocaleLanguage,
				pProgramName,
				pProgramVersion))
				return (-2);
			else
				iInitiated = 1;
		}
	}

	{
		if (pReport) {
			// In non-compact mode zero balances should be typed
			memset(szDim, 0, LENGTH_NUMERIC_STR_MAX);
			if (pReport->pObject && pReport->pObject->pDimension && pReport->pObject->pDimension->Id && pReport->pObject && pReport->pObject->pId && *pReport->pObject->pId) {
				pDimension = pReport->pObject->pDimension;
				pObject = pReport->pObject;
				CounterToStr(pDimension->Id, szDim);
			}

			if (iInitiated != 2) {
				xmlWriteLine(pStream, (*piTabPos)++, 0, "gl-cor:entryHeader", 0, 0);

				switch (BalanceType) {
				case BALANCE_BEGINNING:
				case BALANCE_ENDING:
					if (LastEntryDate) {
						char DateString[LENGTH_LONG_DATE];
						xmlWriteLine(pStream, *piTabPos, pContextRef, "gl-cor:postedDate", DateToStr(LastEntryDate, DateString), "gl-cor:postedDate");
					}
					xmlWriteLine(pStream, *piTabPos, pContextRef, "gl-cor:entryType", "standard", "gl-cor:entryType");
					if (pEntryOrigin && *pEntryOrigin)
						xmlWriteLine(pStream, *piTabPos, pContextRef, "gl-bus:entryOrigin", pEntryOrigin, "gl-bus:entryOrigin");
					if (pTitle && *pTitle)
						xmlWriteLine(pStream, *piTabPos, pContextRef, "gl-cor:entryComment", pTitle, "gl-cor:entryComment");
					xmlWriteLine(pStream, *piTabPos, pContextRef, "gl-cor:qualifierEntry", "balance-brought-forward", "gl-cor:qualifierEntry");
					break;
				case PERIOD_CHANGE:
				{
					char DateString[LENGTH_LONG_DATE];
					xmlWriteLine(pStream, *piTabPos, pContextRef, "gl-cor:postedDate", DateToStr(GetLastDateInMonth(pLedger->YearStart), DateString), "gl-cor:postedDate");
				}
				xmlWriteLine(pStream, *piTabPos, pContextRef, "gl-cor:entryType", "standard", "gl-cor:entryType");
					if (pEntryOrigin && *pEntryOrigin)
						xmlWriteLine(pStream, *piTabPos, pContextRef, "gl-bus:entryOrigin", pEntryOrigin, "gl-bus:entryOrigin");
					if (pTitle && *pTitle)
						xmlWriteLine(pStream, *piTabPos, pContextRef, "gl-cor:entryComment", pTitle, "gl-cor:entryComment");
					xmlWriteLine(pStream, *piTabPos, pContextRef, "gl-cor:qualifierEntry", "standard", "gl-cor:qualifierEntry");
					break;
				case BUDGET:
				{
					char DateString[LENGTH_LONG_DATE];
					xmlWriteLine(pStream, *piTabPos, pContextRef, "gl-cor:postedDate", DateToStr(GetLastDateInMonth(pLedger->YearStart), DateString), "gl-cor:postedDate");
				}
				xmlWriteLine(pStream, *piTabPos, pContextRef, "gl-cor:entryType", "budget", "gl-cor:entryType");
					if (pEntryOrigin && *pEntryOrigin)
						xmlWriteLine(pStream, *piTabPos, pContextRef, "gl-bus:entryOrigin", pEntryOrigin, "gl-bus:entryOrigin");
					if (pTitle && *pTitle)
						xmlWriteLine(pStream, *piTabPos, pContextRef, "gl-cor:entryComment", pTitle, "gl-cor:entryComment");
					xmlWriteLine(pStream, *piTabPos, pContextRef, "gl-cor:qualifierEntry", "standard", "gl-cor:qualifierEntry");
					break;
				}
				iInitiated = 2;
			}
			xmlWriteLine(pStream, (*piTabPos)++, 0, "gl-cor:entryDetail", 0, 0);

			// gl-cor:account
			if (WriteXbrlGlAccountSpecifications(pStream, 
				pAccounting, 
				pAccounting->pGaap, 
				pContextRef, 
				pAccount->pId, 
				GetTextListText(pAccount->pListName, pLocaleLanguage), 
				pAccount->pType, 
				pAccountTypeDescription, 
				0, 
				bQuantityOption, 
				bCompactFileOption, 
				(*piTabPos)))
				return (-2);

			// gl-cor:amount
			{
				char ExtraAttr[64];
				char LwrStringIso4217[LENGTH_ISO4217_CURRENCY + 1];
				memset(LwrStringIso4217, 0, LENGTH_ISO4217_CURRENCY + 1);
				strncpy(LwrStringIso4217, pAccounting->pCurrency, LENGTH_ISO4217_CURRENCY);
				StrLwr((unsigned char *)LwrStringIso4217);
				strcpy(ExtraAttr, ATTRIBUTE_UNITREF"=\"");
				if (*LwrStringIso4217)
					strcat(ExtraAttr, LwrStringIso4217);
				else
					strcat(ExtraAttr, "NotUsed");
				strcat(ExtraAttr, "\" decimals=\"2\" ");
				{
					char *pContextRefExtra = calloc(strlen(ExtraAttr) + strlen(pContextRef) + 1, sizeof(char));
					if (pContextRefExtra) {
						strcpy(pContextRefExtra, ExtraAttr);
						strcat(pContextRefExtra, pContextRef);
						xmlWriteLine(pStream, *piTabPos, pContextRefExtra, "gl-cor:amount", AmountToStr(Amount, szAmount), "gl-cor:amount");
						free(pContextRefExtra);
					}
					else
						xmlWriteLine(pStream, *piTabPos, "Error memory", "gl-cor:amount", AmountToStr(Amount, szAmount), "gl-cor:amount");
				}
			}
			// gl-cor:postingDate
			{
				char szPostingDate[LENGTH_LONG_DATE];
				memset(szPostingDate, 0, LENGTH_LONG_DATE);
				switch (BalanceType) {
				case BALANCE_BEGINNING:
					DateToStr(pLedger->YearStart, szPostingDate);
					break;
				case BALANCE_ENDING:
					if ((GetLastEntryDate(pAccounting) < pLedger->YearEnd) && (!pLedger->Id))
						DateToStr(GetLastEntryDate(pAccounting), szPostingDate);
					else
						DateToStr(pLedger->YearEnd, szPostingDate);
					break;
				case PERIOD_CHANGE:
				case BUDGET:
					DateToStr(EndDate, szPostingDate);
					break;
				}
				xmlWriteLine(pStream, *piTabPos, pContextRef, "gl-cor:postingDate", szPostingDate, "gl-cor:postingDate");
			}

			// gl-cor:identifierReference
			// SIE #DIM #OBJ info
			if (pDimension && pDimension->Id) {
				memset(szDimSuper, 0, LENGTH_NUMERIC_STR_MAX);
				if (pDimension->pSuperDimension) {
					DimensionType *pDimensionSuper = pDimension->pSuperDimension;
					if (pDimensionSuper)
						CounterToStr(pDimensionSuper->Id, szDimSuper);
					pDimSuperTitle = GetTextListText(pDimensionSuper->pListName, pLocaleLanguage);
				}

				if (WriteXbrlGlDimObjSpecificationObject(
					pStream,
					szDim,
					GetTextListText(pDimension->pListName, pLocaleLanguage),
					szDimSuper,
					pDimSuperTitle,
					pObject->pId,
					GetTextListText(pObject->pListName, pLocaleLanguage),
					0,
					pContextRef,
					bCompactFileOption,
					TRUE,
					*piTabPos))
					return (-2);
			}

			// gl-cor:postingStatusDescription
			if (EndDate && ppMonth)
				xmlWriteLine(pStream, *piTabPos, pContextRef, "gl-cor:postingStatusDescription", ppMonth[(((EndDate / 100) % 100) - 1) % 12], "gl-cor:postingStatusDescription"); //  % 12 for MSCS nonsense warning

			// gl-cor:xbrlInfo
			xmlWriteLine(pStream, (*piTabPos)++, 0, "gl-cor:xbrlInfo", 0, 0);
			switch (BalanceType) {
			case BALANCE_BEGINNING:
				xmlWriteLine(pStream, *piTabPos, pContextRef, "gl-cor:xbrlInclude", "beginning_balance", "gl-cor:xbrlInclude");
				break;
			case BALANCE_ENDING:
				xmlWriteLine(pStream, *piTabPos, pContextRef, "gl-cor:xbrlInclude", "ending_balance", "gl-cor:xbrlInclude");
				break;
			case PERIOD_CHANGE:
			case BUDGET:
				xmlWriteLine(pStream, *piTabPos, pContextRef, "gl-cor:xbrlInclude", "period_change", "gl-cor:xbrlInclude");
				break;
			}
			xmlWriteLine(pStream, --(*piTabPos), 0, 0, 0, "gl-cor:xbrlInfo");

			// gl-bus:measurable
			if (bQuantityOption && ((!DecimalCmp(Quant, 0)) || ((!bCompactFileOption) && *GetTextListText(pAccount->pListQuantityUnit, pLocaleLanguage)))) {
				char szQuantity[LENGTH_QUANT_STRING];
				DecimalToStr(Quant, szQuantity);

				if (WriteXbrlGlQuantitySpecifications(pStream, szQuantity, GetTextListText(pAccount->pListQuantityUnit, pLocaleLanguage), pContextRef, (*piTabPos)))
					return (-2);
			}

			// gl-cor:taxes
			if (!bCompactFileOption) {
				if (WriteXbrlGlAccountTaxCodeSpecifications(pStream, pContextRef, pAccount, pAccounting, FALSE, (*piTabPos)))
					return (-2);
			}
			xmlWriteLine(pStream, --(*piTabPos), 0, 0, 0, "gl-cor:entryDetail");
		}
		return iInitiated;
	}
}


/********************************************************************
*
static BOOL WriteXbrlGlLedgerVouchers(
*
*********************************************************************/
static BOOL WriteXbrlGlLedgerVouchers(
	FILE *pStream,
	char *pContextRef,
	AccountingType *pAccounting,
	BOOL bCompactFileOption,
	BOOL bQuantityOption,
	BOOL bObjectOption,
	DATEINT InitialDate,
	DATEINT FinalDate,
	char *FilterEnteredBy,
	char *pContent,
	char *pGenUserName,
	BOOL *pbChartsDeclared,
	int iTabPos,
	char *pLocaleLanguage,
	char *pProgramName,
	char *pProgramVersion)
{
	// The basic matter is that we and SIE regard all journals as one list and XBRL GL as one each for each journal/pSeries
	// So we need to sort the vouchers by joural and then make one XBRL GL journal for each

	// Ledger Vouchers
	BOOL bSeriesItems = FALSE;
	LedgerType *pLedger
		= LListGetFirstRecordData(pAccounting->pLedgersChart->pListLedger);
	while (pLedger) {
		LLISTITEMTYPE *CursorRecord = pAccounting->pLedgersChart->pListLedger->CursorRecord;
		// #VER serie vernr verdatum vertext regdatum sign
		LLIST pVoucherSeriesList = GetSeriesList(pAccounting, 0, pLedger->Id);
		char *VoucherId = 0;
		DATEINT EndDate = FinalDate;
		if (pLedger->YearEnd < FinalDate)
			EndDate = pLedger->YearEnd;
		if (pbChartsDeclared) {
			bCompactFileOption = FALSE;
			*pbChartsDeclared = TRUE;
		}

		VoucherId = LListGetFirstRecordData(pVoucherSeriesList);
		while (VoucherId) {
			LLIST pListVoucher = GetSortedDataStructList(pLedger->pListVoucher, VOUCHERS_BY_SERIES); // VOUCHERS_BY_JORNAL VOUCHERS_BY_DATE:

			VoucherType *pVoucher
				= LListGetFirstRecordData(pListVoucher);
			while (pVoucher) {
				// Matching VoucherId
				if ((*VoucherId && pVoucher->pId && pVoucher->pId->pseries && *pVoucher->pId->pseries && (!strcmp(VoucherId, pVoucher->pId->pseries)))
					|| ((!(*VoucherId)) && ((!pVoucher->pId) || (pVoucher->pId && (!(pVoucher->pId->pseries && *pVoucher->pId->pseries)))))) {
					if (!bSeriesItems) {
						// New journal
						bSeriesItems = TRUE;
						xmlWriteLine(pStream, iTabPos++, 0, "gl-cor:accountingEntries", 0, 0);
						if (WriteXbrlGlInformationHeader(
							pStream,
							pContextRef,
							pAccounting,
							"journal",
							GetEnumFolderTypeText(TXT_VOUCHERS, pLocaleLanguage),
							VoucherId,
							pContent,
							pGenUserName,
							pLedger->YearStart,
							EndDate,
							pLedger->YearStart,
							pLedger->YearEnd,
							pAccounting->pCurrency,
							iTabPos,
							pLocaleLanguage,
							pProgramName,
							pProgramVersion)) {
							LListErase(pVoucherSeriesList);
							xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:accountingEntries");
							return (-2);
						}
					}

					if ((((InitialDate <= pVoucher->Date)
						&& (FinalDate >= pVoucher->Date)))
						&& ((!FilterEnteredBy) ||
						(!*FilterEnteredBy) ||
							(pVoucher->pAccountant && (!strcmp(pVoucher->pAccountant->pOperatorInfo, FilterEnteredBy))))) {
						if ((!FinalDate) || (pVoucher->Date <= FinalDate)) { // The complete year
							char szTempString[LENGTH_AMOUNT_STRING + 1];
							char *pXbrlGlVerTitle = GetTextListText(pVoucher->pListTitle, pLocaleLanguage);
							if (!bCompactFileOption) {
								if (!*pXbrlGlVerTitle) {
									VoucherEntryType *pVoucherEntry
										= LListGetFirstRecordData(pVoucher->pListEntry);
									while (pVoucherEntry && (!*(GetTextListText(pVoucherEntry->pListTitle, pLocaleLanguage))))
										pVoucherEntry = LListGetNextRecordData(pVoucher->pListEntry);

									// For full notification 
									// and for support of programs not having support for 
									// both pVoucher->pTitle and pVoucherEntry->pTitle
									if (pVoucherEntry && (*(GetTextListText(pVoucherEntry->pListTitle, pLocaleLanguage))))
										// Setting an empty pXbrlGlVerTitle as pVoucherEntry->pTitle
										pXbrlGlVerTitle = GetTextListText(pVoucherEntry->pListTitle, pLocaleLanguage);
								}
							}
							xmlWriteLine(pStream, iTabPos++, 0, "gl-cor:entryHeader", 0, 0);

							// postedDate
							xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:postedDate", DateToStr(pVoucher->Date, szTempString), "gl-cor:postedDate");

							// enteredBy
							if (pVoucher->pAccountant && pVoucher->pAccountant->pOperatorInfo && *pVoucher->pAccountant->pOperatorInfo)
								xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:enteredBy", pVoucher->pAccountant->pOperatorInfo, "gl-cor:enteredBy");

							// enteredDate
							if (pVoucher->pAccountant && pVoucher->pAccountant->date)
								xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:enteredDate", DateToStr(pVoucher->pAccountant->date, szTempString), "gl-cor:enteredDate");

							// entryNumber	
							if (pVoucher->pId && pVoucher->pId->SeriesId)
								xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:entryNumber", CounterToStr(pVoucher->pId->SeriesId, szTempString), "gl-cor:entryNumber");
							else
								xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:entryNumber", "0", "gl-cor:entryNumber");

							// entryComment
							if (pXbrlGlVerTitle && *pXbrlGlVerTitle)
								xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:entryComment", pXbrlGlVerTitle, "gl-cor:entryComment");

							// Objects in the dimension, prepare
							{
								VoucherEntryType *pVoucherEntry
									= LListGetFirstRecordData(pVoucher->pListEntry);
								while (pVoucherEntry) {
									DATEINT *pTransDate = &pVoucherEntry->AuxDate;
									char *pXbrlGlVerRecordTitle = GetTextListText(pVoucherEntry->pListTitle, pLocaleLanguage);

									if (pXbrlGlVerTitle && *pXbrlGlVerTitle) {
										if (bCompactFileOption && pXbrlGlVerRecordTitle && !strcmp(pXbrlGlVerTitle, pXbrlGlVerRecordTitle))
											pXbrlGlVerRecordTitle = 0;
									}

									if (pVoucher->Date == *pTransDate)
										pTransDate = 0;

									xmlWriteLine(pStream, iTabPos++, 0, "gl-cor:entryDetail", 0, 0);

									// gl-cor:account
									if (WriteXbrlGlAccountSpecifications(
										pStream,
										pAccounting,
										pAccounting->pGaap,
										pContextRef,
										pVoucherEntry->pAccount->pId,
										GetTextListText(pVoucherEntry->pAccount->pListName, pLocaleLanguage),
										pVoucherEntry->pAccount->pType,
										0,
										0,
										bQuantityOption,
										bCompactFileOption,
										iTabPos)) {
										LListErase(pVoucherSeriesList);
										xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:entryDetail");
										xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:entryHeader");
										xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:accountingEntries");
										return (-2);
									}

									// amount (Accounting currency)
									{
										char TempString[64];
										char LwrStringIso4217[LENGTH_ISO4217_CURRENCY + 1];
										memset(LwrStringIso4217, 0, LENGTH_ISO4217_CURRENCY + 1);
										strncpy(LwrStringIso4217, pAccounting->pCurrency, LENGTH_ISO4217_CURRENCY);
										StrLwr((unsigned char *)LwrStringIso4217);
										strcpy(TempString, ATTRIBUTE_UNITREF"=\"");
										if(*LwrStringIso4217)
											strcat(TempString, LwrStringIso4217);
										else
											strcat(TempString, "NotUsed");
										strcat(TempString, "\" decimals=\"2\" ");
										strcat(TempString, pContextRef);
										xmlWriteLine(pStream, iTabPos, TempString, "gl-cor:amount", AmountToStr(pVoucherEntry->Amount, szTempString), "gl-cor:amount");
									}

									// postingDate
									if (pTransDate && *pTransDate)
										xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:postingDate", DateToStr(*pTransDate, szTempString), "gl-cor:postingDate");

									// gl-cor:identifierReference
									// SIE #DIM #OBJ info
									if ((bObjectOption && pAccounting->pObjectsChart)) {
										if (WriteXbrlGlVoucherDimObjSpecificationList(
											pStream,
											pAccounting,
											pVoucherEntry->pListObject,
											pContextRef,
											bCompactFileOption,
											iTabPos,
											pLocaleLanguage)) {
											LListErase(pVoucherSeriesList);
											xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:entryDetail");
											xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:entryHeader");
											xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:accountingEntries");
											return (-2);
										}
									}
									if (pVoucherEntry->pInvoice && LListGetFirstRecordData(pVoucherEntry->pInvoice->pListOriginalInvoiceFileUrl)) {
										// XBRL only allow one document
										char *pFileUrl
											= LListGetFirstRecordData(pVoucherEntry->pInvoice->pListOriginalInvoiceFileUrl);
										xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:documentReference", pFileUrl, "gl-cor:documentReference");
									}
									else // XBRL only allow one document 
										if (pVoucherEntry->pInvoice && LListGetFirstRecordData(pVoucherEntry->pInvoice->pListRelatedDocumentFileUrl)) {
											// XBRL only allow one document
											char *pFileUrl
												= LListGetFirstRecordData(pVoucherEntry->pInvoice->pListRelatedDocumentFileUrl);
											xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:documentReference", pFileUrl, "gl-cor:documentReference");
										}
									// postingStatus
									if (pVoucherEntry->pDeprecated)
										xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:postingStatus", "cancelled", "gl-cor:postingStatus");
									else
										xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:postingStatus", "posted", "gl-cor:postingStatus");

									// detailComment
									if (pXbrlGlVerRecordTitle && *pXbrlGlVerRecordTitle)
										xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:detailComment", pXbrlGlVerRecordTitle, "gl-cor:detailComment");

									// amount (Quantity)
									if (bQuantityOption && ((!DecimalCmp(pVoucherEntry->Quantity, 0)) || ((!bCompactFileOption) && *GetTextListText(pVoucherEntry->pAccount->pListQuantityUnit, pLocaleLanguage)))) {
										if (WriteXbrlGlQuantitySpecifications(pStream, DecimalToStr(pVoucherEntry->Quantity, szTempString), GetTextListText(pVoucherEntry->pAccount->pListQuantityUnit, pLocaleLanguage), pContextRef, iTabPos)) {
											LListErase(pVoucherSeriesList);
											xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:entryDetail");
											xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:entryHeader");
											xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:accountingEntries");
											return (-2);
										}
									}

									xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:entryDetail");
									pVoucherEntry = LListGetNextRecordData(pVoucher->pListEntry);
								}
							}
						}
					}
					xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:entryHeader");
				}
				pVoucher = LListGetNextRecordData(pListVoucher);
			}
			LListErase(pListVoucher);
			if (bSeriesItems) {
				bSeriesItems = FALSE;
				xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:accountingEntries");
			}
			VoucherId = LListGetNextRecordData(pVoucherSeriesList);
		}
		pAccounting->pLedgersChart->pListLedger->CursorRecord = CursorRecord;
		pLedger = LListGetNextRecordData(pAccounting->pLedgersChart->pListLedger);
		LListErase(pVoucherSeriesList);
	}
	return 0;
}


/********************************************************************
*
static int WriteXbrlGlVoucherDimObjSpecificationList(
*
*********************************************************************/
static int WriteXbrlGlVoucherDimObjSpecificationList(
	FILE *pStream, 
	AccountingType *pAccounting, 
	LLIST pListObject, 
	char *pContextRef, 
	BOOL bCompactFileOption, 
	int iTabPos, 
	char *pLocaleLanguage)
{
	char szDimSuper[LENGTH_NUMERIC_STR_MAX];
	char *pDimSuperTitle = NulString;

	ObjectType *pObject
		= LListGetFirstRecordData(pListObject);
	while (pObject) {
		if (pObject->pDimension->pSuperDimension) {
			DimensionType *pDimensionSuper = (DimensionType *)pObject->pDimension->pSuperDimension;
			memset(szDimSuper, 0, LENGTH_NUMERIC_STR_MAX);
			if (pDimensionSuper) { // Error if not found
				CounterToStr(pDimensionSuper->Id, szDimSuper);
				pDimSuperTitle = GetTextListText(pDimensionSuper->pListName, pLocaleLanguage);
			}
			else
				*szDimSuper = 0;
		}
		else
			*szDimSuper = 0;

		if (pObject->pDimension) {
			char szDimId[LENGTH_NUMERIC_STR_MAX];
			memset(szDimId, 0, LENGTH_NUMERIC_STR_MAX);
			if (WriteXbrlGlDimObjSpecificationObject(
				pStream,
				CounterToStr(pObject->pDimension->Id, szDimId),
				GetTextListText(pObject->pDimension->pListName, pLocaleLanguage),
				szDimSuper,
				pDimSuperTitle,
				pObject->pId,
				GetTextListText(pObject->pListName, pLocaleLanguage),
				0,
				pContextRef,
				bCompactFileOption,
				TRUE,
				iTabPos))
				return (-2);
		}
		pObject = LListGetNextRecordData(pListObject);
	}
	return 0;
}


/********************************************************************
*
static BOOL WriteXbrlGlInformationHeader(
*
*********************************************************************/
static BOOL WriteXbrlGlInformationHeader(
	FILE *pStream,
	char *pContextRef,
	AccountingType *pAccounting,
	char *EntriesType,
	char *pHeaderTitle,
	char *UniqueID,
	char *pContent,
	char *pGenUserName,
	DATEINT PeriodCoveredStart,
	DATEINT PeriodCoveredEnd,
	DATEINT FiscalYearStart,
	DATEINT FiscalYearEnd,
	char *Iso4217Currency,
	int iTabPos,
	char *pLocaleLanguage,
	char *pProgramName,
	char *pProgramVersion)
{
	char szDateString[LENGTH_LONG_DATE];
	memset(szDateString, 0, LENGTH_LONG_DATE);

	// Todays date preparation 
	DateToStr(GetSystemDate(), szDateString);

	xmlWriteLine(pStream, iTabPos++, 0, "gl-cor:documentInfo", 0, 0);
	xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:entriesType", EntriesType, "gl-cor:entriesType");
	if (UniqueID) {
		if (*UniqueID)
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:uniqueID", UniqueID, "gl-cor:uniqueID");
		//	else // Null Voucher IDs
			//	xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:uniqueID", "0", "gl-cor:uniqueID");
	}
	if (pLocaleLanguage && *pLocaleLanguage) {
		char LanguguageIso639[10];
		memset(LanguguageIso639, 0, 10);
		strcat(LanguguageIso639, "ISO639:");
		strncat(LanguguageIso639, pLocaleLanguage, 2);
		StrLwr((unsigned char *)LanguguageIso639);
		xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:language", LanguguageIso639, "gl-cor:language");
	}
	// #GEN
	if (*szDateString)
		xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:creationDate", szDateString, "gl-cor:creationDate");
	if (pGenUserName)
		xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:creator", pGenUserName, "gl-bus:creator");
	if (pHeaderTitle)
		xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:entriesComment", pHeaderTitle, "gl-cor:entriesComment");
	if (PeriodCoveredStart && PeriodCoveredEnd) {
		xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:periodCoveredStart", DateToStr(PeriodCoveredStart, szDateString), "gl-cor:periodCoveredStart");
		xmlWriteLine(pStream, iTabPos, pContextRef, "gl-cor:periodCoveredEnd", DateToStr(PeriodCoveredEnd, szDateString), "gl-cor:periodCoveredEnd");
	}
	xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:periodUnitDescription", GetEnumFolderTypeText(CONTENT_LABEL_LEDGER, pLocaleLanguage), "gl-bus:periodUnitDescription");
	{
		char Program[256];
		memset(Program, 0, 256);
		strncpy(Program, pProgramName, 255);
		Program[255] = 0;
		strncat(Program, " version ", 255 - strlen(Program));
		Program[255] = 0;
		strncat(Program, pProgramVersion, 256 - strlen(Program));
		xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:sourceApplication", Program, "gl-bus:sourceApplication");
	}
	if (Iso4217Currency && *Iso4217Currency) {
		char LwrStringIso4217[8 + LENGTH_ISO4217_CURRENCY + 1];
		memset(LwrStringIso4217, 0, LENGTH_ISO4217_CURRENCY + 1);
		strcat(LwrStringIso4217, PREFIX_ISO4217_CURR);
		strncat(LwrStringIso4217, Iso4217Currency, LENGTH_ISO4217_CURRENCY);
		StrLwr((unsigned char *)LwrStringIso4217);

		xmlWriteLine(pStream, iTabPos, pContextRef, "gl-muc:defaultCurrency", LwrStringIso4217, "gl-muc:defaultCurrency");
	}
	xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:documentInfo");
	if (pAccounting->pEntity && ((!strcmp(pContent, TXT_BALANCE)) || (!strcmp(pContent, TXT_LEDGER)) || (!strcmp(pContent, TXT_VOUCHERS)))) {
		xmlWriteLine(pStream, iTabPos++, 0, "gl-cor:entityInformation", 0, 0);
		if (pAccounting->pEntity->pEntityPhone && *pAccounting->pEntity->pEntityPhone) {
			xmlWriteLine(pStream, iTabPos++, 0, "gl-bus:entityPhoneNumber", 0, 0);
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:phoneNumber", pAccounting->pEntity->pEntityPhone, "gl-bus:phoneNumber");
			xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-bus:entityPhoneNumber");
		}
		if (pAccounting->pEntity->pEntityEmail && *pAccounting->pEntity->pEntityEmail) {
			xmlWriteLine(pStream, iTabPos++, 0, "gl-bus:entityEmailAddressStructure", 0, 0);
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:entityEmailAddress", pAccounting->pEntity->pEntityEmail, "gl-bus:entityEmailAddress");
			xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-bus:entityEmailAddressStructure");
		}
		if (WriteXbrlGlOrganizationIdentifiers(pStream, pContextRef, iTabPos, pAccounting->pEntity->pLegalRegId, "Legal"))
			return (-2);

		if (WriteXbrlGlOrganizationIdentifiers(pStream, pContextRef, iTabPos, pAccounting->pEntity->pVatRegId, "Vat"))
			return (-2);

		if (WriteXbrlGlOrganizationIdentifiers(pStream, pContextRef, iTabPos, pAccounting->pEntity->pStatisticalClass, "Statistical"))
			return (-2);

		if (WriteXbrlGlOrganizationIdentifiers(pStream, pContextRef, iTabPos, pAccounting->pEntity->pJurisdiction, "Jurisdiction"))
			return (-2);

		if (WriteXbrlGlOrganizationAddress(pStream, pContextRef, iTabPos, pAccounting))
			return (-2);

		if (WriteXbrlGlOrganizationWebSite(pStream, pContextRef, iTabPos, pAccounting))
			return (-2);

		if (WriteXbrlGlContactInformation(pStream, pContextRef, iTabPos, pAccounting))
			return (-2);

		if (pAccounting->pEntity->pEntityType && *pAccounting->pEntity->pEntityType && ((!strcmp(pContent, TXT_BALANCE)) || (!strcmp(pContent, TXT_LEDGER))))
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:businessDescription", pAccounting->pEntity->pEntityType, "gl-bus:businessDescription");

		if (FiscalYearStart && FiscalYearEnd) {
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:fiscalYearStart", DateToStr(FiscalYearStart, szDateString), "gl-bus:fiscalYearStart");
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:fiscalYearEnd", DateToStr(FiscalYearEnd, szDateString), "gl-bus:fiscalYearEnd");
		}
		if (WriteXbrlGlAccountantInformation(pStream, pContextRef, iTabPos, pAccounting))
			return (-2);

		if (FiscalYearStart && FiscalYearEnd) {
			xmlWriteLine(pStream, iTabPos++, 0, "gl-bus:reportingCalendar", 0, 0);
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:reportingCalendarPeriodType", "monthly", "gl-bus:reportingCalendarPeriodType");
			xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-bus:reportingCalendar");
		}
		xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-cor:entityInformation");
	}
	return 0;
}


/********************************************************************
*
static int WriteXbrlGlOrganizationIdentifiers(
*
*********************************************************************/
static int WriteXbrlGlOrganizationIdentifiers(
	FILE *pStream, 
	char *pContextRef, 
	int iTabPos, 
	char *OrganizationIdentifier, 
	char *OrganizationDescription)
{
	if (OrganizationIdentifier && *OrganizationIdentifier) {
		xmlWriteLine(pStream, iTabPos++, 0, "gl-bus:organizationIdentifiers", 0, 0);
		xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:organizationIdentifier", OrganizationIdentifier, "gl-bus:organizationIdentifier");
		xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:organizationDescription", OrganizationDescription, "gl-bus:organizationDescription");
		xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-bus:organizationIdentifiers");
	}
	return 0; // OK
}


/********************************************************************
*
static int WriteXbrlGlOrganizationAddress(
*
*********************************************************************/
static int WriteXbrlGlOrganizationAddress(
	FILE *pStream, 
	char *pContextRef, 
	int iTabPos, 
	AccountingType *pAccounting)
{
	if (pAccounting && pAccounting->pEntity
		&& pAccounting->pEntity->pName) {
		char *pAdressLine
			= LListGetFirstRecordData(pAccounting->pEntity->pListAddress);
		xmlWriteLine(pStream, iTabPos++, 0, "gl-bus:organizationAddress", 0, 0);
		if (pAccounting->pEntity->pName && *pAccounting->pEntity->pName)
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:organizationAddressName", pAccounting->pEntity->pName, "gl-bus:organizationAddressName");
		if (pAdressLine && *pAdressLine)
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:organizationBuildingNumber", pAdressLine, "gl-bus:organizationBuildingNumber");
		pAdressLine = LListGetNextRecordData(pAccounting->pEntity->pListAddress);
		if (pAdressLine && *pAdressLine)
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:organizationAddressStreet", pAdressLine, "gl-bus:organizationAddressStreet");
		pAdressLine = LListGetNextRecordData(pAccounting->pEntity->pListAddress);
		if (pAdressLine && *pAdressLine)
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:organizationAddressStreet2", pAdressLine, "gl-bus:organizationAddressStreet2");
		if (pAccounting->pEntity->pPostalAddress && *pAccounting->pEntity->pPostalAddress)
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:organizationAddressCity", pAccounting->pEntity->pPostalAddress, "gl-bus:organizationAddressCity");
		pAdressLine = LListGetNextRecordData(pAccounting->pEntity->pListAddress);
		if (pAdressLine && *pAdressLine)
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:organizationAddressStateOrProvince", pAdressLine, "gl-bus:organizationAddressStateOrProvince");
		if (pAccounting->pEntity->pPostalCode && *pAccounting->pEntity->pPostalCode)
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:organizationAddressZipOrPostalCode", pAccounting->pEntity->pPostalCode, "gl-bus:organizationAddressZipOrPostalCode");
		if (pAccounting->pEntity->pPostalCountry && *pAccounting->pEntity->pPostalCountry)
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:organizationAddressCountry", pAccounting->pEntity->pPostalCountry, "gl-bus:organizationAddressCountry");
		xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-bus:organizationAddress");
	}
	return 0; // OK
}


/********************************************************************
*
static int WriteXbrlGlOrganizationWebSite(
*
*********************************************************************/
static int WriteXbrlGlOrganizationWebSite(
	FILE *pStream, 
	char *pContextRef, 
	int iTabPos, 
	AccountingType *pAccounting)
{
	if (pAccounting && pAccounting->pEntity) {
		char *pWebSiteURL
			= LListGetFirstRecordData(pAccounting->pEntity->pListWebPage);
		while (pWebSiteURL) {
			xmlWriteLine(pStream, iTabPos++, 0, "gl-bus:entityWebSite", 0, 0);
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:webSiteURL", pWebSiteURL, "gl-bus:webSiteURL");
			xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-bus:entityWebSite");

			pWebSiteURL = LListGetNextRecordData(pAccounting->pEntity->pListWebPage);
		}
	}
	return 0; // OK
}


/********************************************************************
*
static int WriteXbrlGlContactInformation(
*
*********************************************************************/
static int WriteXbrlGlContactInformation(
	FILE *pStream, 
	char *pContextRef, 
	int iTabPos, 
	AccountingType *pAccounting)
{
	if (pAccounting && pAccounting->pEntity
		&& LListGetFirstRecordData(pAccounting->pEntity->pListContact)) {
		xmlWriteLine(pStream, iTabPos++, 0, "gl-bus:contactInformation", 0, 0);
		{
			ContactType *pContact
				= LListGetFirstRecordData(pAccounting->pEntity->pListContact);
			while (pContact) {
				xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:contactLastName", pContact->pName, "gl-bus:contactLastName");
				if (LListGetFirstRecordData(pContact->pListPhone)) {
					char *pPhone
						= LListGetFirstRecordData(pContact->pListPhone);
					xmlWriteLine(pStream, iTabPos++, 0, "gl-bus:contactPhone", 0, 0);
					while (pPhone) {
						xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:contactPhoneNumber", pPhone, "gl-bus:contactPhoneNumber");
						pPhone = LListGetNextRecordData(pContact->pListPhone);
					}
					xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-bus:contactPhone");
				}
				if (LListGetFirstRecordData(pContact->pListEmail)) {
					char *pEmail
						= LListGetFirstRecordData(pContact->pListEmail);
					xmlWriteLine(pStream, iTabPos++, 0, "gl-bus:contactEMail", 0, 0);
					while (pEmail) {
						xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:contactEmailAddress", pEmail, "gl-bus:contactEmailAddress");
						pEmail = LListGetNextRecordData(pContact->pListEmail);
					}
					xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-bus:contactEMail");
				}
				//xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:contactActive", "1", "gl-bus:contactActive");
				pContact = LListGetNextRecordData(pAccounting->pEntity->pListContact);
			}
		}
		xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-bus:contactInformation");
	}
	return 0; // OK
}


/********************************************************************
*
static int WriteXbrlGlAccountantInformation(
*
*********************************************************************/
static int WriteXbrlGlAccountantInformation(
	FILE *pStream, 
	char *pContextRef, 
	int iTabPos, 
	AccountingType *pAccounting)
{
	if (pAccounting && pAccounting->pAccountant
		&& pAccounting->pAccountant->pName) {
		char *pAdressLine
			= LListGetFirstRecordData(pAccounting->pAccountant->pListWebPage);
		xmlWriteLine(pStream, iTabPos++, 0, "gl-bus:accountantInformation", 0, 0);
		if (pAccounting->pAccountant->pName && *pAccounting->pAccountant->pName)
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:accountantName", pAccounting->pAccountant->pName, "gl-bus:accountantName");
		xmlWriteLine(pStream, iTabPos++, 0, "gl-bus:accountantAddress", 0, 0);
		xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:accountantAddressName", pAccounting->pAccountant->pName, "gl-bus:accountantAddressName");
		if (pAdressLine && *pAdressLine)
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:accountantBuildingNumber", pAdressLine, "gl-bus:accountantBuildingNumber");
		pAdressLine = LListGetNextRecordData(pAccounting->pAccountant->pListWebPage);
		if (pAdressLine && *pAdressLine)
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:accountantAddressStreet", pAdressLine, "gl-bus:accountantAddressStreet");
		pAdressLine = LListGetNextRecordData(pAccounting->pAccountant->pListWebPage);
		if (pAdressLine && *pAdressLine)
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:accountantAddressStreet2", pAdressLine, "gl-bus:accountantAddressStreet2");
		if (pAccounting->pAccountant->pPostalAddress && *pAccounting->pAccountant->pPostalAddress)
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:accountantAddressCity", pAccounting->pAccountant->pPostalAddress, "gl-bus:accountantAddressCity");
		pAdressLine = LListGetNextRecordData(pAccounting->pAccountant->pListWebPage);
		if (pAdressLine && *pAdressLine)
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:accountantAddressStateOrProvince", pAdressLine, "gl-bus:accountantAddressStateOrProvince");
		if (pAccounting->pAccountant->pPostalCode && *pAccounting->pAccountant->pPostalCode)
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:accountantAddressZipOrPostalCode", pAccounting->pAccountant->pPostalCode, "gl-bus:accountantAddressZipOrPostalCode");
		if (pAccounting->pAccountant->pPostalCountry && *pAccounting->pAccountant->pPostalCountry)
			xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:accountantAddressCountry", pAccounting->pAccountant->pPostalCountry, "gl-bus:accountantAddressCountry");
		xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-bus:organizationAddress");
	}
	if (pAccounting && pAccounting->pAccountant
		&& LListGetFirstRecordData(pAccounting->pAccountant->pListContact)) {
		xmlWriteLine(pStream, iTabPos++, 0, "gl-bus:accountantContactInformation", 0, 0);
		{
			ContactType *pContact
				= LListGetFirstRecordData(pAccounting->pAccountant->pListContact);
			while (pContact) {
				xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:accountantContactLastName", pContact->pName, "gl-bus:accountantContactLastName");
				if (LListGetFirstRecordData(pContact->pListPhone)) {
					char *pPhone
						= LListGetFirstRecordData(pContact->pListPhone);
					xmlWriteLine(pStream, iTabPos++, 0, "gl-bus:accountantContactPhone", 0, 0);
					while (pPhone) {
						xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:accountantContactPhoneNumber", pPhone, "gl-bus:accountantContactPhoneNumber");
						pPhone = LListGetNextRecordData(pContact->pListPhone);
					}
					xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-bus:accountantContactPhone");
				}
				if (LListGetFirstRecordData(pContact->pListEmail)) {
					char *pEmail
						= LListGetFirstRecordData(pContact->pListEmail);
					xmlWriteLine(pStream, iTabPos++, 0, "gl-bus:accountantContactEMail", 0, 0);
					while (pEmail) {
						xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:accountantContactEmailAddress", pEmail, "gl-bus:accountantContactEmailAddress");
						pEmail = LListGetNextRecordData(pContact->pListEmail);
					}
					xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-bus:accountantContactEMail");
				}
				xmlWriteLine(pStream, iTabPos, pContextRef, "gl-bus:accountantContactActive", "1", "gl-bus:accountantContactActive");
				pContact = LListGetNextRecordData(pAccounting->pAccountant->pListContact);
			}
		}
		xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-bus:accountantContactInformation");
		xmlWriteLine(pStream, --iTabPos, 0, 0, 0, "gl-bus:accountantInformation");
	}
	return 0; // OK
}


